# -*- coding: utf-8 -*-
# Taken from http://standards.ieee.org/regauth/oui/oui.txt

vendors = {
    "000000": "XEROX CORPORATION",
    "000001": "XEROX CORPORATION",
    "000002": "XEROX CORPORATION",
    "000003": "XEROX CORPORATION",
    "000004": "XEROX CORPORATION",
    "000005": "XEROX CORPORATION",
    "000006": "XEROX CORPORATION",
    "000007": "XEROX CORPORATION",
    "000008": "XEROX CORPORATION",
    "000009": "XEROX CORPORATION",
    "00000A": "OMRON TATEISI ELECTRONICS CO.",
    "00000B": "MATRIX CORPORATION",
    "00000C": "CISCO SYSTEMS, INC.",
    "00000D": "FIBRONICS LTD.",
    "00000E": "FUJITSU LIMITED",
    "00000F": "NEXT, INC.",
    "000010": "SYTEK INC.",
    "000011": "NORMEREL SYSTEMES",
    "000012": "INFORMATION TECHNOLOGY LIMITED",
    "000013": "CAMEX",
    "000014": "NETRONIX",
    "000015": "DATAPOINT CORPORATION",
    "000016": "DU PONT PIXEL SYSTEMS .",
    "000017": "TEKELEC",
    "000018": "WEBSTER COMPUTER CORPORATION",
    "000019": "APPLIED DYNAMICS INTERNATIONAL",
    "00001A": "ADVANCED MICRO DEVICES",
    "00001B": "NOVELL INC.",
    "00001C": "BELL TECHNOLOGIES",
    "00001D": "CABLETRON SYSTEMS, INC.",
    "00001E": "TELSIST INDUSTRIA ELECTRONICA",
    "00001F": "Telco Systems, Inc.",
    "000020": "DATAINDUSTRIER DIAB AB",
    "000021": "SUREMAN COMP. & COMMUN. CORP.",
    "000022": "VISUAL TECHNOLOGY INC.",
    "000023": "ABB INDUSTRIAL SYSTEMS AB",
    "000024": "CONNECT AS",
    "000025": "RAMTEK CORP.",
    "000026": "SHA-KEN CO., LTD.",
    "000027": "JAPAN RADIO COMPANY",
    "000028": "PRODIGY SYSTEMS CORPORATION",
    "000029": "IMC NETWORKS CORP.",
    "00002A": "TRW - SEDD/INP",
    "00002B": "CRISP AUTOMATION, INC",
    "00002C": "AUTOTOTE LIMITED",
    "00002D": "CHROMATICS INC",
    "00002E": "SOCIETE EVIRA",
    "00002F": "TIMEPLEX INC.",
    "000030": "VG LABORATORY SYSTEMS LTD",
    "000031": "QPSX COMMUNICATIONS PTY LTD",
    "000032": "Marconi plc",
    "000033": "EGAN MACHINERY COMPANY",
    "000034": "NETWORK RESOURCES CORPORATION",
    "000035": "SPECTRAGRAPHICS CORPORATION",
    "000036": "ATARI CORPORATION",
    "000037": "OXFORD METRICS LIMITED",
    "000038": "CSS LABS",
    "000039": "TOSHIBA CORPORATION",
    "00003A": "CHYRON CORPORATION",
    "00003B": "i Controls, Inc.",
    "00003C": "AUSPEX SYSTEMS INC.",
    "00003D": "UNISYS",
    "00003E": "SIMPACT",
    "00003F": "SYNTREX, INC.",
    "000040": "APPLICON, INC.",
    "000041": "ICE CORPORATION",
    "000042": "METIER MANAGEMENT SYSTEMS LTD.",
    "000043": "MICRO TECHNOLOGY",
    "000044": "CASTELLE CORPORATION",
    "000045": "FORD AEROSPACE & COMM. CORP.",
    "000046": "OLIVETTI NORTH AMERICA",
    "000047": "NICOLET INSTRUMENTS CORP.",
    "000048": "SEIKO EPSON CORPORATION",
    "000049": "APRICOT COMPUTERS, LTD",
    "00004A": "ADC CODENOLL TECHNOLOGY CORP.",
    "00004B": "ICL DATA OY",
    "00004C": "NEC CORPORATION",
    "00004D": "DCI CORPORATION",
    "00004E": "AMPEX CORPORATION",
    "00004F": "LOGICRAFT, INC.",
    "000050": "RADISYS CORPORATION",
    "000051": "HOB ELECTRONIC GMBH & CO. KG",
    "000052": "Intrusion.com, Inc.",
    "000053": "COMPUCORP",
    "000054": "MODICON, INC.",
    "000055": "COMMISSARIAT A L`ENERGIE ATOM.",
    "000056": "DR. B. STRUCK",
    "000057": "SCITEX CORPORATION LTD.",
    "000058": "RACORE COMPUTER PRODUCTS INC.",
    "000059": "HELLIGE GMBH",
    "00005A": "SysKonnect GmbH",
    "00005B": "ELTEC ELEKTRONIK AG",
    "00005C": "TELEMATICS INTERNATIONAL INC.",
    "00005D": "CS TELECOM",
    "00005E": "USC INFORMATION SCIENCES INST",
    "00005F": "SUMITOMO ELECTRIC IND., LTD.",
    "000060": "KONTRON ELEKTRONIK GMBH",
    "000061": "GATEWAY COMMUNICATIONS",
    "000062": "BULL HN INFORMATION SYSTEMS",
    "000063": "BARCO CONTROL ROOMS GMBH",
    "000064": "YOKOGAWA DIGITAL COMPUTER CORP",
    "000065": "Network General Corporation",
    "000066": "TALARIS SYSTEMS, INC.",
    "000067": "SOFT * RITE, INC.",
    "000068": "ROSEMOUNT CONTROLS",
    "000069": "CONCORD COMMUNICATIONS INC",
    "00006A": "COMPUTER CONSOLES INC.",
    "00006B": "SILICON GRAPHICS INC./MIPS",
    "00006C": "PRIVATE",
    "00006D": "CRAY COMMUNICATIONS, LTD.",
    "00006E": "ARTISOFT, INC.",
    "00006F": "Madge Ltd.",
    "000070": "HCL LIMITED",
    "000071": "ADRA SYSTEMS INC.",
    "000072": "MINIWARE TECHNOLOGY",
    "000073": "SIECOR CORPORATION",
    "000074": "RICOH COMPANY LTD.",
    "000075": "Nortel Networks",
    "000076": "ABEKAS VIDEO SYSTEM",
    "000077": "INTERPHASE CORPORATION",
    "000078": "LABTAM LIMITED",
    "000079": "NETWORTH INCORPORATED",
    "00007A": "DANA COMPUTER INC.",
    "00007B": "RESEARCH MACHINES",
    "00007C": "AMPERE INCORPORATED",
    "00007D": "SUN MICROSYSTEMS, INC.",
    "00007E": "CLUSTRIX CORPORATION",
    "00007F": "LINOTYPE-HELL AG",
    "000080": "CRAY COMMUNICATIONS A/S",
    "000081": "BAY NETWORKS",
    "000082": "LECTRA SYSTEMES SA",
    "000083": "TADPOLE TECHNOLOGY PLC",
    "000084": "SUPERNET",
    "000085": "CANON INC.",
    "000086": "MEGAHERTZ CORPORATION",
    "000087": "HITACHI, LTD.",
    "000088": "Brocade Communications Systems, Inc.",
    "000089": "CAYMAN SYSTEMS INC.",
    "00008A": "DATAHOUSE INFORMATION SYSTEMS",
    "00008B": "INFOTRON",
    "00008C": "Alloy Computer Products (Australia) Pty Ltd",
    "00008D": "Cryptek Inc.",
    "00008E": "SOLBOURNE COMPUTER, INC.",
    "00008F": "Raytheon",
    "000090": "MICROCOM",
    "000091": "ANRITSU CORPORATION",
    "000092": "COGENT DATA TECHNOLOGIES",
    "000093": "PROTEON INC.",
    "000094": "ASANTE TECHNOLOGIES",
    "000095": "SONY TEKTRONIX CORP.",
    "000096": "MARCONI ELECTRONICS LTD.",
    "000097": "EMC Corporation",
    "000098": "CROSSCOMM CORPORATION",
    "000099": "MTX, INC.",
    "00009A": "RC COMPUTER A/S",
    "00009B": "INFORMATION INTERNATIONAL, INC",
    "00009C": "ROLM MIL-SPEC COMPUTERS",
    "00009D": "LOCUS COMPUTING CORPORATION",
    "00009E": "MARLI S.A.",
    "00009F": "AMERISTAR TECHNOLOGIES INC.",
    "0000A0": "SANYO Electric Co., Ltd.",
    "0000A1": "MARQUETTE ELECTRIC CO.",
    "0000A2": "BAY NETWORKS",
    "0000A3": "NETWORK APPLICATION TECHNOLOGY",
    "0000A4": "ACORN COMPUTERS LIMITED",
    "0000A5": "COMPATIBLE SYSTEMS CORP.",
    "0000A6": "NETWORK GENERAL CORPORATION",
    "0000A7": "NETWORK COMPUTING DEVICES INC.",
    "0000A8": "STRATUS COMPUTER INC.",
    "0000A9": "NETWORK SYSTEMS CORP.",
    "0000AA": "XEROX CORPORATION",
    "0000AB": "LOGIC MODELING CORPORATION",
    "0000AC": "CONWARE COMPUTER CONSULTING",
    "0000AD": "BRUKER INSTRUMENTS INC.",
    "0000AE": "DASSAULT ELECTRONIQUE",
    "0000AF": "NUCLEAR DATA INSTRUMENTATION",
    "0000B0": "RND-RAD NETWORK DEVICES",
    "0000B1": "ALPHA MICROSYSTEMS INC.",
    "0000B2": "TELEVIDEO SYSTEMS, INC.",
    "0000B3": "CIMLINC INCORPORATED",
    "0000B4": "EDIMAX COMPUTER COMPANY",
    "0000B5": "DATABILITY SOFTWARE SYS. INC.",
    "0000B6": "MICRO-MATIC RESEARCH",
    "0000B7": "DOVE COMPUTER CORPORATION",
    "0000B8": "SEIKOSHA CO., LTD.",
    "0000B9": "MCDONNELL DOUGLAS COMPUTER SYS",
    "0000BA": "SIIG, INC.",
    "0000BB": "TRI-DATA",
    "0000BC": "Rockwell Automation",
    "0000BD": "MITSUBISHI CABLE COMPANY",
    "0000BE": "THE NTI GROUP",
    "0000BF": "SYMMETRIC COMPUTER SYSTEMS",
    "0000C0": "WESTERN DIGITAL CORPORATION",
    "0000C1": "Madge Ltd.",
    "0000C2": "INFORMATION PRESENTATION TECH.",
    "0000C3": "HARRIS CORP COMPUTER SYS DIV",
    "0000C4": "WATERS DIV. OF MILLIPORE",
    "0000C5": "FARALLON COMPUTING/NETOPIA",
    "0000C6": "EON SYSTEMS",
    "0000C7": "ARIX CORPORATION",
    "0000C8": "ALTOS COMPUTER SYSTEMS",
    "0000C9": "EMULEX CORPORATION",
    "0000CA": "ARRIS International",
    "0000CB": "COMPU-SHACK ELECTRONIC GMBH",
    "0000CC": "DENSAN CO., LTD.",
    "0000CD": "Allied Telesis Labs Ltd",
    "0000CE": "MEGADATA CORP.",
    "0000CF": "HAYES MICROCOMPUTER PRODUCTS",
    "0000D0": "DEVELCON ELECTRONICS LTD.",
    "0000D1": "ADAPTEC INCORPORATED",
    "0000D2": "SBE, INC.",
    "0000D3": "WANG LABORATORIES INC.",
    "0000D4": "PURE DATA LTD.",
    "0000D5": "MICROGNOSIS INTERNATIONAL",
    "0000D6": "PUNCH LINE HOLDING",
    "0000D7": "DARTMOUTH COLLEGE",
    "0000D8": "NOVELL, INC.",
    "0000D9": "NIPPON TELEGRAPH & TELEPHONE",
    "0000DA": "ATEX",
    "0000DB": "BRITISH TELECOMMUNICATIONS PLC",
    "0000DC": "HAYES MICROCOMPUTER PRODUCTS",
    "0000DD": "TCL INCORPORATED",
    "0000DE": "CETIA",
    "0000DF": "BELL & HOWELL PUB SYS DIV",
    "0000E0": "QUADRAM CORP.",
    "0000E1": "GRID SYSTEMS",
    "0000E2": "ACER TECHNOLOGIES CORP.",
    "0000E3": "INTEGRATED MICRO PRODUCTS LTD",
    "0000E4": "IN2 GROUPE INTERTECHNIQUE",
    "0000E5": "SIGMEX LTD.",
    "0000E6": "APTOR PRODUITS DE COMM INDUST",
    "0000E7": "STAR GATE TECHNOLOGIES",
    "0000E8": "ACCTON TECHNOLOGY CORP.",
    "0000E9": "ISICAD, INC.",
    "0000EA": "UPNOD AB",
    "0000EB": "MATSUSHITA COMM. IND. CO. LTD.",
    "0000EC": "MICROPROCESS",
    "0000ED": "APRIL",
    "0000EE": "NETWORK DESIGNERS, LTD.",
    "0000EF": "KTI",
    "0000F0": "SAMSUNG ELECTRONICS CO., LTD.",
    "0000F1": "MAGNA COMPUTER CORPORATION",
    "0000F2": "SPIDER COMMUNICATIONS",
    "0000F3": "GANDALF DATA LIMITED",
    "0000F4": "ALLIED TELESYN INTERNATIONAL",
    "0000F5": "DIAMOND SALES LIMITED",
    "0000F6": "APPLIED MICROSYSTEMS CORP.",
    "0000F7": "YOUTH KEEP ENTERPRISE CO LTD",
    "0000F8": "DIGITAL EQUIPMENT CORPORATION",
    "0000F9": "QUOTRON SYSTEMS INC.",
    "0000FA": "MICROSAGE COMPUTER SYSTEMS INC",
    "0000FB": "RECHNER ZUR KOMMUNIKATION",
    "0000FC": "MEIKO",
    "0000FD": "HIGH LEVEL HARDWARE",
    "0000FE": "ANNAPOLIS MICRO SYSTEMS",
    "0000FF": "CAMTEC ELECTRONICS LTD.",
    "000100": "EQUIP'TRANS",
    "000101": "PRIVATE",
    "000102": "3COM CORPORATION",
    "000103": "3COM CORPORATION",
    "000104": "DVICO Co., Ltd.",
    "000105": "Beckhoff Automation GmbH",
    "000106": "Tews Datentechnik GmbH",
    "000107": "Leiser GmbH",
    "000108": "AVLAB Technology, Inc.",
    "000109": "Nagano Japan Radio Co., Ltd.",
    "00010A": "CIS TECHNOLOGY INC.",
    "00010B": "Space CyberLink, Inc.",
    "00010C": "System Talks Inc.",
    "00010D": "CORECO, INC.",
    "00010E": "Bri-Link Technologies Co., Ltd",
    "00010F": "Brocade Communications Systems, Inc.",
    "000110": "Gotham Networks",
    "000111": "iDigm Inc.",
    "000112": "Shark Multimedia Inc.",
    "000113": "OLYMPUS CORPORATION",
    "000114": "KANDA TSUSHIN KOGYO CO., LTD.",
    "000115": "EXTRATECH CORPORATION",
    "000116": "Netspect Technologies, Inc.",
    "000117": "CANAL +",
    "000118": "EZ Digital Co., Ltd.",
    "000119": "RTUnet (Australia)",
    "00011A": "EEH DataLink GmbH",
    "00011B": "Unizone Technologies, Inc.",
    "00011C": "Universal Talkware Corporation",
    "00011D": "Centillium Communications",
    "00011E": "Precidia Technologies, Inc.",
    "00011F": "RC Networks, Inc.",
    "000120": "OSCILLOQUARTZ S.A.",
    "000121": "Watchguard Technologies, Inc.",
    "000122": "Trend Communications, Ltd.",
    "000123": "DIGITAL ELECTRONICS CORP.",
    "000124": "Acer Incorporated",
    "000125": "YAESU MUSEN CO., LTD.",
    "000126": "PAC Labs",
    "000127": "OPEN Networks Pty Ltd",
    "000128": "EnjoyWeb, Inc.",
    "000129": "DFI Inc.",
    "00012A": "Telematica Sistems Inteligente",
    "00012B": "TELENET Co., Ltd.",
    "00012C": "Aravox Technologies, Inc.",
    "00012D": "Komodo Technology",
    "00012E": "PC Partner Ltd.",
    "00012F": "Twinhead International Corp",
    "000130": "Extreme Networks",
    "000131": "Detection Systems, Inc.",
    "000132": "Dranetz - BMI",
    "000133": "KYOWA Electronic Instruments C",
    "000134": "SIG Positec Systems AG",
    "000135": "KDC Corp.",
    "000136": "CyberTAN Technology, Inc.",
    "000137": "IT Farm Corporation",
    "000138": "XAVi Technologies Corp.",
    "000139": "Point Multimedia Systems",
    "00013A": "SHELCAD COMMUNICATIONS, LTD.",
    "00013B": "BNA SYSTEMS",
    "00013C": "TIW SYSTEMS",
    "00013D": "RiscStation Ltd.",
    "00013E": "Ascom Tateco AB",
    "00013F": "Neighbor World Co., Ltd.",
    "000140": "Sendtek Corporation",
    "000141": "CABLE PRINT",
    "000142": "Cisco Systems, Inc.",
    "000143": "Cisco Systems, Inc.",
    "000144": "EMC Corporation",
    "000145": "WINSYSTEMS, INC.",
    "000146": "Tesco Controls, Inc.",
    "000147": "Zhone Technologies",
    "000148": "X-traWeb Inc.",
    "000149": "T.D.T. Transfer Data Test GmbH",
    "00014A": "Sony Corporation",
    "00014B": "Ennovate Networks, Inc.",
    "00014C": "Berkeley Process Control",
    "00014D": "Shin Kin Enterprises Co., Ltd",
    "00014E": "WIN Enterprises, Inc.",
    "00014F": "ADTRAN INC",
    "000150": "GILAT COMMUNICATIONS, LTD.",
    "000151": "Ensemble Communications",
    "000152": "CHROMATEK INC.",
    "000153": "ARCHTEK TELECOM CORPORATION",
    "000154": "G3M Corporation",
    "000155": "Promise Technology, Inc.",
    "000156": "FIREWIREDIRECT.COM, INC.",
    "000157": "SYSWAVE CO., LTD",
    "000158": "Electro Industries/Gauge Tech",
    "000159": "S1 Corporation",
    "00015A": "Digital Video Broadcasting",
    "00015B": "ITALTEL S.p.A/RF-UP-I",
    "00015C": "CADANT INC.",
    "00015D": "Sun Microsystems, Inc",
    "00015E": "BEST TECHNOLOGY CO., LTD.",
    "00015F": "DIGITAL DESIGN GmbH",
    "000160": "ELMEX Co., LTD.",
    "000161": "Meta Machine Technology",
    "000162": "Cygnet Technologies, Inc.",
    "000163": "Cisco Systems, Inc.",
    "000164": "Cisco Systems, Inc.",
    "000165": "AirSwitch Corporation",
    "000166": "TC GROUP A/S",
    "000167": "HIOKI E.E. CORPORATION",
    "000168": "VITANA CORPORATION",
    "000169": "Celestix Networks Pte Ltd.",
    "00016A": "ALITEC",
    "00016B": "LightChip, Inc.",
    "00016C": "FOXCONN",
    "00016D": "CarrierComm Inc.",
    "00016E": "Conklin Corporation",
    "00016F": "HAITAI ELECTRONICS CO., LTD.",
    "000170": "ESE Embedded System Engineer'g",
    "000171": "Allied Data Technologies",
    "000172": "TechnoLand Co., LTD.",
    "000173": "AMCC",
    "000174": "CyberOptics Corporation",
    "000175": "Radiant Communications Corp.",
    "000176": "Orient Silver Enterprises",
    "000177": "EDSL",
    "000178": "MARGI Systems, Inc.",
    "000179": "WIRELESS TECHNOLOGY, INC.",
    "00017A": "Chengdu Maipu Electric Industrial Co., Ltd.",
    "00017B": "Heidelberger Druckmaschinen AG",
    "00017C": "AG-E GmbH",
    "00017D": "ThermoQuest",
    "00017E": "ADTEK System Science Co., Ltd.",
    "00017F": "Experience Music Project",
    "000180": "AOpen, Inc.",
    "000181": "Nortel Networks",
    "000182": "DICA TECHNOLOGIES AG",
    "000183": "ANITE TELECOMS",
    "000184": "SIEB & MEYER AG",
    "000185": "Aloka Co., Ltd.",
    "000186": "Uwe Disch",
    "000187": "i2SE GmbH",
    "000188": "LXCO Technologies ag",
    "000189": "Refraction Technology, Inc.",
    "00018A": "ROI COMPUTER AG",
    "00018B": "NetLinks Co., Ltd.",
    "00018C": "Mega Vision",
    "00018D": "AudeSi Technologies",
    "00018E": "Logitec Corporation",
    "00018F": "Kenetec, Inc.",
    "000190": "SMK-M",
    "000191": "SYRED Data Systems",
    "000192": "Texas Digital Systems",
    "000193": "Hanbyul Telecom Co., Ltd.",
    "000194": "Capital Equipment Corporation",
    "000195": "Sena Technologies, Inc.",
    "000196": "Cisco Systems, Inc.",
    "000197": "Cisco Systems, Inc.",
    "000198": "Darim Vision",
    "000199": "HeiSei Electronics",
    "00019A": "LEUNIG GmbH",
    "00019B": "Kyoto Microcomputer Co., Ltd.",
    "00019C": "JDS Uniphase Inc.",
    "00019D": "E-Control Systems, Inc.",
    "00019E": "ESS Technology, Inc.",
    "00019F": "Phonex Broadband",
    "0001A0": "Infinilink Corporation",
    "0001A1": "Mag-Tek, Inc.",
    "0001A2": "Logical Co., Ltd.",
    "0001A3": "GENESYS LOGIC, INC.",
    "0001A4": "Microlink Corporation",
    "0001A5": "Nextcomm, Inc.",
    "0001A6": "Scientific-Atlanta Arcodan A/S",
    "0001A7": "UNEX TECHNOLOGY CORPORATION",
    "0001A8": "Welltech Computer Co., Ltd.",
    "0001A9": "BMW AG",
    "0001AA": "Airspan Communications, Ltd.",
    "0001AB": "Main Street Networks",
    "0001AC": "Sitara Networks, Inc.",
    "0001AD": "Coach Master International d.b.a. CMI Worldwide, Inc.",
    "0001AE": "Trex Enterprises",
    "0001AF": "Emerson Network Power",
    "0001B0": "Fulltek Technology Co., Ltd.",
    "0001B1": "General Bandwidth",
    "0001B2": "Digital Processing Systems, Inc.",
    "0001B3": "Precision Electronic Manufacturing",
    "0001B4": "Wayport, Inc.",
    "0001B5": "Turin Networks, Inc.",
    "0001B6": "SAEJIN T&M Co., Ltd.",
    "0001B7": "Centos, Inc.",
    "0001B8": "Netsensity, Inc.",
    "0001B9": "SKF Condition Monitoring",
    "0001BA": "IC-Net, Inc.",
    "0001BB": "Frequentis",
    "0001BC": "Brains Corporation",
    "0001BD": "Peterson Electro-Musical Products, Inc.",
    "0001BE": "Gigalink Co., Ltd.",
    "0001BF": "Teleforce Co., Ltd.",
    "0001C0": "CompuLab, Ltd.",
    "0001C1": "Vitesse Semiconductor Corporation",
    "0001C2": "ARK Research Corp.",
    "0001C3": "Acromag, Inc.",
    "0001C4": "NeoWave, Inc.",
    "0001C5": "Simpler Networks",
    "0001C6": "Quarry Technologies",
    "0001C7": "Cisco Systems, Inc.",
    "0001C8": "THOMAS CONRAD CORP.",
    "0001C8": "CONRAD CORP.",
    "0001C9": "Cisco Systems, Inc.",
    "0001CA": "Geocast Network Systems, Inc.",
    "0001CB": "EVR",
    "0001CC": "Japan Total Design Communication Co., Ltd.",
    "0001CD": "ARtem",
    "0001CE": "Custom Micro Products, Ltd.",
    "0001CF": "Alpha Data Parallel Systems, Ltd.",
    "0001D0": "VitalPoint, Inc.",
    "0001D1": "CoNet Communications, Inc.",
    "0001D2": "MacPower Peripherals, Ltd.",
    "0001D3": "PAXCOMM, Inc.",
    "0001D4": "Leisure Time, Inc.",
    "0001D5": "HAEDONG INFO & COMM CO., LTD",
    "0001D6": "MAN Roland Druckmaschinen AG",
    "0001D7": "F5 Networks, Inc.",
    "0001D8": "Teltronics, Inc.",
    "0001D9": "Sigma, Inc.",
    "0001DA": "WINCOMM Corporation",
    "0001DB": "Freecom Technologies GmbH",
    "0001DC": "Activetelco",
    "0001DD": "Avail Networks",
    "0001DE": "Trango Systems, Inc.",
    "0001DF": "ISDN Communications, Ltd.",
    "0001E0": "Fast Systems, Inc.",
    "0001E1": "Kinpo Electronics, Inc.",
    "0001E2": "Ando Electric Corporation",
    "0001E3": "Siemens AG",
    "0001E4": "Sitera, Inc.",
    "0001E5": "Supernet, Inc.",
    "0001E6": "Hewlett-Packard Company",
    "0001E7": "Hewlett-Packard Company",
    "0001E8": "Force10 Networks, Inc.",
    "0001E9": "Litton Marine Systems B.V.",
    "0001EA": "Cirilium Corp.",
    "0001EB": "C-COM Corporation",
    "0001EC": "Ericsson Group",
    "0001ED": "SETA Corp.",
    "0001EE": "Comtrol Europe, Ltd.",
    "0001EF": "Camtel Technology Corp.",
    "0001F0": "Tridium, Inc.",
    "0001F1": "Innovative Concepts, Inc.",
    "0001F2": "Mark of the Unicorn, Inc.",
    "0001F3": "QPS, Inc.",
    "0001F4": "Enterasys Networks",
    "0001F5": "ERIM S.A.",
    "0001F6": "Association of Musical Electronics Industry",
    "0001F7": "Image Display Systems, Inc.",
    "0001F8": "Adherent Systems, Ltd.",
    "0001F9": "TeraGlobal Communications Corp.",
    "0001FA": "HOROSCAS",
    "0001FB": "DoTop Technology, Inc.",
    "0001FC": "Keyence Corporation",
    "0001FD": "Digital Voice Systems, Inc.",
    "0001FE": "DIGITAL EQUIPMENT CORPORATION",
    "0001FF": "Data Direct Networks, Inc.",
    "000200": "Net & Sys Co., Ltd.",
    "000201": "IFM Electronic gmbh",
    "000202": "Amino Communications, Ltd.",
    "000203": "Woonsang Telecom, Inc.",
    "000204": "Bodmann Industries Elektronik GmbH",
    "000205": "Hitachi Denshi, Ltd.",
    "000206": "Telital R&D Denmark A/S",
    "000207": "VisionGlobal Network Corp.",
    "000208": "Unify Networks, Inc.",
    "000209": "Shenzhen SED Information Technology Co., Ltd.",
    "00020A": "Gefran Spa",
    "00020B": "Native Networks, Inc.",
    "00020C": "Metro-Optix",
    "00020D": "Micronpc.com",
    "00020E": "ECI Telecom, Ltd., NSD-US",
    "00020F": "AATR",
    "000210": "Fenecom",
    "000211": "Nature Worldwide Technology Corp.",
    "000212": "SierraCom",
    "000213": "S.D.E.L.",
    "000214": "DTVRO",
    "000215": "Cotas Computer Technology A/B",
    "000216": "Cisco Systems, Inc.",
    "000217": "Cisco Systems, Inc.",
    "000218": "Advanced Scientific Corp",
    "000219": "Paralon Technologies",
    "00021A": "Zuma Networks",
    "00021B": "Kollmorgen-Servotronix",
    "00021C": "Network Elements, Inc.",
    "00021D": "Data General Communication Ltd.",
    "00021E": "SIMTEL S.R.L.",
    "00021F": "Aculab PLC",
    "000220": "Canon Aptex, Inc.",
    "000221": "DSP Application, Ltd.",
    "000222": "Chromisys, Inc.",
    "000223": "ClickTV",
    "000224": "C-COR",
    "000225": "One Stop Systems",
    "000226": "XESystems, Inc.",
    "000227": "ESD Electronic System Design GmbH",
    "000228": "Necsom, Ltd.",
    "000229": "Adtec Corporation",
    "00022A": "Asound Electronic",
    "00022B": "SAXA, Inc.",
    "00022C": "ABB Bomem, Inc.",
    "00022D": "Agere Systems",
    "00022E": "TEAC Corp. R& D",
    "00022F": "P-Cube, Ltd.",
    "000230": "Intersoft Electronics",
    "000231": "Ingersoll-Rand",
    "000232": "Avision, Inc.",
    "000233": "Mantra Communications, Inc.",
    "000234": "Imperial Technology, Inc.",
    "000235": "Paragon Networks International",
    "000236": "INIT GmbH",
    "000237": "Cosmo Research Corp.",
    "000238": "Serome Technology, Inc.",
    "000239": "Visicom",
    "00023A": "ZSK Stickmaschinen GmbH",
    "00023B": "Redback Networks",
    "00023C": "Creative Technology, Ltd.",
    "00023D": "NuSpeed, Inc.",
    "00023E": "Selta Telematica S.p.a",
    "00023F": "Compal Electronics, Inc.",
    "000240": "Seedek Co., Ltd.",
    "000241": "Amer.com",
    "000242": "Videoframe Systems",
    "000243": "Raysis Co., Ltd.",
    "000244": "SURECOM Technology Co.",
    "000245": "Lampus Co, Ltd.",
    "000246": "All-Win Tech Co., Ltd.",
    "000247": "Great Dragon Information Technology (Group) Co., Ltd.",
    "000248": "Pilz GmbH & Co.",
    "000249": "Aviv Infocom Co, Ltd.",
    "00024A": "Cisco Systems, Inc.",
    "00024B": "Cisco Systems, Inc.",
    "00024C": "SiByte, Inc.",
    "00024D": "Mannesman Dematic Colby Pty. Ltd.",
    "00024E": "Datacard Group",
    "00024F": "IPM Datacom S.R.L.",
    "000250": "Geyser Networks, Inc.",
    "000251": "Soma Networks, Inc.",
    "000252": "Carrier Corporation",
    "000253": "Televideo, Inc.",
    "000254": "WorldGate",
    "000255": "IBM Corporation",
    "000256": "Alpha Processor, Inc.",
    "000257": "Microcom Corp.",
    "000258": "Flying Packets Communications",
    "000259": "Tsann Kuen China (Shanghai)Enterprise Co., Ltd. IT Group",
    "00025A": "Catena Networks",
    "00025B": "Cambridge Silicon Radio",
    "00025C": "SCI Systems (Kunshan) Co., Ltd.",
    "00025D": "Calix Networks",
    "00025E": "High Technology Ltd",
    "00025F": "Nortel Networks",
    "000260": "Accordion Networks, Inc.",
    "000261": "Tilgin AB",
    "000262": "Soyo Group Soyo Com Tech Co., Ltd",
    "000263": "UPS Manufacturing SRL",
    "000264": "AudioRamp.com",
    "000265": "Virditech Co. Ltd.",
    "000266": "Thermalogic Corporation",
    "000267": "NODE RUNNER, INC.",
    "000268": "Harris Government Communications",
    "000269": "Nadatel Co., Ltd",
    "00026A": "Cocess Telecom Co., Ltd.",
    "00026B": "BCM Computers Co., Ltd.",
    "00026C": "Philips CFT",
    "00026D": "Adept Telecom",
    "00026E": "NeGeN Access, Inc.",
    "00026F": "Senao International Co., Ltd.",
    "000270": "Crewave Co., Ltd.",
    "000271": "Vpacket Communications",
    "000272": "CC&C Technologies, Inc.",
    "000273": "Coriolis Networks",
    "000274": "Tommy Technologies Corp.",
    "000275": "SMART Technologies, Inc.",
    "000276": "Primax Electronics Ltd.",
    "000277": "Cash Systemes Industrie",
    "000278": "Samsung Electro-Mechanics Co., Ltd.",
    "000279": "Control Applications, Ltd.",
    "00027A": "IOI Technology Corporation",
    "00027B": "Amplify Net, Inc.",
    "00027C": "Trilithic, Inc.",
    "00027D": "Cisco Systems, Inc.",
    "00027E": "Cisco Systems, Inc.",
    "00027F": "ask-technologies.com",
    "000280": "Mu Net, Inc.",
    "000281": "Madge Ltd.",
    "000282": "ViaClix, Inc.",
    "000283": "Spectrum Controls, Inc.",
    "000284": "AREVA T&D",
    "000285": "Riverstone Networks",
    "000286": "Occam Networks",
    "000287": "Adapcom",
    "000288": "GLOBAL VILLAGE COMMUNICATION",
    "000289": "DNE Technologies",
    "00028A": "Ambit Microsystems Corporation",
    "00028B": "VDSL Systems OY",
    "00028C": "Micrel-Synergy Semiconductor",
    "00028D": "Movita Technologies, Inc.",
    "00028E": "Rapid 5 Networks, Inc.",
    "00028F": "Globetek, Inc.",
    "000290": "Woorigisool, Inc.",
    "000291": "Open Network Co., Ltd.",
    "000292": "Logic Innovations, Inc.",
    "000293": "Solid Data Systems",
    "000294": "Tokyo Sokushin Co., Ltd.",
    "000295": "IP.Access Limited",
    "000296": "Lectron Co,. Ltd.",
    "000297": "C-COR.net",
    "000298": "Broadframe Corporation",
    "000299": "Apex, Inc.",
    "00029A": "Storage Apps",
    "00029B": "Kreatel Communications AB",
    "00029C": "3COM",
    "00029D": "Merix Corp.",
    "00029E": "Information Equipment Co., Ltd.",
    "00029F": "L-3 Communication Aviation Recorders",
    "0002A0": "Flatstack Ltd.",
    "0002A1": "World Wide Packets",
    "0002A2": "Hilscher GmbH",
    "0002A3": "ABB Power Automation",
    "0002A4": "AddPac Technology Co., Ltd.",
    "0002A5": "Hewlett Packard",
    "0002A6": "Effinet Systems Co., Ltd.",
    "0002A7": "Vivace Networks",
    "0002A8": "Air Link Technology",
    "0002A9": "RACOM, s.r.o.",
    "0002AA": "PLcom Co., Ltd.",
    "0002AB": "CTC Union Technologies Co., Ltd.",
    "0002AC": "3PAR data",
    "0002AD": "HOYA Corporation",
    "0002AE": "Scannex Electronics Ltd.",
    "0002AF": "TeleCruz Technology, Inc.",
    "0002B0": "Hokubu Communication & Industrial Co., Ltd.",
    "0002B1": "Anritsu, Ltd.",
    "0002B2": "Cablevision",
    "0002B3": "Intel Corporation",
    "0002B4": "DAPHNE",
    "0002B5": "Avnet, Inc.",
    "0002B6": "Acrosser Technology Co., Ltd.",
    "0002B7": "Watanabe Electric Industry Co., Ltd.",
    "0002B8": "WHI KONSULT AB",
    "0002B9": "Cisco Systems, Inc.",
    "0002BA": "Cisco Systems, Inc.",
    "0002BB": "Continuous Computing Corp",
    "0002BC": "LVL 7 Systems, Inc.",
    "0002BD": "Bionet Co., Ltd.",
    "0002BE": "Totsu Engineering, Inc.",
    "0002BF": "dotRocket, Inc.",
    "0002C0": "Bencent Tzeng Industry Co., Ltd.",
    "0002C1": "Innovative Electronic Designs, Inc.",
    "0002C2": "Net Vision Telecom",
    "0002C3": "Arelnet Ltd.",
    "0002C4": "Vector International BVBA",
    "0002C5": "Evertz Microsystems Ltd.",
    "0002C6": "Data Track Technology PLC",
    "0002C7": "ALPS ELECTRIC Co., Ltd.",
    "0002C8": "Technocom Communications Technology (pte) Ltd",
    "0002C9": "Mellanox Technologies",
    "0002CA": "EndPoints, Inc.",
    "0002CB": "TriState Ltd.",
    "0002CC": "M.C.C.I",
    "0002CD": "TeleDream, Inc.",
    "0002CE": "FoxJet, Inc.",
    "0002CF": "ZyGate Communications, Inc.",
    "0002D0": "Comdial Corporation",
    "0002D1": "Vivotek, Inc.",
    "0002D2": "Workstation AG",
    "0002D3": "NetBotz, Inc.",
    "0002D4": "PDA Peripherals, Inc.",
    "0002D5": "ACR",
    "0002D6": "NICE Systems",
    "0002D7": "EMPEG Ltd",
    "0002D8": "BRECIS Communications Corporation",
    "0002D9": "Reliable Controls",
    "0002DA": "ExiO Communications, Inc.",
    "0002DB": "NETSEC",
    "0002DC": "Fujitsu General Limited",
    "0002DD": "Bromax Communications, Ltd.",
    "0002DE": "Astrodesign, Inc.",
    "0002DF": "Net Com Systems, Inc.",
    "0002E0": "ETAS GmbH",
    "0002E1": "Integrated Network Corporation",
    "0002E2": "NDC Infared Engineering",
    "0002E3": "LITE-ON Communications, Inc.",
    "0002E4": "JC HYUN Systems, Inc.",
    "0002E5": "Timeware Ltd.",
    "0002E6": "Gould Instrument Systems, Inc.",
    "0002E7": "CAB GmbH & Co KG",
    "0002E8": "E.D.&A.",
    "0002E9": "CS Systemes De Securite - C3S",
    "0002EA": "Focus Enhancements",
    "0002EB": "Pico Communications",
    "0002EC": "Maschoff Design Engineering",
    "0002ED": "DXO Telecom Co., Ltd.",
    "0002EE": "Nokia Danmark A/S",
    "0002EF": "CCC Network Systems Group Ltd.",
    "0002F0": "AME Optimedia Technology Co., Ltd.",
    "0002F1": "Pinetron Co., Ltd.",
    "0002F2": "eDevice, Inc.",
    "0002F3": "Media Serve Co., Ltd.",
    "0002F4": "PCTEL, Inc.",
    "0002F5": "VIVE Synergies, Inc.",
    "0002F6": "Equipe Communications",
    "0002F7": "ARM",
    "0002F8": "SEAKR Engineering, Inc.",
    "0002F9": "Mimos Semiconductor SDN BHD",
    "0002FA": "DX Antenna Co., Ltd.",
    "0002FB": "Baumuller Aulugen-Systemtechnik GmbH",
    "0002FC": "Cisco Systems, Inc.",
    "0002FD": "Cisco Systems, Inc.",
    "0002FE": "Viditec, Inc.",
    "0002FF": "Handan BroadInfoCom",
    "000300": "NetContinuum, Inc.",
    "000301": "Avantas Networks Corporation",
    "000302": "Charles Industries, Ltd.",
    "000303": "JAMA Electronics Co., Ltd.",
    "000304": "Pacific Broadband Communications",
    "000305": "Smart Network Devices GmbH",
    "000306": "Fusion In Tech Co., Ltd.",
    "000307": "Secure Works, Inc.",
    "000308": "AM Communications, Inc.",
    "000309": "Texcel Technology PLC",
    "00030A": "Argus Technologies",
    "00030B": "Hunter Technology, Inc.",
    "00030C": "Telesoft Technologies Ltd.",
    "00030D": "Uniwill Computer Corp.",
    "00030E": "Core Communications Co., Ltd.",
    "00030F": "Digital China (Shanghai) Networks Ltd.",
    "000310": "ITX E-Globaledge Corporation",
    "000311": "Micro Technology Co., Ltd.",
    "000312": "TR-Systemtechnik GmbH",
    "000313": "Access Media SPA",
    "000314": "Teleware Network Systems",
    "000315": "Cidco Incorporated",
    "000316": "Nobell Communications, Inc.",
    "000317": "Merlin Systems, Inc.",
    "000318": "Cyras Systems, Inc.",
    "000319": "Infineon AG",
    "00031A": "Beijing Broad Telecom Ltd., China",
    "00031B": "Cellvision Systems, Inc.",
    "00031C": "Svenska Hardvarufabriken AB",
    "00031D": "Taiwan Commate Computer, Inc.",
    "00031E": "Optranet, Inc.",
    "00031F": "Condev Ltd.",
    "000320": "Xpeed, Inc.",
    "000321": "Reco Research Co., Ltd.",
    "000322": "IDIS Co., Ltd.",
    "000323": "Cornet Technology, Inc.",
    "000324": "SANYO Multimedia Tottori Co., Ltd.",
    "000325": "Arima Computer Corp.",
    "000326": "Iwasaki Information Systems Co., Ltd.",
    "000327": "ACT'L",
    "000328": "Mace Group, Inc.",
    "000329": "F3, Inc.",
    "00032A": "UniData Communication Systems, Inc.",
    "00032B": "GAI Datenfunksysteme GmbH",
    "00032C": "ABB Switzerland Ltd",
    "00032D": "IBASE Technology, Inc.",
    "00032E": "Scope Information Management, Ltd.",
    "00032F": "Global Sun Technology, Inc.",
    "000330": "Imagenics, Co., Ltd.",
    "000331": "Cisco Systems, Inc.",
    "000332": "Cisco Systems, Inc.",
    "000333": "Digitel Co., Ltd.",
    "000334": "Newport Electronics",
    "000335": "Mirae Technology",
    "000336": "Zetes Technologies",
    "000337": "Vaone, Inc.",
    "000338": "Oak Technology",
    "000339": "Eurologic Systems, Ltd.",
    "00033A": "Silicon Wave, Inc.",
    "00033B": "TAMI Tech Co., Ltd.",
    "00033C": "Daiden Co., Ltd.",
    "00033D": "ILSHin Lab",
    "00033E": "Tateyama System Laboratory Co., Ltd.",
    "00033F": "BigBand Networks, Ltd.",
    "000340": "Floware Wireless Systems, Ltd.",
    "000341": "Axon Digital Design",
    "000342": "Nortel Networks",
    "000343": "Martin Professional A/S",
    "000344": "Tietech.Co., Ltd.",
    "000345": "Routrek Networks Corporation",
    "000346": "Hitachi Kokusai Electric, Inc.",
    "000347": "Intel Corporation",
    "000348": "Norscan Instruments, Ltd.",
    "000349": "Vidicode Datacommunicatie B.V.",
    "00034A": "RIAS Corporation",
    "00034B": "Nortel Networks",
    "00034C": "Shanghai DigiVision Technology Co., Ltd.",
    "00034D": "Chiaro Networks, Ltd.",
    "00034E": "Pos Data Company, Ltd.",
    "00034F": "Sur-Gard Security",
    "000350": "BTICINO SPA",
    "000351": "Diebold, Inc.",
    "000352": "Colubris Networks",
    "000353": "Mitac, Inc.",
    "000354": "Fiber Logic Communications",
    "000355": "TeraBeam Internet Systems",
    "000356": "Wincor Nixdorf International GmbH",
    "000357": "Intervoice-Brite, Inc.",
    "000358": "Hanyang Digitech Co., Ltd.",
    "000359": "DigitalSis",
    "00035A": "Photron Limited",
    "00035B": "BridgeWave Communications",
    "00035C": "Saint Song Corp.",
    "00035D": "Bosung Hi-Net Co., Ltd.",
    "00035E": "Metropolitan Area Networks, Inc.",
    "00035F": "Prueftechnik Condition Monitoring GmbH & Co. KG",
    "000360": "PAC Interactive Technology, Inc.",
    "000361": "Widcomm, Inc.",
    "000362": "Vodtel Communications, Inc.",
    "000363": "Miraesys Co., Ltd.",
    "000364": "Scenix Semiconductor, Inc.",
    "000365": "Kira Information & Communications, Ltd.",
    "000366": "ASM Pacific Technology",
    "000367": "Jasmine Networks, Inc.",
    "000368": "Embedone Co., Ltd.",
    "000369": "Nippon Antenna Co., Ltd.",
    "00036A": "Mainnet, Ltd.",
    "00036B": "Cisco Systems, Inc.",
    "00036C": "Cisco Systems, Inc.",
    "00036D": "Runtop, Inc.",
    "00036E": "Nicon Systems (Pty) Limited",
    "00036F": "Telsey SPA",
    "000370": "NXTV, Inc.",
    "000371": "Acomz Networks Corp.",
    "000372": "ULAN",
    "000373": "Aselsan A.S",
    "000374": "Hunter Watertech",
    "000375": "NetMedia, Inc.",
    "000376": "Graphtec Technology, Inc.",
    "000377": "Gigabit Wireless",
    "000378": "HUMAX Co., Ltd.",
    "000379": "Proscend Communications, Inc.",
    "00037A": "Taiyo Yuden Co., Ltd.",
    "00037B": "IDEC IZUMI Corporation",
    "00037C": "Coax Media",
    "00037D": "Stellcom",
    "00037E": "PORTech Communications, Inc.",
    "00037F": "Atheros Communications, Inc.",
    "000380": "SSH Communications Security Corp.",
    "000381": "Ingenico International",
    "000382": "A-One Co., Ltd.",
    "000383": "Metera Networks, Inc.",
    "000384": "AETA",
    "000385": "Actelis Networks, Inc.",
    "000386": "Ho Net, Inc.",
    "000387": "Blaze Network Products",
    "000388": "Fastfame Technology Co., Ltd.",
    "000389": "Plantronics",
    "00038A": "America Online, Inc.",
    "00038B": "PLUS-ONE I&T, Inc.",
    "00038C": "Total Impact",
    "00038D": "PCS Revenue Control Systems, Inc.",
    "00038E": "Atoga Systems, Inc.",
    "00038F": "Weinschel Corporation",
    "000390": "Digital Video Communications, Inc.",
    "000391": "Advanced Digital Broadcast, Ltd.",
    "000392": "Hyundai Teletek Co., Ltd.",
    "000393": "Apple Computer, Inc.",
    "000394": "Connect One",
    "000395": "California Amplifier",
    "000396": "EZ Cast Co., Ltd.",
    "000397": "Watchfront Limited",
    "000398": "WISI",
    "000399": "Dongju Informations & Communications Co., Ltd.",
    "00039A": "SiConnect",
    "00039B": "NetChip Technology, Inc.",
    "00039C": "OptiMight Communications, Inc.",
    "00039D": "Qisda Corporation",
    "00039E": "Tera System Co., Ltd.",
    "00039F": "Cisco Systems, Inc.",
    "0003A0": "Cisco Systems, Inc.",
    "0003A1": "HIPER Information & Communication, Inc.",
    "0003A2": "Catapult Communications",
    "0003A3": "MAVIX, Ltd.",
    "0003A4": "Data Storage and Information Management",
    "0003A5": "Medea Corporation",
    "0003A6": "Traxit Technology, Inc.",
    "0003A7": "Unixtar Technology, Inc.",
    "0003A8": "IDOT Computers, Inc.",
    "0003A9": "AXCENT Media AG",
    "0003AA": "Watlow",
    "0003AB": "Bridge Information Systems",
    "0003AC": "Fronius Schweissmaschinen",
    "0003AD": "Emerson Energy Systems AB",
    "0003AE": "Allied Advanced Manufacturing Pte, Ltd.",
    "0003AF": "Paragea Communications",
    "0003B0": "Xsense Technology Corp.",
    "0003B1": "Hospira Inc.",
    "0003B2": "Radware",
    "0003B3": "IA Link Systems Co., Ltd.",
    "0003B4": "Macrotek International Corp.",
    "0003B5": "Entra Technology Co.",
    "0003B6": "QSI Corporation",
    "0003B7": "ZACCESS Systems",
    "0003B8": "NetKit Solutions, LLC",
    "0003B9": "Hualong Telecom Co., Ltd.",
    "0003BA": "Sun Microsystems Inc.",
    "0003BB": "Signal Communications Limited",
    "0003BC": "COT GmbH",
    "0003BD": "OmniCluster Technologies, Inc.",
    "0003BE": "Netility",
    "0003BF": "Centerpoint Broadband Technologies, Inc.",
    "0003C0": "RFTNC Co., Ltd.",
    "0003C1": "Packet Dynamics Ltd",
    "0003C2": "Solphone K.K.",
    "0003C3": "Micronik Multimedia",
    "0003C4": "Tomra Systems ASA",
    "0003C5": "Mobotix AG",
    "0003C6": "ICUE Systems, Inc.",
    "0003C7": "hopf Elektronik GmbH",
    "0003C8": "CML Emergency Services",
    "0003C9": "TECOM Co., Ltd.",
    "0003CA": "MTS Systems Corp.",
    "0003CB": "Nippon Systems Development Co., Ltd.",
    "0003CC": "Momentum Computer, Inc.",
    "0003CD": "Clovertech, Inc.",
    "0003CE": "ETEN Technologies, Inc.",
    "0003CF": "Muxcom, Inc.",
    "0003D0": "KOANKEISO Co., Ltd.",
    "0003D1": "Takaya Corporation",
    "0003D2": "Crossbeam Systems, Inc.",
    "0003D3": "Internet Energy Systems, Inc.",
    "0003D4": "Alloptic, Inc.",
    "0003D5": "Advanced Communications Co., Ltd.",
    "0003D6": "RADVision, Ltd.",
    "0003D7": "NextNet Wireless, Inc.",
    "0003D8": "iMPath Networks, Inc.",
    "0003D9": "Secheron SA",
    "0003DA": "Takamisawa Cybernetics Co., Ltd.",
    "0003DB": "Apogee Electronics Corp.",
    "0003DC": "Lexar Media, Inc.",
    "0003DD": "Comark Corp.",
    "0003DE": "OTC Wireless",
    "0003DF": "Desana Systems",
    "0003E0": "RadioFrame Networks, Inc.",
    "0003E1": "Winmate Communication, Inc.",
    "0003E2": "Comspace Corporation",
    "0003E3": "Cisco Systems, Inc.",
    "0003E4": "Cisco Systems, Inc.",
    "0003E5": "Hermstedt SG",
    "0003E6": "Entone Technologies, Inc.",
    "0003E7": "Logostek Co. Ltd.",
    "0003E8": "Wavelength Digital Limited",
    "0003E9": "Akara Canada, Inc.",
    "0003EA": "Mega System Technologies, Inc.",
    "0003EB": "Atrica",
    "0003EC": "ICG Research, Inc.",
    "0003ED": "Shinkawa Electric Co., Ltd.",
    "0003EE": "MKNet Corporation",
    "0003EF": "Oneline AG",
    "0003F0": "Redfern Broadband Networks",
    "0003F1": "Cicada Semiconductor, Inc.",
    "0003F2": "Seneca Networks",
    "0003F3": "Dazzle Multimedia, Inc.",
    "0003F4": "NetBurner",
    "0003F5": "Chip2Chip",
    "0003F6": "Allegro Networks, Inc.",
    "0003F7": "Plast-Control GmbH",
    "0003F8": "SanCastle Technologies, Inc.",
    "0003F9": "Pleiades Communications, Inc.",
    "0003FA": "TiMetra Networks",
    "0003FB": "Toko Seiki Company, Ltd.",
    "0003FC": "Intertex Data AB",
    "0003FD": "Cisco Systems, Inc.",
    "0003FE": "Cisco Systems, Inc.",
    "0003FF": "Microsoft Corporation",
    "000400": "LEXMARK INTERNATIONAL, INC.",
    "000401": "Osaki Electric Co., Ltd.",
    "000402": "Nexsan Technologies, Ltd.",
    "000403": "Nexsi Corporation",
    "000404": "Makino Milling Machine Co., Ltd.",
    "000405": "ACN Technologies",
    "000406": "Fa. Metabox AG",
    "000407": "Topcon Positioning Systems, Inc.",
    "000408": "Sanko Electronics Co., Ltd.",
    "000409": "Cratos Networks",
    "00040A": "Sage Systems",
    "00040B": "3com Europe Ltd.",
    "00040C": "KANNO Work's Ltd.",
    "00040D": "Avaya, Inc.",
    "00040E": "AVM GmbH",
    "00040F": "Asus Network Technologies, Inc.",
    "000410": "Spinnaker Networks, Inc.",
    "000411": "Inkra Networks, Inc.",
    "000412": "WaveSmith Networks, Inc.",
    "000413": "SNOM Technology AG",
    "000414": "Umezawa Musen Denki Co., Ltd.",
    "000415": "Rasteme Systems Co., Ltd.",
    "000416": "Parks S/A Comunicacoes Digitais",
    "000417": "ELAU AG",
    "000418": "Teltronic S.A.U.",
    "000419": "Fibercycle Networks, Inc.",
    "00041A": "Ines Test and Measurement GmbH & CoKG",
    "00041B": "Digital Interfaces Ltd.",
    "00041C": "ipDialog, Inc.",
    "00041D": "Corega of America",
    "00041E": "Shikoku Instrumentation Co., Ltd.",
    "00041F": "Sony Computer Entertainment, Inc.",
    "000420": "Slim Devices, Inc.",
    "000421": "Ocular Networks",
    "000422": "Gordon Kapes, Inc.",
    "000423": "Intel Corporation",
    "000424": "TMC s.r.l.",
    "000425": "Atmel Corporation",
    "000426": "Autosys",
    "000427": "Cisco Systems, Inc.",
    "000428": "Cisco Systems, Inc.",
    "000429": "Pixord Corporation",
    "00042A": "Wireless Networks, Inc.",
    "00042B": "IT Access Co., Ltd.",
    "00042C": "Minet, Inc.",
    "00042D": "Sarian Systems, Ltd.",
    "00042E": "Netous Technologies, Ltd.",
    "00042F": "International Communications Products, Inc.",
    "000430": "Netgem",
    "000431": "GlobalStreams, Inc.",
    "000432": "Voyetra Turtle Beach, Inc.",
    "000433": "Cyberboard A/S",
    "000434": "Accelent Systems, Inc.",
    "000435": "Comptek International, Inc.",
    "000436": "ELANsat Technologies, Inc.",
    "000437": "Powin Information Technology, Inc.",
    "000438": "Nortel Networks",
    "000439": "Rosco Entertainment Technology, Inc.",
    "00043A": "Intelligent Telecommunications, Inc.",
    "00043B": "Lava Computer Mfg., Inc.",
    "00043C": "SONOS Co., Ltd.",
    "00043D": "INDEL AG",
    "00043E": "Telencomm",
    "00043F": "Electronic Systems Technology, Inc.",
    "000440": "cyberPIXIE, Inc.",
    "000441": "Half Dome Systems, Inc.",
    "000442": "NACT",
    "000443": "Agilent Technologies, Inc.",
    "000444": "Western Multiplex Corporation",
    "000445": "LMS Skalar Instruments GmbH",
    "000446": "CYZENTECH Co., Ltd.",
    "000447": "Acrowave Systems Co., Ltd.",
    "000448": "Polaroid Professional Imaging",
    "000449": "Mapletree Networks",
    "00044A": "iPolicy Networks, Inc.",
    "00044B": "NVIDIA",
    "00044C": "JENOPTIK",
    "00044D": "Cisco Systems, Inc.",
    "00044E": "Cisco Systems, Inc.",
    "00044F": "Leukhardt Systemelektronik GmbH",
    "000450": "DMD Computers SRL",
    "000451": "Medrad, Inc.",
    "000452": "RocketLogix, Inc.",
    "000453": "YottaYotta, Inc.",
    "000454": "Quadriga UK",
    "000455": "ANTARA.net",
    "000456": "Motorola PTP Inc",
    "000457": "Universal Access Technology, Inc.",
    "000458": "Fusion X Co., Ltd.",
    "000459": "Veristar Corporation",
    "00045A": "The Linksys Group, Inc.",
    "00045B": "Techsan Electronics Co., Ltd.",
    "00045C": "Mobiwave Pte Ltd",
    "00045D": "BEKA Elektronik",
    "00045E": "PolyTrax Information Technology AG",
    "00045F": "Evalue Technology, Inc.",
    "000460": "Knilink Technology, Inc.",
    "000461": "EPOX Computer Co., Ltd.",
    "000462": "DAKOS Data & Communication Co., Ltd.",
    "000463": "Bosch Security Systems",
    "000464": "Fantasma Networks, Inc.",
    "000465": "i.s.t isdn-support technik GmbH",
    "000466": "ARMITEL Co.",
    "000467": "Wuhan Research Institute of MII",
    "000468": "Vivity, Inc.",
    "000469": "Innocom, Inc.",
    "00046A": "Navini Networks",
    "00046B": "Palm Wireless, Inc.",
    "00046C": "Cyber Technology Co., Ltd.",
    "00046D": "Cisco Systems, Inc.",
    "00046E": "Cisco Systems, Inc.",
    "00046F": "Digitel S/A Industria Eletronica",
    "000470": "ipUnplugged AB",
    "000471": "IPrad",
    "000472": "Telelynx, Inc.",
    "000473": "Photonex Corporation",
    "000474": "LEGRAND",
    "000475": "3 Com Corporation",
    "000476": "3 Com Corporation",
    "000477": "Scalant Systems, Inc.",
    "000478": "G. Star Technology Corporation",
    "000479": "Radius Co., Ltd.",
    "00047A": "AXXESSIT ASA",
    "00047B": "Schlumberger",
    "00047C": "Skidata AG",
    "00047D": "Pelco",
    "00047E": "Optelecom=NKF",
    "00047F": "Chr. Mayr GmbH & Co. KG",
    "000480": "Foundry Networks, Inc.",
    "000481": "Econolite Control Products, Inc.",
    "000482": "Medialogic Corp.",
    "000483": "Deltron Technology, Inc.",
    "000484": "Amann GmbH",
    "000485": "PicoLight",
    "000486": "ITTC, University of Kansas",
    "000487": "Cogency Semiconductor, Inc.",
    "000488": "Eurotherm Controls",
    "000489": "YAFO Networks, Inc.",
    "00048A": "Temia Vertriebs GmbH",
    "00048B": "Poscon Corporation",
    "00048C": "Nayna Networks, Inc.",
    "00048D": "Tone Commander Systems, Inc.",
    "00048E": "Ohm Tech Labs, Inc.",
    "00048F": "TD Systems Corporation",
    "000490": "Optical Access",
    "000491": "Technovision, Inc.",
    "000492": "Hive Internet, Ltd.",
    "000493": "Tsinghua Unisplendour Co., Ltd.",
    "000494": "Breezecom, Ltd.",
    "000495": "Tejas Networks",
    "000496": "Extreme Networks",
    "000497": "MacroSystem Digital Video AG",
    "000498": "Mahi Networks",
    "000499": "Chino Corporation",
    "00049A": "Cisco Systems, Inc.",
    "00049B": "Cisco Systems, Inc.",
    "00049C": "Surgient Networks, Inc.",
    "00049D": "Ipanema Technologies",
    "00049E": "Wirelink Co., Ltd.",
    "00049F": "Freescale Semiconductor",
    "0004A0": "Verity Instruments, Inc.",
    "0004A1": "Pathway Connectivity",
    "0004A2": "L.S.I. Japan Co., Ltd.",
    "0004A3": "Microchip Technology, Inc.",
    "0004A4": "NetEnabled, Inc.",
    "0004A5": "Barco Projection Systems NV",
    "0004A6": "SAF Tehnika Ltd.",
    "0004A7": "FabiaTech Corporation",
    "0004A8": "Broadmax Technologies, Inc.",
    "0004A9": "SandStream Technologies, Inc.",
    "0004AA": "Jetstream Communications",
    "0004AB": "Comverse Network Systems, Inc.",
    "0004AC": "IBM CORP.",
    "0004AD": "Malibu Networks",
    "0004AE": "Sullair Corporation",
    "0004AF": "Digital Fountain, Inc.",
    "0004B0": "ELESIGN Co., Ltd.",
    "0004B1": "Signal Technology, Inc.",
    "0004B2": "ESSEGI SRL",
    "0004B3": "Videotek, Inc.",
    "0004B4": "CIAC",
    "0004B5": "Equitrac Corporation",
    "0004B6": "Stratex Networks, Inc.",
    "0004B7": "AMB i.t. Holding",
    "0004B8": "Kumahira Co., Ltd.",
    "0004B9": "S.I. Soubou, Inc.",
    "0004BA": "KDD Media Will Corporation",
    "0004BB": "Bardac Corporation",
    "0004BC": "Giantec, Inc.",
    "0004BD": "Motorola BCS",
    "0004BE": "OptXCon, Inc.",
    "0004BF": "VersaLogic Corp.",
    "0004C0": "Cisco Systems, Inc.",
    "0004C1": "Cisco Systems, Inc.",
    "0004C2": "Magnipix, Inc.",
    "0004C3": "CASTOR Informatique",
    "0004C4": "Allen & Heath Limited",
    "0004C5": "ASE Technologies, USA",
    "0004C6": "Yamaha Motor Co., Ltd.",
    "0004C7": "NetMount",
    "0004C8": "LIBA Maschinenfabrik GmbH",
    "0004C9": "Micro Electron Co., Ltd.",
    "0004CA": "FreeMs Corp.",
    "0004CB": "Tdsoft Communication, Ltd.",
    "0004CC": "Peek Traffic B.V.",
    "0004CD": "Informedia Research Group",
    "0004CE": "Patria Ailon",
    "0004CF": "Seagate Technology",
    "0004D0": "Softlink s.r.o.",
    "0004D1": "Drew Technologies, Inc.",
    "0004D2": "Adcon Telemetry GmbH",
    "0004D3": "Toyokeiki Co., Ltd.",
    "0004D4": "Proview Electronics Co., Ltd.",
    "0004D5": "Hitachi Communication Systems, Inc.",
    "0004D6": "Takagi Industrial Co., Ltd.",
    "0004D7": "Omitec Instrumentation Ltd.",
    "0004D8": "IPWireless, Inc.",
    "0004D9": "Titan Electronics, Inc.",
    "0004DA": "Relax Technology, Inc.",
    "0004DB": "Tellus Group Corp.",
    "0004DC": "Nortel Networks",
    "0004DD": "Cisco Systems, Inc.",
    "0004DE": "Cisco Systems, Inc.",
    "0004DF": "Teracom Telematica Ltda.",
    "0004E0": "Procket Networks",
    "0004E1": "Infinior Microsystems",
    "0004E2": "SMC Networks, Inc.",
    "0004E3": "Accton Technology Corp.",
    "0004E4": "Daeryung Ind., Inc.",
    "0004E5": "Glonet Systems, Inc.",
    "0004E6": "Banyan Network Private Limited",
    "0004E7": "Lightpointe Communications, Inc",
    "0004E8": "IER, Inc.",
    "0004E9": "Infiniswitch Corporation",
    "0004EA": "Hewlett-Packard Company",
    "0004EB": "Paxonet Communications, Inc.",
    "0004EC": "Memobox SA",
    "0004ED": "Billion Electric Co., Ltd.",
    "0004EE": "Lincoln Electric Company",
    "0004EF": "Polestar Corp.",
    "0004F0": "International Computers, Ltd",
    "0004F1": "WhereNet",
    "0004F2": "Polycom",
    "0004F3": "FS FORTH-SYSTEME GmbH",
    "0004F4": "Infinite Electronics Inc.",
    "0004F5": "SnowShore Networks, Inc.",
    "0004F6": "Amphus",
    "0004F7": "Omega Band, Inc.",
    "0004F8": "QUALICABLE TV Industria E Com., Ltda",
    "0004F9": "Xtera Communications, Inc.",
    "0004FA": "NBS Technologies Inc.",
    "0004FB": "Commtech, Inc.",
    "0004FC": "Stratus Computer (DE), Inc.",
    "0004FD": "Japan Control Engineering Co., Ltd.",
    "0004FE": "Pelago Networks",
    "0004FF": "Acronet Co., Ltd.",
    "000500": "Cisco Systems, Inc.",
    "000501": "Cisco Systems, Inc.",
    "000502": "APPLE COMPUTER",
    "000503": "ICONAG",
    "000504": "Naray Information & Communication Enterprise",
    "000505": "Systems Integration Solutions, Inc.",
    "000506": "Reddo Networks AB",
    "000507": "Fine Appliance Corp.",
    "000508": "Inetcam, Inc.",
    "000509": "AVOC Nishimura Ltd.",
    "00050A": "ICS Spa",
    "00050B": "SICOM Systems, Inc.",
    "00050C": "Network Photonics, Inc.",
    "00050D": "Midstream Technologies, Inc.",
    "00050E": "3ware, Inc.",
    "00050F": "Tanaka S/S Ltd.",
    "000510": "Infinite Shanghai Communication Terminals Ltd.",
    "000511": "Complementary Technologies Ltd",
    "000512": "MeshNetworks, Inc.",
    "000513": "VTLinx Multimedia Systems, Inc.",
    "000514": "KDT Systems Co., Ltd.",
    "000515": "Nuark Co., Ltd.",
    "000516": "SMART Modular Technologies",
    "000517": "Shellcomm, Inc.",
    "000518": "Jupiters Technology",
    "000519": "Siemens Building Technologies AG,",
    "00051A": "3Com Europe Ltd.",
    "00051B": "Magic Control Technology Corporation",
    "00051C": "Xnet Technology Corp.",
    "00051D": "Airocon, Inc.",
    "00051E": "Brocade Communications Systems, Inc.",
    "00051F": "Taijin Media Co., Ltd.",
    "000520": "Smartronix, Inc.",
    "000521": "Control Microsystems",
    "000522": "LEA*D Corporation, Inc.",
    "000523": "AVL List GmbH",
    "000524": "BTL System (HK) Limited",
    "000525": "Puretek Industrial Co., Ltd.",
    "000526": "IPAS GmbH",
    "000527": "SJ Tek Co. Ltd",
    "000528": "New Focus, Inc.",
    "000529": "Shanghai Broadan Communication Technology Co., Ltd",
    "00052A": "Ikegami Tsushinki Co., Ltd.",
    "00052B": "HORIBA, Ltd.",
    "00052C": "Supreme Magic Corporation",
    "00052D": "Zoltrix International Limited",
    "00052E": "Cinta Networks",
    "00052F": "Leviton Voice and Data",
    "000530": "Andiamo Systems, Inc.",
    "000531": "Cisco Systems, Inc.",
    "000532": "Cisco Systems, Inc.",
    "000533": "Brocade Communications Systems, Inc.",
    "000534": "Northstar Engineering Ltd.",
    "000535": "Chip PC Ltd.",
    "000536": "Danam Communications, Inc.",
    "000537": "Nets Technology Co., Ltd.",
    "000538": "Merilus, Inc.",
    "000539": "A Brand New World in Sweden AB",
    "00053A": "Willowglen Services Pte Ltd",
    "00053B": "Harbour Networks Ltd., Co. Beijing",
    "00053C": "Xircom",
    "00053D": "Agere Systems",
    "00053E": "KID Systeme GmbH",
    "00053F": "VisionTek, Inc.",
    "000540": "FAST Corporation",
    "000541": "Advanced Systems Co., Ltd.",
    "000542": "Otari, Inc.",
    "000543": "IQ Wireless GmbH",
    "000544": "Valley Technologies, Inc.",
    "000545": "Internet Photonics",
    "000546": "KDDI Network & Solultions Inc.",
    "000547": "Starent Networks",
    "000548": "Disco Corporation",
    "000549": "Salira Optical Network Systems",
    "00054A": "Ario Data Networks, Inc.",
    "00054B": "Micro Innovation AG",
    "00054C": "RF Innovations Pty Ltd",
    "00054D": "Brans Technologies, Inc.",
    "00054E": "Philips",
    "00054F": "PRIVATE",
    "000550": "Vcomms Limited",
    "000551": "F & S Elektronik Systeme GmbH",
    "000552": "Xycotec Computer GmbH",
    "000553": "DVC Company, Inc.",
    "000554": "Rangestar Wireless",
    "000555": "Japan Cash Machine Co., Ltd.",
    "000556": "360 Systems",
    "000557": "Agile TV Corporation",
    "000558": "Synchronous, Inc.",
    "000559": "Intracom S.A.",
    "00055A": "Power Dsine Ltd.",
    "00055B": "Charles Industries, Ltd.",
    "00055C": "Kowa Company, Ltd.",
    "00055D": "D-Link Systems, Inc.",
    "00055E": "Cisco Systems, Inc.",
    "00055F": "Cisco Systems, Inc.",
    "000560": "LEADER COMM.CO., LTD",
    "000561": "nac Image Technology, Inc.",
    "000562": "Digital View Limited",
    "000563": "J-Works, Inc.",
    "000564": "Tsinghua Bitway Co., Ltd.",
    "000565": "Tailyn Communication Company Ltd.",
    "000566": "Secui.com Corporation",
    "000567": "Etymonic Design, Inc.",
    "000568": "Piltofish Networks AB",
    "000569": "VMWARE, Inc.",
    "00056A": "Heuft Systemtechnik GmbH",
    "00056B": "C.P. Technology Co., Ltd.",
    "00056C": "Hung Chang Co., Ltd.",
    "00056D": "Pacific Corporation",
    "00056E": "National Enhance Technology, Inc.",
    "00056F": "Innomedia Technologies Pvt. Ltd.",
    "000570": "Baydel Ltd.",
    "000571": "Seiwa Electronics Co.",
    "000572": "Deonet Co., Ltd.",
    "000573": "Cisco Systems, Inc.",
    "000574": "Cisco Systems, Inc.",
    "000575": "CDS-Electronics BV",
    "000576": "NSM Technology Ltd.",
    "000577": "SM Information & Communication",
    "000578": "PRIVATE",
    "000579": "Universal Control Solution Corp.",
    "00057A": "Hatteras Networks",
    "00057B": "Chung Nam Electronic Co., Ltd.",
    "00057C": "RCO Security AB",
    "00057D": "Sun Communications, Inc.",
    "00057E": "Eckelmann Steuerungstechnik GmbH",
    "00057F": "Acqis Technology",
    "000580": "Fibrolan Ltd.",
    "000581": "Snell & Wilcox Ltd.",
    "000582": "ClearCube Technology",
    "000583": "ImageCom Limited",
    "000584": "AbsoluteValue Systems, Inc.",
    "000585": "Juniper Networks, Inc.",
    "000586": "Lucent Technologies",
    "000587": "Locus, Incorporated",
    "000588": "Sensoria Corp.",
    "000589": "National Datacomputer",
    "00058A": "Netcom Co., Ltd.",
    "00058B": "IPmental, Inc.",
    "00058C": "Opentech Inc.",
    "00058D": "Lynx Photonic Networks, Inc.",
    "00058E": "Flextronics International GmbH & Co. Nfg. KG",
    "00058F": "CLCsoft co.",
    "000590": "Swissvoice Ltd.",
    "000591": "Active Silicon Ltd.",
    "000592": "Pultek Corp.",
    "000593": "Grammar Engine Inc.",
    "000594": "IXXAT Automation GmbH",
    "000595": "Alesis Corporation",
    "000596": "Genotech Co., Ltd.",
    "000597": "Eagle Traffic Control Systems",
    "000598": "CRONOS S.r.l.",
    "000599": "DRS Test and Energy Management or DRS-TEM",
    "00059A": "Cisco Systems, Inc.",
    "00059B": "Cisco Systems, Inc.",
    "00059C": "Kleinknecht GmbH, Ing. Buero",
    "00059D": "Daniel Computing Systems, Inc.",
    "00059E": "Zinwell Corporation",
    "00059F": "Yotta Networks, Inc.",
    "0005A0": "MOBILINE Kft.",
    "0005A1": "Zenocom",
    "0005A2": "CELOX Networks",
    "0005A3": "QEI, Inc.",
    "0005A4": "Lucid Voice Ltd.",
    "0005A5": "KOTT",
    "0005A6": "Extron Electronics",
    "0005A7": "Hyperchip, Inc.",
    "0005A8": "WYLE ELECTRONICS",
    "0005A9": "Princeton Networks, Inc.",
    "0005AA": "Moore Industries International Inc.",
    "0005AB": "Cyber Fone, Inc.",
    "0005AC": "Northern Digital, Inc.",
    "0005AD": "Topspin Communications, Inc.",
    "0005AE": "Mediaport USA",
    "0005AF": "InnoScan Computing A/S",
    "0005B0": "Korea Computer Technology Co., Ltd.",
    "0005B1": "ASB Technology BV",
    "0005B2": "Medison Co., Ltd.",
    "0005B3": "Asahi-Engineering Co., Ltd.",
    "0005B4": "Aceex Corporation",
    "0005B5": "Broadcom Technologies",
    "0005B6": "INSYS Microelectronics GmbH",
    "0005B7": "Arbor Technology Corp.",
    "0005B8": "Electronic Design Associates, Inc.",
    "0005B9": "Airvana, Inc.",
    "0005BA": "Area Netwoeks, Inc.",
    "0005BB": "Myspace AB",
    "0005BC": "Resorsys Ltd.",
    "0005BD": "ROAX BV",
    "0005BE": "Kongsberg Seatex AS",
    "0005BF": "JustEzy Technology, Inc.",
    "0005C0": "Digital Network Alacarte Co., Ltd.",
    "0005C1": "A-Kyung Motion, Inc.",
    "0005C2": "Soronti, Inc.",
    "0005C3": "Pacific Instruments, Inc.",
    "0005C4": "Telect, Inc.",
    "0005C5": "Flaga HF",
    "0005C6": "Triz Communications",
    "0005C7": "I/F-COM A/S",
    "0005C8": "VERYTECH",
    "0005C9": "LG Innotek Co., Ltd.",
    "0005CA": "Hitron Technology, Inc.",
    "0005CB": "ROIS Technologies, Inc.",
    "0005CC": "Sumtel Communications, Inc.",
    "0005CD": "Denon, Ltd.",
    "0005CE": "Prolink Microsystems Corporation",
    "0005CF": "Thunder River Technologies, Inc.",
    "0005D0": "Solinet Systems",
    "0005D1": "Metavector Technologies",
    "0005D2": "DAP Technologies",
    "0005D3": "eProduction Solutions, Inc.",
    "0005D4": "FutureSmart Networks, Inc.",
    "0005D5": "Speedcom Wireless",
    "0005D6": "Titan Wireless",
    "0005D7": "Vista Imaging, Inc.",
    "0005D8": "Arescom, Inc.",
    "0005D9": "Techno Valley, Inc.",
    "0005DA": "Apex Automationstechnik",
    "0005DB": "NENTEC Netzwerktechnologie GmbH",
    "0005DC": "Cisco Systems, Inc.",
    "0005DD": "Cisco Systems, Inc.",
    "0005DE": "Gi Fone Korea, Inc.",
    "0005DF": "Electronic Innovation, Inc.",
    "0005E0": "Empirix Corp.",
    "0005E1": "Trellis Photonics, Ltd.",
    "0005E2": "Creativ Network Technologies",
    "0005E3": "LightSand Communications, Inc.",
    "0005E4": "Red Lion Controls Inc.",
    "0005E5": "Renishaw PLC",
    "0005E6": "Egenera, Inc.",
    "0005E7": "Netrake an AudioCodes Company",
    "0005E8": "TurboWave, Inc.",
    "0005E9": "Unicess Network, Inc.",
    "0005EA": "Rednix",
    "0005EB": "Blue Ridge Networks, Inc.",
    "0005EC": "Mosaic Systems Inc.",
    "0005ED": "Technikum Joanneum GmbH",
    "0005EE": "BEWATOR Group",
    "0005EF": "ADOIR Digital Technology",
    "0005F0": "SATEC",
    "0005F1": "Vrcom, Inc.",
    "0005F2": "Power R, Inc.",
    "0005F3": "Weboyn",
    "0005F4": "System Base Co., Ltd.",
    "0005F5": "OYO Geospace",
    "0005F6": "Young Chang Co. Ltd.",
    "0005F7": "Analog Devices, Inc.",
    "0005F8": "Real Time Access, Inc.",
    "0005F9": "TOA Corporation",
    "0005FA": "IPOptical, Inc.",
    "0005FB": "ShareGate, Inc.",
    "0005FC": "Schenck Pegasus Corp.",
    "0005FD": "PacketLight Networks Ltd.",
    "0005FE": "Traficon N.V.",
    "0005FF": "SNS Solutions, Inc.",
    "000600": "Toshiba Teli Corporation",
    "000601": "Otanikeiki Co., Ltd.",
    "000602": "Cirkitech Electronics Co.",
    "000603": "Baker Hughes Inc.",
    "000604": "@Track Communications, Inc.",
    "000605": "Inncom International, Inc.",
    "000606": "RapidWAN, Inc.",
    "000607": "Omni Directional Control Technology Inc.",
    "000608": "At-Sky SAS",
    "000609": "Crossport Systems",
    "00060A": "Blue2space",
    "00060B": "Paceline Systems Corporation",
    "00060C": "Melco Industries, Inc.",
    "00060D": "Wave7 Optics",
    "00060E": "IGYS Systems, Inc.",
    "00060F": "Narad Networks Inc",
    "000610": "Abeona Networks Inc",
    "000611": "Zeus Wireless, Inc.",
    "000612": "Accusys, Inc.",
    "000613": "Kawasaki Microelectronics Incorporated",
    "000614": "Prism Holdings",
    "000615": "Kimoto Electric Co., Ltd.",
    "000616": "Tel Net Co., Ltd.",
    "000617": "Redswitch Inc.",
    "000618": "DigiPower Manufacturing Inc.",
    "000619": "Connection Technology Systems",
    "00061A": "Zetari Inc.",
    "00061B": "Notebook Development Lab. Lenovo Japan Ltd.",
    "00061C": "Hoshino Metal Industries, Ltd.",
    "00061D": "MIP Telecom, Inc.",
    "00061E": "Maxan Systems",
    "00061F": "Vision Components GmbH",
    "000620": "Serial System Ltd.",
    "000621": "Hinox, Co., Ltd.",
    "000622": "Chung Fu Chen Yeh Enterprise Corp.",
    "000623": "MGE UPS Systems France",
    "000624": "Gentner Communications Corp.",
    "000625": "The Linksys Group, Inc.",
    "000626": "MWE GmbH",
    "000627": "Uniwide Technologies, Inc.",
    "000628": "Cisco Systems, Inc.",
    "000629": "IBM CORPORATION",
    "00062A": "Cisco Systems, Inc.",
    "00062B": "INTRASERVER TECHNOLOGY",
    "00062C": "Network Robots, Inc.",
    "00062D": "TouchStar Technologies, L.L.C.",
    "00062E": "Aristos Logic Corp.",
    "00062F": "Pivotech Systems Inc.",
    "000630": "Adtranz Sweden",
    "000631": "Optical Solutions, Inc.",
    "000632": "Mesco Engineering GmbH",
    "000633": "Cross Match Technologies GmbH",
    "000634": "GTE Airfone Inc.",
    "000635": "PacketAir Networks, Inc.",
    "000636": "Jedai Broadband Networks",
    "000637": "Toptrend-Meta Information (ShenZhen) Inc.",
    "000638": "Sungjin C&C Co., Ltd.",
    "000639": "Newtec",
    "00063A": "Dura Micro, Inc.",
    "00063B": "Arcturus Networks, Inc.",
    "00063C": "Intrinsyc Europe Ltd",
    "00063D": "Microwave Data Systems Inc.",
    "00063E": "Opthos Inc.",
    "00063F": "Everex Communications Inc.",
    "000640": "White Rock Networks",
    "000641": "ITCN",
    "000642": "Genetel Systems Inc.",
    "000643": "SONO Computer Co., Ltd.",
    "000644": "Digital Technologies Corporation",
    "000645": "Meisei Electric Co. Ltd.",
    "000646": "ShenZhen XunBao Network Technology Co Ltd",
    "000647": "Etrali S.A.",
    "000648": "Seedsware, Inc.",
    "000649": "3M Deutschland GmbH",
    "00064A": "Honeywell Co., Ltd. (KOREA)",
    "00064B": "Alexon Co., Ltd.",
    "00064C": "Invicta Networks, Inc.",
    "00064D": "Sencore",
    "00064E": "Broad Net Technology Inc.",
    "00064F": "PRO-NETS Technology Corporation",
    "000650": "Tiburon Networks, Inc.",
    "000651": "Aspen Networks Inc.",
    "000652": "Cisco Systems, Inc.",
    "000653": "Cisco Systems, Inc.",
    "000654": "Winpresa Building Automation Technologies GmbH",
    "000655": "Yipee, Inc.",
    "000656": "Tactel AB",
    "000657": "Market Central, Inc.",
    "000658": "Helmut Fischer GmbH & Co. KG",
    "000659": "EAL (Apeldoorn) B.V.",
    "00065A": "Strix Systems",
    "00065B": "Dell Computer Corp.",
    "00065C": "Malachite Technologies, Inc.",
    "00065D": "Heidelberg Web Systems",
    "00065E": "Photuris, Inc.",
    "00065F": "ECI Telecom - NGTS Ltd.",
    "000660": "NADEX Co., Ltd.",
    "000661": "NIA Home Technologies Corp.",
    "000662": "MBM Technology Ltd.",
    "000663": "Human Technology Co., Ltd.",
    "000664": "Fostex Corporation",
    "000665": "Sunny Giken, Inc.",
    "000666": "Roving Networks",
    "000667": "Tripp Lite",
    "000668": "Vicon Industries Inc.",
    "000669": "Datasound Laboratories Ltd",
    "00066A": "InfiniCon Systems, Inc.",
    "00066B": "Sysmex Corporation",
    "00066C": "Robinson Corporation",
    "00066D": "Compuprint S.P.A.",
    "00066E": "Delta Electronics, Inc.",
    "00066F": "Korea Data Systems",
    "000670": "Upponetti Oy",
    "000671": "Softing AG",
    "000672": "Netezza",
    "000673": "Optelecom-nkf",
    "000674": "Spectrum Control, Inc.",
    "000675": "Banderacom, Inc.",
    "000676": "Novra Technologies Inc.",
    "000677": "SICK AG",
    "000678": "Marantz Brand Company",
    "000679": "Konami Corporation",
    "00067A": "JMP Systems",
    "00067B": "Toplink C&C Corporation",
    "00067C": "CISCO SYSTEMS, INC.",
    "00067D": "Takasago Ltd.",
    "00067E": "WinCom Systems, Inc.",
    "00067F": "Rearden Steel Technologies",
    "000680": "Card Access, Inc.",
    "000681": "Goepel Electronic GmbH",
    "000682": "Convedia",
    "000683": "Bravara Communications, Inc.",
    "000684": "Biacore AB",
    "000685": "NetNearU Corporation",
    "000686": "ZARDCOM Co., Ltd.",
    "000687": "Omnitron Systems Technology, Inc.",
    "000688": "Telways Communication Co., Ltd.",
    "000689": "yLez Technologies Pte Ltd",
    "00068A": "NeuronNet Co. Ltd. R&D Center",
    "00068B": "AirRunner Technologies, Inc.",
    "00068C": "3Com Corporation",
    "00068D": "SEPATON, Inc.",
    "00068E": "HID Corporation",
    "00068F": "Telemonitor, Inc.",
    "000690": "Euracom Communication GmbH",
    "000691": "PT Inovacao",
    "000692": "Intruvert Networks, Inc.",
    "000693": "Flexus Computer Technology, Inc.",
    "000694": "Mobillian Corporation",
    "000695": "Ensure Technologies, Inc.",
    "000696": "Advent Networks",
    "000697": "R & D Center",
    "000698": "egnite Software GmbH",
    "000699": "Vida Design Co.",
    "00069A": "e & Tel",
    "00069B": "AVT Audio Video Technologies GmbH",
    "00069C": "Transmode Systems AB",
    "00069D": "Petards Mobile Intelligence",
    "00069E": "UNIQA, Inc.",
    "00069F": "Kuokoa Networks",
    "0006A0": "Mx Imaging",
    "0006A1": "Celsian Technologies, Inc.",
    "0006A2": "Microtune, Inc.",
    "0006A3": "Bitran Corporation",
    "0006A4": "INNOWELL Corp.",
    "0006A5": "PINON Corp.",
    "0006A6": "Artistic Licence (UK) Ltd",
    "0006A7": "Primarion",
    "0006A8": "KC Technology, Inc.",
    "0006A9": "Universal Instruments Corp.",
    "0006AA": "VT Miltope",
    "0006AB": "W-Link Systems, Inc.",
    "0006AC": "Intersoft Co.",
    "0006AD": "KB Electronics Ltd.",
    "0006AE": "Himachal Futuristic Communications Ltd",
    "0006AF": "PRIVATE",
    "0006B0": "Comtech EF Data Corp.",
    "0006B1": "Sonicwall",
    "0006B2": "Linxtek Co.",
    "0006B3": "Diagraph Corporation",
    "0006B4": "Vorne Industries, Inc.",
    "0006B5": "Luminent, Inc.",
    "0006B6": "Nir-Or Israel Ltd.",
    "0006B7": "TELEM GmbH",
    "0006B8": "Bandspeed Pty Ltd",
    "0006B9": "A5TEK Corp.",
    "0006BA": "Westwave Communications",
    "0006BB": "ATI Technologies Inc.",
    "0006BC": "Macrolink, Inc.",
    "0006BD": "BNTECHNOLOGY Co., Ltd.",
    "0006BE": "Baumer Optronic GmbH",
    "0006BF": "Accella Technologies Co., Ltd.",
    "0006C0": "United Internetworks, Inc.",
    "0006C1": "CISCO SYSTEMS, INC.",
    "0006C2": "Smartmatic Corporation",
    "0006C3": "Schindler Elevator Ltd.",
    "0006C4": "Piolink Inc.",
    "0006C5": "INNOVI Technologies Limited",
    "0006C6": "lesswire AG",
    "0006C7": "RFNET Technologies Pte Ltd (S)",
    "0006C8": "Sumitomo Metal Micro Devices, Inc.",
    "0006C9": "Technical Marketing Research, Inc.",
    "0006CA": "American Computer & Digital Components, Inc. (ACDC)",
    "0006CB": "Jotron Electronics A/S",
    "0006CC": "JMI Electronics Co., Ltd.",
    "0006CD": "Kodak IL Ltd.",
    "0006CE": "DATENO",
    "0006CF": "Thales Avionics In-Flight Systems, LLC",
    "0006D0": "Elgar Electronics Corp.",
    "0006D1": "Tahoe Networks, Inc.",
    "0006D2": "Tundra Semiconductor Corp.",
    "0006D3": "Alpha Telecom, Inc. U.S.A.",
    "0006D4": "Interactive Objects, Inc.",
    "0006D5": "Diamond Systems Corp.",
    "0006D6": "Cisco Systems, Inc.",
    "0006D7": "Cisco Systems, Inc.",
    "0006D8": "Maple Optical Systems",
    "0006D9": "IPM-Net S.p.A.",
    "0006DA": "ITRAN Communications Ltd.",
    "0006DB": "ICHIPS Co., Ltd.",
    "0006DC": "Syabas Technology (Amquest)",
    "0006DD": "AT & T Laboratories - Cambridge Ltd",
    "0006DE": "Flash Technology",
    "0006DF": "AIDONIC Corporation",
    "0006E0": "MAT Co., Ltd.",
    "0006E1": "Techno Trade s.a",
    "0006E2": "Ceemax Technology Co., Ltd.",
    "0006E3": "Quantitative Imaging Corporation",
    "0006E4": "Citel Technologies Ltd.",
    "0006E5": "Fujian Newland Computer Ltd. Co.",
    "0006E6": "DongYang Telecom Co., Ltd.",
    "0006E7": "Bit Blitz Communications Inc.",
    "0006E8": "Optical Network Testing, Inc.",
    "0006E9": "Intime Corp.",
    "0006EA": "ELZET80 Mikrocomputer GmbH&Co. KG",
    "0006EB": "Global Data",
    "0006EC": "M/A COM Private Radio System Inc.",
    "0006ED": "Inara Networks",
    "0006EE": "Shenyang Neu-era Information & Technology Stock Co., Ltd",
    "0006EF": "Maxxan Systems, Inc.",
    "0006F0": "Digeo, Inc.",
    "0006F1": "Optillion",
    "0006F2": "Platys Communications",
    "0006F3": "AcceLight Networks",
    "0006F4": "Prime Electronics & Satellitics Inc.",
    "0006F8": "CPU Technology, Inc.",
    "0006F9": "Mitsui Zosen Systems Research Inc.",
    "0006FA": "IP SQUARE Co, Ltd.",
    "0006FB": "Hitachi Printing Solutions, Ltd.",
    "0006FC": "Fnet Co., Ltd.",
    "0006FD": "Comjet Information Systems Corp.",
    "0006FE": "Celion Networks, Inc.",
    "0006FF": "Sheba Systems Co., Ltd.",
    "000700": "Zettamedia Korea",
    "000701": "RACAL-DATACOM",
    "000702": "Varian Medical Systems",
    "000703": "CSEE Transport",
    "000705": "Endress & Hauser GmbH & Co",
    "000706": "Sanritz Corporation",
    "000707": "Interalia Inc.",
    "000708": "Bitrage Inc.",
    "000709": "Westerstrand Urfabrik AB",
    "00070A": "Unicom Automation Co., Ltd.",
    "00070B": "Octal, SA",
    "00070C": "SVA-Intrusion.com Co. Ltd.",
    "00070D": "Cisco Systems Inc.",
    "00070E": "Cisco Systems Inc.",
    "00070F": "Fujant, Inc.",
    "000710": "Adax, Inc.",
    "000711": "Acterna",
    "000712": "JAL Information Technology",
    "000713": "IP One, Inc.",
    "000714": "Brightcom",
    "000715": "General Research of Electronics, Inc.",
    "000716": "J & S Marine Ltd.",
    "000717": "Wieland Electric GmbH",
    "000718": "iCanTek Co., Ltd.",
    "000719": "Mobiis Co., Ltd.",
    "00071A": "Finedigital Inc.",
    "00071B": "CDV Americas Ltd",
    "00071C": "AT&T Fixed Wireless Services",
    "00071D": "Satelsa Sistemas Y Aplicaciones De Telecomunicaciones, S.A.",
    "00071E": "Tri-M Engineering / Nupak Dev. Corp.",
    "00071F": "European Systems Integration",
    "000720": "Trutzschler GmbH & Co. KG",
    "000721": "Formac Elektronik GmbH",
    "000722": "The Nielsen Company",
    "000723": "ELCON Systemtechnik GmbH",
    "000724": "Telemax Co., Ltd.",
    "000725": "Bematech International Corp.",
    "000727": "Zi Corporation (HK) Ltd.",
    "000728": "Neo Telecom",
    "000729": "Kistler Instrumente AG",
    "00072A": "Innovance Networks",
    "00072B": "Jung Myung Telecom Co., Ltd.",
    "00072C": "Fabricom",
    "00072D": "CNSystems",
    "00072E": "North Node AB",
    "00072F": "Intransa, Inc.",
    "000730": "Hutchison OPTEL Telecom Technology Co., Ltd.",
    "000731": "Ophir-Spiricon Inc",
    "000732": "AAEON Technology Inc.",
    "000733": "DANCONTROL Engineering",
    "000734": "ONStor, Inc.",
    "000735": "Flarion Technologies, Inc.",
    "000736": "Data Video Technologies Co., Ltd.",
    "000737": "Soriya Co. Ltd.",
    "000738": "Young Technology Co., Ltd.",
    "000739": "Motion Media Technology Ltd.",
    "00073A": "Inventel Systemes",
    "00073B": "Tenovis GmbH & Co KG",
    "00073C": "Telecom Design",
    "00073D": "Nanjing Postel Telecommunications Co., Ltd.",
    "00073E": "China Great-Wall Computer Shenzhen Co., Ltd.",
    "00073F": "Woojyun Systec Co., Ltd.",
    "000740": "Melco Inc.",
    "000741": "Sierra Automated Systems",
    "000742": "Current Technologies, LLC",
    "000743": "Chelsio Communications",
    "000744": "Unico, Inc.",
    "000745": "Radlan Computer Communications Ltd.",
    "000746": "TURCK, Inc.",
    "000747": "Mecalc",
    "000748": "The Imaging Source Europe",
    "000749": "CENiX Inc.",
    "00074A": "Carl Valentin GmbH",
    "00074B": "Daihen Corporation",
    "00074C": "Beicom Inc.",
    "00074D": "Zebra Technologies Corp.",
    "00074E": "Naughty boy co., Ltd.",
    "00074F": "Cisco Systems, Inc.",
    "000750": "Cisco Systems, Inc.",
    "000751": "m·u·t AG",
    "000752": "Rhythm Watch Co., Ltd.",
    "000753": "Beijing Qxcomm Technology Co., Ltd.",
    "000754": "Xyterra Computing, Inc.",
    "000755": "Lafon SA",
    "000756": "Juyoung Telecom",
    "000757": "Topcall International AG",
    "000758": "Dragonwave",
    "000759": "Boris Manufacturing Corp.",
    "00075A": "Air Products and Chemicals, Inc.",
    "00075B": "Gibson Guitars",
    "00075C": "Eastman Kodak Company",
    "00075D": "Celleritas Inc.",
    "00075E": "Ametek Power Instruments",
    "00075F": "VCS Video Communication Systems AG",
    "000760": "TOMIS Information & Telecom Corp.",
    "000761": "Logitech SA",
    "000762": "Group Sense Limited",
    "000763": "Sunniwell Cyber Tech. Co., Ltd.",
    "000764": "YoungWoo Telecom Co. Ltd.",
    "000765": "Jade Quantum Technologies, Inc.",
    "000766": "Chou Chin Industrial Co., Ltd.",
    "000767": "Yuxing Electronics Company Limited",
    "000768": "Danfoss A/S",
    "000769": "Italiana Macchi SpA",
    "00076A": "NEXTEYE Co., Ltd.",
    "00076B": "Stralfors AB",
    "00076C": "Daehanet, Inc.",
    "00076D": "Flexlight Networks",
    "00076E": "Sinetica Corporation Limited",
    "00076F": "Synoptics Limited",
    "000770": "Locusnetworks Corporation",
    "000771": "Embedded System Corporation",
    "000772": "Alcatel Shanghai Bell Co., Ltd.",
    "000773": "Ascom Powerline Communications Ltd.",
    "000774": "GuangZhou Thinker Technology Co. Ltd.",
    "000775": "Valence Semiconductor, Inc.",
    "000776": "Federal APD",
    "000777": "Motah Ltd.",
    "000778": "GERSTEL GmbH & Co. KG",
    "000779": "Sungil Telecom Co., Ltd.",
    "00077A": "Infoware System Co., Ltd.",
    "00077B": "Millimetrix Broadband Networks",
    "00077C": "OnTime Networks",
    "00077E": "Elrest GmbH",
    "00077F": "J Communications Co., Ltd.",
    "000780": "Bluegiga Technologies OY",
    "000781": "Itron Inc.",
    "000782": "Sun Microsystems, Inc.",
    "000783": "SynCom Network, Inc.",
    "000784": "Cisco Systems Inc.",
    "000785": "Cisco Systems Inc.",
    "000786": "Wireless Networks Inc.",
    "000787": "Idea System Co., Ltd.",
    "000788": "Clipcomm, Inc.",
    "000789": "Eastel Systems Corporation",
    "00078A": "Mentor Data System Inc.",
    "00078B": "Wegener Communications, Inc.",
    "00078C": "Elektronikspecialisten i Borlange AB",
    "00078D": "NetEngines Ltd.",
    "00078E": "Garz & Friche GmbH",
    "00078F": "Emkay Innovative Products",
    "000790": "Tri-M Technologies (s) Limited",
    "000791": "International Data Communications, Inc.",
    "000792": "Suetron Electronic GmbH",
    "000793": "Shin Satellite Public Company Limited",
    "000794": "Simple Devices, Inc.",
    "000795": "Elitegroup Computer System Co. (ECS)",
    "000796": "LSI Systems, Inc.",
    "000797": "Netpower Co., Ltd.",
    "000798": "Selea SRL",
    "000799": "Tipping Point Technologies, Inc.",
    "00079A": "Verint Systems Inc",
    "00079B": "Aurora Networks",
    "00079C": "Golden Electronics Technology Co., Ltd.",
    "00079D": "Musashi Co., Ltd.",
    "00079E": "Ilinx Co., Ltd.",
    "00079F": "Action Digital Inc.",
    "0007A0": "e-Watch Inc.",
    "0007A1": "VIASYS Healthcare GmbH",
    "0007A2": "Opteon Corporation",
    "0007A3": "Ositis Software, Inc.",
    "0007A4": "GN Netcom Ltd.",
    "0007A5": "Y.D.K Co. Ltd.",
    "0007A6": "Home Automation, Inc.",
    "0007A7": "A-Z Inc.",
    "0007A8": "Haier Group Technologies Ltd.",
    "0007A9": "Novasonics",
    "0007AA": "Quantum Data Inc.",
    "0007AC": "Eolring",
    "0007AD": "Pentacon GmbH Foto-und Feinwerktechnik",
    "0007AE": "Britestream Networks, Inc.",
    "0007AF": "N-Tron Corp.",
    "0007B0": "Office Details, Inc.",
    "0007B1": "Equator Technologies",
    "0007B2": "Transaccess S.A.",
    "0007B3": "Cisco Systems Inc.",
    "0007B4": "Cisco Systems Inc.",
    "0007B5": "Any One Wireless Ltd.",
    "0007B6": "Telecom Technology Ltd.",
    "0007B7": "Samurai Ind. Prods Eletronicos Ltda",
    "0007B8": "American Predator Corp.",
    "0007B9": "Ginganet Corporation",
    "0007BA": "UTStarcom, Inc.",
    "0007BB": "Candera Inc.",
    "0007BC": "Identix Inc.",
    "0007BD": "Radionet Ltd.",
    "0007BE": "DataLogic SpA",
    "0007BF": "Armillaire Technologies, Inc.",
    "0007C0": "NetZerver Inc.",
    "0007C1": "Overture Networks, Inc.",
    "0007C2": "Netsys Telecom",
    "0007C3": "Thomson",
    "0007C4": "JEAN Co. Ltd.",
    "0007C5": "Gcom, Inc.",
    "0007C6": "VDS Vosskuhler GmbH",
    "0007C7": "Synectics Systems Limited",
    "0007C8": "Brain21, Inc.",
    "0007C9": "Technol Seven Co., Ltd.",
    "0007CA": "Creatix Polymedia Ges Fur Kommunikaitonssysteme",
    "0007CB": "Freebox SA",
    "0007CC": "Kaba Benzing GmbH",
    "0007CD": "NMTEL Co., Ltd.",
    "0007CE": "Cabletime Limited",
    "0007CF": "Anoto AB",
    "0007D0": "Automat Engenharia de Automaoa Ltda.",
    "0007D1": "Spectrum Signal Processing Inc.",
    "0007D2": "Logopak Systeme",
    "0007D3": "Stork Digital Imaging B.V.",
    "0007D4": "Zhejiang Yutong Network Communication Co Ltd.",
    "0007D5": "3e Technologies Int;., Inc.",
    "0007D6": "Commil Ltd.",
    "0007D7": "Caporis Networks AG",
    "0007D8": "Hitron Systems Inc.",
    "0007D9": "Splicecom",
    "0007DA": "Neuro Telecom Co., Ltd.",
    "0007DB": "Kirana Networks, Inc.",
    "0007DC": "Atek Co, Ltd.",
    "0007DD": "Cradle Technologies",
    "0007DE": "eCopilt AB",
    "0007DF": "Vbrick Systems Inc.",
    "0007E0": "Palm Inc.",
    "0007E1": "WIS Communications Co. Ltd.",
    "0007E2": "Bitworks, Inc.",
    "0007E3": "Navcom Technology, Inc.",
    "0007E4": "SoftRadio Co., Ltd.",
    "0007E5": "Coup Corporation",
    "0007E6": "edgeflow Canada Inc.",
    "0007E7": "FreeWave Technologies",
    "0007E8": "St. Bernard Software",
    "0007E9": "Intel Corporation",
    "0007EA": "Massana, Inc.",
    "0007EB": "Cisco Systems Inc.",
    "0007EC": "Cisco Systems Inc.",
    "0007ED": "Altera Corporation",
    "0007EE": "telco Informationssysteme GmbH",
    "0007EF": "Lockheed Martin Tactical Systems",
    "0007F0": "Beckett LogiSync LLC",
    "0007F1": "TeraBurst Networks Inc.",
    "0007F2": "IOA Corporation",
    "0007F3": "Thinkengine Networks",
    "0007F4": "Eletex Co., Ltd.",
    "0007F5": "Bridgeco Co AG",
    "0007F6": "Qqest Software Systems",
    "0007F7": "Galtronics",
    "0007F8": "ITDevices, Inc.",
    "0007F9": "Phonetics, Inc.",
    "0007FA": "ITT Co., Ltd.",
    "0007FB": "Giga Stream UMTS Technologies GmbH",
    "0007FC": "Adept Systems Inc.",
    "0007FD": "LANergy Ltd.",
    "0007FE": "Rigaku Corporation",
    "0007FF": "Gluon Networks",
    "000800": "MULTITECH SYSTEMS, INC.",
    "000801": "HighSpeed Surfing Inc.",
    "000802": "Hewlett Packard",
    "000803": "Cos Tron",
    "000804": "ICA Inc.",
    "000805": "Techno-Holon Corporation",
    "000806": "Raonet Systems, Inc.",
    "000807": "Access Devices Limited",
    "000808": "PPT Vision, Inc.",
    "000809": "Systemonic AG",
    "00080A": "Espera-Werke GmbH",
    "00080B": "Birka BPA Informationssystem AB",
    "00080C": "VDA Elettronica spa",
    "00080D": "Toshiba",
    "00080E": "Motorola, BCS",
    "00080F": "Proximion Fiber Optics AB",
    "000810": "Key Technology, Inc.",
    "000811": "VOIX Corporation",
    "000812": "GM-2 Corporation",
    "000813": "Diskbank, Inc.",
    "000814": "TIL Technologies",
    "000815": "CATS Co., Ltd.",
    "000816": "Bluetags A/S",
    "000817": "EmergeCore Networks LLC",
    "000818": "Pixelworks, Inc.",
    "000819": "Banksys",
    "00081A": "Sanrad Intelligence Storage Communications (2000) Ltd.",
    "00081B": "Windigo Systems",
    "00081C": "@pos.com",
    "00081D": "Ipsil, Incorporated",
    "00081E": "Repeatit AB",
    "00081F": "Pou Yuen Tech Corp. Ltd.",
    "000820": "Cisco Systems Inc.",
    "000821": "Cisco Systems Inc.",
    "000822": "InPro Comm",
    "000823": "Texa Corp.",
    "000824": "Copitrak Inc",
    "000825": "Acme Packet",
    "000826": "Colorado Med Tech",
    "000827": "Pirelli Broadband Solutions",
    "000828": "Koei Engineering Ltd.",
    "000829": "Aval Nagasaki Corporation",
    "00082A": "Powerwallz Network Security",
    "00082B": "Wooksung Electronics, Inc.",
    "00082C": "Homag AG",
    "00082D": "Indus Teqsite Private Limited",
    "00082E": "Multitone Electronics PLC",
    "00084E": "DivergeNet, Inc.",
    "00084F": "Qualstar Corporation",
    "000850": "Arizona Instrument Corp.",
    "000851": "Canadian Bank Note Company, Ltd.",
    "000852": "Davolink Co. Inc.",
    "000853": "Schleicher GmbH & Co. Relaiswerke KG",
    "000854": "Netronix, Inc.",
    "000855": "NASA-Goddard Space Flight Center",
    "000856": "Gamatronic Electronic Industries Ltd.",
    "000857": "Polaris Networks, Inc.",
    "000858": "Novatechnology Inc.",
    "000859": "ShenZhen Unitone Electronics Co., Ltd.",
    "00085A": "IntiGate Inc.",
    "00085B": "Hanbit Electronics Co., Ltd.",
    "00085C": "Shanghai Dare Technologies Co. Ltd.",
    "00085D": "Aastra",
    "00085E": "PCO AG",
    "00085F": "Picanol N.V.",
    "000860": "LodgeNet Entertainment Corp.",
    "000861": "SoftEnergy Co., Ltd.",
    "000862": "NEC Eluminant Technologies, Inc.",
    "000863": "Entrisphere Inc.",
    "000864": "Fasy S.p.A.",
    "000865": "JASCOM CO., LTD",
    "000866": "DSX Access Systems, Inc.",
    "000867": "Uptime Devices",
    "000868": "PurOptix",
    "000869": "Command-e Technology Co.,Ltd.",
    "00086A": "Industrie Technik IPS GmbH",
    "00086B": "MIPSYS",
    "00086C": "Plasmon LMS",
    "00086D": "Missouri FreeNet",
    "00086E": "Hyglo AB",
    "00086F": "Resources Computer Network Ltd.",
    "000870": "Rasvia Systems, Inc.",
    "000871": "NORTHDATA Co., Ltd.",
    "000872": "Sorenson Technologies, Inc.",
    "000873": "DAP Design B.V.",
    "000874": "Dell Computer Corp.",
    "000875": "Acorp Electronics Corp.",
    "000876": "SDSystem",
    "000877": "Liebert-Hiross Spa",
    "000878": "Benchmark Storage Innovations",
    "000879": "CEM Corporation",
    "00087A": "Wipotec GmbH",
    "00087B": "RTX Telecom A/S",
    "00087C": "Cisco Systems, Inc.",
    "00087D": "Cisco Systems Inc.",
    "00087E": "Bon Electro-Telecom Inc.",
    "00087F": "SPAUN electronic GmbH & Co. KG",
    "000880": "BroadTel Canada Communications inc.",
    "000881": "DIGITAL HANDS CO.,LTD.",
    "000882": "SIGMA CORPORATION",
    "000883": "Hewlett-Packard Company",
    "000884": "Index Braille AB",
    "000885": "EMS Dr. Thomas Wuensche",
    "000886": "Hansung Teliann, Inc.",
    "000887": "Maschinenfabrik Reinhausen GmbH",
    "000888": "OULLIM Information Technology Inc,.",
    "000889": "Echostar Technologies Corp",
    "00088A": "Minds@Work",
    "00088B": "Tropic Networks Inc.",
    "00088C": "Quanta Network Systems Inc.",
    "00088D": "Sigma-Links Inc.",
    "00088E": "Nihon Computer Co., Ltd.",
    "00088F": "ADVANCED DIGITAL TECHNOLOGY",
    "000890": "AVILINKS SA",
    "000891": "Lyan Inc.",
    "000892": "EM Solutions",
    "000893": "LE INFORMATION COMMUNICATION INC.",
    "000894": "InnoVISION Multimedia Ltd.",
    "000895": "DIRC Technologie GmbH & Co.KG",
    "000896": "Printronix, Inc.",
    "000897": "Quake Technologies",
    "000898": "Gigabit Optics Corporation",
    "000899": "Netbind, Inc.",
    "00089A": "Alcatel Microelectronics",
    "00089B": "ICP Electronics Inc.",
    "00089C": "Elecs Industry Co., Ltd.",
    "00089D": "UHD-Elektronik",
    "00089E": "Beijing Enter-Net co.LTD",
    "00089F": "EFM Networks",
    "0008A0": "Stotz Feinmesstechnik GmbH",
    "0008A1": "CNet Technology Inc.",
    "0008A2": "ADI Engineering, Inc.",
    "0008A3": "Cisco Systems",
    "0008A4": "Cisco Systems",
    "0008A5": "Peninsula Systems Inc.",
    "0008A6": "Multiware & Image Co., Ltd.",
    "0008A7": "iLogic Inc.",
    "0008A8": "Systec Co., Ltd.",
    "0008A9": "SangSang Technology, Inc.",
    "0008AA": "KARAM",
    "0008AB": "EnerLinx.com, Inc.",
    "0008AC": "PRIVATE",
    "0008AD": "Toyo-Linx Co., Ltd.",
    "0008AE": "PacketFront Sweden AB",
    "0008AF": "Novatec Corporation",
    "0008B0": "BKtel communications GmbH",
    "0008B1": "ProQuent Systems",
    "0008B2": "SHENZHEN COMPASS TECHNOLOGY DEVELOPMENT CO.,LTD",
    "0008B3": "Fastwel",
    "0008B4": "SYSPOL",
    "0008B5": "TAI GUEN ENTERPRISE CO., LTD",
    "0008B6": "RouteFree, Inc.",
    "0008B7": "HIT Incorporated",
    "0008B8": "E.F. Johnson",
    "0008B9": "KAON MEDIA Co., Ltd.",
    "0008BA": "Erskine Systems Ltd",
    "0008BB": "NetExcell",
    "0008BC": "Ilevo AB",
    "0008BD": "TEPG-US",
    "0008BE": "XENPAK MSA Group",
    "0008BF": "Aptus Elektronik AB",
    "0008C0": "ASA SYSTEMS",
    "0008C1": "Avistar Communications Corporation",
    "0008C2": "Cisco Systems",
    "0008C3": "Contex A/S",
    "0008C4": "Hikari Co.,Ltd.",
    "0008C5": "Liontech Co., Ltd.",
    "0008C6": "Philips Consumer Communications",
    "0008C7": "Hewlett Packard",
    "0008C8": "Soneticom, Inc.",
    "0008C9": "TechniSat Digital GmbH",
    "0008CA": "TwinHan Technology Co.,Ltd",
    "0008CB": "Zeta Broadband Inc.",
    "0008CC": "Remotec, Inc.",
    "0008CD": "With-Net Inc",
    "0008CE": "IPMobileNet Inc.",
    "0008CF": "Nippon Koei Power Systems Co., Ltd.",
    "0008D0": "Musashi Engineering Co., LTD.",
    "0008D1": "KAREL INC.",
    "0008D2": "ZOOM Networks Inc.",
    "0008D3": "Hercules Technologies S.A.",
    "0008D4": "IneoQuest Technologies, Inc",
    "0008D5": "Vanguard Managed Solutions",
    "0008D6": "HASSNET Inc.",
    "0008D7": "HOW CORPORATION",
    "0008D8": "Dowkey Microwave",
    "0008D9": "Mitadenshi Co.,LTD",
    "0008DA": "SofaWare Technologies Ltd.",
    "0008DB": "Corrigent Systems",
    "0008DC": "Wiznet",
    "0008DD": "Telena Communications, Inc.",
    "0008DE": "3UP Systems",
    "0008DF": "Alistel Inc.",
    "0008E0": "ATO Technology Ltd.",
    "0008E1": "Barix AG",
    "0008E2": "Cisco Systems",
    "0008E3": "Cisco Systems",
    "0008E4": "Envenergy Inc",
    "0008E5": "IDK Corporation",
    "0008E6": "Littlefeet",
    "0008E7": "SHI ControlSystems,Ltd.",
    "0008E8": "Excel Master Ltd.",
    "0008E9": "NextGig",
    "0008EA": "Motion Control Engineering, Inc",
    "0008EB": "ROMWin Co.,Ltd.",
    "0008EC": "Zonu, Inc.",
    "0008ED": "ST&T Instrument Corp.",
    "0008EE": "Logic Product Development",
    "0008EF": "DIBAL,S.A.",
    "0008F0": "Next Generation Systems, Inc.",
    "0008F1": "Voltaire",
    "0008F2": "C&S Technology",
    "0008F3": "WANY",
    "0008F4": "Bluetake Technology Co., Ltd.",
    "0008F5": "YESTECHNOLOGY Co.,Ltd.",
    "0008F6": "Sumitomo Electric System Solutions Co.,Ltd.",
    "0008F7": "Hitachi Ltd, Semiconductor &amp; Integrated Circuits Gr",
    "0008F8": "Guardall Ltd",
    "0008F9": "Padcom, Inc.",
    "0008FA": "Karl E.Brinkmann GmbH",
    "0008FB": "SonoSite, Inc.",
    "0008FC": "Gigaphoton Inc.",
    "0008FD": "BlueKorea Co., Ltd.",
    "0008FE": "UNIK C&C Co.,Ltd.",
    "0008FF": "Trilogy Communications Ltd",
    "000900": "TMT",
    "000901": "Shenzhen Shixuntong Information & Technoligy Co",
    "000902": "Redline Communications Inc.",
    "000903": "Panasas, Inc",
    "000904": "MONDIAL electronic",
    "000905": "iTEC Technologies Ltd.",
    "000906": "Esteem Networks",
    "000907": "Chrysalis Development",
    "000908": "VTech Technology Corp.",
    "000909": "Telenor Connect A/S",
    "00090A": "SnedFar Technology Co., Ltd.",
    "00090B": "MTL Instruments PLC",
    "00090C": "Mayekawa Mfg. Co. Ltd.",
    "00090D": "LEADER ELECTRONICS CORP.",
    "00090E": "Helix Technology Inc.",
    "00090F": "Fortinet Inc.",
    "000910": "Simple Access Inc.",
    "000911": "Cisco Systems",
    "000912": "Cisco Systems",
    "000913": "SystemK Corporation",
    "000914": "COMPUTROLS INC.",
    "000915": "CAS Corp.",
    "000916": "Listman Home Technologies, Inc.",
    "000917": "WEM Technology Inc",
    "000918": "SAMSUNG TECHWIN CO.,LTD",
    "000919": "MDS Gateways",
    "00091A": "Macat Optics & Electronics Co., Ltd.",
    "00091B": "Digital Generation Inc.",
    "00091C": "CacheVision, Inc",
    "00091D": "Proteam Computer Corporation",
    "00091E": "Firstech Technology Corp.",
    "00091F": "A&amp;D Co., Ltd.",
    "000920": "EpoX COMPUTER CO.,LTD.",
    "000921": "Planmeca Oy",
    "000922": "TST Biometrics GmbH",
    "000923": "Heaman System Co., Ltd",
    "000924": "Telebau GmbH",
    "000925": "VSN Systemen BV",
    "000926": "YODA COMMUNICATIONS, INC.",
    "000927": "TOYOKEIKI CO.,LTD.",
    "000928": "Telecore Inc",
    "000929": "Sanyo Industries (UK) Limited",
    "00092A": "MYTECS Co.,Ltd.",
    "00092B": "iQstor Networks, Inc.",
    "00092C": "Hitpoint Inc.",
    "00092D": "High Tech Computer, Corp.",
    "00092E": "B&Tech System Inc.",
    "00092F": "Akom Technology Corporation",
    "000930": "AeroConcierge Inc.",
    "000931": "Future Internet, Inc.",
    "000932": "Omnilux",
    "000933": "OPTOVALLEY Co. Ltd.",
    "000934": "Dream-Multimedia-Tv GmbH",
    "000935": "Sandvine Incorporated",
    "000936": "Ipetronik GmbH & Co.KG",
    "000937": "Inventec Appliance Corp",
    "000938": "Allot Communications",
    "000939": "ShibaSoku Co.,Ltd.",
    "00093A": "Molex Fiber Optics",
    "00093B": "HYUNDAI NETWORKS INC.",
    "00093C": "Jacques Technologies P/L",
    "00093D": "Newisys,Inc.",
    "00093E": "C&I Technologies",
    "00093F": "Double-Win Enterpirse CO., LTD",
    "000940": "AGFEO GmbH & Co. KG",
    "000941": "Allied Telesis K.K.",
    "000942": "CRESCO, LTD.",
    "000943": "Cisco Systems",
    "000944": "Cisco Systems",
    "000945": "Palmmicro Communications Inc",
    "000946": "Cluster Labs GmbH",
    "000947": "Aztek, Inc.",
    "000948": "Vista Control Systems, Corp.",
    "000949": "Glyph Technologies Inc.",
    "00094A": "Homenet Communications",
    "00094B": "FillFactory NV",
    "00094C": "Communication Weaver Co.,Ltd.",
    "00094D": "Braintree Communications Pty Ltd",
    "00094E": "BARTECH SYSTEMS INTERNATIONAL, INC",
    "00094F": "elmegt GmbH & Co. KG",
    "000950": "Independent Storage Corporation",
    "000951": "Apogee Instruments, Inc",
    "000952": "Auerswald GmbH & Co. KG",
    "000953": "Linkage System Integration Co.Ltd.",
    "000954": "AMiT spol. s. r. o.",
    "000955": "Young Generation International Corp.",
    "000956": "Network Systems Group, Ltd. (NSG)",
    "000957": "Supercaller, Inc.",
    "000958": "INTELNET S.A.",
    "000959": "Sitecsoft",
    "00095A": "RACEWOOD TECHNOLOGY",
    "00095B": "Netgear, Inc.",
    "00095C": "Philips Medical Systems - Cardiac and Monitoring Systems (CM",
    "00095D": "Dialogue Technology Corp.",
    "00095E": "Masstech Group Inc.",
    "00095F": "Telebyte, Inc.",
    "000960": "YOZAN Inc.",
    "000961": "Switchgear and Instrumentation Ltd",
    "000962": "Sonitor Technologies AS",
    "000963": "Dominion Lasercom Inc.",
    "000964": "Hi-Techniques, Inc.",
    "000965": "PRIVATE",
    "000966": "Thales Navigation",
    "000967": "Tachyon, Inc",
    "000968": "TECHNOVENTURE, INC.",
    "000969": "Meret Optical Communications",
    "00096A": "Cloverleaf Communications Inc.",
    "00096B": "IBM Corporation",
    "00096C": "Imedia Semiconductor Corp.",
    "00096D": "Powernet Technologies Corp.",
    "00096E": "GIANT ELECTRONICS LTD.",
    "00096F": "Beijing Zhongqing Elegant Tech. Corp.,Limited",
    "000970": "Vibration Research Corporation",
    "000971": "Time Management, Inc.",
    "000972": "Securebase,Inc",
    "000973": "Lenten Technology Co., Ltd.",
    "000974": "Innopia Technologies, Inc.",
    "000975": "fSONA Communications Corporation",
    "000976": "Datasoft ISDN Systems GmbH",
    "000977": "Brunner Elektronik AG",
    "000978": "AIJI System Co., Ltd.",
    "000979": "Advanced Television Systems Committee, Inc.",
    "00097A": "Louis Design Labs.",
    "00097B": "Cisco Systems",
    "00097C": "Cisco Systems",
    "00097D": "SecWell Networks Oy",
    "00097E": "IMI TECHNOLOGY CO., LTD",
    "00097F": "Vsecure 2000 LTD.",
    "000980": "Power Zenith Inc.",
    "000981": "Newport Networks",
    "000982": "Loewe Opta GmbH",
    "000983": "Gvision Incorporated",
    "000984": "MyCasa Network Inc.",
    "000985": "Auto Telecom Company",
    "000986": "Metalink LTD.",
    "000987": "NISHI NIPPON ELECTRIC WIRE & CABLE CO.,LTD.",
    "000988": "Nudian Electron Co., Ltd.",
    "000989": "VividLogic Inc.",
    "00098A": "EqualLogic Inc",
    "00098B": "Entropic Communications, Inc.",
    "00098C": "Option Wireless Sweden",
    "00098D": "Velocity Semiconductor",
    "00098E": "ipcas GmbH",
    "00098F": "Cetacean Networks",
    "000990": "ACKSYS Communications & systems",
    "000991": "GE Fanuc Automation Manufacturing, Inc.",
    "000992": "InterEpoch Technology,INC.",
    "000993": "Visteon Corporation",
    "000994": "Cronyx Engineering",
    "000995": "Castle Technology Ltd",
    "000996": "RDI",
    "000997": "Nortel Networks",
    "000998": "Capinfo Company Limited",
    "000999": "CP GEORGES RENAULT",
    "00099A": "ELMO COMPANY, LIMITED",
    "00099B": "Western Telematic Inc.",
    "00099C": "Naval Research Laboratory",
    "00099D": "Haliplex Communications",
    "00099E": "Testech, Inc.",
    "00099F": "VIDEX INC.",
    "0009A0": "Microtechno Corporation",
    "0009A1": "Telewise Communications, Inc.",
    "0009A2": "Interface Co., Ltd.",
    "0009A3": "Leadfly Techologies Corp. Ltd.",
    "0009A4": "HARTEC Corporation",
    "0009A5": "HANSUNG ELETRONIC INDUSTRIES DEVELOPMENT CO., LTD",
    "0009A6": "Ignis Optics, Inc.",
    "0009A7": "Bang & Olufsen A/S",
    "0009A8": "Eastmode Pte Ltd",
    "0009A9": "Ikanos Communications",
    "0009AA": "Data Comm for Business, Inc.",
    "0009AB": "Netcontrol Oy",
    "0009AC": "LANVOICE",
    "0009AD": "HYUNDAI SYSCOMM, INC.",
    "0009AE": "OKANO ELECTRIC CO.,LTD",
    "0009AF": "e-generis",
    "0009B0": "Onkyo Corporation",
    "0009B1": "Kanematsu Electronics, Ltd.",
    "0009B2": "L&F Inc.",
    "0009B3": "MCM Systems Ltd",
    "0009B4": "KISAN TELECOM CO., LTD.",
    "0009B5": "3J Tech. Co., Ltd.",
    "0009B6": "Cisco Systems",
    "0009B7": "Cisco Systems",
    "0009B8": "Entise Systems",
    "0009B9": "Action Imaging Solutions",
    "0009BA": "MAKU Informationstechik GmbH",
    "0009BB": "MathStar, Inc.",
    "0009BC": "Integrian, Inc.",
    "0009BD": "Epygi Technologies, Ltd.",
    "0009BE": "Mamiya-OP Co.,Ltd.",
    "0009BF": "Nintendo Co.,Ltd.",
    "0009C0": "6WIND",
    "0009C1": "PROCES-DATA A/S",
    "0009C2": "PRIVATE",
    "0009C3": "NETAS",
    "0009C4": "Medicore Co., Ltd",
    "0009C5": "KINGENE Technology Corporation",
    "0009C6": "Visionics Corporation",
    "0009C7": "Movistec",
    "0009C8": "SINAGAWA TSUSHIN KEISOU SERVICE",
    "0009C9": "BlueWINC Co., Ltd.",
    "0009CA": "iMaxNetworks(Shenzhen)Limited.",
    "0009CB": "HBrain",
    "0009CC": "Moog GmbH",
    "0009CD": "HUDSON SOFT CO.,LTD.",
    "0009CE": "SpaceBridge Semiconductor Corp.",
    "0009CF": "iAd GmbH",
    "0009D0": "Solacom Technologies Inc.",
    "0009D1": "SERANOA NETWORKS INC",
    "0009D2": "Mai Logic Inc.",
    "0009D3": "Western DataCom Co., Inc.",
    "0009D4": "Transtech Networks",
    "0009D5": "Signal Communication, Inc.",
    "0009D6": "KNC One GmbH",
    "0009D7": "DC Security Products",
    "0009D8": "PRIVATE",
    "0009D9": "Neoscale Systems, Inc",
    "0009DA": "Control Module Inc.",
    "0009DB": "eSpace",
    "0009DC": "Galaxis Technology AG",
    "0009DD": "Mavin Technology Inc.",
    "0009DE": "Samjin Information & Communications Co., Ltd.",
    "0009DF": "Vestel Komunikasyon Sanayi ve Ticaret A.S.",
    "0009E0": "XEMICS S.A.",
    "0009E1": "Gemtek Technology Co., Ltd.",
    "0009E2": "Sinbon Electronics Co., Ltd.",
    "0009E3": "Angel Iglesias S.A.",
    "0009E4": "K Tech Infosystem Inc.",
    "0009E5": "Hottinger Baldwin Messtechnik GmbH",
    "0009E6": "Cyber Switching Inc.",
    "0009E7": "ADC Techonology",
    "0009E8": "Cisco Systems",
    "0009E9": "Cisco Systems",
    "0009EA": "YEM Inc.",
    "0009EB": "HuMANDATA LTD.",
    "0009EC": "Daktronics, Inc.",
    "0009ED": "CipherOptics",
    "0009EE": "MEIKYO ELECTRIC CO.,LTD",
    "0009EF": "Vocera Communications",
    "0009F0": "Shimizu Technology Inc.",
    "0009F1": "Yamaki Electric Corporation",
    "0009F2": "Cohu, Inc., Electronics Division",
    "0009F3": "WELL Communication Corp.",
    "0009F4": "Alcon Laboratories, Inc.",
    "0009F5": "Emerson Network Power Co.,Ltd",
    "0009F6": "Shenzhen Eastern Digital Tech Ltd.",
    "0009F7": "SED, a division of Calian",
    "0009F8": "UNIMO TECHNOLOGY CO., LTD.",
    "0009F9": "ART JAPAN CO., LTD.",
    "0009FB": "Philips Medizinsysteme Boeblingen GmbH",
    "0009FC": "IPFLEX Inc.",
    "0009FD": "Ubinetics Limited",
    "0009FE": "Daisy Technologies, Inc.",
    "0009FF": "X.net 2000 GmbH",
    "000A00": "Mediatek Corp.",
    "000A01": "SOHOware, Inc.",
    "000A02": "ANNSO CO., LTD.",
    "000A03": "ENDESA SERVICIOS, S.L.",
    "000A04": "3Com Ltd",
    "000A05": "Widax Corp.",
    "000A06": "Teledex LLC",
    "000A07": "WebWayOne Ltd",
    "000A08": "ALPINE ELECTRONICS, INC.",
    "000A09": "TaraCom Integrated Products, Inc.",
    "000A0A": "SUNIX Co., Ltd.",
    "000A0B": "Sealevel Systems, Inc.",
    "000A0C": "Scientific Research Corporation",
    "000A0D": "MergeOptics GmbH",
    "000A0E": "Invivo Research Inc.",
    "000A0F": "Ilryung Telesys, Inc",
    "000A10": "FAST media integrations AG",
    "000A11": "ExPet Technologies, Inc",
    "000A12": "Azylex Technology, Inc",
    "000A13": "Honeywell Video Systems",
    "000A14": "TECO a.s.",
    "000A15": "Silicon Data, Inc",
    "000A16": "Lassen Research",
    "000A17": "NESTAR COMMUNICATIONS, INC",
    "000A18": "Vichel Inc.",
    "000A19": "Valere Power, Inc.",
    "000A1A": "Imerge Ltd",
    "000A1B": "Stream Labs",
    "000A1C": "Bridge Information Co., Ltd.",
    "000A1D": "Optical Communications Products Inc.",
    "000A1E": "Red-M Products Limited",
    "000A1F": "ART WARE Telecommunication Co., Ltd.",
    "000A20": "SVA Networks, Inc.",
    "000A21": "Integra Telecom Co. Ltd",
    "000A22": "Amperion Inc",
    "000A23": "Parama Networks Inc",
    "000A24": "Octave Communications",
    "000A25": "CERAGON NETWORKS",
    "000A26": "CEIA S.p.A.",
    "000A27": "Apple Computer, Inc.",
    "000A28": "Motorola",
    "000A29": "Pan Dacom Networking AG",
    "000A2A": "QSI Systems Inc.",
    "000A2B": "Etherstuff",
    "000A2C": "Active Tchnology Corporation",
    "000A2D": "PRIVATE",
    "000A2E": "MAPLE NETWORKS CO., LTD",
    "000A2F": "Artnix Inc.",
    "000A30": "Johnson Controls-ASG",
    "000A31": "HCV Wireless",
    "000A32": "Xsido Corporation",
    "000A33": "Emulex Corporation",
    "000A34": "Identicard Systems Incorporated",
    "000A35": "Xilinx",
    "000A36": "Synelec Telecom Multimedia",
    "000A37": "Procera Networks, Inc.",
    "000A38": "Apani Networks",
    "000A39": "LoPA Information Technology",
    "000A3A": "J-THREE INTERNATIONAL Holding Co., Ltd.",
    "000A3B": "GCT Semiconductor, Inc",
    "000A3C": "Enerpoint Ltd.",
    "000A3D": "Elo Sistemas Eletronicos S.A.",
    "000A3E": "EADS Telecom",
    "000A3F": "Data East Corporation",
    "000A40": "Crown Audio",
    "000A41": "Cisco Systems",
    "000A42": "Cisco Systems",
    "000A43": "Chunghwa Telecom Co., Ltd.",
    "000A44": "Avery Dennison Deutschland GmbH",
    "000A45": "Audio-Technica Corp.",
    "000A46": "ARO Controls SAS",
    "000A47": "Allied Vision Technologies",
    "000A48": "Albatron Technology",
    "000A49": "F5 Networks, Inc.",
    "000A4A": "Targa Systems Ltd.",
    "000A4B": "DataPower Technology, Inc.",
    "000A4C": "Molecular Devices Corporation",
    "000A4D": "Noritz Corporation",
    "000A4E": "UNITEK Electronics INC.",
    "000A4F": "Brain Boxes Limited",
    "000A50": "REMOTEK CORPORATION",
    "000A51": "GyroSignal Technology Co., Ltd.",
    "000A52": "AsiaRF Ltd.",
    "000A53": "Intronics, Incorporated",
    "000A54": "Laguna Hills, Inc.",
    "000A55": "MARKEM Corporation",
    "000A56": "HITACHI Maxell Ltd.",
    "000A57": "Hewlett-Packard Company - Standards",
    "000A58": "Ingenieur-Buero Freyer & Siegel",
    "000A59": "HW server",
    "000A5A": "GreenNET Technologies Co.,Ltd.",
    "000A5B": "Power-One as",
    "000A5C": "Carel s.p.a.",
    "000A5D": "PUC Founder (MSC) Berhad",
    "000A5E": "3COM Corporation",
    "000A5F": "almedio inc.",
    "000A60": "Autostar Technology Pte Ltd",
    "000A61": "Cellinx Systems Inc.",
    "000A62": "Crinis Networks, Inc.",
    "000A63": "DHD GmbH",
    "000A64": "Eracom Technologies",
    "000A65": "GentechMedia.co.,ltd.",
    "000A66": "MITSUBISHI ELECTRIC SYSTEM & SERVICE CO.,LTD.",
    "000A67": "OngCorp",
    "000A68": "SolarFlare Communications, Inc.",
    "000A69": "SUNNY bell Technology Co., Ltd.",
    "000A6A": "SVM Microwaves s.r.o.",
    "000A6B": "Tadiran Telecom Business Systems LTD",
    "000A6C": "Walchem Corporation",
    "000A6D": "EKS Elektronikservice GmbH",
    "000A6E": "Broadcast Technology Limited",
    "000A6F": "ZyFLEX Technologies Inc",
    "000A70": "MPLS Forum",
    "000A71": "Avrio Technologies, Inc",
    "000A72": "STEC, INC.",
    "000A73": "Scientific Atlanta",
    "000A74": "Manticom Networks Inc.",
    "000A75": "Caterpillar, Inc",
    "000A76": "Beida Jade Bird Huaguang Technology Co.,Ltd",
    "000A77": "Bluewire Technologies LLC",
    "000A78": "OLITEC",
    "000A79": "corega K.K.",
    "000A7A": "Kyoritsu Electric Co., Ltd.",
    "000A7B": "Cornelius Consult",
    "000A7C": "Tecton Ltd",
    "000A7D": "Valo, Inc.",
    "000A7E": "The Advantage Group",
    "000A7F": "Teradon Industries, Inc",
    "000A80": "Telkonet Inc.",
    "000A81": "TEIMA Audiotex S.L.",
    "000A82": "TATSUTA SYSTEM ELECTRONICS CO.,LTD.",
    "000A83": "SALTO SYSTEMS S.L.",
    "000A84": "Rainsun Enterprise Co., Ltd.",
    "000A85": "PLAT'C2,Inc",
    "000A86": "Lenze",
    "000A87": "Integrated Micromachines Inc.",
    "000A88": "InCypher S.A.",
    "000A89": "Creval Systems, Inc.",
    "000A8A": "Cisco Systems",
    "000A8B": "Cisco Systems",
    "000A8C": "Guardware Systems Ltd.",
    "000A8D": "EUROTHERM LIMITED",
    "000A8E": "Invacom Ltd",
    "000A8F": "Aska International Inc.",
    "000A90": "Bayside Interactive, Inc.",
    "000A91": "HemoCue AB",
    "000A92": "Presonus Corporation",
    "000A93": "W2 Networks, Inc.",
    "000A94": "ShangHai cellink CO., LTD",
    "000A95": "Apple Computer, Inc.",
    "000A96": "MEWTEL TECHNOLOGY INC.",
    "000A97": "SONICblue, Inc.",
    "000A98": "M+F Gwinner GmbH & Co",
    "000A99": "Dataradio Inc.",
    "000A9A": "Aiptek International Inc",
    "000A9B": "Towa Meccs Corporation",
    "000A9C": "Server Technology, Inc.",
    "000A9D": "King Young Technology Co. Ltd.",
    "000A9E": "BroadWeb Corportation",
    "000A9F": "Pannaway Technologies, Inc.",
    "000AA0": "Cedar Point Communications",
    "000AA1": "V V S Limited",
    "000AA2": "SYSTEK INC.",
    "000AA3": "SHIMAFUJI ELECTRIC CO.,LTD.",
    "000AA4": "SHANGHAI SURVEILLANCE TECHNOLOGY CO,LTD",
    "000AA5": "MAXLINK INDUSTRIES LIMITED",
    "000AA6": "Hochiki Corporation",
    "000AA7": "FEI Electron Optics",
    "000AA8": "ePipe Pty. Ltd.",
    "000AA9": "Brooks Automation GmbH",
    "000AAA": "AltiGen Communications Inc.",
    "000AAB": "Toyota Technical Development Corporation",
    "000AAC": "TerraTec Electronic GmbH",
    "000AAD": "Stargames Corporation",
    "000AAE": "Rosemount Process Analytical",
    "000AAF": "Pipal Systems",
    "000AB0": "LOYTEC electronics GmbH",
    "000AB1": "GENETEC Corporation",
    "000AB2": "Fresnel Wireless Systems",
    "000AB3": "Fa. GIRA",
    "000AB4": "ETIC Telecommunications",
    "000AB5": "Digital Electronic Network",
    "000AB6": "COMPUNETIX, INC",
    "000AB7": "Cisco Systems",
    "000AB8": "Cisco Systems",
    "000AB9": "Astera Technologies Corp.",
    "000ABA": "Arcon Technology Limited",
    "000ABB": "Taiwan Secom Co,. Ltd",
    "000ABC": "Seabridge Ltd.",
    "000ABD": "Rupprecht & Patashnick Co.",
    "000ABE": "OPNET Technologies CO., LTD.",
    "000ABF": "HIROTA SS",
    "000AC0": "Fuyoh Video Industry CO., LTD.",
    "000AC1": "Futuretel",
    "000AC2": "FiberHome Telecommunication Technologies CO.,LTD",
    "000AC3": "eM Technics Co., Ltd.",
    "000AC4": "Daewoo Teletech Co., Ltd",
    "000AC5": "Color Kinetics",
    "000AC6": "Ceterus Networks, Inc.",
    "000AC7": "Unication Group",
    "000AC8": "ZPSYS CO.,LTD. (Planning&Management)",
    "000AC9": "Zambeel Inc",
    "000ACA": "YOKOYAMA SHOKAI CO.,Ltd.",
    "000ACB": "XPAK MSA Group",
    "000ACC": "Winnow Networks, Inc.",
    "000ACD": "Sunrich Technology Limited",
    "000ACE": "RADIANTECH, INC.",
    "000ACF": "PROVIDEO Multimedia Co. Ltd.",
    "000AD0": "Niigata Develoment Center, F.I.T. Co., Ltd.",
    "000AD1": "MWS",
    "000AD2": "JEPICO Corporation",
    "000AD3": "INITECH Co., Ltd",
    "000AD4": "CoreBell Systems Inc.",
    "000AD5": "Brainchild Electronic Co., Ltd.",
    "000AD6": "BeamReach Networks",
    "000AD7": "Origin ELECTRIC CO.,LTD.",
    "000AD8": "IPCserv Technology Corp.",
    "000AD9": "Sony Ericsson Mobile Communications AB",
    "000ADA": "Vindicator Technologies",
    "000ADB": "SkyPilot Network, Inc",
    "000ADC": "RuggedCom Inc.",
    "000ADD": "Allworx Corp.",
    "000ADE": "Happy Communication Co., Ltd.",
    "000ADF": "Gennum Corporation",
    "000AE0": "Fujitsu Softek",
    "000AE1": "EG Technology",
    "000AE2": "Binatone Electronics International, Ltd",
    "000AE3": "YANG MEI TECHNOLOGY CO., LTD",
    "000AE4": "Wistron Corp.",
    "000AE5": "ScottCare Corporation",
    "000AE6": "Elitegroup Computer System Co. (ECS)",
    "000AE7": "ELIOP S.A.",
    "000AE8": "Cathay Roxus Information Technology Co. LTD",
    "000AE9": "AirVast Technology Inc.",
    "000AEA": "ADAM ELEKTRONIK LTD.STI.",
    "000AEB": "Shenzhen Tp-Link Technology Co; Ltd.",
    "000AEC": "Koatsu Gas Kogyo Co., Ltd.",
    "000AED": "HARTING Systems GmbH & Co KG",
    "000AEE": "GCD Hard- & Software GmbH",
    "000AEF": "OTRUM ASA",
    "000AF0": "SHIN-OH ELECTRONICS CO., LTD. R&D",
    "000AF1": "Clarity Design, Inc.",
    "000AF2": "NeoAxiom Corp.",
    "000AF3": "Cisco Systems",
    "000AF4": "Cisco Systems",
    "000AF5": "Airgo Networks, Inc.",
    "000AF6": "Computer Process Controls",
    "000AF7": "Broadcom Corp.",
    "000AF8": "American Telecare Inc.",
    "000AF9": "HiConnect, Inc.",
    "000AFA": "Traverse Technologies Australia",
    "000AFB": "Ambri Limited",
    "000AFC": "Core Tec Communications, LLC",
    "000AFD": "Viking Electronic Services",
    "000AFE": "NovaPal Ltd",
    "000AFF": "Kilchherr Elektronik AG",
    "000B00": "FUJIAN START COMPUTER EQUIPMENT CO.,LTD",
    "000B01": "DAIICHI ELECTRONICS CO., LTD.",
    "000B02": "Dallmeier electronic",
    "000B03": "Taekwang Industrial Co., Ltd",
    "000B04": "Volktek Corporation",
    "000B05": "Pacific Broadband Networks",
    "000B06": "Motorola BCS",
    "000B07": "Voxpath Networks",
    "000B08": "Pillar Data Systems",
    "000B09": "Ifoundry Systems Singapore",
    "000B0A": "dBm Optics",
    "000B0B": "Corrent Corporation",
    "000B0C": "Agile Systems Inc.",
    "000B0D": "Air2U, Inc.",
    "000B0E": "Trapeze Networks",
    "000B0F": "Nyquist Industrial Control BV",
    "000B10": "11wave Technonlogy Co.,Ltd",
    "000B11": "HIMEJI ABC TRADING CO.,LTD.",
    "000B12": "NURI Telecom Co., Ltd.",
    "000B13": "ZETRON INC",
    "000B14": "ViewSonic Corporation",
    "000B15": "Platypus Technology",
    "000B16": "Communication Machinery Corporation",
    "000B17": "MKS Instruments",
    "000B18": "PRIVATE",
    "000B19": "Vernier Networks, Inc.",
    "000B1A": "Teltone Corporation",
    "000B1B": "Systronix, Inc.",
    "000B1C": "SIBCO bv",
    "000B1D": "LayerZero Power Systems, Inc.",
    "000B1E": "KAPPA opto-electronics GmbH",
    "000B1F": "I CON Computer Co.",
    "000B20": "Hirata corporation",
    "000B21": "G-Star Communications Inc.",
    "000B22": "Environmental Systems and Services",
    "000B23": "Siemens Subscriber Networks",
    "000B24": "AirLogic",
    "000B25": "Aeluros",
    "000B26": "Wetek Corporation",
    "000B27": "Scion Corporation",
    "000B28": "Quatech Inc.",
    "000B29": "LG Industrial Systems Co.,Ltd.",
    "000B2A": "HOWTEL Co., Ltd.",
    "000B2B": "HOSTNET CORPORATION",
    "000B2C": "Eiki Industrial Co. Ltd.",
    "000B2D": "Danfoss Inc.",
    "000B2E": "Cal-Comp Electronics (Thailand) Public Company Limited Taipe",
    "000B2F": "bplan GmbH",
    "000B30": "Beijing Gongye Science & Technology Co.,Ltd",
    "000B31": "Yantai ZhiYang Scientific and technology industry CO., LTD",
    "000B32": "VORMETRIC, INC.",
    "000B33": "Vivato",
    "000B34": "ShangHai Broadband Technologies CO.LTD",
    "000B35": "Quad Bit System co., Ltd.",
    "000B36": "Productivity Systems, Inc.",
    "000B37": "MANUFACTURE DES MONTRES ROLEX SA",
    "000B38": "Knuerr AG",
    "000B39": "Keisoku Giken Co.,Ltd.",
    "000B3A": "QuStream Corporation",
    "000B3B": "devolo AG",
    "000B3C": "Cygnal Integrated Products, Inc.",
    "000B3D": "CONTAL OK Ltd.",
    "000B3E": "BittWare, Inc",
    "000B3F": "Anthology Solutions Inc.",
    "000B40": "OpNext Inc.",
    "000B41": "Ing. Buero Dr. Beutlhauser",
    "000B42": "commax Co., Ltd.",
    "000B43": "Microscan Systems, Inc.",
    "000B44": "Concord IDea Corp.",
    "000B45": "Cisco",
    "000B46": "Cisco",
    "000B47": "Advanced Energy",
    "000B48": "sofrel",
    "000B49": "RF-Link System Inc.",
    "000B4A": "Visimetrics (UK) Ltd",
    "000B4B": "VISIOWAVE SA",
    "000B4C": "Clarion (M) Sdn Bhd",
    "000B4D": "Emuzed",
    "000B4E": "VertexRSI, General Dynamics SatCOM Technologies, Inc.",
    "000B4F": "Verifone, INC.",
    "000B50": "Oxygnet",
    "000B51": "Micetek International Inc.",
    "000B52": "JOYMAX ELECTRONICS CORP.",
    "000B53": "INITIUM Co., Ltd.",
    "000B54": "BiTMICRO Networks, Inc.",
    "000B55": "ADInstruments",
    "000B56": "Cybernetics",
    "000B57": "Silicon Laboratories",
    "000B58": "Astronautics C.A LTD",
    "000B59": "ScriptPro, LLC",
    "000B5A": "HyperEdge",
    "000B5B": "Rincon Research Corporation",
    "000B5C": "Newtech Co.,Ltd",
    "000B5D": "FUJITSU LIMITED",
    "000B5E": "Audio Engineering Society Inc.",
    "000B5F": "Cisco Systems",
    "000B60": "Cisco Systems",
    "000B61": "Friedrich Lütze GmbH &Co.",
    "000B62": "Ingenieurbuero fuer Elektronikdesign Ingo Mohnen",
    "000B63": "Kaleidescape",
    "000B64": "Kieback & Peter GmbH & Co KG",
    "000B65": "Sy.A.C. srl",
    "000B66": "Teralink Communications",
    "000B67": "Topview Technology Corporation",
    "000B68": "Addvalue Communications Pte Ltd",
    "000B69": "Franke Finland Oy",
    "000B6A": "Asiarock Incorporation",
    "000B6B": "Wistron Neweb Corp.",
    "000B6C": "Sychip Inc.",
    "000B6D": "SOLECTRON JAPAN NAKANIIDA",
    "000B6E": "Neff Instrument Corp.",
    "000B6F": "Media Streaming Networks Inc",
    "000B70": "Load Technology, Inc.",
    "000B71": "Litchfield Communications Inc.",
    "000B72": "Lawo AG",
    "000B73": "Kodeos Communications",
    "000B74": "Kingwave Technology Co., Ltd.",
    "000B75": "Iosoft Ltd.",
    "000B76": "ET&T Technology Co. Ltd.",
    "000B77": "Cogent Systems, Inc.",
    "000B78": "TAIFATECH INC.",
    "000B79": "X-COM, Inc.",
    "000B7A": "Wave Science Inc.",
    "000B7B": "Test-Um Inc.",
    "000B7C": "Telex Communications",
    "000B7D": "SOLOMON EXTREME INTERNATIONAL LTD.",
    "000B7E": "SAGINOMIYA Seisakusho Inc.",
    "000B7F": "OmniWerks",
    "000B80": "Lycium Networks",
    "000B81": "Kaparel Corporation",
    "000B82": "Grandstream Networks, Inc.",
    "000B83": "DATAWATT B.V.",
    "000B84": "BODET",
    "000B85": "Cisco Systems",
    "000B86": "Aruba Networks",
    "000B87": "American Reliance Inc.",
    "000B88": "Vidisco ltd.",
    "000B89": "Top Global Technology, Ltd.",
    "000B8A": "MITEQ Inc.",
    "000B8B": "KERAJET, S.A.",
    "000B8C": "Flextronics",
    "000B8D": "Avvio Networks",
    "000B8E": "Ascent Corporation",
    "000B8F": "AKITA ELECTRONICS SYSTEMS CO.,LTD.",
    "000B90": "Adva Optical Networking Inc.",
    "000B91": "Aglaia Gesellschaft für Bildverarbeitung und Kommunikation m",
    "000B92": "Ascom Danmark A/S",
    "000B93": "Ritter Elektronik",
    "000B94": "Digital Monitoring Products, Inc.",
    "000B95": "eBet Gaming Systems Pty Ltd",
    "000B96": "Innotrac Diagnostics Oy",
    "000B97": "Matsushita Electric Industrial Co.,Ltd.",
    "000B98": "NiceTechVision",
    "000B99": "SensAble Technologies, Inc.",
    "000B9A": "Shanghai Ulink Telecom Equipment Co. Ltd.",
    "000B9B": "Sirius System Co, Ltd.",
    "000B9C": "TriBeam Technologies, Inc.",
    "000B9D": "TwinMOS Technologies Inc.",
    "000B9E": "Yasing Technology Corp.",
    "000B9F": "Neue ELSA GmbH",
    "000BA0": "T&L Information Inc.",
    "000BA1": "SYSCOM Ltd.",
    "000BA2": "Sumitomo Electric Networks, Inc",
    "000BA3": "Siemens AG, I&S",
    "000BA4": "Shiron Satellite Communications Ltd. (1996)",
    "000BA5": "Quasar Cipta Mandiri, PT",
    "000BA6": "Miyakawa Electric Works Ltd.",
    "000BA7": "Maranti Networks",
    "000BA8": "HANBACK ELECTRONICS CO., LTD.",
    "000BA9": "CloudShield Technologies, Inc.",
    "000BAA": "Aiphone co.,Ltd",
    "000BAB": "Advantech Technology (CHINA) Co., Ltd.",
    "000BAC": "3Com Ltd",
    "000BAD": "PC-PoS Inc.",
    "000BAE": "Vitals System Inc.",
    "000BAF": "WOOJU COMMUNICATIONS Co,.Ltd",
    "000BB0": "Sysnet Telematica srl",
    "000BB1": "Super Star Technology Co., Ltd.",
    "000BB2": "SMALLBIG TECHNOLOGY",
    "000BB3": "RiT technologies Ltd.",
    "000BB4": "RDC Semiconductor Inc.,",
    "000BB5": "nStor Technologies, Inc.",
    "000BB6": "Mototech Inc.",
    "000BB7": "Micro Systems Co.,Ltd.",
    "000BB8": "Kihoku Electronic Co.",
    "000BB9": "Imsys AB",
    "000BBA": "Harmonic Broadband Access Networks",
    "000BBB": "Etin Systems Co., Ltd",
    "000BBC": "En Garde Systems, Inc.",
    "000BBD": "Connexionz Limited",
    "000BBE": "Cisco Systems",
    "000BBF": "Cisco Systems",
    "000BC0": "China IWNComm Co., Ltd.",
    "000BC1": "Bay Microsystems, Inc.",
    "000BC2": "Corinex Communication Corp.",
    "000BC3": "Multiplex, Inc.",
    "000BC4": "BIOTRONIK GmbH & Co",
    "000BC5": "SMC Networks, Inc.",
    "000BC6": "ISAC, Inc.",
    "000BC7": "ICET S.p.A.",
    "000BC8": "AirFlow Networks",
    "000BC9": "Electroline Equipment",
    "000BCA": "DATAVAN International Corporation",
    "000BCB": "Fagor Automation , S. Coop",
    "000BCC": "JUSAN, S.A.",
    "000BCD": "Hewlett Packard",
    "000BCE": "Free2move AB",
    "000BCF": "AGFA NDT INC.",
    "000BD0": "XiMeta Technology Americas Inc.",
    "000BD1": "Aeronix, Inc.",
    "000BD2": "Remopro Technology Inc.",
    "000BD3": "cd3o",
    "000BD4": "Beijing Wise Technology & Science Development Co.Ltd",
    "000BD5": "Nvergence, Inc.",
    "000BD6": "Paxton Access Ltd",
    "000BD7": "MBB Gelma GmbH",
    "000BD8": "Industrial Scientific Corp.",
    "000BD9": "General Hydrogen",
    "000BDA": "EyeCross Co.,Inc.",
    "000BDB": "Dell ESG PCBA Test",
    "000BDC": "AKCP",
    "000BDD": "TOHOKU RICOH Co., LTD.",
    "000BDE": "TELDIX GmbH",
    "000BDF": "Shenzhen RouterD Networks Limited",
    "000BE0": "SercoNet Ltd.",
    "000BE1": "Nokia NET Product Operations",
    "000BE2": "Lumenera Corporation",
    "000BE3": "Key Stream Co., Ltd.",
    "000BE4": "Hosiden Corporation",
    "000BE5": "HIMS Korea Co., Ltd.",
    "000BE6": "Datel Electronics",
    "000BE7": "COMFLUX TECHNOLOGY INC.",
    "000BE8": "AOIP",
    "000BE9": "Actel Corporation",
    "000BEA": "Zultys Technologies",
    "000BEB": "Systegra AG",
    "000BEC": "NIPPON ELECTRIC INSTRUMENT, INC.",
    "000BED": "ELM Inc.",
    "000BEE": "inc.jet, Incorporated",
    "000BEF": "Code Corporation",
    "000BF0": "MoTEX Products Co., Ltd.",
    "000BF1": "LAP Laser Applikations",
    "000BF2": "Chih-Kan Technology Co., Ltd.",
    "000BF3": "BAE SYSTEMS",
    "000BF4": "PRIVATE",
    "000BF5": "Shanghai Sibo Telecom Technology Co.,Ltd",
    "000BF6": "Nitgen Co., Ltd",
    "000BF7": "NIDEK CO.,LTD",
    "000BF8": "Infinera",
    "000BF9": "Gemstone communications, Inc.",
    "000BFA": "EXEMYS SRL",
    "000BFB": "D-NET International Corporation",
    "000BFC": "Cisco Systems",
    "000BFD": "Cisco Systems",
    "000BFE": "CASTEL Broadband Limited",
    "000BFF": "Berkeley Camera Engineering",
    "000C00": "BEB Industrie-Elektronik AG",
    "000C01": "Abatron AG",
    "000C02": "ABB Oy",
    "000C03": "HDMI Licensing, LLC",
    "000C04": "Tecnova",
    "000C05": "RPA Reserch Co., Ltd.",
    "000C06": "Nixvue Systems Pte Ltd",
    "000C07": "Iftest AG",
    "000C08": "HUMEX Technologies Corp.",
    "000C09": "Hitachi IE Systems Co., Ltd",
    "000C0A": "Guangdong Province Electronic Technology Research Institute",
    "000C0B": "Broadbus Technologies",
    "000C0C": "APPRO TECHNOLOGY INC.",
    "000C0D": "Communications & Power Industries / Satcom Division",
    "000C0E": "XtremeSpectrum, Inc.",
    "000C0F": "Techno-One Co., Ltd",
    "000C10": "PNI Corporation",
    "000C11": "NIPPON DEMPA CO.,LTD.",
    "000C12": "Micro-Optronic-Messtechnik GmbH",
    "000C13": "MediaQ",
    "000C14": "Diagnostic Instruments, Inc.",
    "000C15": "CyberPower Systems, Inc.",
    "000C16": "Concorde Microsystems Inc.",
    "000C17": "AJA Video Systems Inc",
    "000C18": "Zenisu Keisoku Inc.",
    "000C19": "Telio Communications GmbH",
    "000C1A": "Quest Technical Solutions Inc.",
    "000C1B": "ORACOM Co, Ltd.",
    "000C1C": "MicroWeb Co., Ltd.",
    "000C1D": "Mettler & Fuchs AG",
    "000C1E": "Global Cache",
    "000C1F": "Glimmerglass Networks",
    "000C20": "Fi WIn, Inc.",
    "000C21": "Faculty of Science and Technology, Keio University",
    "000C22": "Double D Electronics Ltd",
    "000C23": "Beijing Lanchuan Tech. Co., Ltd.",
    "000C24": "ANATOR",
    "000C25": "Allied Telesyn Networks",
    "000C26": "Weintek Labs. Inc.",
    "000C27": "Sammy Corporation",
    "000C28": "RIFATRON",
    "000C29": "VMware, Inc.",
    "000C2A": "OCTTEL Communication Co., Ltd.",
    "000C2B": "ELIAS Technology, Inc.",
    "000C2C": "Enwiser Inc.",
    "000C2D": "FullWave Technology Co., Ltd.",
    "000C2E": "Openet information technology(shenzhen) Co., Ltd.",
    "000C2F": "SeorimTechnology Co.,Ltd.",
    "000C30": "Cisco",
    "000C31": "Cisco",
    "000C32": "Avionic Design Development GmbH",
    "000C33": "Compucase Enterprise Co. Ltd.",
    "000C34": "Vixen Co., Ltd.",
    "000C35": "KaVo Dental GmbH & Co. KG",
    "000C36": "SHARP TAKAYA ELECTRONICS INDUSTRY CO.,LTD.",
    "000C37": "Geomation, Inc.",
    "000C38": "TelcoBridges Inc.",
    "000C39": "Sentinel Wireless Inc.",
    "000C3A": "Oxance",
    "000C3B": "Orion Electric Co., Ltd.",
    "000C3C": "MediaChorus, Inc.",
    "000C3D": "Glsystech Co., Ltd.",
    "000C3E": "Crest Audio",
    "000C3F": "Cogent Defence & Security Networks,",
    "000C40": "Altech Controls",
    "000C41": "Cisco-Linksys",
    "000C42": "Routerboard.com",
    "000C43": "Ralink Technology, Corp.",
    "000C44": "Automated Interfaces, Inc.",
    "000C45": "Animation Technologies Inc.",
    "000C46": "Allied Telesyn Inc.",
    "000C47": "SK Teletech(R&D Planning Team)",
    "000C48": "QoStek Corporation",
    "000C49": "Dangaard Telecom RTC Division A/S",
    "000C4A": "Cygnus Microsystems (P) Limited",
    "000C4B": "Cheops Elektronik",
    "000C4C": "Arcor AG&Co.",
    "000C4D": "ACRA CONTROL",
    "000C4E": "Winbest Technology CO,LT",
    "000C4F": "UDTech Japan Corporation",
    "000C50": "Seagate Technology",
    "000C51": "Scientific Technologies Inc.",
    "000C52": "Roll Systems Inc.",
    "000C53": "PRIVATE",
    "000C54": "Pedestal Networks, Inc",
    "000C55": "Microlink Communications Inc.",
    "000C56": "Megatel Computer (1986) Corp.",
    "000C57": "MACKIE Engineering Services Belgium BVBA",
    "000C58": "M&S Systems",
    "000C59": "Indyme Electronics, Inc.",
    "000C5A": "IBSmm Industrieelektronik Multimedia",
    "000C5B": "HANWANG TECHNOLOGY CO.,LTD",
    "000C5C": "GTN Systems B.V.",
    "000C5D": "CHIC TECHNOLOGY (CHINA) CORP.",
    "000C5E": "Calypso Medical",
    "000C5F": "Avtec, Inc.",
    "000C60": "ACM Systems",
    "000C61": "AC Tech corporation DBA Advanced Digital",
    "000C62": "ABB Automation Technology Products AB, Control",
    "000C63": "Zenith Electronics Corporation",
    "000C64": "X2 MSA Group",
    "000C65": "Sunin Telecom",
    "000C66": "Pronto Networks Inc",
    "000C67": "OYO ELECTRIC CO.,LTD",
    "000C68": "SigmaTel, Inc.",
    "000C69": "National Radio Astronomy Observatory",
    "000C6A": "MBARI",
    "000C6B": "Kurz Industrie-Elektronik GmbH",
    "000C6C": "Elgato Systems LLC",
    "000C6D": "BOC Edwards",
    "000C6E": "ASUSTEK COMPUTER INC.",
    "000C6F": "Amtek system co.,LTD.",
    "000C70": "ACC GmbH",
    "000C71": "Wybron, Inc",
    "000C72": "Tempearl Industrial Co., Ltd.",
    "000C73": "TELSON ELECTRONICS CO., LTD",
    "000C74": "RIVERTEC CORPORATION",
    "000C75": "Oriental integrated electronics. LTD",
    "000C76": "MICRO-STAR INTERNATIONAL CO., LTD.",
    "000C77": "Life Racing Ltd",
    "000C78": "In-Tech Electronics Limited",
    "000C79": "Extel Communications P/L",
    "000C7A": "DaTARIUS Technologies GmbH",
    "000C7B": "ALPHA PROJECT Co.,Ltd.",
    "000C7C": "Internet Information Image Inc.",
    "000C7D": "TEIKOKU ELECTRIC MFG. CO., LTD",
    "000C7E": "Tellium Incorporated",
    "000C7F": "synertronixx GmbH",
    "000C80": "Opelcomm Inc.",
    "000C81": "Nulec Industries Pty Ltd",
    "000C82": "NETWORK TECHNOLOGIES INC",
    "000C83": "Logical Solutions",
    "000C84": "Eazix, Inc.",
    "000C85": "Cisco Systems",
    "000C86": "Cisco Systems",
    "000C87": "AMD",
    "000C88": "Apache Micro Peripherals, Inc.",
    "000C89": "AC Electric Vehicles, Ltd.",
    "000C8A": "Bose Corporation",
    "000C8B": "Connect Tech Inc",
    "000C8C": "KODICOM CO.,LTD.",
    "000C8D": "MATRIX VISION GmbH",
    "000C8E": "Mentor Engineering Inc",
    "000C8F": "Nergal s.r.l.",
    "000C90": "Octasic Inc.",
    "000C91": "Riverhead Networks Inc.",
    "000C92": "WolfVision Gmbh",
    "000C93": "Xeline Co., Ltd.",
    "000C94": "United Electronic Industries, Inc. (EUI)",
    "000C95": "PrimeNet",
    "000C96": "OQO, Inc.",
    "000C97": "NV ADB TTV Technologies SA",
    "000C98": "LETEK Communications Inc.",
    "000C99": "HITEL LINK Co.,Ltd",
    "000C9A": "Hitech Electronics Corp.",
    "000C9B": "EE Solutions, Inc",
    "000C9C": "Chongho information & communications",
    "000C9D": "AirWalk Communications, Inc.",
    "000C9E": "MemoryLink Corp.",
    "000C9F": "NKE Corporation",
    "000CA0": "StorCase Technology, Inc.",
    "000CA1": "SIGMACOM Co., LTD.",
    "000CA2": "Scopus Network Technologies Ltd",
    "000CA3": "Rancho Technology, Inc.",
    "000CA4": "Prompttec Product Management GmbH",
    "000CA5": "Naman NZ LTd",
    "000CA6": "Mintera Corporation",
    "000CA7": "Metro (Suzhou) Technologies Co., Ltd.",
    "000CA8": "Garuda Networks Corporation",
    "000CA9": "Ebtron Inc.",
    "000CAA": "Cubic Transportation Systems Inc",
    "000CAB": "COMMEND International",
    "000CAC": "Citizen Watch Co., Ltd.",
    "000CAD": "BTU International",
    "000CAE": "Ailocom Oy",
    "000CAF": "TRI TERM CO.,LTD.",
    "000CB0": "Star Semiconductor Corporation",
    "000CB1": "Salland Engineering (Europe) BV",
    "000CB2": "Comstar Co., Ltd.",
    "000CB3": "ROUND Co.,Ltd.",
    "000CB4": "AutoCell Laboratories, Inc.",
    "000CB5": "Premier Technolgies, Inc",
    "000CB6": "NANJING SEU MOBILE & INTERNET TECHNOLOGY CO.,LTD",
    "000CB7": "Nanjing Huazhuo Electronics Co., Ltd.",
    "000CB8": "MEDION AG",
    "000CB9": "LEA",
    "000CBA": "Jamex",
    "000CBB": "ISKRAEMECO",
    "000CBC": "Iscutum",
    "000CBD": "Interface Masters, Inc",
    "000CBE": "PRIVATE",
    "000CBF": "Holy Stone Ent. Co., Ltd.",
    "000CC0": "Genera Oy",
    "000CC1": "Cooper Industries Inc.",
    "000CC2": "ControlNet (India) Private Limited",
    "000CC3": "BeWAN systems",
    "000CC4": "Tiptel AG",
    "000CC5": "Nextlink Co., Ltd.",
    "000CC6": "Ka-Ro electronics GmbH",
    "000CC7": "Intelligent Computer Solutions Inc.",
    "000CC8": "Xytronix Research & Design, Inc.",
    "000CC9": "ILWOO DATA & TECHNOLOGY CO.,LTD",
    "000CCA": "Hitachi Global Storage Technologies",
    "000CCB": "Design Combus Ltd",
    "000CCC": "Aeroscout Ltd.",
    "000CCD": "IEC - TC57",
    "000CCE": "Cisco Systems",
    "000CCF": "Cisco Systems",
    "000CD0": "Symetrix",
    "000CD1": "SFOM Technology Corp.",
    "000CD2": "Schaffner EMV AG",
    "000CD3": "Prettl Elektronik Radeberg GmbH",
    "000CD4": "Positron Public Safety Systems inc.",
    "000CD5": "Passave Inc.",
    "000CD6": "PARTNER TECH",
    "000CD7": "Nallatech Ltd",
    "000CD8": "M. K. Juchheim GmbH & Co",
    "000CD9": "Itcare Co., Ltd",
    "000CDA": "FreeHand Systems, Inc.",
    "000CDB": "Foundry Networks",
    "000CDC": "BECS Technology, Inc",
    "000CDD": "AOS Technologies AG",
    "000CDE": "ABB STOTZ-KONTAKT GmbH",
    "000CDF": "PULNiX America, Inc",
    "000CE0": "Trek Diagnostics Inc.",
    "000CE1": "The Open Group",
    "000CE2": "Rolls-Royce",
    "000CE3": "Option International N.V.",
    "000CE4": "NeuroCom International, Inc.",
    "000CE5": "Motorola BCS",
    "000CE6": "Meru Networks Inc",
    "000CE7": "MediaTek Inc.",
    "000CE8": "GuangZhou AnJuBao Co., Ltd",
    "000CE9": "BLOOMBERG L.P.",
    "000CEA": "aphona Kommunikationssysteme",
    "000CEB": "CNMP Networks, Inc.",
    "000CEC": "Spectracom Corp.",
    "000CED": "Real Digital Media",
    "000CEE": "jp-embedded",
    "000CEF": "Open Networks Engineering Ltd",
    "000CF0": "M & N GmbH",
    "000CF1": "Intel Corporation",
    "000CF2": "GAMESA EÓLICA",
    "000CF3": "CALL IMAGE SA",
    "000CF4": "AKATSUKI ELECTRIC MFG.CO.,LTD.",
    "000CF5": "InfoExpress",
    "000CF6": "Sitecom Europe BV",
    "000CF7": "Nortel Networks",
    "000CF8": "Nortel Networks",
    "000CF9": "ITT Flygt AB",
    "000CFA": "Digital Systems Corp",
    "000CFB": "Korea Network Systems",
    "000CFC": "S2io Technologies Corp",
    "000CFD": "PRIVATE",
    "000CFE": "Grand Electronic Co., Ltd",
    "000CFF": "MRO-TEK LIMITED",
    "000D00": "Seaway Networks Inc.",
    "000D01": "P&E Microcomputer Systems, Inc.",
    "000D02": "NEC AccessTechnica,Ltd",
    "000D03": "Matrics, Inc.",
    "000D04": "Foxboro Eckardt Development GmbH",
    "000D05": "cybernet manufacturing inc.",
    "000D06": "Compulogic Limited",
    "000D07": "Calrec Audio Ltd",
    "000D08": "AboveCable, Inc.",
    "000D09": "Yuehua(Zhuhai) Electronic CO. LTD",
    "000D0A": "Projectiondesign as",
    "000D0B": "Buffalo Inc.",
    "000D0C": "MDI Security Systems",
    "000D0D": "ITSupported, LLC",
    "000D0E": "Inqnet Systems, Inc.",
    "000D0F": "Finlux Ltd",
    "000D10": "Embedtronics Oy",
    "000D11": "DENTSPLY - Gendex",
    "000D12": "AXELL Corporation",
    "000D13": "Wilhelm Rutenbeck GmbH&Co.",
    "000D14": "Vtech Innovation LP dba Advanced American Telephones",
    "000D15": "Voipac s.r.o.",
    "000D16": "UHS Systems Pty Ltd",
    "000D17": "Turbo Networks Co.Ltd",
    "000D18": "Mega-Trend Electronics CO., LTD.",
    "000D19": "ROBE Show lighting",
    "000D1A": "Mustek System Inc.",
    "000D1B": "Kyoto Electronics Manufacturing Co., Ltd.",
    "000D1C": "Amesys Defense",
    "000D1D": "HIGH-TEK HARNESS ENT. CO., LTD.",
    "000D1E": "Control Techniques",
    "000D1F": "AV Digital",
    "000D20": "ASAHIKASEI TECHNOSYSTEM CO.,LTD.",
    "000D21": "WISCORE Inc.",
    "000D22": "Unitronics",
    "000D23": "Smart Solution, Inc",
    "000D24": "SENTEC E&E CO., LTD.",
    "000D25": "SANDEN CORPORATION",
    "000D26": "Primagraphics Limited",
    "000D27": "MICROPLEX Printware AG",
    "000D28": "Cisco",
    "000D29": "Cisco",
    "000D2A": "Scanmatic AS",
    "000D2B": "Racal Instruments",
    "000D2C": "Patapsco Designs Ltd",
    "000D2D": "NCT Deutschland GmbH",
    "000D2E": "Matsushita Avionics Systems Corporation",
    "000D2F": "AIN Comm.Tech.Co., LTD",
    "000D30": "IceFyre Semiconductor",
    "000D31": "Compellent Technologies, Inc.",
    "000D32": "DispenseSource, Inc.",
    "000D33": "Prediwave Corp.",
    "000D34": "Shell International Exploration and Production, Inc.",
    "000D35": "PAC International Ltd",
    "000D36": "Wu Han Routon Electronic Co., Ltd",
    "000D37": "WIPLUG",
    "000D38": "NISSIN INC.",
    "000D39": "Network Electronics",
    "000D3A": "Microsoft Corp.",
    "000D3B": "Microelectronics Technology Inc.",
    "000D3C": "i.Tech Dynamic Ltd",
    "000D3D": "Hammerhead Systems, Inc.",
    "000D3E": "APLUX Communications Ltd.",
    "000D3F": "VXI Technology",
    "000D40": "Verint Loronix Video Solutions",
    "000D41": "Siemens AG ICM MP UC RD IT KLF1",
    "000D42": "Newbest Development Limited",
    "000D43": "DRS Tactical Systems Inc.",
    "000D44": "PRIVATE",
    "000D45": "Tottori SANYO Electric Co., Ltd.",
    "000D46": "Parker SSD Drives",
    "000D47": "Collex",
    "000D48": "AEWIN Technologies Co., Ltd.",
    "000D49": "Triton Systems of Delaware, Inc.",
    "000D4A": "Steag ETA-Optik",
    "000D4B": "Roku, LLC",
    "000D4C": "Outline Electronics Ltd.",
    "000D4D": "Ninelanes",
    "000D4E": "NDR Co.,LTD.",
    "000D4F": "Kenwood Corporation",
    "000D50": "Galazar Networks",
    "000D51": "DIVR Systems, Inc.",
    "000D52": "Comart system",
    "000D53": "Beijing 5w Communication Corp.",
    "000D54": "3Com Ltd",
    "000D55": "SANYCOM Technology Co.,Ltd",
    "000D56": "Dell PCBA Test",
    "000D57": "Fujitsu I-Network Systems Limited.",
    "000D58": "PRIVATE",
    "000D59": "Amity Systems, Inc.",
    "000D5A": "Tiesse SpA",
    "000D5B": "Smart Empire Investments Limited",
    "000D5C": "Robert Bosch GmbH, VT-ATMO",
    "000D5D": "Raritan Computer, Inc",
    "000D5E": "NEC CustomTechnica, Ltd.",
    "000D5F": "Minds Inc",
    "000D60": "IBM Corporation",
    "000D61": "Giga-Byte Technology Co., Ltd.",
    "000D62": "Funkwerk Dabendorf GmbH",
    "000D63": "DENT Instruments, Inc.",
    "000D64": "COMAG Handels AG",
    "000D65": "Cisco Systems",
    "000D66": "Cisco Systems",
    "000D67": "BelAir Networks Inc.",
    "000D68": "Vinci Systems, Inc.",
    "000D69": "TMT&D Corporation",
    "000D6A": "Redwood Technologies LTD",
    "000D6B": "Mita-Teknik A/S",
    "000D6C": "M-Audio",
    "000D6D": "K-Tech Devices Corp.",
    "000D6E": "K-Patents Oy",
    "000D6F": "Ember Corporation",
    "000D70": "Datamax Corporation",
    "000D71": "boca systems",
    "000D72": "2Wire, Inc",
    "000D73": "Technical Support, Inc.",
    "000D74": "Sand Network Systems, Inc.",
    "000D75": "Kobian Pte Ltd - Taiwan Branch",
    "000D76": "Hokuto Denshi Co,. Ltd.",
    "000D77": "FalconStor Software",
    "000D78": "Engineering & Security",
    "000D79": "Dynamic Solutions Co,.Ltd.",
    "000D7A": "DiGATTO Asia Pacific Pte Ltd",
    "000D7B": "Consensys Computers Inc.",
    "000D7C": "Codian Ltd",
    "000D7D": "Afco Systems",
    "000D7E": "Axiowave Networks, Inc.",
    "000D7F": "MIDAS COMMUNICATION TECHNOLOGIES PTE LTD ( Foreign Branch)",
    "000D80": "Online Development Inc",
    "000D81": "Pepperl+Fuchs GmbH",
    "000D82": "PHS srl",
    "000D83": "Sanmina-SCI Hungary Ltd.",
    "000D84": "Makus Inc.",
    "000D85": "Tapwave, Inc.",
    "000D86": "Huber + Suhner AG",
    "000D87": "Elitegroup Computer System Co. (ECS)",
    "000D88": "D-Link Corporation",
    "000D89": "Bils Technology Inc",
    "000D8A": "Winners Electronics Co., Ltd.",
    "000D8B": "T&D Corporation",
    "000D8C": "Shanghai Wedone Digital Ltd. CO.",
    "000D8D": "ProLinx Communication Gateways, Inc.",
    "000D8E": "Koden Electronics Co., Ltd.",
    "000D8F": "King Tsushin Kogyo Co., LTD.",
    "000D90": "Factum Electronics AB",
    "000D91": "Eclipse (HQ Espana) S.L.",
    "000D92": "Arima Communication Corporation",
    "000D93": "Apple Computer",
    "000D94": "AFAR Communications,Inc",
    "000D95": "Opti-cell, Inc.",
    "000D96": "Vtera Technology Inc.",
    "000D97": "Tropos Networks, Inc.",
    "000D98": "S.W.A.C. Schmitt-Walter Automation Consult GmbH",
    "000D99": "Orbital Sciences Corp.; Launch Systems Group",
    "000D9A": "INFOTEC LTD",
    "000D9B": "Heraeus Electro-Nite International N.V.",
    "000D9C": "Elan GmbH & Co KG",
    "000D9D": "Hewlett Packard",
    "000D9E": "TOKUDEN OHIZUMI SEISAKUSYO Co.,Ltd.",
    "000D9F": "RF Micro Devices",
    "000DA0": "NEDAP N.V.",
    "000DA1": "MIRAE ITS Co.,LTD.",
    "000DA2": "Infrant Technologies, Inc.",
    "000DA3": "Emerging Technologies Limited",
    "000DA4": "DOSCH & AMAND SYSTEMS AG",
    "000DA5": "Fabric7 Systems, Inc",
    "000DA6": "Universal Switching Corporation",
    "000DA7": "PRIVATE",
    "000DA8": "Teletronics Technology Corporation",
    "000DA9": "T.E.A.M. S.L.",
    "000DAA": "S.A.Tehnology co.,Ltd.",
    "000DAB": "Parker Hannifin GmbH Electromechanical Division Europe",
    "000DAC": "Japan CBM Corporation",
    "000DAD": "Dataprobe Inc",
    "000DAE": "SAMSUNG HEAVY INDUSTRIES CO., LTD.",
    "000DAF": "Plexus Corp (UK) Ltd",
    "000DB0": "Olym-tech Co.,Ltd.",
    "000DB1": "Japan Network Service Co., Ltd.",
    "000DB2": "Ammasso, Inc.",
    "000DB3": "SDO Communication Corperation",
    "000DB4": "NETASQ",
    "000DB5": "GLOBALSAT TECHNOLOGY CORPORATION",
    "000DB6": "Teknovus, Inc.",
    "000DB7": "SANKO ELECTRIC CO,.LTD",
    "000DB8": "SCHILLER AG",
    "000DB9": "PC Engines GmbH",
    "000DBA": "Océ Document Technologies GmbH",
    "000DBB": "Nippon Dentsu Co.,Ltd.",
    "000DBC": "Cisco Systems",
    "000DBD": "Cisco Systems",
    "000DBE": "Bel Fuse Europe Ltd.,UK",
    "000DBF": "TekTone Sound & Signal Mfg., Inc.",
    "000DC0": "Spagat AS",
    "000DC1": "SafeWeb Inc",
    "000DC2": "PRIVATE",
    "000DC3": "First Communication, Inc.",
    "000DC4": "Emcore Corporation",
    "000DC5": "EchoStar International Corporation",
    "000DC6": "DigiRose Technology Co., Ltd.",
    "000DC7": "COSMIC ENGINEERING INC.",
    "000DC8": "AirMagnet, Inc",
    "000DC9": "THALES Elektronik Systeme GmbH",
    "000DCA": "Tait Electronics",
    "000DCB": "Petcomkorea Co., Ltd.",
    "000DCC": "NEOSMART Corp.",
    "000DCD": "GROUPE TXCOM",
    "000DCE": "Dynavac Technology Pte Ltd",
    "000DCF": "Cidra Corp.",
    "000DD0": "TetraTec Instruments GmbH",
    "000DD1": "Stryker Corporation",
    "000DD2": "Simrad Optronics ASA",
    "000DD3": "SAMWOO Telecommunication Co.,Ltd.",
    "000DD4": "Symantec Corporation",
    "000DD5": "O'RITE TECHNOLOGY CO.,LTD",
    "000DD6": "ITI LTD",
    "000DD7": "Bright",
    "000DD8": "BBN",
    "000DD9": "Anton Paar GmbH",
    "000DDA": "ALLIED TELESIS K.K.",
    "000DDB": "AIRWAVE TECHNOLOGIES INC.",
    "000DDC": "VAC",
    "000DDD": "PROFÝLO TELRA ELEKTRONÝK SANAYÝ VE TÝCARET A.Þ.",
    "000DDE": "Joyteck Co., Ltd.",
    "000DDF": "Japan Image & Network Inc.",
    "000DE0": "ICPDAS Co.,LTD",
    "000DE1": "Control Products, Inc.",
    "000DE2": "CMZ Sistemi Elettronici",
    "000DE3": "AT Sweden AB",
    "000DE4": "DIGINICS, Inc.",
    "000DE5": "Samsung Thales",
    "000DE6": "YOUNGBO ENGINEERING CO.,LTD",
    "000DE7": "Snap-on OEM Group",
    "000DE8": "Nasaco Electronics Pte. Ltd",
    "000DE9": "Napatech Aps",
    "000DEA": "Kingtel Telecommunication Corp.",
    "000DEB": "CompXs Limited",
    "000DEC": "Cisco Systems",
    "000DED": "Cisco Systems",
    "000DEE": "Andrew RF Power Amplifier Group",
    "000DEF": "Soc. Coop. Bilanciai",
    "000DF0": "QCOM TECHNOLOGY INC.",
    "000DF1": "IONIX INC.",
    "000DF2": "PRIVATE",
    "000DF3": "Asmax Solutions",
    "000DF4": "Watertek Co.",
    "000DF5": "Teletronics International Inc.",
    "000DF6": "Technology Thesaurus Corp.",
    "000DF7": "Space Dynamics Lab",
    "000DF8": "ORGA Kartensysteme GmbH",
    "000DF9": "NDS Limited",
    "000DFA": "Micro Control Systems Ltd.",
    "000DFB": "Komax AG",
    "000DFC": "ITFOR Inc.",
    "000DFD": "Huges Hi-Tech Inc.,",
    "000DFE": "Hauppauge Computer Works, Inc.",
    "000DFF": "CHENMING MOLD INDUSTRY CORP.",
    "000E00": "Atrie",
    "000E01": "ASIP Technologies Inc.",
    "000E02": "Advantech AMT Inc.",
    "000E03": "Emulex",
    "000E04": "CMA/Microdialysis AB",
    "000E05": "WIRELESS MATRIX CORP.",
    "000E06": "Team Simoco Ltd",
    "000E07": "Sony Ericsson Mobile Communications AB",
    "000E08": "Cisco Linksys LLC",
    "000E09": "Shenzhen Coship Software Co.,LTD.",
    "000E0A": "SAKUMA DESIGN OFFICE",
    "000E0B": "Netac Technology Co., Ltd.",
    "000E0C": "Intel Corporation",
    "000E0D": "HESCH Schröder GmbH",
    "000E0E": "ESA elettronica S.P.A.",
    "000E0F": "ERMME",
    "000E10": "PRIVATE",
    "000E11": "BDT Büro- und Datentechnik GmbH & Co. KG",
    "000E12": "Adaptive Micro Systems Inc.",
    "000E13": "Accu-Sort Systems inc.",
    "000E14": "Visionary Solutions, Inc.",
    "000E15": "Tadlys LTD",
    "000E16": "SouthWing S.L.",
    "000E17": "PRIVATE",
    "000E18": "MyA Technology",
    "000E19": "LogicaCMG Pty Ltd",
    "000E1A": "JPS Communications",
    "000E1B": "IAV GmbH",
    "000E1C": "Hach Company",
    "000E1D": "ARION Technology Inc.",
    "000E1E": "PRIVATE",
    "000E1F": "TCL Networks Equipment Co., Ltd.",
    "000E20": "ACCESS Systems Americas, Inc.",
    "000E21": "MTU Friedrichshafen GmbH",
    "000E22": "PRIVATE",
    "000E23": "Incipient, Inc.",
    "000E24": "Huwell Technology Inc.",
    "000E25": "Hannae Technology Co., Ltd",
    "000E26": "Gincom Technology Corp.",
    "000E27": "Crere Networks, Inc.",
    "000E28": "Dynamic Ratings P/L",
    "000E29": "Shester Communications Inc",
    "000E2A": "PRIVATE",
    "000E2B": "Safari Technologies",
    "000E2C": "Netcodec co.",
    "000E2D": "Hyundai Digital Technology Co.,Ltd.",
    "000E2E": "Edimax Technology Co., Ltd.",
    "000E2F": "Disetronic Medical Systems AG",
    "000E30": "AERAS Networks, Inc.",
    "000E31": "Olympus Soft Imaging Solutions GmbH",
    "000E32": "Kontron Medical",
    "000E33": "Shuko Electronics Co.,Ltd",
    "000E34": "NexGen City, LP",
    "000E35": "Intel Corp",
    "000E36": "HEINESYS, Inc.",
    "000E37": "Harms & Wende GmbH & Co.KG",
    "000E38": "Cisco Systems",
    "000E39": "Cisco Systems",
    "000E3A": "Cirrus Logic",
    "000E3B": "Hawking Technologies, Inc.",
    "000E3C": "Transact Technologies Inc",
    "000E3D": "Televic N.V.",
    "000E3E": "Sun Optronics Inc",
    "000E3F": "Soronti, Inc.",
    "000E40": "Nortel Networks",
    "000E41": "NIHON MECHATRONICS CO.,LTD.",
    "000E42": "Motic Incoporation Ltd.",
    "000E43": "G-Tek Electronics Sdn. Bhd.",
    "000E44": "Digital 5, Inc.",
    "000E45": "Beijing Newtry Electronic Technology Ltd",
    "000E46": "Niigata Seimitsu Co.,Ltd.",
    "000E47": "NCI System Co.,Ltd.",
    "000E48": "Lipman TransAction Solutions",
    "000E49": "Forsway Scandinavia AB",
    "000E4A": "Changchun Huayu WEBPAD Co.,LTD",
    "000E4B": "atrium c and i",
    "000E4C": "Bermai Inc.",
    "000E4D": "Numesa Inc.",
    "000E4E": "Waveplus Technology Co., Ltd.",
    "000E4F": "Trajet GmbH",
    "000E50": "Thomson Telecom Belgium",
    "000E51": "tecna elettronica srl",
    "000E52": "Optium Corporation",
    "000E53": "AV TECH CORPORATION",
    "000E54": "AlphaCell Wireless Ltd.",
    "000E55": "AUVITRAN",
    "000E56": "4G Systems GmbH",
    "000E57": "Iworld Networking, Inc.",
    "000E58": "Sonos, Inc.",
    "000E59": "SAGEM SA",
    "000E5A": "TELEFIELD inc.",
    "000E5B": "ParkerVision - Direct2Data",
    "000E5C": "Motorola BCS",
    "000E5D": "Triple Play Technologies A/S",
    "000E5E": "Raisecom Technology",
    "000E5F": "activ-net GmbH & Co. KG",
    "000E60": "360SUN Digital Broadband Corporation",
    "000E61": "MICROTROL LIMITED",
    "000E62": "Nortel Networks",
    "000E63": "Lemke Diagnostics GmbH",
    "000E64": "Elphel, Inc",
    "000E65": "TransCore",
    "000E66": "Hitachi Advanced Digital, Inc.",
    "000E67": "Eltis Microelectronics Ltd.",
    "000E68": "E-TOP Network Technology Inc.",
    "000E69": "China Electric Power Research Institute",
    "000E6A": "3Com Ltd",
    "000E6B": "Janitza electronics GmbH",
    "000E6C": "Device Drivers Limited",
    "000E6D": "Murata Manufacturing Co., Ltd.",
    "000E6E": "MICRELEC ELECTRONICS S.A",
    "000E6F": "IRIS Corporation Berhad",
    "000E70": "in2 Networks",
    "000E71": "Gemstar Technology Development Ltd.",
    "000E72": "CTS electronics",
    "000E73": "Tpack A/S",
    "000E74": "Solar Telecom. Tech",
    "000E75": "New York Air Brake Corp.",
    "000E76": "GEMSOC INNOVISION INC.",
    "000E77": "Decru, Inc.",
    "000E78": "Amtelco",
    "000E79": "Ample Communications Inc.",
    "000E7A": "GemWon Communications Co., Ltd.",
    "000E7B": "Toshiba",
    "000E7C": "Televes S.A.",
    "000E7D": "Electronics Line 3000 Ltd.",
    "000E7E": "Comprog Oy",
    "000E7F": "Hewlett Packard",
    "000E80": "Thomson Technology Inc",
    "000E81": "Devicescape Software, Inc.",
    "000E82": "Commtech Wireless",
    "000E83": "Cisco Systems",
    "000E84": "Cisco Systems",
    "000E85": "Catalyst Enterprises, Inc.",
    "000E86": "Alcatel North America",
    "000E87": "adp Gauselmann GmbH",
    "000E88": "VIDEOTRON CORP.",
    "000E89": "CLEMATIC",
    "000E8A": "Avara Technologies Pty. Ltd.",
    "000E8B": "Astarte Technology Co, Ltd.",
    "000E8C": "Siemens AG A&D ET",
    "000E8D": "Systems in Progress Holding GmbH",
    "000E8E": "SparkLAN Communications, Inc.",
    "000E8F": "Sercomm Corp.",
    "000E90": "PONICO CORP.",
    "000E91": "Navico",
    "000E92": "Millinet Co., Ltd.",
    "000E93": "Milénio 3 Sistemas Electrónicos, Lda.",
    "000E94": "Maas International BV",
    "000E95": "Fujiya Denki Seisakusho Co.,Ltd.",
    "000E96": "Cubic Defense Applications, Inc.",
    "000E97": "Ultracker Technology CO., Inc",
    "000E98": "Vitec CC, INC.",
    "000E99": "Spectrum Digital, Inc",
    "000E9A": "BOE TECHNOLOGY GROUP CO.,LTD",
    "000E9B": "Ambit Microsystems Corporation",
    "000E9C": "Pemstar",
    "000E9D": "Tiscali UK Ltd",
    "000E9E": "Topfield Co., Ltd",
    "000E9F": "TEMIC SDS GmbH",
    "000EA0": "NetKlass Technology Inc.",
    "000EA1": "Formosa Teletek Corporation",
    "000EA2": "CyberGuard Corporation",
    "000EA3": "CNCR-IT CO.,LTD,HangZhou P.R.CHINA",
    "000EA4": "Certance Inc.",
    "000EA5": "BLIP Systems",
    "000EA6": "ASUSTEK COMPUTER INC.",
    "000EA7": "Endace Technology",
    "000EA8": "United Technologists Europe Limited",
    "000EA9": "Shanghai Xun Shi Communications Equipment Ltd. Co.",
    "000EAA": "Scalent Systems, Inc.",
    "000EAB": "Cray Inc",
    "000EAC": "MINTRON ENTERPRISE CO., LTD.",
    "000EAD": "Metanoia Technologies, Inc.",
    "000EAE": "GAWELL TECHNOLOGIES CORP.",
    "000EAF": "CASTEL",
    "000EB0": "Solutions Radio BV",
    "000EB1": "Newcotech,Ltd",
    "000EB2": "Micro-Research Finland Oy",
    "000EB3": "LeftHand Networks",
    "000EB4": "GUANGZHOU GAOKE COMMUNICATIONS TECHNOLOGY CO.LTD.",
    "000EB5": "Ecastle Electronics Co., Ltd.",
    "000EB6": "Riverbed Technology, Inc.",
    "000EB7": "Knovative, Inc.",
    "000EB8": "Iiga co.,Ltd",
    "000EB9": "HASHIMOTO Electronics Industry Co.,Ltd.",
    "000EBA": "HANMI SEMICONDUCTOR CO., LTD.",
    "000EBB": "Everbee Networks",
    "000EBC": "Paragon Fidelity GmbH",
    "000EBD": "Burdick, a Quinton Compny",
    "000EBE": "B&B Electronics Manufacturing Co.",
    "000EBF": "Remsdaq Limited",
    "000EC0": "Nortel Networks",
    "000EC1": "MYNAH Technologies",
    "000EC2": "Lowrance Electronics, Inc.",
    "000EC3": "Logic Controls, Inc.",
    "000EC4": "Iskra Transmission d.d.",
    "000EC5": "Digital Multitools Inc",
    "000EC6": "ASIX ELECTRONICS CORP.",
    "000EC7": "Motorola Korea",
    "000EC8": "Zoran Corporation",
    "000EC9": "YOKO Technology Corp.",
    "000ECA": "WTSS Inc",
    "000ECB": "VineSys Technology",
    "000ECC": "Tableau, LLC",
    "000ECD": "SKOV A/S",
    "000ECE": "S.I.T.T.I. S.p.A.",
    "000ECF": "PROFIBUS Nutzerorganisation e.V.",
    "000ED0": "Privaris, Inc.",
    "000ED1": "Osaka Micro Computer.",
    "000ED2": "Filtronic plc",
    "000ED3": "Epicenter, Inc.",
    "000ED4": "CRESITT INDUSTRIE",
    "000ED5": "COPAN Systems Inc.",
    "000ED6": "Cisco Systems",
    "000ED7": "Cisco Systems",
    "000ED8": "Aktino, Inc.",
    "000ED9": "Aksys, Ltd.",
    "000EDA": "C-TECH UNITED CORP.",
    "000EDB": "XiNCOM Corp.",
    "000EDC": "Tellion INC.",
    "000EDD": "SHURE INCORPORATED",
    "000EDE": "REMEC, Inc.",
    "000EDF": "PLX Technology",
    "000EE0": "Mcharge",
    "000EE1": "ExtremeSpeed Inc.",
    "000EE2": "Custom Engineering S.p.A.",
    "000EE3": "Chiyu Technology Co.,Ltd",
    "000EE4": "BOE TECHNOLOGY GROUP CO.,LTD",
    "000EE5": "bitWallet, Inc.",
    "000EE6": "Adimos Systems LTD",
    "000EE7": "AAC ELECTRONICS CORP.",
    "000EE8": "zioncom",
    "000EE9": "WayTech Development, Inc.",
    "000EEA": "Shadong Luneng Jicheng Electronics,Co.,Ltd",
    "000EEB": "Sandmartin(zhong shan)Electronics Co.,Ltd",
    "000EEC": "Orban",
    "000EED": "Nokia Danmark A/S",
    "000EEE": "Muco Industrie BV",
    "000EEF": "PRIVATE",
    "000EF0": "Festo AG & Co. KG",
    "000EF1": "EZQUEST INC.",
    "000EF2": "Infinico Corporation",
    "000EF3": "Smarthome",
    "000EF4": "Shenzhen Kasda Digital Technology Co.,Ltd",
    "000EF5": "iPAC Technology Co., Ltd.",
    "000EF6": "E-TEN Information Systems Co., Ltd.",
    "000EF7": "Vulcan Portals Inc",
    "000EF8": "SBC ASI",
    "000EF9": "REA Elektronik GmbH",
    "000EFA": "Optoway Technology Incorporation",
    "000EFB": "Macey Enterprises",
    "000EFC": "JTAG Technologies B.V.",
    "000EFD": "FUJINON CORPORATION",
    "000EFE": "EndRun Technologies LLC",
    "000EFF": "Megasolution,Inc.",
    "000F00": "Legra Systems, Inc.",
    "000F01": "DIGITALKS INC",
    "000F02": "Digicube Technology Co., Ltd",
    "000F03": "COM&C CO., LTD",
    "000F04": "cim-usa inc",
    "000F05": "3B SYSTEM INC.",
    "000F06": "Nortel Networks",
    "000F07": "Mangrove Systems, Inc.",
    "000F08": "Indagon Oy",
    "000F09": "PRIVATE",
    "000F0A": "Clear Edge Networks",
    "000F0B": "Kentima Technologies AB",
    "000F0C": "SYNCHRONIC ENGINEERING",
    "000F0D": "Hunt Electronic Co., Ltd.",
    "000F0E": "WaveSplitter Technologies, Inc.",
    "000F0F": "Real ID Technology Co., Ltd.",
    "000F10": "RDM Corporation",
    "000F11": "Prodrive B.V.",
    "000F12": "Panasonic Europe Ltd.",
    "000F13": "Nisca corporation",
    "000F14": "Mindray Co., Ltd.",
    "000F15": "Kjaerulff1 A/S",
    "000F16": "JAY HOW TECHNOLOGY CO.,",
    "000F17": "Insta Elektro GmbH",
    "000F18": "Industrial Control Systems",
    "000F19": "Boston Scientific",
    "000F1A": "Gaming Support B.V.",
    "000F1B": "Ego Systems Inc.",
    "000F1C": "DigitAll World Co., Ltd",
    "000F1D": "Cosmo Techs Co., Ltd.",
    "000F1E": "Chengdu KT Electric Co.of High & New Technology",
    "000F1F": "WW PCBA Test",
    "000F20": "Hewlett Packard",
    "000F21": "Scientific Atlanta, Inc",
    "000F22": "Helius, Inc.",
    "000F23": "Cisco Systems",
    "000F24": "Cisco Systems",
    "000F25": "AimValley B.V.",
    "000F26": "WorldAccxx LLC",
    "000F27": "TEAL Electronics, Inc.",
    "000F28": "Itronix Corporation",
    "000F29": "Augmentix Corporation",
    "000F2A": "Cableware Electronics",
    "000F2B": "GREENBELL SYSTEMS",
    "000F2C": "Uplogix, Inc.",
    "000F2D": "CHUNG-HSIN ELECTRIC & MACHINERY MFG.CORP.",
    "000F2E": "Megapower International Corp.",
    "000F2F": "W-LINX TECHNOLOGY CO., LTD.",
    "000F30": "Raza Microelectronics Inc",
    "000F31": "Prosilica",
    "000F32": "LuTong Electronic Technology Co.,Ltd",
    "000F33": "DUALi Inc.",
    "000F34": "Cisco Systems",
    "000F35": "Cisco Systems",
    "000F36": "Accurate Techhnologies, Inc.",
    "000F37": "Xambala Incorporated",
    "000F38": "Netstar",
    "000F39": "IRIS SENSORS",
    "000F3A": "HISHARP",
    "000F3B": "Fuji System Machines Co., Ltd.",
    "000F3C": "Endeleo Limited",
    "000F3D": "D-Link Corporation",
    "000F3E": "CardioNet, Inc",
    "000F3F": "Big Bear Networks",
    "000F40": "Optical Internetworking Forum",
    "000F41": "Zipher Ltd",
    "000F42": "Xalyo Systems",
    "000F43": "Wasabi Systems Inc.",
    "000F44": "Tivella Inc.",
    "000F45": "Stretch, Inc.",
    "000F46": "SINAR AG",
    "000F47": "ROBOX SPA",
    "000F48": "Polypix Inc.",
    "000F49": "Northover Solutions Limited",
    "000F4A": "Kyushu-kyohan co.,ltd",
    "000F4B": "Virtual Iron Software, Inc.",
    "000F4C": "Elextech INC",
    "000F4D": "TalkSwitch",
    "000F4E": "Cellink",
    "000F4F": "Cadmus Technology Ltd",
    "000F50": "StreamScale Limited",
    "000F51": "Azul Systems, Inc.",
    "000F52": "YORK Refrigeration, Marine & Controls",
    "000F53": "Solarflare Communications Inc",
    "000F54": "Entrelogic Corporation",
    "000F55": "Datawire Communication Networks Inc.",
    "000F56": "Continuum Photonics Inc",
    "000F57": "CABLELOGIC Co., Ltd.",
    "000F58": "Adder Technology Limited",
    "000F59": "Phonak Communications AG",
    "000F5A": "Peribit Networks",
    "000F5B": "Delta Information Systems, Inc.",
    "000F5C": "Day One Digital Media Limited",
    "000F5D": "42Networks AB",
    "000F5E": "Veo",
    "000F5F": "Nicety Technologies Inc. (NTS)",
    "000F60": "Lifetron Co.,Ltd",
    "000F61": "Kiwi Networks",
    "000F62": "Alcatel Bell Space N.V.",
    "000F63": "Obzerv Technologies",
    "000F64": "D&R Electronica Weesp BV",
    "000F65": "icube Corp.",
    "000F66": "Cisco-Linksys",
    "000F67": "West Instruments",
    "000F68": "Vavic Network Technology, Inc.",
    "000F69": "SEW Eurodrive GmbH & Co. KG",
    "000F6A": "Nortel Networks",
    "000F6B": "GateWare Communications GmbH",
    "000F6C": "ADDI-DATA GmbH",
    "000F6D": "Midas Engineering",
    "000F6E": "BBox",
    "000F6F": "FTA Communication Technologies",
    "000F70": "Wintec Industries, inc.",
    "000F71": "Sanmei Electronics Co.,Ltd",
    "000F72": "Sandburst",
    "000F73": "Rockwell Automation Korea",
    "000F74": "Qamcom Technology AB",
    "000F75": "First Silicon Solutions",
    "000F76": "Digital Keystone, Inc.",
    "000F77": "DENTUM CO.,LTD",
    "000F78": "Datacap Systems Inc",
    "000F79": "Bluetooth Interest Group Inc.",
    "000F7A": "BeiJing NuQX Technology CO.,LTD",
    "000F7B": "Arce Sistemas, S.A.",
    "000F7C": "ACTi Corporation",
    "000F7D": "Xirrus",
    "000F7E": "Ablerex Electronics Co., LTD",
    "000F7F": "UBSTORAGE Co.,Ltd.",
    "000F80": "Trinity Security Systems,Inc.",
    "000F81": "Secure Info Imaging",
    "000F82": "Mortara Instrument, Inc.",
    "000F83": "Brainium Technologies Inc.",
    "000F84": "Astute Networks, Inc.",
    "000F85": "ADDO-Japan Corporation",
    "000F86": "Research In Motion Limited",
    "000F87": "Maxcess International",
    "000F88": "AMETEK, Inc.",
    "000F89": "Winnertec System Co., Ltd.",
    "000F8A": "WideView",
    "000F8B": "Orion MultiSystems Inc",
    "000F8C": "Gigawavetech Pte Ltd",
    "000F8D": "FAST TV-Server AG",
    "000F8E": "DONGYANG TELECOM CO.,LTD.",
    "000F8F": "Cisco Systems",
    "000F90": "Cisco Systems",
    "000F91": "Aerotelecom Co.,Ltd.",
    "000F92": "Microhard Systems Inc.",
    "000F93": "Landis+Gyr Ltd.",
    "000F94": "Genexis",
    "000F95": "ELECOM Co.,LTD Laneed Division",
    "000F96": "Critical Telecom Corp.",
    "000F97": "Avanex Corporation",
    "000F98": "Avamax Co. Ltd.",
    "000F99": "APAC opto Electronics Inc.",
    "000F9A": "Synchrony, Inc.",
    "000F9B": "Ross Video Limited",
    "000F9C": "Panduit Corp",
    "000F9D": "DisplayLink (UK) Ltd",
    "000F9E": "Murrelektronik GmbH",
    "000F9F": "Motorola BCS",
    "000FA0": "CANON KOREA BUSINESS SOLUTIONS INC.",
    "000FA1": "Gigabit Systems Inc.",
    "000FA2": "Digital Path Networks",
    "000FA3": "Alpha Networks Inc.",
    "000FA4": "Sprecher Automation GmbH",
    "000FA5": "SMP / BWA Technology GmbH",
    "000FA6": "S2 Security Corporation",
    "000FA7": "Raptor Networks Technology",
    "000FA8": "Photometrics, Inc.",
    "000FA9": "PC Fabrik",
    "000FAA": "Nexus Technologies",
    "000FAB": "Kyushu Electronics Systems Inc.",
    "000FAC": "IEEE 802.11",
    "000FAD": "FMN communications GmbH",
    "000FAE": "E2O Communications",
    "000FAF": "Dialog Inc.",
    "000FB0": "Compal Electronics,INC.",
    "000FB1": "Cognio Inc.",
    "000FB2": "Broadband Pacenet (India) Pvt. Ltd.",
    "000FB3": "Actiontec Electronics, Inc",
    "000FB4": "Timespace Technology",
    "000FB5": "NETGEAR Inc",
    "000FB6": "Europlex Technologies",
    "000FB7": "Cavium Networks",
    "000FB8": "CallURL Inc.",
    "000FB9": "Adaptive Instruments",
    "000FBA": "Tevebox AB",
    "000FBB": "Nokia Siemens Networks GmbH & Co. KG",
    "000FBC": "Onkey Technologies, Inc.",
    "000FBD": "MRV Communications (Networks) LTD",
    "000FBE": "e-w/you Inc.",
    "000FBF": "DGT Sp. z o.o.",
    "000FC0": "DELCOMp",
    "000FC1": "WAVE Corporation",
    "000FC2": "Uniwell Corporation",
    "000FC3": "PalmPalm Technology, Inc.",
    "000FC4": "NST co.,LTD.",
    "000FC5": "KeyMed Ltd",
    "000FC6": "Eurocom Industries A/S",
    "000FC7": "Dionica R&D Ltd.",
    "000FC8": "Chantry Networks",
    "000FC9": "Allnet GmbH",
    "000FCA": "A-JIN TECHLINE CO, LTD",
    "000FCB": "3Com Ltd",
    "000FCC": "Netopia, Inc.",
    "000FCD": "Nortel Networks",
    "000FCE": "Kikusui Electronics Corp.",
    "000FCF": "Datawind Research",
    "000FD0": "ASTRI",
    "000FD1": "Applied Wireless Identifications Group, Inc.",
    "000FD2": "EWA Technologies, Inc.",
    "000FD3": "Digium",
    "000FD4": "Soundcraft",
    "000FD5": "Schwechat - RISE",
    "000FD6": "Sarotech Co., Ltd",
    "000FD7": "Harman Music Group",
    "000FD8": "Force, Inc.",
    "000FD9": "FlexDSL Telecommunications AG",
    "000FDA": "YAZAKI CORPORATION",
    "000FDB": "Westell Technologies",
    "000FDC": "Ueda Japan Radio Co., Ltd.",
    "000FDD": "SORDIN AB",
    "000FDE": "Sony Ericsson Mobile Communications AB",
    "000FDF": "SOLOMON Technology Corp.",
    "000FE0": "NComputing Co.,Ltd.",
    "000FE1": "ID DIGITAL CORPORATION",
    "000FE2": "Hangzhou H3C Technologies Co., Ltd.",
    "000FE3": "Damm Cellular Systems A/S",
    "000FE4": "Pantech Co.,Ltd",
    "000FE5": "MERCURY SECURITY CORPORATION",
    "000FE6": "MBTech Systems, Inc.",
    "000FE7": "Lutron Electronics Co., Inc.",
    "000FE8": "Lobos, Inc.",
    "000FE9": "GW TECHNOLOGIES CO.,LTD.",
    "000FEA": "Giga-Byte Technology Co.,LTD.",
    "000FEB": "Cylon Controls",
    "000FEC": "Arkus Inc.",
    "000FED": "Anam Electronics Co., Ltd",
    "000FEE": "XTec, Incorporated",
    "000FEF": "Thales e-Transactions GmbH",
    "000FF0": "Sunray Co. Ltd.",
    "000FF1": "nex-G Systems Pte.Ltd",
    "000FF2": "Loud Technologies Inc.",
    "000FF3": "Jung Myoung Communications&Technology",
    "000FF4": "Guntermann & Drunck GmbH",
    "000FF5": "GN&S company",
    "000FF6": "Darfon Electronics Corp.",
    "000FF7": "Cisco Systems",
    "000FF8": "Cisco Systems",
    "000FF9": "Valcretec, Inc.",
    "000FFA": "Optinel Systems, Inc.",
    "000FFB": "Nippon Denso Industry Co., Ltd.",
    "000FFC": "Merit Li-Lin Ent.",
    "000FFD": "Glorytek Network Inc.",
    "000FFE": "G-PRO COMPUTER",
    "000FFF": "Control4",
    "001000": "CABLE TELEVISION LABORATORIES, INC.",
    "001001": "Citel plc",
    "001002": "ACTIA",
    "001003": "IMATRON, INC.",
    "001004": "THE BRANTLEY COILE COMPANY,INC",
    "001005": "UEC COMMERCIAL",
    "001006": "Thales Contact Solutions Ltd.",
    "001007": "CISCO SYSTEMS, INC.",
    "001008": "VIENNA SYSTEMS CORPORATION",
    "001009": "HORO QUARTZ",
    "00100A": "WILLIAMS COMMUNICATIONS GROUP",
    "00100B": "CISCO SYSTEMS, INC.",
    "00100C": "ITO CO., LTD.",
    "00100D": "CISCO SYSTEMS, INC.",
    "00100E": "MICRO LINEAR COPORATION",
    "00100F": "INDUSTRIAL CPU SYSTEMS",
    "001010": "INITIO CORPORATION",
    "001011": "CISCO SYSTEMS, INC.",
    "001012": "PROCESSOR SYSTEMS (I) PVT LTD",
    "001013": "Kontron",
    "001014": "CISCO SYSTEMS, INC.",
    "001015": "OOmon Inc.",
    "001016": "T.SQWARE",
    "001017": "MICOS GmbH",
    "001018": "BROADCOM CORPORATION",
    "001019": "SIRONA DENTAL SYSTEMS GmbH & Co. KG",
    "00101A": "PictureTel Corp.",
    "00101B": "CORNET TECHNOLOGY, INC.",
    "00101C": "OHM TECHNOLOGIES INTL, LLC",
    "00101D": "WINBOND ELECTRONICS CORP.",
    "00101E": "MATSUSHITA ELECTRONIC INSTRUMENTS CORP.",
    "00101F": "CISCO SYSTEMS, INC.",
    "001020": "Hand Held Products Inc",
    "001021": "ENCANTO NETWORKS, INC.",
    "001022": "SatCom Media Corporation",
    "001023": "Network Equipment Technologies",
    "001024": "NAGOYA ELECTRIC WORKS CO., LTD",
    "001025": "Grayhill, Inc",
    "001026": "ACCELERATED NETWORKS, INC.",
    "001027": "L-3 COMMUNICATIONS EAST",
    "001028": "COMPUTER TECHNICA, INC.",
    "001029": "CISCO SYSTEMS, INC.",
    "00102A": "ZF MICROSYSTEMS, INC.",
    "00102B": "UMAX DATA SYSTEMS, INC.",
    "00102C": "Lasat Networks A/S",
    "00102D": "HITACHI SOFTWARE ENGINEERING",
    "00102E": "NETWORK SYSTEMS & TECHNOLOGIES PVT. LTD.",
    "00102F": "CISCO SYSTEMS, INC.",
    "001030": "EION Inc.",
    "001031": "OBJECTIVE COMMUNICATIONS, INC.",
    "001032": "ALTA TECHNOLOGY",
    "001033": "ACCESSLAN COMMUNICATIONS, INC.",
    "001034": "GNP Computers",
    "001035": "ELITEGROUP COMPUTER SYSTEMS CO., LTD",
    "001036": "INTER-TEL INTEGRATED SYSTEMS",
    "001037": "CYQ've Technology Co., Ltd.",
    "001038": "MICRO RESEARCH INSTITUTE, INC.",
    "001039": "Vectron Systems AG",
    "00103A": "DIAMOND NETWORK TECH",
    "00103B": "HIPPI NETWORKING FORUM",
    "00103C": "IC ENSEMBLE, INC.",
    "00103D": "PHASECOM, LTD.",
    "00103E": "NETSCHOOLS CORPORATION",
    "00103F": "TOLLGRADE COMMUNICATIONS, INC.",
    "001040": "INTERMEC CORPORATION",
    "001041": "BRISTOL BABCOCK, INC.",
    "001042": "Alacritech, Inc.",
    "001043": "A2 CORPORATION",
    "001044": "InnoLabs Corporation",
    "001045": "Nortel Networks",
    "001046": "ALCORN MCBRIDE INC.",
    "001047": "ECHO ELETRIC CO. LTD.",
    "001048": "HTRC AUTOMATION, INC.",
    "001049": "ShoreTel, Inc",
    "00104A": "THE PARVUC CORPORATION",
    "00104B": "3COM CORPORATION",
    "00104C": "LeCroy Corporation",
    "00104D": "SURTEC INDUSTRIES, INC.",
    "00104E": "CEOLOGIC",
    "00104F": "Sun Microsystems, Inc.",
    "001050": "RION CO., LTD.",
    "001051": "CMICRO CORPORATION",
    "001052": "METTLER-TOLEDO (ALBSTADT) GMBH",
    "001053": "COMPUTER TECHNOLOGY CORP.",
    "001054": "CISCO SYSTEMS, INC.",
    "001055": "FUJITSU MICROELECTRONICS, INC.",
    "001056": "SODICK CO., LTD.",
    "001057": "Rebel.com, Inc.",
    "001058": "ArrowPoint Communications",
    "001059": "DIABLO RESEARCH CO. LLC",
    "00105A": "3COM CORPORATION",
    "00105B": "NET INSIGHT AB",
    "00105C": "QUANTUM DESIGNS (H.K.) LTD.",
    "00105D": "Draeger Medical",
    "00105E": "HEKIMIAN LABORATORIES, INC.",
    "00105F": "IN-SNEC",
    "001060": "BILLIONTON SYSTEMS, INC.",
    "001061": "HOSTLINK CORP.",
    "001062": "NX SERVER, ILNC.",
    "001063": "STARGUIDE DIGITAL NETWORKS",
    "001064": "DNPG, LLC",
    "001065": "RADYNE CORPORATION",
    "001066": "ADVANCED CONTROL SYSTEMS, INC.",
    "001067": "REDBACK NETWORKS, INC.",
    "001068": "COMOS TELECOM",
    "001069": "HELIOSS COMMUNICATIONS, INC.",
    "00106A": "DIGITAL MICROWAVE CORPORATION",
    "00106B": "SONUS NETWORKS, INC.",
    "00106C": "Infratec AG",
    "00106D": "Axxcelera Broadband Wireless",
    "00106E": "TADIRAN COM. LTD.",
    "00106F": "TRENTON TECHNOLOGY INC.",
    "001070": "CARADON TREND LTD.",
    "001071": "ADVANET INC.",
    "001072": "GVN TECHNOLOGIES, INC.",
    "001073": "Technobox, Inc.",
    "001074": "ATEN INTERNATIONAL CO., LTD.",
    "001075": "Maxtor Corporation",
    "001076": "EUREM GmbH",
    "001077": "SAF DRIVE SYSTEMS, LTD.",
    "001078": "NUERA COMMUNICATIONS, INC.",
    "001079": "CISCO SYSTEMS, INC.",
    "00107A": "AmbiCom, Inc.",
    "00107B": "CISCO SYSTEMS, INC.",
    "00107C": "P-COM, INC.",
    "00107D": "AURORA COMMUNICATIONS, LTD.",
    "00107E": "BACHMANN ELECTRONIC GmbH",
    "00107F": "CRESTRON ELECTRONICS, INC.",
    "001080": "METAWAVE COMMUNICATIONS",
    "001081": "DPS, INC.",
    "001082": "JNA TELECOMMUNICATIONS LIMITED",
    "001083": "HEWLETT-PACKARD COMPANY",
    "001084": "K-BOT COMMUNICATIONS",
    "001085": "POLARIS COMMUNICATIONS, INC.",
    "001086": "ATTO Technology, Inc.",
    "001087": "Xstreamis PLC",
    "001088": "AMERICAN NETWORKS INC.",
    "001089": "WebSonic",
    "00108A": "TeraLogic, Inc.",
    "00108B": "LASERANIMATION SOLLINGER GmbH",
    "00108C": "FUJITSU TELECOMMUNICATIONS EUROPE, LTD.",
    "00108D": "Johnson Controls, Inc.",
    "00108E": "HUGH SYMONS CONCEPT Technologies Ltd.",
    "00108F": "RAPTOR SYSTEMS",
    "001090": "CIMETRICS, INC.",
    "001091": "NO WIRES NEEDED BV",
    "001092": "NETCORE INC.",
    "001093": "CMS COMPUTERS, LTD.",
    "001094": "Performance Analysis Broadband, Spirent plc",
    "001095": "Thomson Inc.",
    "001096": "TRACEWELL SYSTEMS, INC.",
    "001097": "WinNet Metropolitan Communications Systems, Inc.",
    "001098": "STARNET TECHNOLOGIES, INC.",
    "001099": "InnoMedia, Inc.",
    "00109A": "NETLINE",
    "00109B": "Emulex Corporation",
    "00109C": "M-SYSTEM CO., LTD.",
    "00109D": "CLARINET SYSTEMS, INC.",
    "00109E": "AWARE, INC.",
    "00109F": "PAVO, INC.",
    "0010A0": "INNOVEX TECHNOLOGIES, INC.",
    "0010A1": "KENDIN SEMICONDUCTOR, INC.",
    "0010A2": "TNS",
    "0010A3": "OMNITRONIX, INC.",
    "0010A4": "XIRCOM",
    "0010A5": "OXFORD INSTRUMENTS",
    "0010A6": "CISCO SYSTEMS, INC.",
    "0010A7": "UNEX TECHNOLOGY CORPORATION",
    "0010A8": "RELIANCE COMPUTER CORP.",
    "0010A9": "ADHOC TECHNOLOGIES",
    "0010AA": "MEDIA4, INC.",
    "0010AB": "KOITO INDUSTRIES, LTD.",
    "0010AC": "IMCI TECHNOLOGIES",
    "0010AD": "SOFTRONICS USB, INC.",
    "0010AE": "SHINKO ELECTRIC INDUSTRIES CO.",
    "0010AF": "TAC SYSTEMS, INC.",
    "0010B0": "MERIDIAN TECHNOLOGY CORP.",
    "0010B1": "FOR-A CO., LTD.",
    "0010B2": "COACTIVE AESTHETICS",
    "0010B3": "NOKIA MULTIMEDIA TERMINALS",
    "0010B4": "ATMOSPHERE NETWORKS",
    "0010B5": "ACCTON TECHNOLOGY CORPORATION",
    "0010B6": "ENTRATA COMMUNICATIONS CORP.",
    "0010B7": "COYOTE TECHNOLOGIES, LLC",
    "0010B8": "ISHIGAKI COMPUTER SYSTEM CO.",
    "0010B9": "MAXTOR CORP.",
    "0010BA": "MARTINHO-DAVIS SYSTEMS, INC.",
    "0010BB": "DATA & INFORMATION TECHNOLOGY",
    "0010BC": "Aastra Telecom",
    "0010BD": "THE TELECOMMUNICATION TECHNOLOGY COMMITTEE",
    "0010BE": "TELEXIS CORP.",
    "0010BF": "InterAir Wireless",
    "0010C0": "ARMA, Inc.",
    "0010C1": "OI ELECTRIC CO., LTD.",
    "0010C2": "WILLNET, INC.",
    "0010C3": "CSI-CONTROL SYSTEMS",
    "0010C4": "MEDIA LINKS CO., LTD.",
    "0010C5": "PROTOCOL TECHNOLOGIES, INC.",
    "0010C6": "USI",
    "0010C7": "DATA TRANSMISSION NETWORK",
    "0010C8": "COMMUNICATIONS ELECTRONICS SECURITY GROUP",
    "0010C9": "MITSUBISHI ELECTRONICS LOGISTIC SUPPORT CO.",
    "0010CA": "INTEGRAL ACCESS",
    "0010CB": "FACIT K.K.",
    "0010CC": "CLP COMPUTER LOGISTIK PLANUNG GmbH",
    "0010CD": "INTERFACE CONCEPT",
    "0010CE": "VOLAMP, LTD.",
    "0010CF": "FIBERLANE COMMUNICATIONS",
    "0010D0": "WITCOM, LTD.",
    "0010D1": "Top Layer Networks, Inc.",
    "0010D2": "NITTO TSUSHINKI CO., LTD",
    "0010D3": "GRIPS ELECTRONIC GMBH",
    "0010D4": "STORAGE COMPUTER CORPORATION",
    "0010D5": "IMASDE CANARIAS, S.A.",
    "0010D6": "ITT - A/CD",
    "0010D7": "ARGOSY RESEARCH INC.",
    "0010D8": "CALISTA",
    "0010D9": "IBM JAPAN, FUJISAWA MT+D",
    "0010DA": "MOTION ENGINEERING, INC.",
    "0010DB": "Juniper Networks, Inc.",
    "0010DC": "MICRO-STAR INTERNATIONAL CO., LTD.",
    "0010DD": "ENABLE SEMICONDUCTOR, INC.",
    "0010DE": "INTERNATIONAL DATACASTING CORPORATION",
    "0010DF": "RISE COMPUTER INC.",
    "0010E0": "Sun Microsystems, Inc.",
    "0010E1": "S.I. TECH, INC.",
    "0010E2": "ArrayComm, Inc.",
    "0010E3": "Hewlett Packard",
    "0010E4": "NSI CORPORATION",
    "0010E5": "SOLECTRON TEXAS",
    "0010E6": "APPLIED INTELLIGENT SYSTEMS, INC.",
    "0010E7": "BreezeCom",
    "0010E8": "TELOCITY, INCORPORATED",
    "0010E9": "RAIDTEC LTD.",
    "0010EA": "ADEPT TECHNOLOGY",
    "0010EB": "SELSIUS SYSTEMS, INC.",
    "0010EC": "RPCG, LLC",
    "0010ED": "SUNDANCE TECHNOLOGY, INC.",
    "0010EE": "CTI PRODUCTS, INC.",
    "0010EF": "DBTEL INCORPORATED",
    "0010F1": "I-O CORPORATION",
    "0010F2": "ANTEC",
    "0010F3": "Nexcom International Co., Ltd.",
    "0010F4": "Vertical Communications",
    "0010F5": "AMHERST SYSTEMS, INC.",
    "0010F6": "CISCO SYSTEMS, INC.",
    "0010F7": "IRIICHI TECHNOLOGIES Inc.",
    "0010F8": "TEXIO CORPORATION",
    "0010F9": "UNIQUE SYSTEMS, INC.",
    "0010FA": "Apple Inc",
    "0010FB": "ZIDA TECHNOLOGIES LIMITED",
    "0010FC": "BROADBAND NETWORKS, INC.",
    "0010FD": "COCOM A/S",
    "0010FE": "DIGITAL EQUIPMENT CORPORATION",
    "0010FF": "CISCO SYSTEMS, INC.",
    "001100": "RAM Industries, LLC",
    "001101": "CET Technologies Pte Ltd",
    "001102": "Aurora Multimedia Corp.",
    "001103": "kawamura electric inc.",
    "001104": "TELEXY",
    "001105": "Sunplus Technology Co., Ltd.",
    "001106": "Siemens NV (Belgium)",
    "001107": "RGB Networks Inc.",
    "001108": "Orbital Data Corporation",
    "001109": "Micro-Star International",
    "00110A": "Hewlett Packard",
    "00110B": "Franklin Technology Systems",
    "00110C": "Atmark Techno, Inc.",
    "00110D": "SANBlaze Technology, Inc.",
    "00110E": "Tsurusaki Sealand Transportation Co. Ltd.",
    "00110F": "netplat,Inc.",
    "001110": "Maxanna Technology Co., Ltd.",
    "001111": "Intel Corporation",
    "001112": "Honeywell CMSS",
    "001113": "Fraunhofer FOKUS",
    "001114": "EverFocus Electronics Corp.",
    "001115": "EPIN Technologies, Inc.",
    "001116": "COTEAU VERT CO., LTD.",
    "001117": "CESNET",
    "001118": "BLX IC Design Corp., Ltd.",
    "001119": "Solteras, Inc.",
    "00111A": "Motorola BCS",
    "00111B": "Targa Systems Div L-3 Communications Canada",
    "00111C": "Pleora Technologies Inc.",
    "00111D": "Hectrix Limited",
    "00111E": "EPSG (Ethernet Powerlink Standardization Group)",
    "00111F": "Doremi Labs, Inc.",
    "001120": "Cisco Systems",
    "001121": "Cisco Systems",
    "001122": "CIMSYS Inc",
    "001123": "Appointech, Inc.",
    "001124": "Apple Computer",
    "001125": "IBM Corporation",
    "001126": "Venstar Inc.",
    "001127": "TASI, Inc",
    "001128": "Streamit",
    "001129": "Paradise Datacom Ltd.",
    "00112A": "Niko NV",
    "00112B": "NetModule",
    "00112C": "IZT GmbH",
    "00112D": "Guys Without Ties",
    "00112E": "CEICOM",
    "00112F": "ASUSTek Computer Inc.",
    "001130": "Allied Telesis (Hong Kong) Ltd.",
    "001131": "UNATECH. CO.,LTD",
    "001132": "Synology Incorporated",
    "001133": "Siemens Austria SIMEA",
    "001134": "MediaCell, Inc.",
    "001135": "Grandeye Ltd",
    "001136": "Goodrich Sensor Systems",
    "001137": "AICHI ELECTRIC CO., LTD.",
    "001138": "TAISHIN CO., LTD.",
    "001139": "STOEBER ANTRIEBSTECHNIK GmbH + Co. KG.",
    "00113A": "SHINBORAM",
    "00113B": "Micronet Communications Inc.",
    "00113C": "Micronas GmbH",
    "00113D": "KN SOLTEC CO.,LTD.",
    "00113E": "JL Corporation",
    "00113F": "Alcatel DI",
    "001140": "Nanometrics Inc.",
    "001141": "GoodMan Corporation",
    "001142": "e-SMARTCOM INC.",
    "001143": "DELL INC.",
    "001144": "Assurance Technology Corp",
    "001145": "ValuePoint Networks",
    "001146": "Telecard-Pribor Ltd",
    "001147": "Secom-Industry co.LTD.",
    "001148": "Prolon Control Systems",
    "001149": "Proliphix Inc.",
    "00114A": "KAYABA INDUSTRY Co,.Ltd.",
    "00114B": "Francotyp-Postalia AG & Co. KG",
    "00114C": "caffeina applied research ltd.",
    "00114D": "Atsumi Electric Co.,LTD.",
    "00114E": "690885 Ontario Inc.",
    "00114F": "US Digital Television, Inc",
    "001150": "Belkin Corporation",
    "001151": "Mykotronx",
    "001152": "Eidsvoll Electronics AS",
    "001153": "Trident Tek, Inc.",
    "001154": "Webpro Technologies Inc.",
    "001155": "Sevis Systems",
    "001156": "Pharos Systems NZ",
    "001157": "OF Networks Co., Ltd.",
    "001158": "Nortel Networks",
    "001159": "MATISSE NETWORKS INC",
    "00115A": "Ivoclar Vivadent AG",
    "00115B": "Elitegroup Computer System Co. (ECS)",
    "00115C": "Cisco",
    "00115D": "Cisco",
    "00115E": "ProMinent Dosiertechnik GmbH",
    "00115F": "Intellix Co., Ltd.",
    "001160": "ARTDIO Company Co., LTD",
    "001161": "NetStreams, LLC",
    "001162": "STAR MICRONICS CO.,LTD.",
    "001163": "SYSTEM SPA DEPT. ELECTRONICS",
    "001164": "ACARD Technology Corp.",
    "001165": "Znyx Networks",
    "001166": "Taelim Electronics Co., Ltd.",
    "001167": "Integrated System Solution Corp.",
    "001168": "HomeLogic LLC",
    "001169": "EMS Satcom",
    "00116A": "Domo Ltd",
    "00116B": "Digital Data Communications Asia Co.,Ltd",
    "00116C": "Nanwang Multimedia Inc.,Ltd",
    "00116D": "American Time and Signal",
    "00116E": "PePLink Ltd.",
    "00116F": "Netforyou Co., LTD.",
    "001170": "GSC SRL",
    "001171": "DEXTER Communications, Inc.",
    "001172": "COTRON CORPORATION",
    "001173": "Adtron Corporation",
    "001174": "Wibhu Technologies, Inc.",
    "001175": "PathScale, Inc.",
    "001176": "Intellambda Systems, Inc.",
    "001177": "COAXIAL NETWORKS, INC.",
    "001178": "Chiron Technology Ltd",
    "001179": "Singular Technology Co. Ltd.",
    "00117A": "Singim International Corp.",
    "00117B": "Büchi Labortechnik AG",
    "00117C": "e-zy.net",
    "00117D": "ZMD America, Inc.",
    "00117E": "Progeny Inc.",
    "00117F": "Neotune Information Technology Corporation,.LTD",
    "001180": "Motorola BCS",
    "001181": "InterEnergy Co.Ltd,",
    "001182": "IMI Norgren Ltd",
    "001183": "PSC Scanning, Inc",
    "001184": "Humo Laboratory,Ltd.",
    "001185": "Hewlett Packard",
    "001186": "Prime Systems, Inc.",
    "001187": "Category Solutions, Inc",
    "001188": "Enterasys",
    "001189": "Aerotech Inc",
    "00118A": "Viewtran Technology Limited",
    "00118B": "Alcatel-Lucent, Enterprise Business Group",
    "00118C": "Missouri Department of Transportation",
    "00118D": "Hanchang System Corp.",
    "00118E": "Halytech Mace",
    "00118F": "EUTECH INSTRUMENTS PTE. LTD.",
    "001190": "Digital Design Corporation",
    "001191": "CTS-Clima Temperatur Systeme GmbH",
    "001192": "Cisco Systems",
    "001193": "Cisco Systems",
    "001194": "Chi Mei Communication Systems, Inc.",
    "001195": "D-Link Corporation",
    "001196": "Actuality Systems, Inc.",
    "001197": "Monitoring Technologies Limited",
    "001198": "Prism Media Products Limited",
    "001199": "2wcom GmbH",
    "00119A": "Alkeria srl",
    "00119B": "Telesynergy Research Inc.",
    "00119C": "EP&T Energy",
    "00119D": "Diginfo Technology Corporation",
    "00119E": "Solectron Brazil",
    "00119F": "Nokia Danmark A/S",
    "0011A0": "Vtech Engineering Canada Ltd",
    "0011A1": "VISION NETWARE CO.,LTD",
    "0011A2": "Manufacturing Technology Inc",
    "0011A3": "LanReady Technologies Inc.",
    "0011A4": "JStream Technologies Inc.",
    "0011A5": "Fortuna Electronic Corp.",
    "0011A6": "Sypixx Networks",
    "0011A7": "Infilco Degremont Inc.",
    "0011A8": "Quest Technologies",
    "0011A9": "MOIMSTONE Co., LTD",
    "0011AA": "Uniclass Technology, Co., LTD",
    "0011AB": "TRUSTABLE TECHNOLOGY CO.,LTD.",
    "0011AC": "Simtec Electronics",
    "0011AD": "Shanghai Ruijie Technology",
    "0011AE": "Motorola BCS",
    "0011AF": "Medialink-i,Inc",
    "0011B0": "Fortelink Inc.",
    "0011B1": "BlueExpert Technology Corp.",
    "0011B2": "2001 Technology Inc.",
    "0011B3": "YOSHIMIYA CO.,LTD.",
    "0011B4": "Westermo Teleindustri AB",
    "0011B5": "Shenzhen Powercom Co.,Ltd",
    "0011B6": "Open Systems International",
    "0011B7": "Onroerend Groen B.V.",
    "0011B8": "Liebherr - Elektronik GmbH",
    "0011B9": "Inner Range Pty. Ltd.",
    "0011BA": "Elexol Pty Ltd",
    "0011BB": "Cisco Systems",
    "0011BC": "Cisco Systems",
    "0011BD": "Bombardier Transportation",
    "0011BE": "AGP Telecom Co. Ltd",
    "0011BF": "AESYS S.p.A.",
    "0011C0": "Aday Technology Inc",
    "0011C1": "4P MOBILE DATA PROCESSING",
    "0011C2": "United Fiber Optic Communication",
    "0011C3": "Transceiving System Technology Corporation",
    "0011C4": "Terminales de Telecomunicacion Terrestre, S.L.",
    "0011C5": "TEN Technology",
    "0011C6": "Seagate Technology LLC",
    "0011C7": "RAYMARINE Group Ltd.",
    "0011C8": "Powercom Co., Ltd.",
    "0011C9": "MTT Corporation",
    "0011CA": "Long Range Systems, Inc.",
    "0011CB": "Jacobsons AB",
    "0011CC": "Guangzhou Jinpeng Group Co.,Ltd.",
    "0011CD": "Axsun Technologies",
    "0011CE": "Ubisense Limited",
    "0011CF": "Thrane & Thrane A/S",
    "0011D0": "Tandberg Data ASA",
    "0011D1": "Soft Imaging System GmbH",
    "0011D2": "Perception Digital Ltd",
    "0011D3": "NextGenTel Holding ASA",
    "0011D4": "NetEnrich, Inc",
    "0011D5": "Hangzhou Sunyard System Engineering Co.,Ltd.",
    "0011D6": "HandEra, Inc.",
    "0011D7": "eWerks Inc",
    "0011D8": "ASUSTek Computer Inc.",
    "0011D9": "TiVo",
    "0011DA": "Vivaas Technology Inc.",
    "0011DB": "Land-Cellular Corporation",
    "0011DC": "Glunz & Jensen",
    "0011DD": "FROMUS TEC. Co., Ltd.",
    "0011DE": "EURILOGIC",
    "0011DF": "Arecont Systems",
    "0011E0": "U-MEDIA Communications, Inc.",
    "0011E1": "BEKO Electronics Co.",
    "0011E2": "Hua Jung Components Co., Ltd.",
    "0011E3": "Thomson, Inc.",
    "0011E4": "Danelec Electronics A/S",
    "0011E5": "KCodes Corporation",
    "0011E6": "Scientific Atlanta",
    "0011E7": "WORLDSAT - Texas de France",
    "0011E8": "Tixi.Com",
    "0011E9": "STARNEX CO., LTD.",
    "0011EA": "IWICS Inc.",
    "0011EB": "Innovative Integration",
    "0011EC": "AVIX INC.",
    "0011ED": "802 Global",
    "0011EE": "Estari, Inc.",
    "0011EF": "Conitec Datensysteme GmbH",
    "0011F0": "Wideful Limited",
    "0011F1": "QinetiQ Ltd",
    "0011F2": "Institute of Network Technologies",
    "0011F3": "Gavitec AG- mobile digit",
    "0011F4": "woori-net",
    "0011F5": "ASKEY COMPUTER CORP.",
    "0011F6": "Asia Pacific Microsystems , Inc.",
    "0011F7": "Shenzhen Forward Industry Co., Ltd",
    "0011F8": "AIRAYA Corp",
    "0011F9": "Nortel Networks",
    "0011FA": "Rane Corporation",
    "0011FB": "Heidelberg Engineering GmbH",
    "0011FC": "HARTING Electric Gmbh & Co.KG",
    "0011FD": "KORG INC.",
    "0011FE": "Keiyo System Research, Inc.",
    "0011FF": "Digitro Tecnologia Ltda",
    "001200": "Cisco",
    "001201": "Cisco",
    "001202": "Decrane Aerospace - Audio International Inc.",
    "001203": "Activ Networks",
    "001204": "u10 Networks, Inc.",
    "001205": "Terrasat Communications, Inc.",
    "001206": "iQuest (NZ) Ltd",
    "001207": "Head Strong International Limited",
    "001208": "Gantner Electronic GmbH",
    "001209": "Fastrax Ltd",
    "00120A": "Emerson Electric GmbH & Co. OHG",
    "00120B": "Chinasys Technologies Limited",
    "00120C": "CE-Infosys Pte Ltd",
    "00120D": "Advanced Telecommunication Technologies, Inc.",
    "00120E": "AboCom",
    "00120F": "IEEE 802.3",
    "001210": "WideRay Corp",
    "001211": "Protechna Herbst GmbH & Co. KG",
    "001212": "PLUS Vision Corporation",
    "001213": "Metrohm AG",
    "001214": "Koenig & Bauer AG",
    "001215": "iStor Networks, Inc.",
    "001216": "ICP Internet Communication Payment AG",
    "001217": "Cisco-Linksys, LLC",
    "001218": "ARUZE Corporation",
    "001219": "Ahead Communication Systems Inc",
    "00121A": "Techno Soft Systemnics Inc.",
    "00121B": "Sound Devices, LLC",
    "00121C": "PARROT S.A.",
    "00121D": "Netfabric Corporation",
    "00121E": "Juniper Networks, Inc.",
    "00121F": "Harding Intruments",
    "001220": "Cadco Systems",
    "001221": "B.Braun Melsungen AG",
    "001222": "Skardin (UK) Ltd",
    "001223": "Pixim",
    "001224": "NexQL Corporation",
    "001225": "Motorola BCS",
    "001226": "Japan Direx Corporation",
    "001227": "Franklin Electric Co., Inc.",
    "001228": "Data Ltd.",
    "001229": "BroadEasy Technologies Co.,Ltd",
    "00122A": "VTech Telecommunications Ltd.",
    "00122B": "Virbiage Pty Ltd",
    "00122C": "Soenen Controls N.V.",
    "00122D": "SiNett Corporation",
    "00122E": "Signal Technology - AISD",
    "00122F": "Sanei Electric Inc.",
    "001230": "Picaso Infocommunication CO., LTD.",
    "001231": "Motion Control Systems, Inc.",
    "001232": "LeWiz Communications Inc.",
    "001233": "JRC TOKKI Co.,Ltd.",
    "001234": "Camille Bauer",
    "001235": "Andrew Corporation",
    "001236": "ConSentry Networks",
    "001237": "Texas Instruments",
    "001238": "SetaBox Technology Co., Ltd.",
    "001239": "S Net Systems Inc.",
    "00123A": "Posystech Inc., Co.",
    "00123B": "KeRo Systems ApS",
    "00123C": "Second Rule LLC",
    "00123D": "GES",
    "00123E": "ERUNE technology Co., Ltd.",
    "00123F": "Dell Inc",
    "001240": "AMOI ELECTRONICS CO.,LTD",
    "001241": "a2i marketing center",
    "001242": "Millennial Net",
    "001243": "Cisco",
    "001244": "Cisco",
    "001245": "Zellweger Analytics, Inc.",
    "001246": "T.O.M TECHNOLOGY INC..",
    "001247": "Samsung Electronics Co., Ltd.",
    "001248": "EMC Corporation (Kashya)",
    "001249": "Delta Elettronica S.p.A.",
    "00124A": "Dedicated Devices, Inc.",
    "00124B": "Texas Instruments",
    "00124C": "BBWM Corporation",
    "00124D": "Inducon BV",
    "00124E": "XAC AUTOMATION CORP.",
    "00124F": "Tyco Thermal Controls LLC.",
    "001250": "Tokyo Aircaft Instrument Co., Ltd.",
    "001251": "SILINK",
    "001252": "Citronix, LLC",
    "001253": "AudioDev AB",
    "001254": "Spectra Technologies Holdings Company Ltd",
    "001255": "NetEffect Incorporated",
    "001256": "LG INFORMATION & COMM.",
    "001257": "LeapComm Communication Technologies Inc.",
    "001258": "Activis Polska",
    "001259": "THERMO ELECTRON KARLSRUHE",
    "00125A": "Microsoft Corporation",
    "00125B": "KAIMEI ELECTRONI",
    "00125C": "Green Hills Software, Inc.",
    "00125D": "CyberNet Inc.",
    "00125E": "CAEN",
    "00125F": "AWIND Inc.",
    "001260": "Stanton Magnetics,inc.",
    "001261": "Adaptix, Inc",
    "001262": "Nokia Danmark A/S",
    "001263": "Data Voice Technologies GmbH",
    "001264": "daum electronic gmbh",
    "001265": "Enerdyne Technologies, Inc.",
    "001266": "Swisscom Hospitality Services SA",
    "001267": "Matsushita Electronic Components Co., Ltd.",
    "001268": "IPS d.o.o.",
    "001269": "Value Electronics",
    "00126A": "OPTOELECTRONICS Co., Ltd.",
    "00126B": "Ascalade Communications Limited",
    "00126C": "Visonic Ltd.",
    "00126D": "University of California, Berkeley",
    "00126E": "Seidel Elektronik GmbH Nfg.KG",
    "00126F": "Rayson Technology Co., Ltd.",
    "001270": "NGES Denro Systems",
    "001271": "Measurement Computing Corp",
    "001272": "Redux Communications Ltd.",
    "001273": "Stoke Inc",
    "001274": "NIT lab",
    "001275": "Sentilla Corporation",
    "001276": "Microsol Holdings Ltd.",
    "001277": "Korenix Technologies Co., Ltd.",
    "001278": "International Bar Code",
    "001279": "Hewlett Packard",
    "00127A": "Sanyu Industry Co.,Ltd.",
    "00127B": "VIA Networking Technologies, Inc.",
    "00127C": "SWEGON AB",
    "00127D": "MobileAria",
    "00127E": "Digital Lifestyles Group, Inc.",
    "00127F": "Cisco",
    "001280": "Cisco",
    "001281": "CIEFFE srl",
    "001282": "Qovia",
    "001283": "Nortel Networks",
    "001284": "Lab33 Srl",
    "001285": "Gizmondo Europe Ltd",
    "001286": "ENDEVCO CORP",
    "001287": "Digital Everywhere Unterhaltungselektronik GmbH",
    "001288": "2Wire, Inc",
    "001289": "Advance Sterilization Products",
    "00128A": "Motorola PCS",
    "00128B": "Sensory Networks Inc",
    "00128C": "Woodward Governor",
    "00128D": "STB Datenservice GmbH",
    "00128E": "Q-Free ASA",
    "00128F": "Montilio",
    "001290": "KYOWA Electric & Machinery Corp.",
    "001291": "KWS Computersysteme GmbH",
    "001292": "Griffin Technology",
    "001293": "GE Energy",
    "001294": "Eudyna Devices Inc.",
    "001295": "Aiware Inc.",
    "001296": "Addlogix",
    "001297": "O2Micro, Inc.",
    "001298": "MICO ELECTRIC(SHENZHEN) LIMITED",
    "001299": "Ktech Telecommunications Inc",
    "00129A": "IRT Electronics Pty Ltd",
    "00129B": "E2S Electronic Engineering Solutions, S.L.",
    "00129C": "Yulinet",
    "00129D": "First International Computer do Brasil",
    "00129E": "Surf Communications Inc.",
    "00129F": "RAE Systems, Inc.",
    "0012A0": "NeoMeridian Sdn Bhd",
    "0012A1": "BluePacket Communications Co., Ltd.",
    "0012A2": "VITA",
    "0012A3": "Trust International B.V.",
    "0012A4": "ThingMagic, LLC",
    "0012A5": "Stargen, Inc.",
    "0012A6": "Dolby Australia",
    "0012A7": "ISR TECHNOLOGIES Inc",
    "0012A8": "intec GmbH",
    "0012A9": "3Com Ltd",
    "0012AA": "IEE, Inc.",
    "0012AB": "WiLife, Inc.",
    "0012AC": "ONTIMETEK INC.",
    "0012AD": "IDS GmbH",
    "0012AE": "HLS HARD-LINE Solutions Inc.",
    "0012AF": "ELPRO Technologies",
    "0012B0": "Efore Oyj (Plc)",
    "0012B1": "Dai Nippon Printing Co., Ltd",
    "0012B2": "AVOLITES LTD.",
    "0012B3": "Advance Wireless Technology Corp.",
    "0012B4": "Work Microwave GmbH",
    "0012B5": "Vialta, Inc.",
    "0012B6": "Santa Barbara Infrared, Inc.",
    "0012B7": "PTW Freiburg",
    "0012B8": "G2 Microsystems",
    "0012B9": "Fusion Digital Technology",
    "0012BA": "FSI Systems, Inc.",
    "0012BB": "Telecommunications Industry Association TR-41 Committee",
    "0012BC": "Echolab LLC",
    "0012BD": "Avantec Manufacturing Limited",
    "0012BE": "Astek Corporation",
    "0012BF": "Arcadyan Technology Corporation",
    "0012C0": "HotLava Systems, Inc.",
    "0012C1": "Check Point Software Technologies",
    "0012C2": "Apex Electronics Factory",
    "0012C3": "WIT S.A.",
    "0012C4": "Viseon, Inc.",
    "0012C5": "V-Show Technology Co.Ltd",
    "0012C6": "TGC America, Inc",
    "0012C7": "SECURAY Technologies Ltd.Co.",
    "0012C8": "Perfect tech",
    "0012C9": "Motorola BCS",
    "0012CA": "Mechatronic Brick Aps",
    "0012CB": "CSS Inc.",
    "0012CC": "Bitatek CO., LTD",
    "0012CD": "ASEM SpA",
    "0012CE": "Advanced Cybernetics Group",
    "0012CF": "Accton Technology Corporation",
    "0012D0": "Gossen-Metrawatt-GmbH",
    "0012D1": "Texas Instruments Inc",
    "0012D2": "Texas Instruments",
    "0012D3": "Zetta Systems, Inc.",
    "0012D4": "Princeton Technology, Ltd",
    "0012D5": "Motion Reality Inc.",
    "0012D6": "Jiangsu Yitong High-Tech Co.,Ltd",
    "0012D7": "Invento Networks, Inc.",
    "0012D8": "International Games System Co., Ltd.",
    "0012D9": "Cisco Systems",
    "0012DA": "Cisco Systems",
    "0012DB": "ZIEHL industrie-elektronik GmbH + Co KG",
    "0012DC": "SunCorp Industrial Limited",
    "0012DD": "Shengqu Information Technology (Shanghai) Co., Ltd.",
    "0012DE": "Radio Components Sweden AB",
    "0012DF": "Novomatic AG",
    "0012E0": "Codan Limited",
    "0012E1": "Alliant Networks, Inc",
    "0012E2": "ALAXALA Networks Corporation",
    "0012E3": "Agat-RT, Ltd.",
    "0012E4": "ZIEHL industrie-electronik GmbH + Co KG",
    "0012E5": "Time America, Inc.",
    "0012E6": "SPECTEC COMPUTER CO., LTD.",
    "0012E7": "Projectek Networking Electronics Corp.",
    "0012E8": "Fraunhofer IMS",
    "0012E9": "Abbey Systems Ltd",
    "0012EA": "Trane",
    "0012EB": "R2DI, LLC",
    "0012EC": "Movacolor b.v.",
    "0012ED": "AVG Advanced Technologies",
    "0012EE": "Sony Ericsson Mobile Communications AB",
    "0012EF": "OneAccess SA",
    "0012F0": "Intel Corporate",
    "0012F1": "IFOTEC",
    "0012F2": "Foundry Networks",
    "0012F3": "connectBlue AB",
    "0012F4": "Belco International Co.,Ltd.",
    "0012F5": "Prolificx Ltd",
    "0012F6": "MDK CO.,LTD.",
    "0012F7": "Xiamen Xinglian Electronics Co., Ltd.",
    "0012F8": "WNI Resources, LLC",
    "0012F9": "URYU SEISAKU, LTD.",
    "0012FA": "THX LTD",
    "0012FB": "Samsung Electronics",
    "0012FC": "PLANET System Co.,LTD",
    "0012FD": "OPTIMUS IC S.A.",
    "0012FE": "Lenovo Mobile Communication Technology Ltd.",
    "0012FF": "Lely Industries N.V.",
    "001300": "IT-FACTORY, INC.",
    "001301": "IronGate S.L.",
    "001302": "Intel Corporate",
    "001303": "GateConnect Technologies GmbH",
    "001304": "Flaircomm Technologies Co. LTD",
    "001305": "Epicom, Inc.",
    "001306": "Always On Wireless",
    "001307": "Paravirtual Corporation",
    "001308": "Nuvera Fuel Cells",
    "001309": "Ocean Broadband Networks",
    "00130A": "Nortel",
    "00130B": "Mextal B.V.",
    "00130C": "HF System Corporation",
    "00130D": "GALILEO AVIONICA",
    "00130E": "Focusrite Audio Engineering Limited",
    "00130F": "EGEMEN Bilgisayar Muh San ve Tic LTD STI",
    "001310": "Cisco-Linksys, LLC",
    "001311": "ARRIS International",
    "001312": "Amedia Networks Inc.",
    "001313": "GuangZhou Post & Telecom Equipment ltd",
    "001314": "Asiamajor Inc.",
    "001315": "SONY Computer Entertainment inc,",
    "001316": "L-S-B GmbH",
    "001317": "GN Netcom as",
    "001318": "DGSTATION Co., Ltd.",
    "001319": "Cisco Systems",
    "00131A": "Cisco Systems",
    "00131B": "BeCell Innovations Corp.",
    "00131C": "LiteTouch, Inc.",
    "00131D": "Scanvaegt International A/S",
    "00131E": "Peiker acustic GmbH & Co. KG",
    "00131F": "NxtPhase T&D, Corp.",
    "001320": "Intel Corporate",
    "001321": "Hewlett Packard",
    "001322": "DAQ Electronics, Inc.",
    "001323": "Cap Co., Ltd.",
    "001324": "Schneider Electric Ultra Terminal",
    "001325": "Cortina Systems Inc",
    "001326": "ECM Systems Ltd",
    "001327": "Data Acquisitions limited",
    "001328": "Westech Korea Inc.,",
    "001329": "VSST Co., LTD",
    "00132A": "Sitronics Telecom Solutions",
    "00132B": "Phoenix Digital",
    "00132C": "MAZ Brandenburg GmbH",
    "00132D": "iWise Communications",
    "00132E": "ITian Coporation",
    "00132F": "Interactek",
    "001330": "EURO PROTECTION SURVEILLANCE",
    "001331": "CellPoint Connect",
    "001332": "Beijing Topsec Network Security Technology Co., Ltd.",
    "001333": "Baud Technology Inc.",
    "001334": "Arkados, Inc.",
    "001335": "VS Industry Berhad",
    "001336": "Tianjin 712 Communication Broadcasting co., ltd.",
    "001337": "Orient Power Home Network Ltd.",
    "001338": "FRESENIUS-VIAL",
    "001339": "EL-ME AG",
    "00133A": "VadaTech Inc.",
    "00133B": "Speed Dragon Multimedia Limited",
    "00133C": "QUINTRON SYSTEMS INC.",
    "00133D": "Micro Memory LLC",
    "00133E": "MetaSwitch",
    "00133F": "Eppendorf Instrumente GmbH",
    "001340": "AD.EL s.r.l.",
    "001341": "Shandong New Beiyang Information Technology Co.,Ltd",
    "001342": "Vision Research, Inc.",
    "001343": "Matsushita Electronic Components (Europe) GmbH",
    "001344": "Fargo Electronics Inc.",
    "001345": "Eaton Corporation",
    "001346": "D-Link Corporation",
    "001347": "BlueTree Wireless Data Inc.",
    "001348": "Artila Electronics Co., Ltd.",
    "001349": "ZyXEL Communications Corporation",
    "00134A": "Engim, Inc.",
    "00134B": "ToGoldenNet Technology Inc.",
    "00134C": "YDT Technology International",
    "00134D": "IPC systems",
    "00134E": "Valox Systems, Inc.",
    "00134F": "Tranzeo Wireless Technologies Inc.",
    "001350": "Silver Spring Networks, Inc",
    "001351": "Niles Audio Corporation",
    "001352": "Naztec, Inc.",
    "001353": "HYDAC Filtertechnik GMBH",
    "001354": "Zcomax Technologies, Inc.",
    "001355": "TOMEN Cyber-business Solutions, Inc.",
    "001356": "target systemelectronic gmbh",
    "001357": "Soyal Technology Co., Ltd.",
    "001358": "Realm Systems, Inc.",
    "001359": "ProTelevision Technologies A/S",
    "00135A": "Project T&E Limited",
    "00135B": "PanelLink Cinema, LLC",
    "00135C": "OnSite Systems, Inc.",
    "00135D": "NTTPC Communications, Inc.",
    "00135E": "EAB/RWI/K",
    "00135F": "Cisco Systems",
    "001360": "Cisco Systems",
    "001361": "Biospace Co., Ltd.",
    "001362": "ShinHeung Precision Co., Ltd.",
    "001363": "Verascape, Inc.",
    "001364": "Paradigm Technology Inc..",
    "001365": "Nortel",
    "001366": "Neturity Technologies Inc.",
    "001367": "Narayon. Co., Ltd.",
    "001368": "Maersk Data Defence",
    "001369": "Honda Electron Co., LED.",
    "00136A": "Hach Ultra Analytics",
    "00136B": "E-TEC",
    "00136C": "TomTom",
    "00136D": "Tentaculus AB",
    "00136E": "Techmetro Corp.",
    "00136F": "PacketMotion, Inc.",
    "001370": "Nokia Danmark A/S",
    "001371": "Motorola CHS",
    "001372": "Dell Inc.",
    "001373": "BLwave Electronics Co., Ltd",
    "001374": "Atheros Communications, Inc.",
    "001375": "American Security Products Co.",
    "001376": "Tabor Electronics Ltd.",
    "001377": "Samsung Electronics CO., LTD",
    "001378": "QSAN Technology, Inc.",
    "001379": "PONDER INFORMATION INDUSTRIES LTD.",
    "00137A": "Netvox Technology Co., Ltd.",
    "00137B": "Movon Corporation",
    "00137C": "Kaicom co., Ltd.",
    "00137D": "Dynalab, Inc.",
    "00137E": "CorEdge Networks, Inc.",
    "00137F": "Cisco Systems",
    "001380": "Cisco Systems",
    "001381": "CHIPS & Systems, Inc.",
    "001382": "Cetacea Networks Corporation",
    "001383": "Application Technologies and Engineering Research Laboratory",
    "001384": "Advanced Motion Controls",
    "001385": "Add-On Technology Co., LTD.",
    "001386": "ABB Inc./Totalflow",
    "001387": "27M Technologies AB",
    "001388": "WiMedia Alliance",
    "001389": "Redes de Telefonía Móvil S.A.",
    "00138A": "QINGDAO GOERTEK ELECTRONICS CO.,LTD.",
    "00138B": "Phantom Technologies LLC",
    "00138C": "Kumyoung.Co.Ltd",
    "00138D": "Kinghold",
    "00138E": "FOAB Elektronik AB",
    "00138F": "Asiarock Incorporation",
    "001390": "Termtek Computer Co., Ltd",
    "001391": "OUEN CO.,LTD.",
    "001392": "Ruckus Wireless",
    "001393": "Panta Systems, Inc.",
    "001394": "Infohand Co.,Ltd",
    "001395": "congatec AG",
    "001396": "Acbel Polytech Inc.",
    "001397": "Xsigo Systems, Inc.",
    "001398": "TrafficSim Co.,Ltd",
    "001399": "STAC Corporation.",
    "00139A": "K-ubique ID Corp.",
    "00139B": "ioIMAGE Ltd.",
    "00139C": "Exavera Technologies, Inc.",
    "00139D": "Design of Systems on Silicon S.A.",
    "00139E": "Ciara Technologies Inc.",
    "00139F": "Electronics Design Services, Co., Ltd.",
    "0013A0": "ALGOSYSTEM Co., Ltd.",
    "0013A1": "Crow Electronic Engeneering",
    "0013A2": "MaxStream, Inc",
    "0013A3": "Siemens Com CPE Devices",
    "0013A4": "KeyEye Communications",
    "0013A5": "General Solutions, LTD.",
    "0013A6": "Extricom Ltd",
    "0013A7": "BATTELLE MEMORIAL INSTITUTE",
    "0013A8": "Tanisys Technology",
    "0013A9": "Sony Corporation",
    "0013AA": "ALS & TEC Ltd.",
    "0013AB": "Telemotive AG",
    "0013AC": "Sunmyung Electronics Co., LTD",
    "0013AD": "Sendo Ltd",
    "0013AE": "Radiance Technologies",
    "0013AF": "NUMA Technology,Inc.",
    "0013B0": "Jablotron",
    "0013B1": "Intelligent Control Systems (Asia) Pte Ltd",
    "0013B2": "Carallon Limited",
    "0013B3": "Ecom Communications Technology Co., Ltd.",
    "0013B4": "Appear TV",
    "0013B5": "Wavesat",
    "0013B6": "Sling Media, Inc.",
    "0013B7": "Scantech ID",
    "0013B8": "RyCo Electronic Systems Limited",
    "0013B9": "BM SPA",
    "0013BA": "ReadyLinks Inc",
    "0013BB": "Smartvue Corporation",
    "0013BC": "Artimi Ltd",
    "0013BD": "HYMATOM SA",
    "0013BE": "Virtual Conexions",
    "0013BF": "Media System Planning Corp.",
    "0013C0": "Trix Tecnologia Ltda.",
    "0013C1": "Asoka USA Corporation",
    "0013C2": "WACOM Co.,Ltd",
    "0013C3": "Cisco Systems",
    "0013C4": "Cisco Systems",
    "0013C5": "LIGHTRON FIBER-OPTIC DEVICES INC.",
    "0013C6": "OpenGear, Inc",
    "0013C7": "IONOS Co.,Ltd.",
    "0013C8": "PIRELLI BROADBAND SOLUTIONS S.P.A.",
    "0013C9": "Beyond Achieve Enterprises Ltd.",
    "0013CA": "X-Digital Systems, Inc.",
    "0013CB": "Zenitel Norway AS",
    "0013CC": "Tall Maple Systems",
    "0013CD": "MTI co. LTD",
    "0013CE": "Intel Corporate",
    "0013CF": "4Access Communications",
    "0013D0": "t+ Medical Ltd",
    "0013D1": "KIRK telecom A/S",
    "0013D2": "PAGE IBERICA, S.A.",
    "0013D3": "MICRO-STAR INTERNATIONAL CO., LTD.",
    "0013D4": "ASUSTek COMPUTER INC.",
    "0013D5": "WiNetworks LTD",
    "0013D6": "TII NETWORK TECHNOLOGIES, INC.",
    "0013D7": "SPIDCOM Technologies SA",
    "0013D8": "Princeton Instruments",
    "0013D9": "Matrix Product Development, Inc.",
    "0013DA": "Diskware Co., Ltd",
    "0013DB": "SHOEI Electric Co.,Ltd",
    "0013DC": "IBTEK INC.",
    "0013DD": "Abbott Diagnostics",
    "0013DE": "Adapt4",
    "0013DF": "Ryvor Corp.",
    "0013E0": "Murata Manufacturing Co., Ltd.",
    "0013E1": "Iprobe AB",
    "0013E2": "GeoVision Inc.",
    "0013E3": "CoVi Technologies, Inc.",
    "0013E4": "YANGJAE SYSTEMS CORP.",
    "0013E5": "TENOSYS, INC.",
    "0013E6": "Technolution",
    "0013E7": "Halcro",
    "0013E8": "Intel Corporate",
    "0013E9": "VeriWave, Inc.",
    "0013EA": "Kamstrup A/S",
    "0013EB": "Sysmaster Corporation",
    "0013EC": "Sunbay Software AG",
    "0013ED": "PSIA",
    "0013EE": "JBX Designs Inc.",
    "0013EF": "Kingjon Digital Technology Co.,Ltd",
    "0013F0": "Wavefront Semiconductor",
    "0013F1": "AMOD Technology Co., Ltd.",
    "0013F2": "Klas Ltd",
    "0013F3": "Giga-byte Communications Inc.",
    "0013F4": "Psitek (Pty) Ltd",
    "0013F5": "Akimbi Systems",
    "0013F6": "Cintech",
    "0013F7": "SMC Networks, Inc.",
    "0013F8": "Dex Security Solutions",
    "0013F9": "Cavera Systems",
    "0013FA": "LifeSize Communications, Inc",
    "0013FB": "RKC INSTRUMENT INC.",
    "0013FC": "SiCortex, Inc",
    "0013FD": "Nokia Danmark A/S",
    "0013FE": "GRANDTEC ELECTRONIC CORP.",
    "0013FF": "Dage-MTI of MC, Inc.",
    "001400": "MINERVA KOREA CO., LTD",
    "001401": "Rivertree Networks Corp.",
    "001402": "kk-electronic a/s",
    "001403": "Renasis, LLC",
    "001404": "Motorola CHS",
    "001405": "OpenIB, Inc.",
    "001406": "Go Networks",
    "001407": "Sperian Protection Instrumentation",
    "001408": "Eka Systems Inc.",
    "001409": "MAGNETI MARELLI S.E. S.p.A.",
    "00140A": "WEPIO Co., Ltd.",
    "00140B": "FIRST INTERNATIONAL COMPUTER, INC.",
    "00140C": "GKB CCTV CO., LTD.",
    "00140D": "Nortel",
    "00140E": "Nortel",
    "00140F": "Federal State Unitary Enterprise Leningrad R&D Institute of",
    "001410": "Suzhou Keda Technology CO.,Ltd",
    "001411": "Deutschmann Automation GmbH & Co. KG",
    "001412": "S-TEC electronics AG",
    "001413": "Trebing & Himstedt Prozessautomation GmbH & Co. KG",
    "001414": "Jumpnode Systems LLC.",
    "001415": "Intec Automation Inc.",
    "001416": "Scosche Industries, Inc.",
    "001417": "RSE Informations Technologie GmbH",
    "001418": "C4Line",
    "001419": "SIDSA",
    "00141A": "DEICY CORPORATION",
    "00141B": "Cisco Systems",
    "00141C": "Cisco Systems",
    "00141D": "Lust Antriebstechnik GmbH",
    "00141E": "P.A. Semi, Inc.",
    "00141F": "SunKwang Electronics Co., Ltd",
    "001420": "G-Links networking company",
    "001421": "Total Wireless Technologies Pte. Ltd.",
    "001422": "Dell Inc.",
    "001423": "J-S Co. NEUROCOM",
    "001424": "Merry Electrics CO., LTD.",
    "001425": "Galactic Computing Corp.",
    "001426": "NL Technology",
    "001427": "JazzMutant",
    "001428": "Vocollect, Inc",
    "001429": "V Center Technologies Co., Ltd.",
    "00142A": "Elitegroup Computer System Co., Ltd",
    "00142B": "Edata Communication Inc.",
    "00142C": "Koncept International, Inc.",
    "00142D": "Toradex AG",
    "00142E": "77 Elektronika Kft.",
    "00142F": "WildPackets",
    "001430": "ViPowER, Inc",
    "001431": "PDL Electronics Ltd",
    "001432": "Tarallax Wireless, Inc.",
    "001433": "Empower Technologies(Canada) Inc.",
    "001434": "Keri Systems, Inc",
    "001435": "CityCom Corp.",
    "001436": "Qwerty Elektronik AB",
    "001437": "GSTeletech Co.,Ltd.",
    "001438": "Hewlett Packard",
    "001439": "Blonder Tongue Laboratories, Inc.",
    "00143A": "RAYTALK INTERNATIONAL SRL",
    "00143B": "Sensovation AG",
    "00143C": "Rheinmetall Canada Inc.",
    "00143D": "Aevoe Inc.",
    "00143E": "AirLink Communications, Inc.",
    "00143F": "Hotway Technology Corporation",
    "001440": "ATOMIC Corporation",
    "001441": "Innovation Sound Technology Co., LTD.",
    "001442": "ATTO CORPORATION",
    "001443": "Consultronics Europe Ltd",
    "001444": "Grundfos Electronics",
    "001445": "Telefon-Gradnja d.o.o.",
    "001446": "SuperVision Solutions LLC",
    "001447": "BOAZ Inc.",
    "001448": "Inventec Multimedia & Telecom Corporation",
    "001449": "Sichuan Changhong Electric Ltd.",
    "00144A": "Taiwan Thick-Film Ind. Corp.",
    "00144B": "Hifn, Inc.",
    "00144C": "General Meters Corp.",
    "00144D": "Intelligent Systems",
    "00144E": "SRISA",
    "00144F": "Sun Microsystems, Inc.",
    "001450": "Heim Systems GmbH",
    "001451": "Apple Computer Inc.",
    "001452": "CALCULEX,INC.",
    "001453": "ADVANTECH TECHNOLOGIES CO.,LTD",
    "001454": "Symwave",
    "001455": "Coder Electronics Corporation",
    "001456": "Edge Products",
    "001457": "T-VIPS AS",
    "001458": "HS Automatic ApS",
    "001459": "Moram Co., Ltd.",
    "00145A": "Elektrobit AG",
    "00145B": "SeekerNet Inc.",
    "00145C": "Intronics B.V.",
    "00145D": "WJ Communications, Inc.",
    "00145E": "IBM",
    "00145F": "ADITEC CO. LTD",
    "001460": "Kyocera Wireless Corp.",
    "001461": "CORONA CORPORATION",
    "001462": "Digiwell Technology, inc",
    "001463": "IDCS N.V.",
    "001464": "Cryptosoft",
    "001465": "Novo Nordisk A/S",
    "001466": "Kleinhenz Elektronik GmbH",
    "001467": "ArrowSpan Inc.",
    "001468": "CelPlan International, Inc.",
    "001469": "Cisco Systems",
    "00146A": "Cisco Systems",
    "00146B": "Anagran, Inc.",
    "00146C": "Netgear Inc.",
    "00146D": "RF Technologies",
    "00146E": "H. Stoll GmbH & Co. KG",
    "00146F": "Kohler Co",
    "001470": "Prokom Software SA",
    "001471": "Eastern Asia Technology Limited",
    "001472": "China Broadband Wireless IP Standard Group",
    "001473": "Bookham Inc",
    "001474": "K40 Electronics",
    "001475": "Wiline Networks, Inc.",
    "001476": "MultiCom Industries Limited",
    "001477": "Nertec Inc.",
    "001478": "ShenZhen TP-LINK Technologies Co., Ltd.",
    "001479": "NEC Magnus Communications,Ltd.",
    "00147A": "Eubus GmbH",
    "00147B": "Iteris, Inc.",
    "00147C": "3Com Ltd",
    "00147D": "Aeon Digital International",
    "00147E": "InnerWireless",
    "00147F": "Thomson Telecom Belgium",
    "001480": "Hitachi-LG Data Storage Korea, Inc",
    "001481": "Multilink Inc",
    "001482": "GoBackTV, Inc",
    "001483": "eXS Inc.",
    "001484": "Cermate Technologies Inc.",
    "001485": "Giga-Byte",
    "001486": "Echo Digital Audio Corporation",
    "001487": "American Technology Integrators",
    "001488": "Akorri",
    "001489": "B15402100 - JANDEI, S.L.",
    "00148A": "Elin Ebg Traction Gmbh",
    "00148B": "Globo Electronic GmbH & Co. KG",
    "00148C": "Fortress Technologies",
    "00148D": "Cubic Defense Simulation Systems",
    "00148E": "Tele Power Inc.",
    "00148F": "Protronic (Far East) Ltd.",
    "001490": "ASP Corporation",
    "001491": "Daniels Electronics Ltd.",
    "001492": "Liteon, Mobile Media Solution SBU",
    "001493": "Systimax Solutions",
    "001494": "ESU AG",
    "001495": "2Wire, Inc.",
    "001496": "Phonic Corp.",
    "001497": "ZHIYUAN Eletronics co.,ltd.",
    "001498": "Viking Design Technology",
    "001499": "Helicomm Inc",
    "00149A": "Motorola Mobile Devices Business",
    "00149B": "Nokota Communications, LLC",
    "00149C": "HF Company",
    "00149D": "Sound ID Inc.",
    "00149E": "UbONE Co., Ltd",
    "00149F": "System and Chips, Inc.",
    "0014A0": "Accsense, Inc.",
    "0014A1": "Synchronous Communication Corp",
    "0014A2": "Core Micro Systems Inc.",
    "0014A3": "Vitelec BV",
    "0014A4": "Hon Hai Precision Ind. Co., Ltd.",
    "0014A5": "Gemtek Technology Co., Ltd.",
    "0014A6": "Teranetics, Inc.",
    "0014A7": "Nokia Danmark A/S",
    "0014A8": "Cisco Systems",
    "0014A9": "Cisco Systems",
    "0014AA": "Ashly Audio, Inc.",
    "0014AB": "Senhai Electronic Technology Co., Ltd.",
    "0014AC": "Bountiful WiFi",
    "0014AD": "Gassner Wiege- u. Meßtechnik GmbH",
    "0014AE": "Wizlogics Co., Ltd.",
    "0014AF": "Datasym Inc.",
    "0014B0": "Naeil Community",
    "0014B1": "Avitec AB",
    "0014B2": "mCubelogics Corporation",
    "0014B3": "CoreStar International Corp",
    "0014B4": "General Dynamics United Kingdom Ltd",
    "0014B5": "PHYSIOMETRIX,INC",
    "0014B6": "Enswer Technology Inc.",
    "0014B7": "AR Infotek Inc.",
    "0014B8": "Hill-Rom",
    "0014B9": "MSTAR SEMICONDUCTOR",
    "0014BA": "Carvers SA de CV",
    "0014BB": "Open Interface North America",
    "0014BC": "SYNECTIC TELECOM EXPORTS PVT. LTD.",
    "0014BD": "incNETWORKS, Inc",
    "0014BE": "Wink communication technology CO.LTD",
    "0014BF": "Cisco-Linksys LLC",
    "0014C0": "Symstream Technology Group Ltd",
    "0014C1": "U.S. Robotics Corporation",
    "0014C2": "Hewlett Packard",
    "0014C3": "Seagate Technology LLC",
    "0014C4": "Vitelcom Mobile Technology",
    "0014C5": "Alive Technologies Pty Ltd",
    "0014C6": "Quixant Ltd",
    "0014C7": "Nortel",
    "0014C8": "Contemporary Research Corp",
    "0014C9": "Brocade Communications Systems, Inc.",
    "0014CA": "Key Radio Systems Limited",
    "0014CB": "LifeSync Corporation",
    "0014CC": "Zetec, Inc.",
    "0014CD": "DigitalZone Co., Ltd.",
    "0014CE": "NF CORPORATION",
    "0014CF": "Nextlink.to A/S",
    "0014D0": "BTI Photonics",
    "0014D1": "TRENDware International, Inc.",
    "0014D2": "KYUKI CORPORATION",
    "0014D3": "SEPSA",
    "0014D4": "K Technology Corporation",
    "0014D5": "Datang Telecom Technology CO. , LCD,Optical Communication Br",
    "0014D6": "Jeongmin Electronics Co.,Ltd.",
    "0014D7": "Datastore Technology Corp",
    "0014D8": "bio-logic SA",
    "0014D9": "IP Fabrics, Inc.",
    "0014DA": "Huntleigh Healthcare",
    "0014DB": "Elma Trenew Electronic GmbH",
    "0014DC": "Communication System Design & Manufacturing (CSDM)",
    "0014DD": "Covergence Inc.",
    "0014DE": "Sage Instruments Inc.",
    "0014DF": "HI-P Tech Corporation",
    "0014E0": "LET'S Corporation",
    "0014E1": "Data Display AG",
    "0014E2": "datacom systems inc.",
    "0014E3": "mm-lab GmbH",
    "0014E4": "Integral Technologies",
    "0014E5": "Alticast",
    "0014E6": "AIM Infrarotmodule GmbH",
    "0014E7": "Stolinx,. Inc",
    "0014E8": "Motorola CHS",
    "0014E9": "Nortech International",
    "0014EA": "S Digm Inc. (Safe Paradigm Inc.)",
    "0014EB": "AwarePoint Corporation",
    "0014EC": "Acro Telecom",
    "0014ED": "Airak, Inc.",
    "0014EE": "Western Digital Technologies, Inc.",
    "0014EF": "TZero Technologies, Inc.",
    "0014F0": "Business Security OL AB",
    "0014F1": "Cisco Systems",
    "0014F2": "Cisco Systems",
    "0014F3": "ViXS Systems Inc",
    "0014F4": "DekTec Digital Video B.V.",
    "0014F5": "OSI Security Devices",
    "0014F6": "Juniper Networks, Inc.",
    "0014F7": "Crevis",
    "0014F8": "Scientific Atlanta",
    "0014F9": "Vantage Controls",
    "0014FA": "AsGa S.A.",
    "0014FB": "Technical Solutions Inc.",
    "0014FC": "Extandon, Inc.",
    "0014FD": "Thecus Technology Corp.",
    "0014FE": "Artech Electronics",
    "0014FF": "Precise Automation, Inc.",
    "001500": "Intel Corporate",
    "001501": "LexBox",
    "001502": "BETA tech",
    "001503": "PROFIcomms s.r.o.",
    "001504": "GAME PLUS CO., LTD.",
    "001505": "Actiontec Electronics, Inc",
    "001506": "Neo Photonics",
    "001507": "Renaissance Learning Inc",
    "001508": "Global Target Enterprise Inc",
    "001509": "Plus Technology Co., Ltd",
    "00150A": "Sonoa Systems, Inc",
    "00150B": "SAGE INFOTECH LTD.",
    "00150C": "AVM GmbH",
    "00150D": "Hoana Medical, Inc.",
    "00150E": "OPENBRAIN TECHNOLOGIES CO., LTD.",
    "00150F": "mingjong",
    "001510": "Techsphere Co., Ltd",
    "001511": "Data Center Systems",
    "001512": "Zurich University of Applied Sciences",
    "001513": "EFS sas",
    "001514": "Hu Zhou NAVA Networks&Electronics Ltd.",
    "001515": "Leipold+Co.GmbH",
    "001516": "URIEL SYSTEMS INC.",
    "001517": "Intel Corporate",
    "001518": "Shenzhen 10MOONS Technology Development CO.,Ltd",
    "001519": "StoreAge Networking Technologies",
    "00151A": "Hunter Engineering Company",
    "00151B": "Isilon Systems Inc.",
    "00151C": "LENECO",
    "00151D": "M2I CORPORATION",
    "00151E": "Ethernet Powerlink Standardization Group (EPSG)",
    "00151F": "Multivision Intelligent Surveillance (Hong Kong) Ltd",
    "001520": "Radiocrafts AS",
    "001521": "Horoquartz",
    "001522": "Dea Security",
    "001523": "Meteor Communications Corporation",
    "001524": "Numatics, Inc.",
    "001525": "PTI Integrated Systems, Inc.",
    "001526": "Remote Technologies Inc",
    "001527": "Balboa Instruments",
    "001528": "Beacon Medical Products LLC d.b.a. BeaconMedaes",
    "001529": "N3 Corporation",
    "00152A": "Nokia GmbH",
    "00152B": "Cisco Systems",
    "00152C": "Cisco Systems",
    "00152D": "TenX Networks, LLC",
    "00152E": "PacketHop, Inc.",
    "00152F": "Motorola CHS",
    "001530": "Bus-Tech, Inc.",
    "001531": "KOCOM",
    "001532": "Consumer Technologies Group, LLC",
    "001533": "NADAM.CO.,LTD",
    "001534": "A BELTRÓNICA, Companhia de Comunicações, Lda",
    "001535": "OTE Spa",
    "001536": "Powertech co.,Ltd",
    "001537": "Ventus Networks",
    "001538": "RFID, Inc.",
    "001539": "Technodrive SRL",
    "00153A": "Shenzhen Syscan Technology Co.,Ltd.",
    "00153B": "EMH Elektrizitätszähler GmbH & CoKG",
    "00153C": "Kprotech Co., Ltd.",
    "00153D": "ELIM PRODUCT CO.",
    "00153E": "Q-Matic Sweden AB",
    "00153F": "Alcatel Alenia Space Italia",
    "001540": "Nortel",
    "001541": "StrataLight Communications, Inc.",
    "001542": "MICROHARD S.R.L.",
    "001543": "Aberdeen Test Center",
    "001544": "coM.s.a.t. AG",
    "001545": "SEECODE Co., Ltd.",
    "001546": "ITG Worldwide Sdn Bhd",
    "001547": "AiZen Solutions Inc.",
    "001548": "CUBE TECHNOLOGIES",
    "001549": "Dixtal Biomedica Ind. Com. Ltda",
    "00154A": "WANSHIH ELECTRONIC CO., LTD",
    "00154B": "Wonde Proud Technology Co., Ltd",
    "00154C": "Saunders Electronics",
    "00154D": "Netronome Systems, Inc.",
    "00154E": "Hirschmann Automation and Control GmbH",
    "00154F": "one RF Technology",
    "001550": "Nits Technology Inc",
    "001551": "RadioPulse Inc.",
    "001552": "Wi-Gear Inc.",
    "001553": "Cytyc Corporation",
    "001554": "Atalum Wireless S.A.",
    "001555": "DFM GmbH",
    "001556": "SAGEM COMMUNICATION",
    "001557": "Olivetti",
    "001558": "FOXCONN",
    "001559": "Securaplane Technologies, Inc.",
    "00155A": "DAINIPPON PHARMACEUTICAL CO., LTD.",
    "00155B": "Sampo Corporation",
    "00155C": "Dresser Wayne",
    "00155D": "Microsoft Corporation",
    "00155E": "Morgan Stanley",
    "00155F": "GreenPeak",
    "001560": "Hewlett Packard",
    "001561": "JJPlus Corporation",
    "001562": "Cisco Systems",
    "001563": "Cisco Systems",
    "001564": "BEHRINGER Spezielle Studiotechnik GmbH",
    "001565": "XIAMEN YEALINK NETWORK TECHNOLOGY CO.,LTD",
    "001566": "A-First Technology Co., Ltd.",
    "001567": "RADWIN Inc.",
    "001568": "Dilithium Networks",
    "001569": "PECO II, Inc.",
    "00156A": "DG2L Technologies Pvt. Ltd.",
    "00156B": "Perfisans Networks Corp.",
    "00156C": "SANE SYSTEM CO., LTD",
    "00156D": "Ubiquiti Networks Inc.",
    "00156E": "A. W. Communication Systems Ltd",
    "00156F": "Xiranet Communications GmbH",
    "001570": "Symbol Technologies",
    "001571": "Nolan Systems",
    "001572": "Red-Lemon",
    "001573": "NewSoft Technology Corporation",
    "001574": "Horizon Semiconductors Ltd.",
    "001575": "Nevis Networks Inc.",
    "001576": "scil animal care company GmbH",
    "001577": "Allied Telesyn, Inc.",
    "001578": "Audio / Video Innovations",
    "001579": "Lunatone Industrielle Elektronik GmbH",
    "00157A": "Telefin S.p.A.",
    "00157B": "Leuze electronic GmbH + Co. KG",
    "00157C": "Dave Networks, Inc.",
    "00157D": "POSDATA CO., LTD.",
    "00157E": "HEYFRA ELECTRONIC gmbH",
    "00157F": "ChuanG International Holding CO.,LTD.",
    "001580": "U-WAY CORPORATION",
    "001581": "MAKUS Inc.",
    "001582": "TVonics Ltd",
    "001583": "IVT corporation",
    "001584": "Schenck Process GmbH",
    "001585": "Aonvision Technolopy Corp.",
    "001586": "Xiamen Overseas Chinese Electronic Co., Ltd.",
    "001587": "Takenaka Seisakusho Co.,Ltd",
    "001588": "Balda-Thong Fook Solutions Sdn. Bhd.",
    "001589": "D-MAX Technology Co.,Ltd",
    "00158A": "SURECOM Technology Corp.",
    "00158B": "Park Air Systems Ltd",
    "00158C": "Liab ApS",
    "00158D": "Jennic Ltd",
    "00158E": "Plustek.INC",
    "00158F": "NTT Advanced Technology Corporation",
    "001590": "Hectronic GmbH",
    "001591": "RLW Inc.",
    "001592": "Facom UK Ltd (Melksham)",
    "001593": "U4EA Technologies Inc.",
    "001594": "BIXOLON CO.,LTD",
    "001595": "Quester Tangent Corporation",
    "001596": "ARRIS International",
    "001597": "AETA AUDIO SYSTEMS",
    "001598": "Kolektor group",
    "001599": "Samsung Electronics Co., LTD",
    "00159A": "Motorola CHS",
    "00159B": "Nortel",
    "00159C": "B-KYUNG SYSTEM Co.,Ltd.",
    "00159D": "Minicom Advanced Systems ltd",
    "00159E": "Saitek plc",
    "00159F": "Terascala, Inc.",
    "0015A0": "Nokia Danmark A/S",
    "0015A1": "SINTERS SAS",
    "0015A2": "ARRIS International",
    "0015A3": "ARRIS International",
    "0015A4": "ARRIS International",
    "0015A5": "DCI Co., Ltd.",
    "0015A6": "Digital Electronics Products Ltd.",
    "0015A7": "Robatech AG",
    "0015A8": "Motorola Mobile Devices",
    "0015A9": "KWANG WOO I&C CO.,LTD",
    "0015AA": "Rextechnik International Co.,",
    "0015AB": "PRO CO SOUND INC",
    "0015AC": "Capelon AB",
    "0015AD": "Accedian Networks",
    "0015AE": "kyung il",
    "0015AF": "AzureWave Technologies, Inc.",
    "0015B0": "AUTOTELENET CO.,LTD",
    "0015B1": "Ambient Corporation",
    "0015B2": "Advanced Industrial Computer, Inc.",
    "0015B3": "Caretech AB",
    "0015B4": "Polymap Wireless LLC",
    "0015B5": "CI Network Corp.",
    "0015B6": "ShinMaywa Industries, Ltd.",
    "0015B7": "Toshiba",
    "0015B8": "Tahoe",
    "0015B9": "Samsung Electronics Co., Ltd.",
    "0015BA": "iba AG",
    "0015BB": "SMA Technologie AG",
    "0015BC": "Develco",
    "0015BD": "Group 4 Technology Ltd",
    "0015BE": "Iqua Ltd.",
    "0015BF": "technicob",
    "0015C0": "DIGITAL TELEMEDIA CO.,LTD.",
    "0015C1": "SONY Computer Entertainment inc,",
    "0015C2": "3M Germany",
    "0015C3": "Ruf Telematik AG",
    "0015C4": "FLOVEL CO., LTD.",
    "0015C5": "Dell Inc",
    "0015C6": "Cisco Systems",
    "0015C7": "Cisco Systems",
    "0015C8": "FlexiPanel Ltd",
    "0015C9": "Gumstix, Inc",
    "0015CA": "TeraRecon, Inc.",
    "0015CB": "Surf Communication Solutions Ltd.",
    "0015CC": "TEPCO UQUEST, LTD.",
    "0015CD": "Exartech International Corp.",
    "0015CE": "ARRIS International",
    "0015CF": "ARRIS International",
    "0015D0": "ARRIS International",
    "0015D1": "ARRIS Group, Inc.",
    "0015D2": "Xantech Corporation",
    "0015D3": "Pantech&Curitel Communications, Inc.",
    "0015D4": "Emitor AB",
    "0015D5": "NICEVT",
    "0015D6": "OSLiNK Sp. z o.o.",
    "0015D7": "Reti Corporation",
    "0015D8": "Interlink Electronics",
    "0015D9": "PKC Electronics Oy",
    "0015DA": "IRITEL A.D.",
    "0015DB": "Canesta Inc.",
    "0015DC": "KT&C Co., Ltd.",
    "0015DD": "IP Control Systems Ltd.",
    "0015DE": "Nokia Danmark A/S",
    "0015DF": "Clivet S.p.A.",
    "0015E0": "Ericsson Mobile Platforms",
    "0015E1": "picoChip Designs Ltd",
    "0015E2": "Wissenschaftliche Geraetebau Dr. Ing. H. Knauer GmbH",
    "0015E3": "Dream Technologies Corporation",
    "0015E4": "Zimmer Elektromedizin",
    "0015E5": "Cheertek Inc.",
    "0015E6": "MOBILE TECHNIKA Inc.",
    "0015E7": "Quantec ProAudio",
    "0015E8": "Nortel",
    "0015E9": "D-Link Corporation",
    "0015EA": "Tellumat (Pty) Ltd",
    "0015EB": "ZTE CORPORATION",
    "0015EC": "Boca Devices LLC",
    "0015ED": "Fulcrum Microsystems, Inc.",
    "0015EE": "Omnex Control Systems",
    "0015EF": "NEC TOKIN Corporation",
    "0015F0": "EGO BV",
    "0015F1": "KYLINK Communications Corp.",
    "0015F2": "ASUSTek COMPUTER INC.",
    "0015F3": "PELTOR AB",
    "0015F4": "Eventide",
    "0015F5": "Sustainable Energy Systems",
    "0015F6": "SCIENCE AND ENGINEERING SERVICES, INC.",
    "0015F7": "Wintecronics Ltd.",
    "0015F8": "Kingtronics Industrial Co. Ltd.",
    "0015F9": "Cisco Systems",
    "0015FA": "Cisco Systems",
    "0015FB": "setex schermuly textile computer gmbh",
    "0015FC": "Startco Engineering Ltd.",
    "0015FD": "Complete Media Systems",
    "0015FE": "SCHILLING ROBOTICS LLC",
    "0015FF": "Novatel Wireless, Inc.",
    "001600": "CelleBrite Mobile Synchronization",
    "001601": "Buffalo Inc.",
    "001602": "CEYON TECHNOLOGY CO.,LTD.",
    "001603": "COOLKSKY Co., LTD",
    "001604": "Sigpro",
    "001605": "YORKVILLE SOUND INC.",
    "001606": "Ideal Industries",
    "001607": "Curves International Inc.",
    "001608": "Sequans Communications",
    "001609": "Unitech electronics co., ltd.",
    "00160A": "SWEEX Europe BV",
    "00160B": "TVWorks LLC",
    "00160C": "LPL DEVELOPMENT S.A. DE C.V",
    "00160D": "Be Here Corporation",
    "00160E": "Optica Technologies Inc.",
    "00160F": "BADGER METER INC",
    "001610": "Carina Technology",
    "001611": "Altecon Srl",
    "001612": "Otsuka Electronics Co., Ltd.",
    "001613": "LibreStream Technologies Inc.",
    "001614": "Picosecond Pulse Labs",
    "001615": "Nittan Company, Limited",
    "001616": "BROWAN COMMUNICATION INC.",
    "001617": "MSI",
    "001618": "HIVION Co., Ltd.",
    "001619": "La Factoría de Comunicaciones Aplicadas,S.L.",
    "00161A": "Dametric AB",
    "00161B": "Micronet Corporation",
    "00161C": "e:cue",
    "00161D": "Innovative Wireless Technologies, Inc.",
    "00161E": "Woojinnet",
    "00161F": "SUNWAVETEC Co., Ltd.",
    "001620": "Sony Ericsson Mobile Communications AB",
    "001621": "Colorado Vnet",
    "001622": "BBH SYSTEMS GMBH",
    "001623": "Interval Media",
    "001624": "Teneros, Inc.",
    "001625": "Impinj, Inc.",
    "001626": "Motorola CHS",
    "001627": "embedded-logic DESIGN AND MORE GmbH",
    "001628": "Ultra Electronics Manufacturing and Card Systems",
    "001629": "Nivus GmbH",
    "00162A": "Antik computers & communications s.r.o.",
    "00162B": "Togami Electric Mfg.co.,Ltd.",
    "00162C": "Xanboo",
    "00162D": "STNet Co., Ltd.",
    "00162E": "Space Shuttle Hi-Tech Co., Ltd.",
    "00162F": "Geutebrück GmbH",
    "001630": "Vativ Technologies",
    "001631": "Xteam",
    "001632": "SAMSUNG ELECTRONICS CO., LTD.",
    "001633": "Oxford Diagnostics Ltd.",
    "001634": "Mathtech, Inc.",
    "001635": "Hewlett Packard",
    "001636": "Quanta Computer Inc.",
    "001637": "Citel Srl",
    "001638": "TECOM Co., Ltd.",
    "001639": "UBIQUAM Co.,Ltd",
    "00163A": "YVES TECHNOLOGY CO., LTD.",
    "00163B": "VertexRSI/General Dynamics",
    "00163C": "Rebox B.V.",
    "00163D": "Tsinghua Tongfang Legend Silicon Tech. Co., Ltd.",
    "00163E": "Xensource, Inc.",
    "00163F": "CReTE SYSTEMS Inc.",
    "001640": "Asmobile Communication Inc.",
    "001641": "USI",
    "001642": "Pangolin",
    "001643": "Sunhillo Corporation",
    "001644": "LITE-ON Technology Corp.",
    "001645": "Power Distribution, Inc.",
    "001646": "Cisco Systems",
    "001647": "Cisco Systems",
    "001648": "SSD Company Limited",
    "001649": "SetOne GmbH",
    "00164A": "Vibration Technology Limited",
    "00164B": "Quorion Data Systems GmbH",
    "00164C": "PLANET INT Co., Ltd",
    "00164D": "Alcatel North America IP Division",
    "00164E": "Nokia Danmark A/S",
    "00164F": "World Ethnic Broadcastin Inc.",
    "001650": "EYAL MICROWAVE",
    "001651": "PRIVATE",
    "001652": "Hoatech Technologies, Inc.",
    "001653": "LEGO System A/S IE Electronics Division",
    "001654": "Flex-P Industries Sdn. Bhd.",
    "001655": "FUHO TECHNOLOGY Co., LTD",
    "001656": "Nintendo Co., Ltd.",
    "001657": "Aegate Ltd",
    "001658": "Fusiontech Technologies Inc.",
    "001659": "Z.M.P. RADWAG",
    "00165A": "Harman Specialty Group",
    "00165B": "Grip Audio",
    "00165C": "Trackflow Ltd",
    "00165D": "AirDefense, Inc.",
    "00165E": "Precision I/O",
    "00165F": "Fairmount Automation",
    "001660": "Nortel",
    "001661": "Novatium Solutions (P) Ltd",
    "001662": "Liyuh Technology Ltd.",
    "001663": "KBT Mobile",
    "001664": "Prod-El SpA",
    "001665": "Cellon France",
    "001666": "Quantier Communication Inc.",
    "001667": "A-TEC Subsystem INC.",
    "001668": "Eishin Electronics",
    "001669": "MRV Communication (Networks) LTD",
    "00166A": "TPS",
    "00166B": "Samsung Electronics",
    "00166C": "Samsung Electonics Digital Video System Division",
    "00166D": "Yulong Computer Telecommunication Scientific(shenzhen)Co.,Lt",
    "00166E": "Arbitron Inc.",
    "00166F": "Intel Corporation",
    "001670": "SKNET Corporation",
    "001671": "Symphox Information Co.",
    "001672": "Zenway enterprise ltd",
    "001673": "Bury GmbH & Co. KG",
    "001674": "EuroCB (Phils.), Inc.",
    "001675": "Motorola MDb",
    "001676": "Intel Corporation",
    "001677": "Bihl+Wiedemann GmbH",
    "001678": "SHENZHEN BAOAN GAOKE ELECTRONICS CO., LTD",
    "001679": "eOn Communications",
    "00167A": "Skyworth Overseas Dvelopment Ltd.",
    "00167B": "Haver&Boecker",
    "00167C": "iRex Technologies BV",
    "00167D": "Sky-Line Information Co., Ltd.",
    "00167E": "DIBOSS.CO.,LTD",
    "00167F": "Bluebird Soft Inc.",
    "001680": "Bally Gaming + Systems",
    "001681": "Vector Informatik GmbH",
    "001682": "Pro Dex, Inc",
    "001683": "WEBIO International Co.,.Ltd.",
    "001684": "Donjin Co.,Ltd.",
    "001685": "Elisa Oyj",
    "001686": "Karl Storz Imaging",
    "001687": "Chubb CSC-Vendor AP",
    "001688": "ServerEngines LLC",
    "001689": "Pilkor Electronics Co., Ltd",
    "00168A": "id-Confirm Inc",
    "00168B": "Paralan Corporation",
    "00168C": "DSL Partner AS",
    "00168D": "KORWIN CO., Ltd.",
    "00168E": "Vimicro corporation",
    "00168F": "GN Netcom as",
    "001690": "J-TEK INCORPORATION",
    "001691": "Moser-Baer AG",
    "001692": "Scientific-Atlanta, Inc.",
    "001693": "PowerLink Technology Inc.",
    "001694": "Sennheiser Communications A/S",
    "001695": "AVC Technology Limited",
    "001696": "QDI Technology (H.K.) Limited",
    "001697": "NEC Corporation",
    "001698": "T&A Mobile Phones",
    "001699": "Tonic DVB Marketing Ltd",
    "00169A": "Quadrics Ltd",
    "00169B": "Alstom Transport",
    "00169C": "Cisco Systems",
    "00169D": "Cisco Systems",
    "00169E": "TV One Ltd",
    "00169F": "Vimtron Electronics Co., Ltd.",
    "0016A0": "Auto-Maskin",
    "0016A1": "3Leaf Networks",
    "0016A2": "CentraLite Systems, Inc.",
    "0016A3": "TEAM ARTECHE, S.A.",
    "0016A4": "Ezurio Ltd",
    "0016A5": "Tandberg Storage ASA",
    "0016A6": "Dovado FZ-LLC",
    "0016A7": "AWETA G&P",
    "0016A8": "CWT CO., LTD.",
    "0016A9": "2EI",
    "0016AA": "Kei Communication Technology Inc.",
    "0016AB": "PBI-Dansensor A/S",
    "0016AC": "Toho Technology Corp.",
    "0016AD": "BT-Links Company Limited",
    "0016AE": "INVENTEL",
    "0016AF": "Shenzhen Union Networks Equipment Co.,Ltd.",
    "0016B0": "VK Corporation",
    "0016B1": "KBS",
    "0016B2": "DriveCam Inc",
    "0016B3": "Photonicbridges (China) Co., Ltd.",
    "0016B4": "PRIVATE",
    "0016B5": "Motorola CHS",
    "0016B6": "Cisco-Linksys",
    "0016B7": "Seoul Commtech",
    "0016B8": "Sony Ericsson Mobile Communications",
    "0016B9": "ProCurve Networking",
    "0016BA": "WEATHERNEWS INC.",
    "0016BB": "Law-Chain Computer Technology Co Ltd",
    "0016BC": "Nokia Danmark A/S",
    "0016BD": "ATI Industrial Automation",
    "0016BE": "INFRANET, Inc.",
    "0016BF": "PaloDEx Group Oy",
    "0016C0": "Semtech Corporation",
    "0016C1": "Eleksen Ltd",
    "0016C2": "Avtec Systems Inc",
    "0016C3": "BA Systems Inc",
    "0016C4": "SiRF Technology, Inc.",
    "0016C5": "Shenzhen Xing Feng Industry Co.,Ltd",
    "0016C6": "North Atlantic Industries",
    "0016C7": "Cisco Systems",
    "0016C8": "Cisco Systems",
    "0016C9": "NAT Seattle, Inc.",
    "0016CA": "Nortel",
    "0016CB": "Apple Computer",
    "0016CC": "Xcute Mobile Corp.",
    "0016CD": "HIJI HIGH-TECH CO., LTD.",
    "0016CE": "Hon Hai Precision Ind. Co., Ltd.",
    "0016CF": "Hon Hai Precision Ind. Co., Ltd.",
    "0016D0": "ATech elektronika d.o.o.",
    "0016D1": "ZAT a.s.",
    "0016D2": "Caspian",
    "0016D3": "Wistron Corporation",
    "0016D4": "Compal Communications, Inc.",
    "0016D5": "Synccom Co., Ltd",
    "0016D6": "TDA Tech Pty Ltd",
    "0016D7": "Sunways AG",
    "0016D8": "Senea AB",
    "0016D9": "NINGBO BIRD CO.,LTD.",
    "0016DA": "Futronic Technology Co. Ltd.",
    "0016DB": "Samsung Electronics Co., Ltd.",
    "0016DC": "ARCHOS",
    "0016DD": "Gigabeam Corporation",
    "0016DE": "FAST Inc",
    "0016DF": "Lundinova AB",
    "0016E0": "3Com Ltd",
    "0016E1": "SiliconStor, Inc.",
    "0016E2": "American Fibertek, Inc.",
    "0016E3": "ASKEY COMPUTER CORP.",
    "0016E4": "VANGUARD SECURITY ENGINEERING CORP.",
    "0016E5": "FORDLEY DEVELOPMENT LIMITED",
    "0016E6": "GIGA-BYTE TECHNOLOGY CO.,LTD.",
    "0016E7": "Dynamix Promotions Limited",
    "0016E8": "Sigma Designs, Inc.",
    "0016E9": "Tiba Medical Inc",
    "0016EA": "Intel Corporation",
    "0016EB": "Intel Corporation",
    "0016EC": "Elitegroup Computer Systems Co., Ltd.",
    "0016ED": "Integrian, Inc.",
    "0016EE": "RoyalDigital Inc.",
    "0016EF": "Koko Fitness, Inc.",
    "0016F0": "Zermatt Systems, Inc",
    "0016F1": "OmniSense, LLC",
    "0016F2": "Dmobile System Co., Ltd.",
    "0016F3": "CAST Information Co., Ltd",
    "0016F4": "Eidicom Co., Ltd.",
    "0016F5": "Dalian Golden Hualu Digital Technology Co.,Ltd",
    "0016F6": "Video Products Group",
    "0016F7": "L-3 Communications, Electrodynamics, Inc.",
    "0016F8": "AVIQTECH TECHNOLOGY CO., LTD.",
    "0016F9": "CETRTA POT, d.o.o., Kranj",
    "0016FA": "ECI Telecom Ltd.",
    "0016FB": "SHENZHEN MTC CO.,LTD.",
    "0016FC": "TOHKEN CO.,LTD.",
    "0016FD": "Jaty Electronics",
    "0016FE": "Alps Electric Co., Ltd",
    "0016FF": "Wamin Optocomm Mfg Corp",
    "001700": "Motorola MDb",
    "001701": "KDE, Inc.",
    "001702": "Osung Midicom Co., Ltd",
    "001703": "MOSDAN Internation Co.,Ltd",
    "001704": "Shinco Electronics Group Co.,Ltd",
    "001705": "Methode Electronics",
    "001706": "Techfaith Wireless Communication Technology Limited.",
    "001707": "InGrid, Inc",
    "001708": "Hewlett Packard",
    "001709": "Exalt Communications",
    "00170A": "INEW DIGITAL COMPANY",
    "00170B": "Contela, Inc.",
    "00170C": "GeoSentric OYj",
    "00170D": "Dust Networks Inc.",
    "00170E": "Cisco Systems",
    "00170F": "Cisco Systems",
    "001710": "Casa Systems Inc.",
    "001711": "GE Healthcare Bio-Sciences AB",
    "001712": "ISCO International",
    "001713": "Tiger NetCom",
    "001714": "BR Controls Nederland bv",
    "001715": "Qstik",
    "001716": "Qno Technology Inc.",
    "001717": "Leica Geosystems AG",
    "001718": "Vansco Electronics Oy",
    "001719": "AudioCodes USA, Inc",
    "00171A": "Winegard Company",
    "00171B": "Innovation Lab Corp.",
    "00171C": "NT MicroSystems, Inc.",
    "00171D": "DIGIT",
    "00171E": "Theo Benning GmbH & Co. KG",
    "00171F": "IMV Corporation",
    "001720": "Image Sensing Systems, Inc.",
    "001721": "FITRE S.p.A.",
    "001722": "Hanazeder Electronic GmbH",
    "001723": "Summit Data Communications",
    "001724": "Studer Professional Audio GmbH",
    "001725": "Liquid Computing",
    "001726": "m2c Electronic Technology Ltd.",
    "001727": "Thermo Ramsey Italia s.r.l.",
    "001728": "Selex Communications",
    "001729": "Ubicod Co.LTD",
    "00172A": "Proware Technology Corp.",
    "00172B": "Global Technologies Inc.",
    "00172C": "TAEJIN INFOTECH",
    "00172D": "Axcen Photonics Corporation",
    "00172E": "FXC Inc.",
    "00172F": "NeuLion Incorporated",
    "001730": "Automation Electronics",
    "001731": "ASUSTek COMPUTER INC.",
    "001732": "Science-Technical Center \"RISSA\"",
    "001733": "neuf cegetel",
    "001734": "LGC Wireless Inc.",
    "001735": "PRIVATE",
    "001736": "iiTron Inc.",
    "001737": "Industrie Dial Face S.p.A.",
    "001738": "XIV",
    "001739": "Bright Headphone Electronics Company",
    "00173A": "Reach Systems Inc.",
    "00173B": "Arched Rock Corporation",
    "00173C": "Extreme Engineering Solutions",
    "00173D": "Neology",
    "00173E": "LeucotronEquipamentos Ltda.",
    "00173F": "Belkin Corporation",
    "001740": "Technologies Labtronix",
    "001741": "DEFIDEV",
    "001742": "FUJITSU LIMITED",
    "001743": "Deck Srl",
    "001744": "Araneo Ltd.",
    "001745": "INNOTZ CO., Ltd",
    "001746": "Freedom9 Inc.",
    "001747": "Trimble",
    "001748": "Neokoros Brasil Ltda",
    "001749": "HYUNDAE YONG-O-SA CO.,LTD",
    "00174A": "SOCOMEC",
    "00174B": "Nokia Danmark A/S",
    "00174C": "Millipore",
    "00174D": "DYNAMIC NETWORK FACTORY, INC.",
    "00174E": "Parama-tech Co.,Ltd.",
    "00174F": "iCatch Inc.",
    "001750": "GSI Group, MicroE Systems",
    "001751": "Online Corporation",
    "001752": "DAGS, Inc",
    "001753": "nFore Technology Inc.",
    "001754": "Arkino Corporation., Ltd",
    "001755": "GE Security",
    "001756": "Vinci Labs Oy",
    "001757": "RIX TECHNOLOGY LIMITED",
    "001758": "ThruVision Ltd",
    "001759": "Cisco Systems",
    "00175A": "Cisco Systems",
    "00175B": "ACS Solutions Switzerland Ltd.",
    "00175C": "SHARP CORPORATION",
    "00175D": "Dongseo system.",
    "00175E": "Anta Systems, Inc.",
    "00175F": "XENOLINK Communications Co., Ltd.",
    "001760": "Naito Densei Machida MFG.CO.,LTD",
    "001761": "ZKSoftware Inc.",
    "001762": "Solar Technology, Inc.",
    "001763": "Essentia S.p.A.",
    "001764": "ATMedia GmbH",
    "001765": "Nortel",
    "001766": "Accense Technology, Inc.",
    "001767": "Earforce AS",
    "001768": "Zinwave Ltd",
    "001769": "Cymphonix Corp",
    "00176A": "Avago Technologies",
    "00176B": "Kiyon, Inc.",
    "00176C": "Pivot3, Inc.",
    "00176D": "CORE CORPORATION",
    "00176E": "DUCATI SISTEMI",
    "00176F": "PAX Computer Technology(Shenzhen) Ltd.",
    "001770": "Arti Industrial Electronics Ltd.",
    "001771": "APD Communications Ltd",
    "001772": "ASTRO Strobel Kommunikationssysteme GmbH",
    "001773": "Laketune Technologies Co. Ltd",
    "001774": "Elesta GmbH",
    "001775": "TTE Germany GmbH",
    "001776": "Meso Scale Diagnostics, LLC",
    "001777": "Obsidian Research Corporation",
    "001778": "Central Music Co.",
    "001779": "QuickTel",
    "00177A": "ASSA ABLOY AB",
    "00177B": "Azalea Networks inc",
    "00177C": "D-Link India Ltd",
    "00177D": "IDT International Limited",
    "00177E": "Meshcom Technologies Inc.",
    "00177F": "Worldsmart Retech",
    "001780": "Applera Holding B.V. Singapore Operations",
    "001781": "Greystone Data System, Inc.",
    "001782": "LoBenn Inc.",
    "001783": "Texas Instruments",
    "001784": "Motorola Mobile Devices",
    "001785": "Sparr Electronics Ltd",
    "001786": "wisembed",
    "001787": "Brother, Brother & Sons ApS",
    "001788": "Philips Lighting BV",
    "001789": "Zenitron Corporation",
    "00178A": "DARTS TECHNOLOGIES CORP.",
    "00178B": "Teledyne Technologies Incorporated",
    "00178C": "Independent Witness, Inc",
    "00178D": "Checkpoint Systems, Inc.",
    "00178E": "Gunnebo Cash Automation AB",
    "00178F": "NINGBO YIDONG ELECTRONIC CO.,LTD.",
    "001790": "HYUNDAI DIGITECH Co, Ltd.",
    "001791": "LinTech GmbH",
    "001792": "Falcom Wireless Comunications Gmbh",
    "001793": "Tigi Corporation",
    "001794": "Cisco Systems",
    "001795": "Cisco Systems",
    "001796": "Rittmeyer AG",
    "001797": "Telsy Elettronica S.p.A.",
    "001798": "Azonic Technology Co., LTD",
    "001799": "SmarTire Systems Inc.",
    "00179A": "D-Link Corporation",
    "00179B": "Chant Sincere CO., LTD.",
    "00179C": "DEPRAG SCHULZ GMBH u. CO.",
    "00179D": "Kelman Limited",
    "00179E": "Sirit Inc",
    "00179F": "Apricorn",
    "0017A0": "RoboTech srl",
    "0017A1": "3soft inc.",
    "0017A2": "Camrivox Ltd.",
    "0017A3": "MIX s.r.l.",
    "0017A4": "Hewlett Packard",
    "0017A5": "TrendChip Technologies Corp.",
    "0017A6": "YOSIN ELECTRONICS CO., LTD.",
    "0017A7": "Mobile Computing Promotion Consortium",
    "0017A8": "EDM Corporation",
    "0017A9": "Sentivision",
    "0017AA": "elab-experience inc.",
    "0017AB": "Nintendo Co., Ltd.",
    "0017AC": "O'Neil Product Development Inc.",
    "0017AD": "AceNet Corporation",
    "0017AE": "GAI-Tronics",
    "0017AF": "Enermet",
    "0017B0": "Nokia Danmark A/S",
    "0017B1": "ACIST Medical Systems, Inc.",
    "0017B2": "SK Telesys",
    "0017B3": "Aftek Infosys Limited",
    "0017B4": "Remote Security Systems, LLC",
    "0017B5": "Peerless Systems Corporation",
    "0017B6": "Aquantia",
    "0017B7": "Tonze Technology Co.",
    "0017B8": "NOVATRON CO., LTD.",
    "0017B9": "Gambro Lundia AB",
    "0017BA": "SEDO CO., LTD.",
    "0017BB": "Syrinx Industrial Electronics",
    "0017BC": "Touchtunes Music Corporation",
    "0017BD": "Tibetsystem",
    "0017BE": "Tratec Telecom B.V.",
    "0017BF": "Coherent Research Limited",
    "0017C0": "PureTech Systems, Inc.",
    "0017C1": "CM Precision Technology LTD.",
    "0017C2": "Pirelli Broadband Solutions",
    "0017C3": "KTF Technologies Inc.",
    "0017C4": "Quanta Microsystems, INC.",
    "0017C5": "SonicWALL",
    "0017C6": "Cross Match Technologies Inc",
    "0017C7": "MARA Systems Consulting AB",
    "0017C8": "Kyocera Mita Corporation",
    "0017C9": "Samsung Electronics Co., Ltd.",
    "0017CA": "Qisda Corporation",
    "0017CB": "Juniper Networks",
    "0017CC": "Alcatel-Lucent",
    "0017CD": "CEC Wireless R&D Ltd.",
    "0017CE": "MB International Telecom Labs srl",
    "0017CF": "iMCA-GmbH",
    "0017D0": "Opticom Communications, LLC",
    "0017D1": "Nortel",
    "0017D2": "THINLINX PTY LTD",
    "0017D3": "Etymotic Research, Inc.",
    "0017D4": "Monsoon Multimedia, Inc",
    "0017D5": "Samsung Electronics Co., Ltd.",
    "0017D6": "Bluechips Microhouse Co.,Ltd.",
    "0017D7": "ION Geophysical Corporation Inc.",
    "0017D8": "Magnum Semiconductor, Inc.",
    "0017D9": "AAI Corporation",
    "0017DA": "Spans Logic",
    "0017DB": "CANKO TECHNOLOGIES INC.",
    "0017DC": "DAEMYUNG ZERO1",
    "0017DD": "Clipsal Australia",
    "0017DE": "Advantage Six Ltd",
    "0017DF": "Cisco Systems",
    "0017E0": "Cisco Systems",
    "0017E1": "DACOS Technologies Co., Ltd.",
    "0017E2": "Motorola Mobile Devices",
    "0017E3": "Texas Instruments",
    "0017E4": "Texas Instruments",
    "0017E5": "Texas Instruments",
    "0017E6": "Texas Instruments",
    "0017E7": "Texas Instruments",
    "0017E8": "Texas Instruments",
    "0017E9": "Texas Instruments",
    "0017EA": "Texas Instruments",
    "0017EB": "Texas Instruments",
    "0017EC": "Texas Instruments",
    "0017ED": "WooJooIT Ltd.",
    "0017EE": "Motorola CHS",
    "0017EF": "Blade Network Technologies, Inc.",
    "0017F0": "SZCOM Broadband Network Technology Co.,Ltd",
    "0017F1": "Renu Electronics Pvt Ltd",
    "0017F2": "Apple Computer",
    "0017F3": "M/A-COM Wireless Systems",
    "0017F4": "ZERON ALLIANCE",
    "0017F5": "LIG NEOPTEK",
    "0017F6": "Pyramid Meriden Inc.",
    "0017F7": "CEM Solutions Pvt Ltd",
    "0017F8": "Motech Industries Inc.",
    "0017F9": "Forcom Sp. z o.o.",
    "0017FA": "Microsoft Corporation",
    "0017FB": "FA",
    "0017FC": "Suprema Inc.",
    "0017FD": "Amulet Hotkey",
    "0017FE": "TALOS SYSTEM INC.",
    "0017FF": "PLAYLINE Co.,Ltd.",
    "001800": "UNIGRAND LTD",
    "001801": "Actiontec Electronics, Inc",
    "001802": "Alpha Networks Inc.",
    "001803": "ArcSoft Shanghai Co. LTD",
    "001804": "E-TEK DIGITAL TECHNOLOGY LIMITED",
    "001805": "Beijing InHand Networking Technology Co.,Ltd.",
    "001806": "Hokkei Industries Co., Ltd.",
    "001807": "Fanstel Corp.",
    "001808": "SightLogix, Inc.",
    "001809": "CRESYN",
    "00180A": "Meraki Networks, Inc.",
    "00180B": "Brilliant Telecommunications",
    "00180C": "Optelian Access Networks",
    "00180D": "Terabytes Server Storage Tech Corp",
    "00180E": "Avega Systems",
    "00180F": "Nokia Danmark A/S",
    "001810": "IPTrade S.A.",
    "001811": "Neuros Technology International, LLC.",
    "001812": "Beijing Xinwei Telecom Technology Co., Ltd.",
    "001813": "Sony Ericsson Mobile Communications",
    "001814": "Mitutoyo Corporation",
    "001815": "GZ Technologies, Inc.",
    "001816": "Ubixon Co., Ltd.",
    "001817": "D. E. Shaw Research, LLC",
    "001818": "Cisco Systems",
    "001819": "Cisco Systems",
    "00181A": "AVerMedia Technologies Inc.",
    "00181B": "TaiJin Metal Co., Ltd.",
    "00181C": "Exterity Limited",
    "00181D": "ASIA ELECTRONICS CO.,LTD",
    "00181E": "GDX Technologies Ltd.",
    "00181F": "Palmmicro Communications",
    "001820": "w5networks",
    "001821": "SINDORICOH",
    "001822": "CEC TELECOM CO.,LTD.",
    "001823": "Delta Electronics, Inc.",
    "001824": "Kimaldi Electronics, S.L.",
    "001825": "Wavion LTD",
    "001826": "Cale Access AB",
    "001827": "NEC PHILIPS UNIFIED SOLUTIONS NEDERLAND BV",
    "001828": "e2v technologies (UK) ltd.",
    "001829": "Gatsometer",
    "00182A": "Taiwan Video & Monitor",
    "00182B": "Softier",
    "00182C": "Ascend Networks, Inc.",
    "00182D": "Artec Group OÜ",
    "00182E": "XStreamHD, LLC",
    "00182F": "Texas Instruments",
    "001830": "Texas Instruments",
    "001831": "Texas Instruments",
    "001832": "Texas Instruments",
    "001833": "Texas Instruments",
    "001834": "Texas Instruments",
    "001835": "Thoratec / ITC",
    "001836": "Reliance Electric Limited",
    "001837": "Universal ABIT Co., Ltd.",
    "001838": "PanAccess Communications,Inc.",
    "001839": "Cisco-Linksys LLC",
    "00183A": "Westell Technologies",
    "00183B": "CENITS Co., Ltd.",
    "00183C": "Encore Software Limited",
    "00183D": "Vertex Link Corporation",
    "00183E": "Digilent, Inc",
    "00183F": "2Wire, Inc",
    "001840": "3 Phoenix, Inc.",
    "001841": "High Tech Computer Corp",
    "001842": "Nokia Danmark A/S",
    "001843": "Dawevision Ltd",
    "001844": "Heads Up Technologies, Inc.",
    "001845": "NPL Pulsar Ltd.",
    "001846": "Crypto S.A.",
    "001847": "AceNet Technology Inc.",
    "001848": "Vecima Networks Inc.",
    "001849": "Pigeon Point Systems",
    "00184A": "Catcher, Inc.",
    "00184B": "Las Vegas Gaming, Inc.",
    "00184C": "Bogen Communications",
    "00184D": "Netgear Inc.",
    "00184E": "Lianhe Technologies, Inc.",
    "00184F": "8 Ways Technology Corp.",
    "001850": "Secfone Kft",
    "001851": "SWsoft",
    "001852": "StorLink Semiconductors, Inc.",
    "001853": "Atera Networks LTD.",
    "001854": "Argard Co., Ltd",
    "001855": "Aeromaritime Systembau GmbH",
    "001856": "EyeFi, Inc",
    "001857": "Unilever R&D",
    "001858": "TagMaster AB",
    "001859": "Strawberry Linux Co.,Ltd.",
    "00185A": "uControl, Inc.",
    "00185B": "Network Chemistry, Inc",
    "00185C": "EDS Lab Pte Ltd",
    "00185D": "TAIGUEN TECHNOLOGY (SHEN-ZHEN) CO., LTD.",
    "00185E": "Nexterm Inc.",
    "00185F": "TAC Inc.",
    "001860": "SIM Technology Group Shanghai Simcom Ltd.,",
    "001861": "Ooma, Inc.",
    "001862": "Seagate Technology",
    "001863": "Veritech Electronics Limited",
    "001864": "Cybectec Inc.",
    "001865": "Siemens Medical Solutions Diagnostics Manufacturing Ltd",
    "001866": "Leutron Vision",
    "001867": "Evolution Robotics Retail",
    "001868": "Scientific Atlanta, A Cisco Company",
    "001869": "KINGJIM",
    "00186A": "Global Link Digital Technology Co,.LTD",
    "00186B": "Sambu Communics CO., LTD.",
    "00186C": "Neonode AB",
    "00186D": "Zhenjiang Sapphire Electronic Industry CO.",
    "00186E": "3Com Ltd",
    "00186F": "Setha Industria Eletronica LTDA",
    "001870": "E28 Shanghai Limited",
    "001871": "Hewlett Packard",
    "001872": "Expertise Engineering",
    "001873": "Cisco Systems",
    "001874": "Cisco Systems",
    "001875": "AnaCise Testnology Pte Ltd",
    "001876": "WowWee Ltd.",
    "001877": "Amplex A/S",
    "001878": "Mackware GmbH",
    "001879": "dSys",
    "00187A": "Wiremold",
    "00187B": "4NSYS Co. Ltd.",
    "00187C": "INTERCROSS, LLC",
    "00187D": "Armorlink shanghai Co. Ltd",
    "00187E": "RGB Spectrum",
    "00187F": "ZODIANET",
    "001880": "Mobilygen",
    "001881": "Buyang Electronics Industrial Co., Ltd",
    "001882": "Huawei Technologies Co., Ltd.",
    "001883": "FORMOSA21 INC.",
    "001884": "FON",
    "001885": "Avigilon Corporation",
    "001886": "EL-TECH, INC.",
    "001887": "Metasystem SpA",
    "001888": "GOTIVE a.s.",
    "001889": "WinNet Solutions Limited",
    "00188A": "Infinova LLC",
    "00188B": "Dell",
    "00188C": "Mobile Action Technology Inc.",
    "00188D": "Nokia Danmark A/S",
    "00188E": "Ekahau, Inc.",
    "00188F": "Montgomery Technology, Inc.",
    "001890": "RadioCOM, s.r.o.",
    "001891": "Zhongshan General K-mate Electronics Co., Ltd",
    "001892": "ads-tec GmbH",
    "001893": "SHENZHEN PHOTON BROADBAND TECHNOLOGY CO.,LTD",
    "001894": "zimocom",
    "001895": "Hansun Technologies Inc.",
    "001896": "Great Well Electronic LTD",
    "001897": "JESS-LINK PRODUCTS Co., LTD",
    "001898": "KINGSTATE ELECTRONICS CORPORATION",
    "001899": "ShenZhen jieshun Science&Technology Industry CO,LTD.",
    "00189A": "HANA Micron Inc.",
    "00189B": "Thomson Inc.",
    "00189C": "Weldex Corporation",
    "00189D": "Navcast Inc.",
    "00189E": "OMNIKEY GmbH.",
    "00189F": "Lenntek Corporation",
    "0018A0": "Cierma Ascenseurs",
    "0018A1": "Tiqit Computers, Inc.",
    "0018A2": "XIP Technology AB",
    "0018A3": "ZIPPY TECHNOLOGY CORP.",
    "0018A4": "Motorola Mobile Devices",
    "0018A5": "ADigit Technologies Corp.",
    "0018A6": "Persistent Systems, LLC",
    "0018A7": "Yoggie Security Systems LTD.",
    "0018A8": "AnNeal Technology Inc.",
    "0018A9": "Ethernet Direct Corporation",
    "0018AA": "Protec Fire Detection plc",
    "0018AB": "BEIJING LHWT MICROELECTRONICS INC.",
    "0018AC": "Shanghai Jiao Da HISYS Technology Co. Ltd.",
    "0018AD": "NIDEC SANKYO CORPORATION",
    "0018AE": "TVT CO.,LTD",
    "0018AF": "Samsung Electronics Co., Ltd.",
    "0018B0": "Nortel",
    "0018B1": "Blade Network Technologies",
    "0018B2": "ADEUNIS RF",
    "0018B3": "TEC WizHome Co., Ltd.",
    "0018B4": "Dawon Media Inc.",
    "0018B5": "Magna Carta",
    "0018B6": "S3C, Inc.",
    "0018B7": "D3 LED, LLC",
    "0018B8": "New Voice International AG",
    "0018B9": "Cisco Systems",
    "0018BA": "Cisco Systems",
    "0018BB": "Eliwell Controls srl",
    "0018BC": "ZAO NVP Bolid",
    "0018BD": "SHENZHEN DVBWORLD TECHNOLOGY CO., LTD.",
    "0018BE": "ANSA Corporation",
    "0018BF": "Essence Technology Solution, Inc.",
    "0018C0": "Motorola CHS",
    "0018C1": "Almitec Informática e Comércio Ltda.",
    "0018C2": "Firetide, Inc",
    "0018C3": "C&S Microwave",
    "0018C4": "Raba Technologies LLC",
    "0018C5": "Nokia Danmark A/S",
    "0018C6": "OPW Fuel Management Systems",
    "0018C7": "Real Time Automation",
    "0018C8": "ISONAS Inc.",
    "0018C9": "EOps Technology Limited",
    "0018CA": "Viprinet GmbH",
    "0018CB": "Tecobest Technology Limited",
    "0018CC": "AXIOHM SAS",
    "0018CD": "Erae Electronics Industry Co., Ltd",
    "0018CE": "Dreamtech Co., Ltd",
    "0018CF": "Baldor Electric Company",
    "0018D0": "AtRoad, A Trimble Company",
    "0018D1": "Siemens Home & Office Comm. Devices",
    "0018D2": "High-Gain Antennas LLC",
    "0018D3": "TEAMCAST",
    "0018D4": "Unified Display Interface SIG",
    "0018D5": "REIGNCOM",
    "0018D6": "Swirlnet A/S",
    "0018D7": "Javad Navigation Systems Inc.",
    "0018D8": "ARCH METER Corporation",
    "0018D9": "Santosha Internatonal, Inc",
    "0018DA": "AMBER wireless GmbH",
    "0018DB": "EPL Technology Ltd",
    "0018DC": "Prostar Co., Ltd.",
    "0018DD": "Silicondust Engineering Ltd",
    "0018DE": "Intel Corporation",
    "0018DF": "The Morey Corporation",
    "0018E0": "ANAVEO",
    "0018E1": "Verkerk Service Systemen",
    "0018E2": "Topdata Sistemas de Automacao Ltda",
    "0018E3": "Visualgate Systems, Inc.",
    "0018E4": "YIGUANG",
    "0018E5": "Adhoco AG",
    "0018E6": "Computer Hardware Design SIA",
    "0018E7": "Cameo Communications, INC.",
    "0018E8": "Hacetron Corporation",
    "0018E9": "Numata Corporation",
    "0018EA": "Alltec GmbH",
    "0018EB": "BroVis Wireless Networks",
    "0018EC": "Welding Technology Corporation",
    "0018ED": "Accutech Ultrasystems Co., Ltd.",
    "0018EE": "Videology Imaging Solutions, Inc.",
    "0018EF": "Escape Communications, Inc.",
    "0018F0": "JOYTOTO Co., Ltd.",
    "0018F1": "Chunichi Denshi Co.,LTD.",
    "0018F2": "Beijing Tianyu Communication Equipment Co., Ltd",
    "0018F3": "ASUSTek COMPUTER INC.",
    "0018F4": "EO TECHNICS Co., Ltd.",
    "0018F5": "Shenzhen Streaming Video Technology Company Limited",
    "0018F6": "Thomson Telecom Belgium",
    "0018F7": "Kameleon Technologies",
    "0018F8": "Cisco-Linksys LLC",
    "0018F9": "VVOND, Inc.",
    "0018FA": "Yushin Precision Equipment Co.,Ltd.",
    "0018FB": "Compro Technology",
    "0018FC": "Altec Electronic AG",
    "0018FD": "Optimal Technologies International Inc.",
    "0018FE": "Hewlett Packard",
    "0018FF": "PowerQuattro Co.",
    "001900": "Intelliverese - DBA Voicecom",
    "001901": "F1MEDIA",
    "001902": "Cambridge Consultants Ltd",
    "001903": "Bigfoot Networks Inc",
    "001904": "WB Electronics Sp. z o.o.",
    "001905": "SCHRACK Seconet AG",
    "001906": "Cisco Systems",
    "001907": "Cisco Systems",
    "001908": "Duaxes Corporation",
    "001909": "Devi A/S",
    "00190A": "HASWARE INC.",
    "00190B": "Southern Vision Systems, Inc.",
    "00190C": "Encore Electronics, Inc.",
    "00190D": "IEEE 1394c",
    "00190E": "Atech Technology Co., Ltd.",
    "00190F": "Advansus Corp.",
    "001910": "Knick Elektronische Messgeraete GmbH & Co. KG",
    "001911": "Just In Mobile Information Technologies (Shanghai) Co., Ltd.",
    "001912": "Welcat Inc",
    "001913": "Chuang-Yi Network Equipment Co.Ltd.",
    "001914": "Winix Co., Ltd",
    "001915": "TECOM Co., Ltd.",
    "001916": "PayTec AG",
    "001917": "Posiflex Inc.",
    "001918": "Interactive Wear AG",
    "001919": "ASTEL Inc.",
    "00191A": "IRLINK",
    "00191B": "Sputnik Engineering AG",
    "00191C": "Sensicast Systems",
    "00191D": "Nintendo Co.,Ltd.",
    "00191E": "Beyondwiz Co., Ltd.",
    "00191F": "Microlink communications Inc.",
    "001920": "KUME electric Co.,Ltd.",
    "001921": "Elitegroup Computer System Co.",
    "001922": "CM Comandos Lineares",
    "001923": "Phonex Korea Co., LTD.",
    "001924": "LBNL Engineering",
    "001925": "Intelicis Corporation",
    "001926": "BitsGen Co., Ltd.",
    "001927": "ImCoSys Ltd",
    "001928": "Siemens AG, Transportation Systems",
    "001929": "2M2B Montadora de Maquinas Bahia Brasil LTDA",
    "00192A": "Antiope Associates",
    "00192B": "Aclara RF Systems Inc.",
    "00192C": "Motorola Mobile Devices",
    "00192D": "Nokia Corporation",
    "00192E": "Spectral Instruments, Inc.",
    "00192F": "Cisco Systems",
    "001930": "Cisco Systems",
    "001931": "Balluff GmbH",
    "001932": "Gude Analog- und Digialsysteme GmbH",
    "001933": "Strix Systems, Inc.",
    "001934": "TRENDON TOUCH TECHNOLOGY CORP.",
    "001935": "Duerr Dental GmbH & Co. KG",
    "001936": "STERLITE OPTICAL TECHNOLOGIES LIMITED",
    "001937": "CommerceGuard AB",
    "001938": "UMB Communications Co., Ltd.",
    "001939": "Gigamips",
    "00193A": "OESOLUTIONS",
    "00193B": "Deliberant LLC",
    "00193C": "HighPoint Technologies Incorporated",
    "00193D": "GMC Guardian Mobility Corp.",
    "00193E": "PIRELLI BROADBAND SOLUTIONS",
    "00193F": "RDI technology(Shenzhen) Co.,LTD",
    "001940": "Rackable Systems",
    "001941": "Pitney Bowes, Inc",
    "001942": "ON SOFTWARE INTERNATIONAL LIMITED",
    "001943": "Belden",
    "001944": "Fossil Partners, L.P.",
    "001945": "Ten-Tec Inc.",
    "001946": "Cianet Industria e Comercio S/A",
    "001947": "Scientific Atlanta, A Cisco Company",
    "001948": "AireSpider Networks",
    "001949": "TENTEL COMTECH CO., LTD.",
    "00194A": "TESTO AG",
    "00194B": "SAGEM COMMUNICATION",
    "00194C": "Fujian Stelcom information & Technology CO.,Ltd",
    "00194D": "Avago Technologies Sdn Bhd",
    "00194E": "Ultra Electronics - TCS (Tactical Communication Systems)",
    "00194F": "Nokia Danmark A/S",
    "001950": "Harman Multimedia",
    "001951": "NETCONS, s.r.o.",
    "001952": "ACOGITO Co., Ltd",
    "001953": "Chainleader Communications Corp.",
    "001954": "Leaf Corporation.",
    "001955": "Cisco Systems",
    "001956": "Cisco Systems",
    "001957": "Saafnet Canada Inc.",
    "001958": "Bluetooth SIG, Inc.",
    "001959": "Staccato Communications Inc.",
    "00195A": "Jenaer Antriebstechnik GmbH",
    "00195B": "D-Link Corporation",
    "00195C": "Innotech Corporation",
    "00195D": "ShenZhen XinHuaTong Opto Electronics Co.,Ltd",
    "00195E": "Motorola CHS",
    "00195F": "Valemount Networks Corporation",
    "001960": "DoCoMo Systems, Inc.",
    "001961": "Blaupunkt GmbH",
    "001962": "Commerciant, LP",
    "001963": "Sony Ericsson Mobile Communications AB",
    "001964": "Doorking Inc.",
    "001965": "YuHua TelTech (ShangHai) Co., Ltd.",
    "001966": "Asiarock Technology Limited",
    "001967": "TELDAT Sp.J.",
    "001968": "Digital Video Networks(Shanghai) CO. LTD.",
    "001969": "Nortel",
    "00196A": "MikroM GmbH",
    "00196B": "Danpex Corporation",
    "00196C": "ETROVISION TECHNOLOGY",
    "00196D": "Raybit Systems Korea, Inc",
    "00196E": "Metacom (Pty) Ltd.",
    "00196F": "SensoPart GmbH",
    "001970": "Z-Com, Inc.",
    "001971": "Guangzhou Unicomp Technology Co.,Ltd",
    "001972": "Plexus (Xiamen) Co.,ltd",
    "001973": "Zeugma Systems",
    "001974": "AboCom Systems, Inc.",
    "001975": "Beijing Huisen networks technology Inc",
    "001976": "Xipher Technologies, LLC",
    "001977": "Aerohive Networks, Inc.",
    "001978": "Datum Systems, Inc.",
    "001979": "Nokia Danmark A/S",
    "00197A": "MAZeT GmbH",
    "00197B": "Picotest Corp.",
    "00197C": "Riedel Communications GmbH",
    "00197D": "Hon Hai Precision Ind. Co., Ltd",
    "00197E": "Hon Hai Precision Ind. Co., Ltd",
    "00197F": "PLANTRONICS, INC.",
    "001980": "Gridpoint Systems",
    "001981": "Vivox Inc",
    "001982": "SmarDTV",
    "001983": "CCT R&D Limited",
    "001984": "ESTIC Corporation",
    "001985": "IT Watchdogs, Inc",
    "001986": "Cheng Hongjian",
    "001987": "Panasonic Mobile Communications Co., Ltd.",
    "001988": "Wi2Wi, Inc",
    "001989": "Sonitrol Corporation",
    "00198A": "Northrop Grumman Systems Corp.",
    "00198B": "Novera Optics Korea, Inc.",
    "00198C": "iXSea",
    "00198D": "Ocean Optics, Inc.",
    "00198E": "Oticon A/S",
    "00198F": "Alcatel Bell N.V.",
    "001990": "ELM DATA Co., Ltd.",
    "001991": "avinfo",
    "001992": "Bluesocket, Inc",
    "001993": "Changshu Switchgear MFG. Co.,Ltd. (Former Changshu Switchgea",
    "001994": "Jorjin Technologies Inc.",
    "001995": "Jurong Hi-Tech (Suzhou)Co.ltd",
    "001996": "TurboChef Technologies Inc.",
    "001997": "Soft Device Sdn Bhd",
    "001998": "SATO CORPORATION",
    "001999": "Fujitsu Siemens Computers",
    "00199A": "EDO-EVI",
    "00199B": "Diversified Technical Systems, Inc.",
    "00199C": "CTRING",
    "00199D": "VIZIO, Inc.",
    "00199E": "SHOWADENSHI ELECTRONICS,INC.",
    "00199F": "DKT A/S",
    "0019A0": "NIHON DATA SYSTENS, INC.",
    "0019A1": "LG INFORMATION & COMM.",
    "0019A2": "ORION TELE-EQUIPMENTS PVT LTD",
    "0019A3": "asteel electronique atlantique",
    "0019A4": "Austar Technology (hang zhou) Co.,Ltd",
    "0019A5": "RadarFind Corporation",
    "0019A6": "Motorola CHS",
    "0019A7": "ITU-T",
    "0019A8": "WiQuest Communications",
    "0019A9": "Cisco Systems",
    "0019AA": "Cisco Systems",
    "0019AB": "Raycom CO ., LTD",
    "0019AC": "GSP SYSTEMS Inc.",
    "0019AD": "BOBST SA",
    "0019AE": "Hopling Technologies b.v.",
    "0019AF": "Rigol Technologies, Inc.",
    "0019B0": "HanYang System",
    "0019B1": "Arrow7 Corporation",
    "0019B2": "XYnetsoft Co.,Ltd",
    "0019B3": "Stanford Research Systems",
    "0019B4": "VideoCast Ltd.",
    "0019B5": "Famar Fueguina S.A.",
    "0019B6": "Euro Emme s.r.l.",
    "0019B7": "Nokia Danmark A/S",
    "0019B8": "Boundary Devices",
    "0019B9": "Dell Inc.",
    "0019BA": "Paradox Security Systems Ltd",
    "0019BB": "Hewlett Packard",
    "0019BC": "ELECTRO CHANCE SRL",
    "0019BD": "New Media Life",
    "0019BE": "Altai Technologies Limited",
    "0019BF": "Citiway technology Co.,ltd",
    "0019C0": "Motorola Mobile Devices",
    "0019C1": "Alps Electric Co., Ltd",
    "0019C2": "Equustek Solutions, Inc.",
    "0019C3": "Qualitrol",
    "0019C4": "Infocrypt Inc.",
    "0019C5": "SONY Computer Entertainment inc,",
    "0019C6": "ZTE Corporation",
    "0019C7": "Cambridge Industries(Group) Co.,Ltd.",
    "0019C8": "AnyDATA Corporation",
    "0019C9": "S&C ELECTRIC COMPANY",
    "0019CA": "Broadata Communications, Inc",
    "0019CB": "ZyXEL Communications Corporation",
    "0019CC": "RCG (HK) Ltd",
    "0019CD": "Chengdu ethercom information technology Ltd.",
    "0019CE": "Progressive Gaming International",
    "0019CF": "SALICRU, S.A.",
    "0019D0": "Cathexis",
    "0019D1": "Intel Corporation",
    "0019D2": "Intel Corporation",
    "0019D3": "TRAK Microwave",
    "0019D4": "ICX Technologies",
    "0019D5": "IP Innovations, Inc.",
    "0019D6": "LS Cable Ltd.",
    "0019D7": "FORTUNETEK CO., LTD",
    "0019D8": "MAXFOR",
    "0019D9": "Zeutschel GmbH",
    "0019DA": "Welltrans O&E Technology Co. , Ltd.",
    "0019DB": "MICRO-STAR INTERNATIONAL CO., LTD.",
    "0019DC": "ENENSYS Technologies",
    "0019DD": "FEI-Zyfer, Inc.",
    "0019DE": "MOBITEK",
    "0019DF": "Thomson Inc.",
    "0019E0": "TP-LINK Technologies Co., Ltd.",
    "0019E1": "Nortel",
    "0019E2": "Juniper Networks",
    "0019E3": "Apple Computers",
    "0019E4": "2Wire, Inc",
    "0019E5": "Lynx Studio Technology, Inc.",
    "0019E6": "TOYO MEDIC CO.,LTD.",
    "0019E7": "Cisco Systems",
    "0019E8": "Cisco Systems",
    "0019E9": "S-Information Technolgy, Co., Ltd.",
    "0019EA": "TeraMage Technologies Co., Ltd.",
    "0019EB": "Pyronix Ltd",
    "0019EC": "Sagamore Systems, Inc.",
    "0019ED": "Axesstel Inc.",
    "0019EE": "CARLO GAVAZZI CONTROLS SPA-Controls Division",
    "0019EF": "SHENZHEN LINNKING ELECTRONICS CO.,LTD",
    "0019F0": "UNIONMAN TECHNOLOGY CO.,LTD",
    "0019F1": "Star Communication Network Technology Co.,Ltd",
    "0019F2": "Teradyne K.K.",
    "0019F3": "Telematrix, Inc",
    "0019F4": "Convergens Oy Ltd",
    "0019F5": "Imagination Technologies Ltd",
    "0019F6": "Acconet (PTE) Ltd",
    "0019F7": "Onset Computer Corporation",
    "0019F8": "Embedded Systems Design, Inc.",
    "0019F9": "Lambda",
    "0019FA": "Cable Vision Electronics CO., LTD.",
    "0019FB": "AMSTRAD PLC",
    "0019FC": "PT. Ufoakses Sukses Luarbiasa",
    "0019FD": "Nintendo Co., Ltd.",
    "0019FE": "SHENZHEN SEECOMM TECHNOLOGY CO.,LTD.",
    "0019FF": "Finnzymes",
    "001A00": "MATRIX INC.",
    "001A01": "Smiths Medical",
    "001A02": "SECURE CARE PRODUCTS, INC",
    "001A03": "Angel Electronics Co., Ltd.",
    "001A04": "Interay Solutions BV",
    "001A05": "OPTIBASE LTD",
    "001A06": "OpVista, Inc.",
    "001A07": "Arecont Vision",
    "001A08": "Dalman Technical Services",
    "001A09": "Wayfarer Transit Systems Ltd",
    "001A0A": "Adaptive Micro-Ware Inc.",
    "001A0B": "BONA TECHNOLOGY INC.",
    "001A0C": "Swe-Dish Satellite Systems AB",
    "001A0D": "HandHeld entertainment, Inc.",
    "001A0E": "Cheng Uei Precision Industry Co.,Ltd",
    "001A0F": "Sistemas Avanzados de Control, S.A.",
    "001A10": "LUCENT TRANS ELECTRONICS CO.,LTD",
    "001A11": "Google Inc.",
    "001A12": "Essilor",
    "001A13": "Wanlida Group Co., LTD",
    "001A14": "Xin Hua Control Engineering Co.,Ltd.",
    "001A15": "gemalto e-Payment",
    "001A16": "Nokia Danmark A/S",
    "001A17": "Teak Technologies, Inc.",
    "001A18": "Advanced Simulation Technology inc.",
    "001A19": "Computer Engineering Limited",
    "001A1A": "Gentex Corporation/Electro-Acoustic Products",
    "001A1B": "Motorola Mobile Devices",
    "001A1C": "GT&T Engineering Pte Ltd",
    "001A1D": "PChome Online Inc.",
    "001A1E": "Aruba Networks",
    "001A1F": "Coastal Environmental Systems",
    "001A20": "CMOTECH Co. Ltd.",
    "001A21": "Indac B.V.",
    "001A22": "eq-3 GmbH",
    "001A23": "Ice Qube, Inc",
    "001A24": "Galaxy Telecom Technologies Ltd",
    "001A25": "DELTA DORE",
    "001A26": "Deltanode Solutions AB",
    "001A27": "Ubistar",
    "001A28": "ASWT Co., LTD. Taiwan Branch H.K.",
    "001A29": "Techsonic Industries d/b/a Humminbird",
    "001A2A": "Arcadyan Technology Corporation",
    "001A2B": "Ayecom Technology Co., Ltd.",
    "001A2C": "SATEC Co.,LTD",
    "001A2D": "The Navvo Group",
    "001A2E": "Ziova Coporation",
    "001A2F": "Cisco Systems",
    "001A30": "Cisco Systems",
    "001A31": "SCAN COIN Industries AB",
    "001A32": "ACTIVA MULTIMEDIA",
    "001A33": "ASI Communications, Inc.",
    "001A34": "Konka Group Co., Ltd.",
    "001A35": "BARTEC GmbH",
    "001A36": "Aipermon GmbH & Co. KG",
    "001A37": "Lear Corporation",
    "001A38": "Sanmina-SCI",
    "001A39": "Merten GmbH&CoKG",
    "001A3A": "Dongahelecomm",
    "001A3B": "Doah Elecom Inc.",
    "001A3C": "Technowave Ltd.",
    "001A3D": "Ajin Vision Co.,Ltd",
    "001A3E": "Faster Technology LLC",
    "001A3F": "intelbras",
    "001A40": "A-FOUR TECH CO., LTD.",
    "001A41": "INOCOVA Co.,Ltd",
    "001A42": "Techcity Technology co., Ltd.",
    "001A43": "Logical Link Communications",
    "001A44": "JWTrading Co., Ltd",
    "001A45": "GN Netcom as",
    "001A46": "Digital Multimedia Technology Co., Ltd",
    "001A47": "Agami Systems, Inc.",
    "001A48": "Takacom Corporation",
    "001A49": "Micro Vision Co.,LTD",
    "001A4A": "Qumranet Inc.",
    "001A4B": "Hewlett Packard",
    "001A4C": "Crossbow Technology, Inc",
    "001A4D": "GIGA-BYTE TECHNOLOGY CO.,LTD.",
    "001A4E": "NTI AG / LinMot",
    "001A4F": "AVM GmbH",
    "001A50": "PheeNet Technology Corp.",
    "001A51": "Alfred Mann Foundation",
    "001A52": "Meshlinx Wireless Inc.",
    "001A53": "Zylaya",
    "001A54": "Hip Shing Electronics Ltd.",
    "001A55": "ACA-Digital Corporation",
    "001A56": "ViewTel Co,. Ltd.",
    "001A57": "Matrix Design Group, LLC",
    "001A58": "Celectronic GmbH",
    "001A59": "Ircona",
    "001A5A": "Korea Electric Power Data Network (KDN) Co., Ltd",
    "001A5B": "NetCare Service Co., Ltd.",
    "001A5C": "Euchner GmbH+Co. KG",
    "001A5D": "Mobinnova Corp.",
    "001A5E": "Thincom Technology Co.,Ltd",
    "001A5F": "KitWorks.fi Ltd.",
    "001A60": "Wave Electronics Co.,Ltd.",
    "001A61": "PacStar Corp.",
    "001A62": "Data Robotics, Incorporated",
    "001A63": "Elster Electricity, LLC",
    "001A64": "IBM Corp.",
    "001A65": "Seluxit",
    "001A66": "Motorola CHS",
    "001A67": "Infinite QL Sdn Bhd",
    "001A68": "Weltec Enterprise Co., Ltd.",
    "001A69": "Wuhan Yangtze Optical Technology CO.,Ltd.",
    "001A6A": "Tranzas, Inc.",
    "001A6B": "USI",
    "001A6C": "Cisco Systems",
    "001A6D": "Cisco Systems",
    "001A6E": "Impro Technologies",
    "001A6F": "MI.TEL s.r.l.",
    "001A70": "Cisco-Linksys, LLC",
    "001A71": "Diostech Co., Ltd.",
    "001A72": "Mosart Semiconductor Corp.",
    "001A73": "Gemtek Technology Co., Ltd.",
    "001A74": "Procare International Co",
    "001A75": "Sony Ericsson Mobile Communications",
    "001A76": "SDT information Technology Co.,LTD.",
    "001A77": "Motorola Mobile Devices",
    "001A78": "ubtos",
    "001A79": "TELECOMUNICATION TECHNOLOGIES LTD.",
    "001A7A": "Lismore Instruments Limited",
    "001A7B": "Teleco, Inc.",
    "001A7C": "Hirschmann Automation and Control B.V.",
    "001A7D": "cyber-blue(HK)Ltd",
    "001A7E": "LN Srithai Comm Ltd.",
    "001A7F": "GCI Science&Technology Co.,Ltd.",
    "001A80": "Sony Corporation",
    "001A81": "Zelax",
    "001A82": "PROBA Building Automation Co.,LTD",
    "001A83": "Pegasus Technologies Inc.",
    "001A84": "V One Multimedia Pte Ltd",
    "001A85": "NV Michel Van de Wiele",
    "001A86": "AdvancedIO Systems Inc",
    "001A87": "Canhold International Limited",
    "001A88": "Venergy,Co,Ltd",
    "001A89": "Nokia Danmark A/S",
    "001A8A": "Samsung Electronics Co., Ltd.",
    "001A8B": "CHUNIL ELECTRIC IND., CO.",
    "001A8C": "Astaro AG",
    "001A8D": "AVECS Bergen GmbH",
    "001A8E": "3Way Networks Ltd",
    "001A8F": "Nortel",
    "001A90": "Trópico Sistemas e Telecomunicações da Amazônia LTDA.",
    "001A91": "FusionDynamic Ltd.",
    "001A92": "ASUSTek COMPUTER INC.",
    "001A93": "ERCO Leuchten GmbH",
    "001A94": "Votronic GmbH",
    "001A95": "Hisense Mobile Communications Technoligy Co.,Ltd.",
    "001A96": "ECLER S.A.",
    "001A97": "fitivision technology Inc.",
    "001A98": "Asotel Communication Limited Taiwan Branch",
    "001A99": "Smarty (HZ) Information Electronics Co., Ltd",
    "001A9A": "Skyworth Digital technology(shenzhen)co.ltd.",
    "001A9B": "ADEC & Parter AG",
    "001A9C": "RightHand Technologies, Inc.",
    "001A9D": "Skipper Wireless, Inc.",
    "001A9E": "ICON Digital International Limited",
    "001A9F": "A-Link Europe Ltd",
    "001AA0": "Dell Inc",
    "001AA1": "Cisco Systems",
    "001AA2": "Cisco Systems",
    "001AA3": "DELORME",
    "001AA4": "Future University-Hakodate",
    "001AA5": "BRN Phoenix",
    "001AA6": "Telefunken Radio Communication Systems GmbH &CO.KG",
    "001AA7": "Torian Wireless",
    "001AA8": "Mamiya Digital Imaging Co., Ltd.",
    "001AA9": "FUJIAN STAR-NET COMMUNICATION CO.,LTD",
    "001AAA": "Analogic Corp.",
    "001AAB": "eWings s.r.l.",
    "001AAC": "Corelatus AB",
    "001AAD": "Motorola CHS",
    "001AAE": "Savant Systems LLC",
    "001AAF": "BLUSENS TECHNOLOGY",
    "001AB0": "Signal Networks Pvt. Ltd.,",
    "001AB1": "Asia Pacific Satellite Industries Co., Ltd.",
    "001AB2": "Cyber Solutions Inc.",
    "001AB3": "VISIONITE INC.",
    "001AB4": "FFEI Ltd.",
    "001AB5": "Home Network System",
    "001AB6": "Luminary Micro Inc",
    "001AB7": "Ethos Networks LTD.",
    "001AB8": "Anseri Corporation",
    "001AB9": "PMC",
    "001ABA": "Caton Overseas Limited",
    "001ABB": "Fontal Technology Incorporation",
    "001ABC": "U4EA Technologies Ltd",
    "001ABD": "Impatica Inc.",
    "001ABE": "COMPUTER HI-TECH INC.",
    "001ABF": "TRUMPF Laser Marking Systems AG",
    "001AC0": "JOYBIEN TECHNOLOGIES CO., LTD.",
    "001AC1": "3Com Ltd",
    "001AC2": "YEC Co.,Ltd.",
    "001AC3": "Scientific-Atlanta, Inc",
    "001AC4": "2Wire, Inc",
    "001AC5": "BreakingPoint Systems, Inc.",
    "001AC6": "Micro Control Designs",
    "001AC7": "UNIPOINT",
    "001AC8": "ISL (Instrumentation Scientifique de Laboratoire)",
    "001AC9": "SUZUKEN CO.,LTD",
    "001ACA": "Tilera Corporation",
    "001ACB": "Autocom Products Ltd",
    "001ACC": "Celestial Semiconductor, Ltd",
    "001ACD": "Tidel Engineering LP",
    "001ACE": "YUPITERU CORPORATION",
    "001ACF": "C.T. ELETTRONICA",
    "001AD0": "Siemens Schweiz AG",
    "001AD1": "FARGO CO., LTD.",
    "001AD2": "Eletronica Nitron Ltda",
    "001AD3": "Vamp Ltd.",
    "001AD4": "iPOX Technology Co., Ltd.",
    "001AD5": "KMC CHAIN INDUSTRIAL CO., LTD.",
    "001AD6": "JIAGNSU AETNA ELECTRIC CO.,LTD",
    "001AD7": "Christie Digital Systems, Inc.",
    "001AD8": "AlsterAero GmbH",
    "001AD9": "International Broadband Electric Communications, Inc.",
    "001ADA": "Biz-2-Me Inc.",
    "001ADB": "Motorola Mobile Devices",
    "001ADC": "Nokia Danmark A/S",
    "001ADD": "PePWave Ltd",
    "001ADE": "Motorola CHS",
    "001ADF": "Interactivetv Pty Limited",
    "001AE0": "Mythology Tech Express Inc.",
    "001AE1": "EDGE ACCESS INC",
    "001AE2": "Cisco Systems",
    "001AE3": "Cisco Systems",
    "001AE4": "Liposonix Inc,",
    "001AE5": "Mvox Technologies Inc.",
    "001AE6": "Atlanta Advanced Communications Holdings Limited",
    "001AE7": "Aztek Networks, Inc.",
    "001AE8": "Siemens Enterprise Communications GmbH & Co. KG",
    "001AE9": "Nintendo Co., Ltd.",
    "001AEA": "Radio Terminal Systems Pty Ltd",
    "001AEB": "Allied Telesis K.K.",
    "001AEC": "Keumbee Electronics Co.,Ltd.",
    "001AED": "INCOTEC GmbH",
    "001AEE": "Shenztech Ltd",
    "001AEF": "Loopcomm Technology, Inc.",
    "001AF0": "Alcatel - IPD",
    "001AF1": "Embedded Artists AB",
    "001AF2": "Dynavisions Schweiz AG",
    "001AF3": "Samyoung Electronics",
    "001AF4": "Handreamnet",
    "001AF5": "PENTAONE. CO., LTD.",
    "001AF6": "Woven Systems, Inc.",
    "001AF7": "dataschalt e+a GmbH",
    "001AF8": "Copley Controls Corporation",
    "001AF9": "AeroVIronment (AV Inc)",
    "001AFA": "Welch Allyn, Inc.",
    "001AFB": "Joby Inc.",
    "001AFC": "ModusLink Corporation",
    "001AFD": "EVOLIS",
    "001AFE": "SOFACREAL",
    "001AFF": "Wizyoung Tech.",
    "001B00": "Neopost Technologies",
    "001B01": "Applied Radio Technologies",
    "001B02": "ED Co.Ltd",
    "001B03": "Action Technology (SZ) Co., Ltd",
    "001B04": "Affinity International S.p.a",
    "001B05": "Young Media Concepts GmbH",
    "001B06": "Ateliers R. LAUMONIER",
    "001B07": "Mendocino Software",
    "001B08": "Danfoss Drives A/S",
    "001B09": "Matrix Telecom Pvt. Ltd.",
    "001B0A": "Intelligent Distributed Controls Ltd",
    "001B0B": "Phidgets Inc.",
    "001B0C": "Cisco Systems",
    "001B0D": "Cisco Systems",
    "001B0E": "InoTec GmbH Organisationssysteme",
    "001B0F": "Petratec",
    "001B10": "ShenZhen Kang Hui Technology Co.,ltd",
    "001B11": "D-Link Corporation",
    "001B12": "Apprion",
    "001B13": "Icron Technologies Corporation",
    "001B14": "Carex Lighting Equipment Factory",
    "001B15": "Voxtel, Inc.",
    "001B16": "Celtro Ltd.",
    "001B17": "Palo Alto Networks",
    "001B18": "Tsuken Electric Ind. Co.,Ltd",
    "001B19": "IEEE 1588 Standard",
    "001B1A": "e-trees Japan, Inc.",
    "001B1B": "Siemens AG, A&D AS EWK PU1",
    "001B1C": "Coherent",
    "001B1D": "Phoenix International Co., Ltd",
    "001B1E": "HART Communication Foundation",
    "001B1F": "DELTA - Danish Electronics, Light & Acoustics",
    "001B20": "TPine Technology",
    "001B21": "Intel Corporate",
    "001B22": "Palit Microsystems ( H.K.) Ltd.",
    "001B23": "SimpleComTools",
    "001B24": "Quanta Computer Inc.",
    "001B25": "Nortel",
    "001B26": "RON-Telecom ZAO",
    "001B27": "Merlin CSI",
    "001B28": "POLYGON, JSC",
    "001B29": "Avantis.Co.,Ltd",
    "001B2A": "Cisco Systems",
    "001B2B": "Cisco Systems",
    "001B2C": "ATRON electronic GmbH",
    "001B2D": "PRIVATE",
    "001B2E": "Sinkyo Electron Inc",
    "001B2F": "NETGEAR Inc.",
    "001B30": "Solitech Inc.",
    "001B31": "Neural Image. Co. Ltd.",
    "001B32": "QLogic Corporation",
    "001B33": "Nokia Danmark A/S",
    "001B34": "Focus System Inc.",
    "001B35": "ChongQing JINOU Science & Technology Development CO.,Ltd",
    "001B36": "Tsubata Engineering Co.,Ltd. (Head Office)",
    "001B37": "Computec Oy",
    "001B38": "COMPAL INFORMATION (KUNSHAN) CO., LTD.",
    "001B39": "Proxicast",
    "001B3A": "SIMS Corp.",
    "001B3B": "Yi-Qing CO., LTD",
    "001B3C": "Software Technologies Group,Inc.",
    "001B3D": "EuroTel Spa",
    "001B3E": "Curtis, Inc.",
    "001B3F": "ProCurve Networking by HP",
    "001B40": "Network Automation mxc AB",
    "001B41": "General Infinity Co.,Ltd.",
    "001B42": "Wise & Blue",
    "001B43": "Beijing DG Telecommunications equipment Co.,Ltd",
    "001B44": "SanDisk Corporation",
    "001B45": "ABB AS, Division Automation Products",
    "001B46": "Blueone Technology Co.,Ltd",
    "001B47": "Futarque A/S",
    "001B48": "Shenzhen Lantech Electronics Co., Ltd.",
    "001B49": "Roberts Radio limited",
    "001B4A": "W&W Communications, Inc.",
    "001B4B": "SANION Co., Ltd.",
    "001B4C": "Signtech",
    "001B4D": "Areca Technology Corporation",
    "001B4E": "Navman New Zealand",
    "001B4F": "Avaya Inc.",
    "001B50": "Nizhny Novgorod Factory named after M.Frunze, FSUE (NZiF)",
    "001B51": "Vector Technology Corp.",
    "001B52": "Motorola Mobile Devices",
    "001B53": "Cisco Systems",
    "001B54": "Cisco Systems",
    "001B55": "Hurco Automation Ltd.",
    "001B56": "Tehuti Networks Ltd.",
    "001B57": "SEMINDIA SYSTEMS PRIVATE LIMITED",
    "001B58": "ACE CAD Enterprise Co., Ltd.",
    "001B59": "Sony Ericsson Mobile Communications AB",
    "001B5A": "Apollo Imaging Technologies, Inc.",
    "001B5B": "2Wire, Inc.",
    "001B5C": "Azuretec Co., Ltd.",
    "001B5D": "Vololink Pty Ltd",
    "001B5E": "BPL Limited",
    "001B5F": "Alien Technology",
    "001B60": "NAVIGON AG",
    "001B61": "Digital Acoustics, LLC",
    "001B62": "JHT Optoelectronics Co.,Ltd.",
    "001B63": "Apple Inc.",
    "001B64": "IsaacLandKorea Co., Ltd,",
    "001B65": "China Gridcom Co., Ltd",
    "001B66": "Sennheiser electronic GmbH & Co. KG",
    "001B67": "Ubiquisys Ltd",
    "001B68": "Modnnet Co., Ltd",
    "001B69": "Equaline Corporation",
    "001B6A": "Powerwave UK Ltd",
    "001B6B": "Swyx Solutions AG",
    "001B6C": "LookX Digital Media BV",
    "001B6D": "Midtronics, Inc.",
    "001B6E": "Anue Systems, Inc.",
    "001B6F": "Teletrak Ltd",
    "001B70": "IRI Ubiteq, INC.",
    "001B71": "Telular Corp.",
    "001B72": "Sicep s.p.a.",
    "001B73": "DTL Broadcast Ltd",
    "001B74": "MiraLink Corporation",
    "001B75": "Hypermedia Systems",
    "001B76": "Ripcode, Inc.",
    "001B77": "Intel Corporate",
    "001B78": "Hewlett Packard",
    "001B79": "FAIVELEY TRANSPORT",
    "001B7A": "Nintendo Co., Ltd.",
    "001B7B": "The Tintometer Ltd",
    "001B7C": "A & R Cambridge",
    "001B7D": "CXR Anderson Jacobson",
    "001B7E": "Beckmann GmbH",
    "001B7F": "TMN Technologies Telecomunicacoes Ltda",
    "001B80": "LORD Corporation",
    "001B81": "DATAQ Instruments, Inc.",
    "001B82": "Taiwan Semiconductor Co., Ltd.",
    "001B83": "Finsoft Ltd",
    "001B84": "Scan Engineering Telecom",
    "001B85": "MAN Diesel A/S",
    "001B86": "Bosch Access Systems GmbH",
    "001B87": "Deepsound Tech. Co., Ltd",
    "001B88": "Divinet Access Technologies Ltd",
    "001B89": "EMZA Visual Sense Ltd.",
    "001B8A": "2M Electronic A/S",
    "001B8B": "NEC AccessTechnica,Ltd.",
    "001B8C": "JMicron Technology Corp.",
    "001B8D": "Electronic Computer Systems, Inc.",
    "001B8E": "Hulu Sweden AB",
    "001B8F": "Cisco Systems",
    "001B90": "Cisco Systems",
    "001B91": "EFKON AG",
    "001B92": "l-acoustics",
    "001B93": "JC Decaux SA DNT",
    "001B94": "T.E.M.A. S.p.A.",
    "001B95": "VIDEO SYSTEMS SRL",
    "001B96": "Snif Labs, Inc.",
    "001B97": "Violin Technologies",
    "001B98": "Samsung Electronics Co., Ltd.",
    "001B99": "KS System GmbH",
    "001B9A": "Apollo Fire Detectors Ltd",
    "001B9B": "Hose-McCann Communications",
    "001B9C": "SATEL sp. z o.o.",
    "001B9D": "Novus Security Sp. z o.o.",
    "001B9E": "ASKEY COMPUTER CORP",
    "001B9F": "Calyptech Pty Ltd",
    "001BA0": "Awox",
    "001BA1": "Åmic AB",
    "001BA2": "IDS Imaging Development Systems GmbH",
    "001BA3": "Flexit Group GmbH",
    "001BA4": "S.A.E Afikim",
    "001BA5": "MyungMin Systems, Inc.",
    "001BA6": "intotech inc.",
    "001BA7": "Lorica Solutions",
    "001BA8": "UBI&MOBI,.Inc",
    "001BA9": "BROTHER INDUSTRIES, LTD.",
    "001BAA": "XenICs nv",
    "001BAB": "Telchemy, Incorporated",
    "001BAC": "Curtiss Wright Controls Embedded Computing",
    "001BAD": "iControl Incorporated",
    "001BAE": "Micro Control Systems, Inc",
    "001BAF": "Nokia Danmark A/S",
    "001BB0": "BHARAT ELECTRONICS",
    "001BB1": "Wistron Neweb Corp.",
    "001BB2": "Intellect International NV",
    "001BB3": "Condalo GmbH",
    "001BB4": "Airvod Limited",
    "001BB5": "Cherry GmbH",
    "001BB6": "Bird Electronic Corp.",
    "001BB7": "Alta Heights Technology Corp.",
    "001BB8": "BLUEWAY ELECTRONIC CO;LTD",
    "001BB9": "Elitegroup Computer System Co.",
    "001BBA": "Nortel",
    "001BBB": "RFTech Co.,Ltd",
    "001BBC": "Silver Peak Systems, Inc.",
    "001BBD": "FMC Kongsberg Subsea AS",
    "001BBE": "ICOP Digital",
    "001BBF": "SAGEM COMMUNICATION",
    "001BC0": "Juniper Networks",
    "001BC1": "HOLUX Technology, Inc.",
    "001BC2": "Integrated Control Technology Limitied",
    "001BC3": "Mobisolution Co.,Ltd",
    "001BC4": "Ultratec, Inc.",
    "001BC5": "IEEE Registration Authority",
    "001BC6": "Strato Rechenzentrum AG",
    "001BC7": "StarVedia Technology Inc.",
    "001BC8": "MIURA CO.,LTD",
    "001BC9": "FSN DISPLAY INC",
    "001BCA": "Beijing Run Technology LTD. Company",
    "001BCB": "PEMPEK SYSTEMS PTY LTD",
    "001BCC": "KINGTEK CCTV ALLIANCE CO., LTD.",
    "001BCD": "DAVISCOMMS (S) PTE LTD",
    "001BCE": "Measurement Devices Ltd",
    "001BCF": "Dataupia Corporation",
    "001BD0": "IDENTEC SOLUTIONS",
    "001BD1": "SOGESTMATIC",
    "001BD2": "ULTRA-X ASIA PACIFIC Inc.",
    "001BD3": "Matsushita Electric Panasonic AVC",
    "001BD4": "Cisco Systems",
    "001BD5": "Cisco Systems",
    "001BD6": "Kelvin Hughes Ltd",
    "001BD7": "Scientific Atlanta, A Cisco Company",
    "001BD8": "DVTel, Inc.",
    "001BD9": "Edgewater Computer Systems",
    "001BDA": "UTStarcom Inc",
    "001BDB": "Valeo VECS",
    "001BDC": "Vencer Co., Ltd.",
    "001BDD": "Motorola CHS",
    "001BDE": "Renkus-Heinz, Inc.",
    "001BDF": "Iskra MIS",
    "001BE0": "TELENOT ELECTRONIC GmbH",
    "001BE1": "ViaLogy",
    "001BE2": "AhnLab,Inc.",
    "001BE3": "Health Hero Network, Inc.",
    "001BE4": "TOWNET SRL",
    "001BE5": "802automation Limited",
    "001BE6": "VR AG",
    "001BE7": "Postek Electronics Co., Ltd.",
    "001BE8": "Ultratronik GmbH",
    "001BE9": "Broadcom Corporation",
    "001BEA": "Nintendo Co., Ltd.",
    "001BEB": "DMP Electronics INC.",
    "001BEC": "Netio Technologies Co., Ltd",
    "001BED": "Foundry Networks",
    "001BEE": "Nokia Danmark A/S",
    "001BEF": "Blossoms Digital Technology Co.,Ltd.",
    "001BF0": "Value Platforms Limited",
    "001BF1": "Nanjing SilverNet Software Co., Ltd.",
    "001BF2": "KWORLD COMPUTER CO., LTD",
    "001BF3": "TRANSRADIO SenderSysteme Berlin AG",
    "001BF4": "KENWIN INDUSTRIAL(HK) LTD.",
    "001BF5": "Tellink Sistemas de Telecomunicación S.L.",
    "001BF6": "CONWISE Technology Corporation Ltd.",
    "001BF7": "Lund IP Products AB",
    "001BF8": "Digitrax Inc.",
    "001BF9": "Intellitect Water Ltd",
    "001BFA": "G.i.N. mbH",
    "001BFB": "Alps Electric Co., Ltd",
    "001BFC": "ASUSTek COMPUTER INC.",
    "001BFD": "Dignsys Inc.",
    "001BFE": "Zavio Inc.",
    "001BFF": "Millennia Media inc.",
    "001C00": "Entry Point, LLC",
    "001C01": "ABB Oy Drives",
    "001C02": "Atto Devices, Inc",
    "001C03": "Betty TV Technology AG",
    "001C04": "Airgain, Inc.",
    "001C05": "Nonin Medical Inc.",
    "001C06": "Siemens Numerical Control Ltd., Nanjing",
    "001C07": "Cwlinux Limited",
    "001C08": "Anystream, Inc.",
    "001C09": "SAE Electronic Co.,Ltd.",
    "001C0A": "Shenzhen AEE Technology Co.,Ltd.",
    "001C0B": "SmartAnt Telecom",
    "001C0C": "TANITA Corporation",
    "001C0D": "G-Technology, Inc.",
    "001C0E": "Cisco Systems",
    "001C0F": "Cisco Systems",
    "001C10": "Cisco-Linksys, LLC",
    "001C11": "Motorola CHS",
    "001C12": "Motorola Mobile Devices",
    "001C13": "OPTSYS TECHNOLOGY CO., LTD.",
    "001C14": "VMware, Inc",
    "001C15": "TXP Corporation",
    "001C16": "ThyssenKrupp Elevator",
    "001C17": "Nortel",
    "001C18": "Sicert S.r.L.",
    "001C19": "secunet Security Networks AG",
    "001C1A": "Thomas Instrumentation, Inc",
    "001C1B": "Hyperstone GmbH",
    "001C1C": "Center Communication Systems GmbH",
    "001C1D": "CHENZHOU GOSPELL DIGITAL TECHNOLOGY CO.,LTD",
    "001C1E": "emtrion GmbH",
    "001C1F": "Quest Retail Technology Pty Ltd",
    "001C20": "CLB Benelux",
    "001C21": "Nucsafe Inc.",
    "001C22": "Aeris Elettronica s.r.l.",
    "001C23": "Dell Inc",
    "001C24": "Formosa Wireless Systems Corp.",
    "001C25": "Hon Hai Precision Ind. Co.,Ltd.",
    "001C26": "Hon Hai Precision Ind. Co.,Ltd.",
    "001C27": "Sunell Electronics Co.",
    "001C28": "Sphairon Access Systems GmbH",
    "001C29": "CORE DIGITAL ELECTRONICS CO., LTD",
    "001C2A": "Envisacor Technologies Inc.",
    "001C2B": "Alertme.com Limited",
    "001C2C": "Synapse",
    "001C2D": "FlexRadio Systems",
    "001C2E": "ProCurve Networking by HP",
    "001C2F": "Pfister GmbH",
    "001C30": "Mode Lighting (UK ) Ltd.",
    "001C31": "Mobile XP Technology Co., LTD",
    "001C32": "Telian Corporation",
    "001C33": "Sutron",
    "001C34": "HUEY CHIAO INTERNATIONAL CO., LTD.",
    "001C35": "Nokia Danmark A/S",
    "001C36": "iNEWiT NV",
    "001C37": "Callpod, Inc.",
    "001C38": "Bio-Rad Laboratories, Inc.",
    "001C39": "S Netsystems Inc.",
    "001C3A": "Element Labs, Inc.",
    "001C3B": "AmRoad Technology Inc.",
    "001C3C": "Seon Design Inc.",
    "001C3D": "WaveStorm",
    "001C3E": "ECKey Limited",
    "001C3F": "International Police Technologies, Inc.",
    "001C40": "VDG-Security bv",
    "001C41": "scemtec Transponder Technology GmbH",
    "001C42": "Parallels, Inc.",
    "001C43": "Samsung Electronics Co.,Ltd",
    "001C44": "Bosch Security Systems BV",
    "001C45": "Chenbro Micom Co., Ltd.",
    "001C46": "QTUM",
    "001C47": "Hangzhou Hollysys Automation Co., Ltd",
    "001C48": "WiDeFi, Inc.",
    "001C49": "Zoltan Technology Inc.",
    "001C4A": "AVM GmbH",
    "001C4B": "Gener8, Inc.",
    "001C4C": "Petrotest Instruments",
    "001C4D": "Zeetoo, Inc.",
    "001C4E": "TASA International Limited",
    "001C4F": "MACAB AB",
    "001C50": "TCL Technoly Electronics(Huizhou)Co.,Ltd",
    "001C51": "Celeno Communications",
    "001C52": "VISIONEE SRL",
    "001C53": "Synergy Lighting Controls",
    "001C54": "Hillstone Networks Inc",
    "001C55": "Shenzhen Kaifa Technology Co.",
    "001C56": "Pado Systems, Inc.",
    "001C57": "Cisco Systems",
    "001C58": "Cisco Systems",
    "001C59": "DEVON IT",
    "001C5A": "Advanced Relay Corporation",
    "001C5B": "Chubb Electronic Security Systems Ltd",
    "001C5C": "Integrated Medical Systems, Inc.",
    "001C5D": "Leica Microsystems",
    "001C5E": "ASTON France",
    "001C5F": "Winland Electronics, Inc.",
    "001C60": "CSP Frontier Technologies,Inc.",
    "001C61": "Galaxy Technology (HK) Ltd.",
    "001C62": "LG Electronics Inc",
    "001C63": "TRUEN",
    "001C64": "Cellnet+Hunt",
    "001C65": "JoeScan, Inc.",
    "001C66": "UCAMP CO.,LTD",
    "001C67": "Pumpkin Networks, Inc.",
    "001C68": "Anhui Sun Create Electronics Co., Ltd",
    "001C69": "Packet Vision Ltd",
    "001C6A": "Weiss Engineering Ltd.",
    "001C6B": "COVAX Co. Ltd",
    "001C6C": "Jabil Circuit (Guangzhou) Limited",
    "001C6D": "KYOHRITSU ELECTRONIC INDUSTRY CO., LTD.",
    "001C6E": "Newbury Networks, Inc.",
    "001C6F": "Emfit Ltd",
    "001C70": "NOVACOMM LTDA",
    "001C71": "Emergent Electronics",
    "001C72": "Mayer & Cie GmbH & Co KG",
    "001C73": "Arastra, Inc.",
    "001C74": "Syswan Technologies Inc.",
    "001C75": "RF Systems GmbH",
    "001C76": "The Wandsworth Group Ltd",
    "001C77": "Prodys",
    "001C78": "WYPLAY SAS",
    "001C79": "Cohesive Financial Technologies LLC",
    "001C7A": "Perfectone Netware Company Ltd",
    "001C7B": "Castlenet Technology Inc.",
    "001C7C": "PERQ SYSTEMS CORPORATION",
    "001C7D": "Excelpoint Manufacturing Pte Ltd",
    "001C7E": "Toshiba",
    "001C7F": "Check Point Software Technologies",
    "001C80": "New Business Division/Rhea-Information CO., LTD.",
    "001C81": "NextGen Venturi LTD",
    "001C82": "Genew Technologies",
    "001C83": "New Level Telecom Co., Ltd.",
    "001C84": "STL Solution Co.,Ltd.",
    "001C85": "Eunicorn",
    "001C86": "Cranite Systems, Inc.",
    "001C87": "Uriver Inc.",
    "001C88": "TRANSYSTEM INC.",
    "001C89": "Force Communications, Inc.",
    "001C8A": "Verari Systems Inc",
    "001C8B": "MJ Innovations Ltd.",
    "001C8C": "DIAL TECHNOLOGY LTD.",
    "001C8D": "Mesa Imaging",
    "001C8E": "Alcatel-Lucent IPD",
    "001C8F": "Advanced Electronic Design, Inc.",
    "001C90": "Empacket Corporation",
    "001C91": "Gefen Inc.",
    "001C92": "Tervela",
    "001C93": "ExaDigm Inc",
    "001C94": "LI-COR Biosciences",
    "001C95": "Opticomm Corporation",
    "001C96": "Linkwise Technology Pte Ltd",
    "001C97": "Enzytek Technology Inc.,",
    "001C98": "LUCKY TECHNOLOGY (HK) COMPANY LIMITED",
    "001C99": "Shunra Software Ltd.",
    "001C9A": "Nokia Danmark A/S",
    "001C9B": "FEIG ELECTRONIC GmbH",
    "001C9C": "Nortel",
    "001C9D": "Liecthi AG",
    "001C9E": "Dualtech IT AB",
    "001C9F": "Razorstream, LLC",
    "001CA0": "Production Resource Group, LLC",
    "001CA1": "PRIVATE",
    "001CA2": "PIRELLI BROADBAND SOLUTIONS",
    "001CA3": "Terra",
    "001CA4": "Sony Ericsson Mobile Communications",
    "001CA5": "Zygo Corporation",
    "001CA6": "Win4NET",
    "001CA7": "International Quartz Limited",
    "001CA8": "AirTies Wireless Networks",
    "001CA9": "Audiomatica Srl",
    "001CAA": "Bellon Pty Ltd",
    "001CAB": "Meyer Sound Laboratories, Inc.",
    "001CAC": "Qniq Technology Corp.",
    "001CAD": "Wuhan Telecommunication Devices Co.,Ltd",
    "001CAE": "WiChorus, Inc.",
    "001CAF": "Plato Networks Inc.",
    "001CB0": "Cisco Systems",
    "001CB1": "Cisco Systems",
    "001CB2": "BPT SPA",
    "001CB3": "APPLE, INC",
    "001CB4": "Iridium Satellite LLC",
    "001CB5": "Neihua Network Technology Co.,LTD.(NHN)",
    "001CB6": "Duzon CNT Co., Ltd.",
    "001CB7": "USC DigiArk Corporation",
    "001CB8": "CBC Co., Ltd",
    "001CB9": "KWANG SUNG ELECTRONICS CO., LTD.",
    "001CBA": "VerScient, Inc.",
    "001CBB": "MusicianLink",
    "001CBC": "CastGrabber, LLC",
    "001CBD": "Ezze Mobile Tech., Inc.",
    "001CBE": "Nintendo Co., Ltd.",
    "001CBF": "Intel Corporate",
    "001CC0": "Intel Corporate",
    "001CC1": "Motorola Mobile Devices",
    "001CC2": "Part II Research, Inc.",
    "001CC3": "Pace Micro Technology plc",
    "001CC4": "Hewlett Packard",
    "001CC5": "3COM LTD",
    "001CC6": "ProStor Systems",
    "001CC7": "Rembrandt Technologies, LLC d/b/a REMSTREAM",
    "001CC8": "INDUSTRONIC Industrie-Electronic GmbH & Co. KG",
    "001CC9": "Kaise Electronic Technology Co., Ltd.",
    "001CCA": "Shanghai Gaozhi Science & Technology Development Co.",
    "001CCB": "Forth Corporation Public Company Limited",
    "001CCC": "Research In Motion Limited",
    "001CCD": "Alektrona Corporation",
    "001CCE": "By Techdesign",
    "001CCF": "LIMETEK",
    "001CD0": "Circleone Co.,Ltd.",
    "001CD1": "Waves Audio LTD",
    "001CD2": "King Champion (Hong Kong) Limited",
    "001CD3": "ZP Engineering SEL",
    "001CD4": "Nokia Danmark A/S",
    "001CD5": "Zazzle Technologies, Inc.",
    "001CD6": "Nokia Danmark A/S",
    "001CD7": "Harman/Becker Automotive Systems GmbH",
    "001CD8": "BlueAnt Wireless",
    "001CD9": "GlobalTop Technology Inc.",
    "001CDA": "Exegin Technologies Limited",
    "001CDB": "CARPOINT CO.,LTD",
    "001CDC": "Custom Computer Services, Inc.",
    "001CDD": "COWBELL ENGINEERING CO., LTD.",
    "001CDE": "Interactive Multimedia eXchange Inc.",
    "001CDF": "Belkin International Inc.",
    "001CE0": "DASAN TPS",
    "001CE1": "INDRA SISTEMAS, S.A.",
    "001CE2": "Attero Tech, LLC.",
    "001CE3": "Optimedical Systems",
    "001CE4": "EleSy JSC",
    "001CE5": "MBS Electronic Systems GmbH",
    "001CE6": "INNES",
    "001CE7": "Rocon PLC Research Centre",
    "001CE8": "Cummins Inc",
    "001CE9": "Galaxy Technology Limited",
    "001CEA": "Scientific-Atlanta, Inc",
    "001CEB": "Nortel",
    "001CEC": "Mobilesoft (Aust.) Pty Ltd",
    "001CED": "ENVIRONNEMENT SA",
    "001CEE": "SHARP Corporation",
    "001CEF": "Primax Electronics LTD",
    "001CF0": "D-Link Corporation",
    "001CF1": "SUPoX Technology Co. , LTD.",
    "001CF2": "Tenlon Technology Co.,Ltd.",
    "001CF3": "EVS BROADCAST EQUIPMENT",
    "001CF4": "Media Technology Systems Inc",
    "001CF5": "Wiseblue Technology Limited",
    "001CF6": "Cisco Systems",
    "001CF7": "AudioScience",
    "001CF8": "Parade Technologies, Ltd.",
    "001CF9": "Cisco Systems",
    "001CFA": "PRIVATE",
    "001CFB": "Motorola CHS",
    "001CFC": "Suminet Communication Technologies (Shanghai) Co., Ltd.",
    "001CFD": "Universal Electronics",
    "001CFE": "Quartics Inc",
    "001CFF": "Napera Networks Inc",
    "001D00": "Brivo Systems, LLC",
    "001D01": "Neptune Digital",
    "001D02": "Cybertech Telecom Development",
    "001D03": "Design Solutions Inc.",
    "001D04": "Zipit Wireless, Inc.",
    "001D05": "iLight",
    "001D06": "HM Electronics, Inc.",
    "001D07": "Shenzhen Sang Fei Consumer Communications Co.,Ltd",
    "001D08": "JIANGSU YINHE ELECTRONICS CO., LTD",
    "001D09": "Dell Inc",
    "001D0A": "Davis Instruments, Inc.",
    "001D0B": "Power Standards Lab",
    "001D0C": "MobileCompia",
    "001D0D": "Sony Computer Entertainment inc.",
    "001D0E": "Agapha Technology co., Ltd.",
    "001D0F": "TP-LINK Technologies Co., Ltd.",
    "001D10": "LightHaus Logic, Inc.",
    "001D11": "Analogue & Micro Ltd",
    "001D12": "ROHM CO., LTD.",
    "001D13": "NextGTV",
    "001D14": "SPERADTONE INFORMATION TECHNOLOGY LIMITED",
    "001D15": "Shenzhen Dolphin Electronic Co., Ltd",
    "001D16": "Efixo",
    "001D17": "Digital Sky Corporation",
    "001D18": "Power Innovation GmbH",
    "001D19": "Arcadyan Technology Corporation",
    "001D1A": "OvisLink S.A.",
    "001D1B": "Sangean Electronics Inc.",
    "001D1C": "Gennet s.a.",
    "001D1D": "Inter-M Corporation",
    "001D1E": "KYUSHU TEN CO.,LTD",
    "001D1F": "Siauliu Tauro Televizoriai, JSC",
    "001D20": "COMTREND CO.",
    "001D21": "Alcad S.A.",
    "001D22": "Foss Analytical A/S",
    "001D23": "SENSUS METERING SYSTEMS",
    "001D24": "DCSI",
    "001D25": "Samsung Electronics Co.,Ltd",
    "001D26": "Rockridgesound Technology Co.",
    "001D27": "NAC-INTERCOM",
    "001D28": "Sony Ericsson Mobile Communications AB",
    "001D29": "Doro AB",
    "001D2A": "Tideway Electronic LTD",
    "001D2B": "Wuhan Pont Technology CO. , LTD",
    "001D2C": "Wavetrend Technologies (Pty) Limited",
    "001D2D": "Pylone, Inc.",
    "001D2E": "Ruckus Wireless",
    "001D2F": "QuantumVision Corporation",
    "001D30": "YX Wireless S.A.",
    "001D31": "HIGHPRO INTERNATIONAL R&D CO,.LTD.",
    "001D32": "Longkay Communication & Technology (Shanghai) Co. Ltd",
    "001D33": "Maverick Systems Inc.",
    "001D34": "SYRIS Technology Corp",
    "001D35": "Viconics Electronics Inc.",
    "001D36": "ELECTRONICS CORPORATION OF INDIA LIMITED",
    "001D37": "Thales-Panda Transportation System",
    "001D38": "Seagate Technology",
    "001D39": "MOOHADIGITAL CO., LTD",
    "001D3A": "mh acoustics LLC",
    "001D3B": "Nokia Danmark A/S",
    "001D3C": "Muscle Corporation",
    "001D3D": "Avidyne Corporation",
    "001D3E": "SAKA TECHNO SCIENCE CO.,LTD",
    "001D3F": "Mitron Pty Ltd",
    "001D40": "Living Independently Group, Inc.",
    "001D41": "Hardy Instruments",
    "001D42": "Nortel",
    "001D43": "Shenzhen G-link Digital Technology Co., Ltd.",
    "001D44": "Krohne",
    "001D45": "Cisco Systems",
    "001D46": "Cisco Systems",
    "001D47": "Covote GmbH & Co KG",
    "001D48": "Sensor-Technik Wiedemann GmbH",
    "001D49": "Innovation Wireless Inc.",
    "001D4A": "Carestream Health, Inc.",
    "001D4B": "Grid Connect Inc.",
    "001D4C": "Alcatel-Lucent",
    "001D4D": "Adaptive Recognition Hungary, Inc",
    "001D4E": "TCM Mobile LLC",
    "001D4F": "Apple Inc",
    "001D50": "SPINETIX SA",
    "001D51": "GE Energy",
    "001D52": "Defzone B.V.",
    "001D53": "SHARP-ROXY CORP.(M) SDN. BHD.",
    "001D54": "Sunnic Technology & Merchandise INC.",
    "001D55": "ZANTAZ, Inc",
    "001D56": "Kramer Electronics Ltd.",
    "001D57": "CAETEC Messtechnik",
    "001D58": "CQ Inc",
    "001D59": "CHEROKEE INTERNATIONAL",
    "001D5A": "2Wire Inc.",
    "001D5B": "Tecvan Informatica Ltda",
    "001D5C": "Tom Communication Industrial Co.,Ltd.",
    "001D5D": "Control Dynamics Pty. Ltd.",
    "001D5E": "COMING MEDIA CORP.",
    "001D5F": "OverSpeed SARL",
    "001D60": "ASUSTek COMPUTER INC.",
    "001D61": "BIJ Corporation",
    "001D62": "InPhase Technologies",
    "001D63": "Miele & Cie. KG",
    "001D64": "Adam Communications Systems Int Ltd",
    "001D65": "Microwave Radio Communications",
    "001D66": "Hyundai Telecom",
    "001D67": "AMEC",
    "001D68": "Thomson Telecom Belgium",
    "001D69": "Knorr-Bremse AG",
    "001D6A": "Alpha Networks Inc.",
    "001D6B": "Motorola (formerly Netopia, Inc",
    "001D6C": "ClariPhy Communications, Inc.",
    "001D6D": "Confidant International LLC",
    "001D6E": "Nokia Danmark A/S",
    "001D6F": "Chainzone Technology Co., Ltd",
    "001D70": "Cisco Systems",
    "001D71": "Cisco Systems",
    "001D72": "Wistron Corporation",
    "001D73": "Buffalo Inc.",
    "001D74": "Tianjin China-Silicon Microelectronics Co., Ltd.",
    "001D75": "Radioscape PLC",
    "001D76": "Eyeheight Ltd.",
    "001D77": "NSGate",
    "001D78": "Invengo Information Technology Co.,Ltd",
    "001D79": "SIGNAMAX LLC",
    "001D7A": "Wideband Semiconductor, Inc.",
    "001D7B": "Ice Energy, Inc.",
    "001D7C": "ABE Elettronica S.p.A.",
    "001D7D": "GIGA-BYTE TECHNOLOGY CO.,LTD.",
    "001D7E": "Cisco-Linksys, LLC",
    "001D7F": "Tekron International Ltd",
    "001D80": "Beijing Huahuan Eletronics Co.,Ltd",
    "001D81": "GUANGZHOU GATEWAY ELECTRONICS CO., LTD",
    "001D82": "GN A/S (GN Netcom A/S)",
    "001D83": "Emitech Corporation",
    "001D84": "Gateway, Inc.",
    "001D85": "Call Direct Cellular Solutions",
    "001D86": "Shinwa Industries(China) Ltd.",
    "001D87": "VigTech Labs Sdn Bhd",
    "001D88": "Clearwire",
    "001D89": "VaultStor Corporation",
    "001D8A": "TechTrex Inc",
    "001D8B": "PIRELLI BROADBAND SOLUTIONS",
    "001D8C": "La Crosse Technology LTD",
    "001D8D": "Raytek GmbH",
    "001D8E": "Alereon, Inc.",
    "001D8F": "PureWave Networks",
    "001D90": "EMCO Flow Systems",
    "001D91": "Digitize, Inc",
    "001D92": "MICRO-STAR INT'L CO.,LTD.",
    "001D93": "Modacom",
    "001D94": "Climax Technology Co., Ltd",
    "001D95": "Flash, Inc.",
    "001D96": "WatchGuard Video",
    "001D97": "Alertus Technologies LLC",
    "001D98": "Nokia Danmark A/S",
    "001D99": "Cyan Optic, Inc.",
    "001D9A": "GODEX INTERNATIONAL CO., LTD",
    "001D9B": "Hokuyo Automatic Co., Ltd.",
    "001D9C": "Rockwell Automation",
    "001D9D": "ARTJOY INTERNATIONAL LIMITED",
    "001D9E": "AXION TECHNOLOGIES",
    "001D9F": "MATT R.P.Traczynscy Sp.J.",
    "001DA0": "Heng Yu Electronic Manufacturing Company Limited",
    "001DA1": "Cisco Systems",
    "001DA2": "Cisco Systems",
    "001DA3": "SabiOso",
    "001DA4": "Hangzhou System Technology CO., LTD",
    "001DA5": "WB Electronics",
    "001DA6": "Media Numerics Limited",
    "001DA7": "Seamless Internet",
    "001DA8": "Takahata Electronics Co.,Ltd",
    "001DA9": "Castles Technology, Co., LTD",
    "001DAA": "DrayTek Corp.",
    "001DAB": "SwissQual",
    "001DAC": "Gigamon Systems LLC",
    "001DAD": "Sinotech Engineering Consultants, Inc. Geotechnical Enginee",
    "001DAE": "CHANG TSENG TECHNOLOGY CO., LTD",
    "001DAF": "Nortel",
    "001DB0": "FuJian HengTong Information Technology Co.,Ltd",
    "001DB1": "Crescendo Networks",
    "001DB2": "HOKKAIDO ELECTRIC ENGINEERING CO.,LTD.",
    "001DB3": "ProCurve Networking by HP",
    "001DB4": "KUMHO ENG CO.,LTD",
    "001DB5": "Juniper networks",
    "001DB6": "BestComm Networks, Inc.",
    "001DB7": "Tendril Networks, Inc.",
    "001DB8": "Intoto Inc.",
    "001DB9": "Wellspring Wireless",
    "001DBA": "Sony Corporation",
    "001DBB": "Dynamic System Electronics Corp.",
    "001DBC": "Nintendo Co., Ltd.",
    "001DBD": "Versamed Inc.",
    "001DBE": "Motorola Mobile Devices",
    "001DBF": "Radiient Technologies, Inc.",
    "001DC0": "Enphase Energy",
    "001DC1": "Audinate",
    "001DC2": "XORTEC OY",
    "001DC3": "RIKOR TV, Ltd",
    "001DC4": "AIOI Systems Co., Ltd.",
    "001DC5": "Beijing Jiaxun Feihong Electricial Co., Ltd.",
    "001DC6": "SNR Inc.",
    "001DC7": "L-3 Communications Geneva Aerospace",
    "001DC8": "ScadaMetrcs, LLC.",
    "001DC9": "GainSpan Corp.",
    "001DCA": "PAV Electronics Limited",
    "001DCB": "Exéns Development Oy",
    "001DCC": "Hetra Secure Solutions",
    "001DCD": "ARRIS Group, Inc.",
    "001DCE": "ARRIS Group, Inc.",
    "001DCF": "ARRIS Group, Inc.",
    "001DD0": "ARRIS Group, Inc.",
    "001DD1": "ARRIS Group, Inc.",
    "001DD2": "ARRIS Group, Inc.",
    "001DD3": "ARRIS Group, Inc.",
    "001DD4": "ARRIS Group, Inc.",
    "001DD5": "ARRIS Group, Inc.",
    "001DD6": "ARRIS Group, Inc.",
    "001DD7": "Algolith",
    "001DD8": "Microsoft Corporation",
    "001DD9": "Hon Hai Precision Ind.Co.,Ltd.",
    "001DDA": "Mikroelektronika spol. s r. o.",
    "001DDB": "C-BEL Corporation",
    "001DDC": "HangZhou DeChangLong Tech&Info Co.,Ltd",
    "001DDD": "DAT H.K. LIMITED",
    "001DDE": "Zhejiang Broadcast&Television Technology Co.,Ltd.",
    "001DDF": "Sunitec Enterprise Co., Ltd.",
    "001DE0": "Intel Corporate",
    "001DE1": "Intel Corporate",
    "001DE2": "Radionor Communications",
    "001DE3": "Intuicom",
    "001DE4": "Visioneered Image Systems",
    "001DE5": "Cisco Systems",
    "001DE6": "Cisco Systems",
    "001DE7": "Marine Sonic Technology, Ltd.",
    "001DE8": "Nikko Denki Tsushin Company(NDTC)",
    "001DE9": "Nokia Danmark A/S",
    "001DEA": "Commtest Instruments Ltd",
    "001DEB": "DINEC International",
    "001DEC": "Marusys",
    "001DED": "Grid Net, Inc.",
    "001DEE": "NEXTVISION SISTEMAS DIGITAIS DE TELEVISÃO LTDA.",
    "001DEF": "TRIMM, INC.",
    "001DF0": "Vidient Systems, Inc.",
    "001DF1": "Intego Systems, Inc.",
    "001DF2": "Netflix, Inc.",
    "001DF3": "SBS Science & Technology Co., Ltd",
    "001DF4": "Magellan Technology Pty Limited",
    "001DF5": "Sunshine Co,LTD",
    "001DF6": "Samsung Electronics Co.,Ltd",
    "001DF7": "R. STAHL Schaltgeräte GmbH",
    "001DF8": "Webpro Vision Technology Corporation",
    "001DF9": "Cybiotronics (Far East) Limited",
    "001DFA": "Fujian LANDI Commercial Equipment Co.,Ltd",
    "001DFB": "NETCLEUS Systems Corporation",
    "001DFC": "KSIC",
    "001DFD": "Nokia Danmark A/S",
    "001DFE": "Palm, Inc",
    "001DFF": "Network Critical Solutions Ltd",
    "001E00": "Shantou Institute of Ultrasonic Instruments",
    "001E01": "Renesas Technology Sales Co., Ltd.",
    "001E02": "Sougou Keikaku Kougyou Co.,Ltd.",
    "001E03": "LiComm Co., Ltd.",
    "001E04": "Hanson Research Corporation",
    "001E05": "Xseed Technologies & Computing",
    "001E06": "WIBRAIN",
    "001E07": "Winy Technology Co., Ltd.",
    "001E08": "Centec Networks Inc",
    "001E09": "ZEFATEK Co.,LTD",
    "001E0A": "Syba Tech Limited",
    "001E0B": "Hewlett Packard",
    "001E0C": "Sherwood Information Partners, Inc.",
    "001E0D": "Micran Ltd.",
    "001E0E": "MAXI VIEW HOLDINGS LIMITED",
    "001E0F": "Briot International",
    "001E10": "ShenZhen Huawei Communication Technologies Co.,Ltd.",
    "001E11": "ELELUX INTERNATIONAL LTD",
    "001E12": "Ecolab",
    "001E13": "Cisco Systems",
    "001E14": "Cisco Systems",
    "001E15": "Beech Hill Electronics",
    "001E16": "Keytronix",
    "001E17": "STN BV",
    "001E18": "Radio Activity srl",
    "001E19": "GTRI",
    "001E1A": "Best Source Taiwan Inc.",
    "001E1B": "Digital Stream Technology, Inc.",
    "001E1C": "SWS Australia Pty Limited",
    "001E1D": "East Coast Datacom, Inc.",
    "001E1E": "Honeywell Life Safety",
    "001E1F": "Nortel",
    "001E20": "Intertain Inc.",
    "001E21": "Qisda Co.",
    "001E22": "ARVOO Imaging Products BV",
    "001E23": "Electronic Educational Devices, Inc",
    "001E24": "Zhejiang Bell Technology Co.,ltd",
    "001E25": "Intek Digital Inc",
    "001E26": "Digifriends Co. Ltd",
    "001E27": "SBN TECH Co.,Ltd.",
    "001E28": "Lumexis Corporation",
    "001E29": "Hypertherm Inc",
    "001E2A": "Netgear Inc.",
    "001E2B": "Radio Systems Design, Inc.",
    "001E2C": "CyVerse Corporation",
    "001E2D": "STIM",
    "001E2E": "SIRTI S.p.A.",
    "001E2F": "DiMoto Pty Ltd",
    "001E30": "Shireen Inc",
    "001E31": "INFOMARK CO.,LTD.",
    "001E32": "Zensys",
    "001E33": "Inventec Corporation",
    "001E34": "CryptoMetrics",
    "001E35": "Nintendo Co., Ltd.",
    "001E36": "IPTE",
    "001E37": "USI",
    "001E38": "Bluecard Software Technology Co., Ltd.",
    "001E39": "Comsys Communication Ltd.",
    "001E3A": "Nokia Danmark A/S",
    "001E3B": "Nokia Danmark A/S",
    "001E3C": "Lyngbox Media AB",
    "001E3D": "Alps Electric Co., Ltd",
    "001E3E": "KMW Inc.",
    "001E3F": "PRIVATE",
    "001E40": "Shanghai DareGlobal Technologies Co.,Ltd.",
    "001E41": "Microwave Communication & Component, Inc.",
    "001E42": "Teltonika",
    "001E43": "AISIN AW CO.,LTD.",
    "001E44": "SANTEC",
    "001E45": "Sony Ericsson Mobile Communications AB",
    "001E46": "Motorola CHS",
    "001E47": "PT. Hariff Daya Tunggal Engineering",
    "001E48": "Wi-Links",
    "001E49": "Cisco Systems",
    "001E4A": "Cisco Systems",
    "001E4B": "City Theatrical",
    "001E4C": "Hon Hai Precision Ind.Co., Ltd.",
    "001E4D": "Welkin Sciences, LLC",
    "001E4E": "DAKO EDV-Ingenieur- und Systemhaus GmbH",
    "001E4F": "Dell Inc.",
    "001E50": "BATTISTONI RESEARCH",
    "001E51": "Converter Industry Srl",
    "001E52": "Apple Inc",
    "001E53": "Further Tech Co., LTD",
    "001E54": "TOYO ELECTRIC Corporation",
    "001E55": "COWON SYSTEMS,Inc.",
    "001E56": "Bally Wulff Entertainment GmbH",
    "001E57": "ALCOMA, spol. s r.o.",
    "001E58": "D-Link Corporation",
    "001E59": "Silicon Turnkey Express, LLC",
    "001E5A": "Motorola CHS",
    "001E5B": "Unitron Company, Inc.",
    "001E5C": "RB GeneralEkonomik",
    "001E5D": "Holosys d.o.o.",
    "001E5E": "COmputime Ltd.",
    "001E5F": "KwikByte, LLC",
    "001E60": "Digital Lighting Systems, Inc",
    "001E61": "ITEC GmbH",
    "001E62": "Siemon",
    "001E63": "Vibro-Meter SA",
    "001E64": "Intel Corporate",
    "001E65": "Intel Corporate",
    "001E66": "RESOL Elektronische Regelungen GmbH",
    "001E67": "Intel Corporate",
    "001E68": "Quanta Computer",
    "001E69": "Thomson Inc.",
    "001E6A": "Beijing Bluexon Technology Co.,Ltd",
    "001E6B": "Scientific Atlanta, A Cisco Company",
    "001E6C": "Spliced Networks LLC",
    "001E6D": "IT R&D Center",
    "001E6E": "Shenzhen First Mile Communications Ltd",
    "001E6F": "Magna-Power Electronics, Inc.",
    "001E70": "Cobham Defence Communications Ltd",
    "001E71": "IgeaCare Systems Inc.",
    "001E72": "PCS",
    "001E73": "ZTE CORPORATION",
    "001E74": "SAGEM COMMUNICATION",
    "001E75": "LG Electronics",
    "001E76": "Thermo Fisher Scientific",
    "001E77": "Air2App",
    "001E78": "Owitek Technology Ltd.,",
    "001E79": "Cisco Systems",
    "001E7A": "Cisco Systems",
    "001E7B": "R.I.CO. S.r.l.",
    "001E7C": "Taiwick Limited",
    "001E7D": "Samsung Electronics Co.,Ltd",
    "001E7E": "Nortel",
    "001E7F": "CBM of America",
    "001E80": "Last Mile Ltd.",
    "001E81": "CNB Technology Inc.",
    "001E82": "Pliant Technology, Inc.",
    "001E83": "LAN/MAN Standards Association (LMSC)",
    "001E84": "Pika Technologies Inc.",
    "001E85": "Lagotek Corporation",
    "001E86": "MEL Co.,Ltd.",
    "001E87": "Realease Limited",
    "001E88": "ANDOR SYSTEM SUPPORT CO., LTD.",
    "001E89": "CRFS Limited",
    "001E8A": "eCopy, Inc",
    "001E8B": "Hannet Telecom Co., Ltd.",
    "001E8C": "ASUSTek COMPUTER INC.",
    "001E8D": "Motorola Mobile Devices",
    "001E8E": "Hunkeler AG",
    "001E8F": "CANON INC.",
    "001E90": "Elitegroup Computer Systems Co",
    "001E91": "KIMIN Electronic Co., Ltd.",
    "001E92": "JEULIN S.A.",
    "001E93": "CiriTech Systems Inc",
    "001E94": "SUPERCOM TECHNOLOGY CORPORATION",
    "001E95": "SIGMALINK",
    "001E96": "Sepura Plc",
    "001E97": "Medium Link System Technology CO., LTD,",
    "001E98": "GreenLine Communications",
    "001E99": "Vantanol Industrial Corporation",
    "001E9A": "HAMILTON Bonaduz AG",
    "001E9B": "San-Eisha, Ltd.",
    "001E9C": "Fidustron INC",
    "001E9D": "Recall Technologies, Inc.",
    "001E9E": "ddm hopt + schuler Gmbh + Co. KG",
    "001E9F": "Visioneering Systems, Inc.",
    "001EA0": "XLN-t",
    "001EA1": "Brunata a/s",
    "001EA2": "Symx Systems, Inc.",
    "001EA3": "Nokia Danmark A/S",
    "001EA4": "Nokia Danmark A/S",
    "001EA5": "ROBOTOUS, Inc.",
    "001EA6": "Best IT World (India) Pvt. Ltd.",
    "001EA7": "ActionTec Electronics, Inc",
    "001EA8": "Datang Mobile Communications Equipment CO.,LTD",
    "001EA9": "Nintendo Co., Ltd.",
    "001EAA": "E-Senza Technologies GmbH",
    "001EAB": "TeleWell Oy",
    "001EAC": "Armadeus Systems",
    "001EAD": "Wingtech Group Limited",
    "001EAE": "Continental Automotive Systems",
    "001EAF": "Ophir Optronics Ltd",
    "001EB0": "ImesD Electronica S.L.",
    "001EB1": "Cryptsoft Pty Ltd",
    "001EB2": "LG innotek",
    "001EB3": "Primex Wireless",
    "001EB4": "UNIFAT TECHNOLOGY LTD.",
    "001EB5": "Ever Sparkle Technologies Ltd",
    "001EB6": "TAG Heuer SA",
    "001EB7": "TBTech, Co., Ltd.",
    "001EB8": "Fortis, Inc.",
    "001EB9": "Sing Fai Technology Limited",
    "001EBA": "High Density Devices AS",
    "001EBB": "BLUELIGHT TECHNOLOGY INC.",
    "001EBC": "WINTECH AUTOMATION CO.,LTD.",
    "001EBD": "Cisco Systems",
    "001EBE": "Cisco Systems",
    "001EBF": "Haas Automation Inc.",
    "001EC0": "PRIVATE",
    "001EC1": "3COM EUROPE LTD",
    "001EC2": "Apple, Inc",
    "001EC3": "Kozio, Inc.",
    "001EC4": "Celio Corp",
    "001EC5": "Middle Atlantic Products Inc",
    "001EC6": "Obvius Holdings LLC",
    "001EC7": "2Wire",
    "001EC8": "Rapid Mobile (Pty) Ltd",
    "001EC9": "Dell Inc",
    "001ECA": "Nortel",
    "001ECB": "RPC Energoautomatika Ltd",
    "001ECC": "CDVI",
    "001ECD": "KYLAND",
    "001ECE": "BISA Technologies (Hong Kong) Limited",
    "001ECF": "PHILIPS ELECTRONICS UK LTD",
    "001ED0": "CONNEXIUM",
    "001ED1": "Keyprocessor B.V.",
    "001ED2": "Ray Shine Video Technology Inc",
    "001ED3": "Dot Technology Int'l Co., Ltd.",
    "001ED4": "Doble Engineering",
    "001ED5": "Tekon-Automatics",
    "001ED6": "Alentec & Orion AB",
    "001ED7": "H-Stream Wireless, Inc.",
    "001ED8": "Digital United Inc.",
    "001ED9": "Mitsubishi Precision Co.,LTd.",
    "001EDA": "Wesemann Elektrotechniek B.V.",
    "001EDB": "Giken Trastem Co., Ltd.",
    "001EDC": "Sony Ericsson Mobile Communications AB",
    "001EDD": "WASKO S.A.",
    "001EDE": "BYD COMPANY LIMITED",
    "001EDF": "Master Industrialization Center Kista",
    "001EE0": "Urmet Domus SpA",
    "001EE1": "Samsung Electronics Co.,Ltd",
    "001EE2": "Samsung Electronics Co.,Ltd",
    "001EE3": "T&W Electronics (ShenZhen) Co.,Ltd",
    "001EE4": "ACS Solutions France",
    "001EE5": "Cisco-Linksys, LLC",
    "001EE6": "Shenzhen Advanced Video Info-Tech Co., Ltd.",
    "001EE7": "Epic Systems Inc",
    "001EE8": "Mytek",
    "001EE9": "Stoneridge Electronics AB",
    "001EEA": "Sensor Switch, Inc.",
    "001EEB": "Talk-A-Phone Co.",
    "001EEC": "COMPAL INFORMATION (KUNSHAN) CO., LTD.",
    "001EED": "Adventiq Ltd.",
    "001EEE": "ETL Systems Ltd",
    "001EEF": "Cantronic International Limited",
    "001EF0": "Gigafin Networks",
    "001EF1": "Servimat",
    "001EF2": "Micro Motion Inc",
    "001EF3": "From2",
    "001EF4": "L-3 Communications Display Systems",
    "001EF5": "Hitek Automated Inc.",
    "001EF6": "Cisco Systems",
    "001EF7": "Cisco Systems",
    "001EF8": "Emfinity Inc.",
    "001EF9": "Pascom Kommunikations systeme GmbH.",
    "001EFA": "PROTEI Ltd.",
    "001EFB": "Trio Motion Technology Ltd",
    "001EFC": "JSC \"MASSA-K\"",
    "001EFD": "Microbit 2.0 AB",
    "001EFE": "LEVEL s.r.o.",
    "001EFF": "Mueller-Elektronik GmbH & Co. KG",
    "001F00": "Nokia Danmark A/S",
    "001F01": "Nokia Danmark A/S",
    "001F02": "Pixelmetrix Corporation Pte Ltd",
    "001F03": "NUM AG",
    "001F04": "Granch Ltd.",
    "001F05": "iTAS Technology Corp.",
    "001F06": "Integrated Dispatch Solutions",
    "001F07": "AZTEQ Mobile",
    "001F08": "RISCO LTD",
    "001F09": "JASTEC CO., LTD.",
    "001F0A": "Nortel",
    "001F0B": "Federal State Unitary Enterprise Industrial Union \"Electropribor\"",
    "001F0C": "Intelligent Digital Services GmbH",
    "001F0D": "L3 Communications - Telemetry West",
    "001F0E": "Japan Kyastem Co., Ltd",
    "001F0F": "Select Engineered Systems",
    "001F10": "TOLEDO DO BRASIL INDUSTRIA DE BALANCAS LTDA",
    "001F11": "OPENMOKO, INC.",
    "001F12": "Juniper Networks",
    "001F13": "S.& A.S. Ltd.",
    "001F14": "NexG",
    "001F15": "Bioscrypt Inc",
    "001F16": "Wistron Corporation",
    "001F17": "IDX Company, Ltd.",
    "001F18": "Hakusan.Mfg.Co,.Ltd",
    "001F19": "BEN-RI ELECTRONICA S.A.",
    "001F1A": "Prominvest",
    "001F1B": "RoyalTek Company Ltd.",
    "001F1C": "KOBISHI ELECTRIC Co.,Ltd.",
    "001F1D": "Atlas Material Testing Technology LLC",
    "001F1E": "Astec Technology Co., Ltd",
    "001F1F": "Edimax Technology Co. Ltd.",
    "001F20": "Logitech Europe SA",
    "001F21": "Inner Mongolia Yin An Science & Technology Development Co.,L",
    "001F22": "Fiberxon, Inc.",
    "001F23": "Interacoustics",
    "001F24": "DIGITVIEW TECHNOLOGY CO., LTD.",
    "001F25": "MBS GmbH",
    "001F26": "Cisco Systems",
    "001F27": "Cisco Systems",
    "001F28": "ProCurve Networking by HP",
    "001F29": "Hewlett Packard",
    "001F2A": "ACCM",
    "001F2B": "Orange Logic",
    "001F2C": "Starbridge Networks",
    "001F2D": "Electro-Optical Imaging, Inc.",
    "001F2E": "Triangle Research Int'l Pte Ltd",
    "001F2F": "Berker GmbH & Co. KG",
    "001F30": "Travelping",
    "001F31": "Radiocomp",
    "001F32": "Nintendo Co., Ltd.",
    "001F33": "Netgear Inc.",
    "001F34": "Lung Hwa Electronics Co., Ltd.",
    "001F35": "AIR802 LLC",
    "001F36": "Bellwin Information Co. Ltd.,",
    "001F37": "Genesis I&C",
    "001F38": "POSITRON",
    "001F39": "Construcciones y Auxiliar de Ferrocarriles, S.A.",
    "001F3A": "Hon Hai Precision Ind.Co., Ltd.",
    "001F3B": "Intel Corporate",
    "001F3C": "Intel Corporate",
    "001F3D": "Qbit GmbH",
    "001F3E": "RP-Technik e.K.",
    "001F3F": "AVM GmbH",
    "001F40": "Speakercraft Inc.",
    "001F41": "Ruckus Wireless",
    "001F42": "Etherstack Pty Ltd",
    "001F43": "ENTES ELEKTRONIK",
    "001F44": "GE Transportation Systems",
    "001F45": "Enterasys",
    "001F46": "Nortel",
    "001F47": "MCS Logic Inc.",
    "001F48": "Mojix Inc.",
    "001F49": "Eurosat Distribution Ltd",
    "001F4A": "Albentia Systems S.A.",
    "001F4B": "Lineage Power",
    "001F4C": "Roseman Engineering Ltd",
    "001F4D": "Segnetics LLC",
    "001F4E": "ConMed Linvatec",
    "001F4F": "Thinkware Co. Ltd.",
    "001F50": "Swissdis AG",
    "001F51": "HD Communications Corp",
    "001F52": "UVT Unternehmensberatung für Verkehr und Technik GmbH",
    "001F53": "GEMAC Gesellschaft für Mikroelektronikanwendung Chemnitz mbH",
    "001F54": "Lorex Technology Inc.",
    "001F55": "Honeywell Security (China) Co., Ltd.",
    "001F56": "DIGITAL FORECAST",
    "001F57": "Phonik Innovation Co.,LTD",
    "001F58": "EMH Energiemesstechnik GmbH",
    "001F59": "Kronback Tracers",
    "001F5A": "Beckwith Electric Co.",
    "001F5B": "Apple, Inc.",
    "001F5C": "Nokia Danmark A/S",
    "001F5D": "Nokia Danmark A/S",
    "001F5E": "Dyna Technology Co.,Ltd.",
    "001F5F": "Blatand GmbH",
    "001F60": "COMPASS SYSTEMS CORP.",
    "001F61": "Talent Communication Networks Inc.",
    "001F62": "JSC \"Stilsoft\"",
    "001F63": "JSC Goodwin-Europa",
    "001F64": "Beijing Autelan Technology Inc.",
    "001F65": "KOREA ELECTRIC TERMINAL CO., LTD.",
    "001F66": "PLANAR LLC",
    "001F67": "Hitachi,Ltd.",
    "001F68": "Martinsson Elektronik AB",
    "001F69": "Pingood Technology Co., Ltd.",
    "001F6A": "PacketFlux Technologies, Inc.",
    "001F6B": "LG Electronics",
    "001F6C": "Cisco Systems",
    "001F6D": "Cisco Systems",
    "001F6E": "Vtech Engineering Corporation",
    "001F6F": "Fujian Sunnada Communication Co.,Ltd.",
    "001F70": "Botik Technologies LTD",
    "001F71": "xG Technology, Inc.",
    "001F72": "QingDao Hiphone Technology Co,.Ltd",
    "001F73": "Teraview Technology Co., Ltd.",
    "001F74": "Eigen Development",
    "001F75": "GiBahn Media",
    "001F76": "AirLogic Systems Inc.",
    "001F77": "HEOL DESIGN",
    "001F78": "Blue Fox Porini Textile",
    "001F79": "Lodam Electronics A/S",
    "001F7A": "WiWide Inc.",
    "001F7B": "TechNexion Ltd.",
    "001F7C": "Witelcom AS",
    "001F7D": "embedded wireless GmbH",
    "001F7E": "Motorola Mobile Devices",
    "001F7F": "Phabrix Limited",
    "001F80": "Lucas Holding bv",
    "001F81": "Accel Semiconductor Corp",
    "001F82": "Cal-Comp Electronics & Communications Co., Ltd",
    "001F83": "Teleplan Technology Services Sdn Bhd",
    "001F84": "Gigle Semiconductor",
    "001F85": "Apriva ISS, LLC",
    "001F86": "digEcor",
    "001F87": "Skydigital Inc.",
    "001F88": "FMS Force Measuring Systems AG",
    "001F89": "Signalion GmbH",
    "001F8A": "Ellion Digital Inc.",
    "001F8B": "Storspeed, Inc.",
    "001F8C": "CCS Inc.",
    "001F8D": "Ingenieurbuero Stark GmbH und Ko. KG",
    "001F8E": "Metris USA Inc.",
    "001F8F": "Shanghai Bellmann Digital Source Co.,Ltd.",
    "001F90": "Actiontec Electronics, Inc",
    "001F91": "DBS Lodging Technologies, LLC",
    "001F92": "VideoIQ, Inc.",
    "001F93": "Xiotech Corporation",
    "001F94": "Lascar Electronics Ltd",
    "001F95": "SAGEM COMMUNICATION",
    "001F96": "APROTECH CO.LTD",
    "001F97": "BERTANA SRL",
    "001F98": "DAIICHI-DENTSU LTD.",
    "001F99": "SERONICS co.ltd",
    "001F9A": "Nortel Networks",
    "001F9B": "POSBRO",
    "001F9C": "LEDCO",
    "001F9D": "Cisco Systems",
    "001F9E": "Cisco Systems",
    "001F9F": "Thomson Telecom Belgium",
    "001FA0": "A10 Networks",
    "001FA1": "Gtran Inc",
    "001FA2": "Datron World Communications, Inc.",
    "001FA3": "T&W Electronics(Shenzhen)Co.,Ltd.",
    "001FA4": "ShenZhen Gongjin Electronics Co.,Ltd",
    "001FA5": "Blue-White Industries",
    "001FA6": "Stilo srl",
    "001FA7": "Sony Computer Entertainment Inc.",
    "001FA8": "ANI Technologies Corp.",
    "001FA9": "Atlanta DTH, Inc.",
    "001FAA": "Taseon, Inc.",
    "001FAB": "I.S HIGH TECH.INC",
    "001FAC": "Goodmill Systems Ltd",
    "001FAD": "Brown Innovations, Inc",
    "001FAE": "Blick South Africa (Pty) Ltd",
    "001FAF": "NextIO, Inc.",
    "001FB0": "TimeIPS, Inc.",
    "001FB1": "Cybertech Inc.",
    "001FB2": "Sontheim Industrie Elektronik GmbH",
    "001FB3": "2Wire",
    "001FB4": "SmartShare Systems",
    "001FB5": "I/O Interconnect Inc.",
    "001FB6": "Chi Lin Technology Co., Ltd.",
    "001FB7": "WiMate Technologies Corp.",
    "001FB8": "Universal Remote Control, Inc.",
    "001FB9": "Paltronics",
    "001FBA": "BoYoung Tech. & Marketing, Inc.",
    "001FBB": "Xenatech Co.,LTD",
    "001FBC": "EVGA Corporation",
    "001FBD": "Kyocera Wireless Corp.",
    "001FBE": "Shenzhen Mopnet Industrial Co.,Ltd",
    "001FBF": "Fulhua Microelectronics Corp. Taiwan Branch",
    "001FC0": "Control Express Finland Oy",
    "001FC1": "Hanlong Technology Co.,LTD",
    "001FC2": "Jow Tong Technology Co Ltd",
    "001FC3": "SmartSynch, Inc",
    "001FC4": "Motorola CHS",
    "001FC5": "Nintendo Co., Ltd.",
    "001FC6": "ASUSTek COMPUTER INC.",
    "001FC7": "Casio Hitachi Mobile Comunications Co., Ltd.",
    "001FC8": "Up-Today Industrial Co., Ltd.",
    "001FC9": "Cisco Systems",
    "001FCA": "Cisco Systems",
    "001FCB": "NIW Solutions",
    "001FCC": "Samsung Electronics Co.,Ltd",
    "001FCD": "Samsung Electronics",
    "001FCE": "QTECH LLC",
    "001FCF": "MSI Technology GmbH",
    "001FD0": "GIGA-BYTE TECHNOLOGY CO.,LTD.",
    "001FD1": "OPTEX CO.,LTD.",
    "001FD2": "COMMTECH TECHNOLOGY MACAO COMMERCIAL OFFSHORE LTD.",
    "001FD3": "RIVA Networks Inc.",
    "001FD4": "4IPNET, INC.",
    "001FD5": "MICRORISC s.r.o.",
    "001FD6": "Shenzhen Allywll",
    "001FD7": "TELERAD SA",
    "001FD8": "A-TRUST COMPUTER CORPORATION",
    "001FD9": "RSD Communications Ltd",
    "001FDA": "Nortel Networks",
    "001FDB": "Network Supply Corp.,",
    "001FDC": "Mobile Safe Track Ltd",
    "001FDD": "GDI LLC",
    "001FDE": "Nokia Danmark A/S",
    "001FDF": "Nokia Danmark A/S",
    "001FE0": "EdgeVelocity Corp",
    "001FE1": "Hon Hai Precision Ind. Co., Ltd.",
    "001FE2": "Hon Hai Precision Ind. Co., Ltd.",
    "001FE3": "LG Electronics",
    "001FE4": "Sony Ericsson Mobile Communications",
    "001FE5": "In-Circuit GmbH",
    "001FE6": "Alphion Corporation",
    "001FE7": "Simet",
    "001FE8": "KURUSUGAWA Electronics Industry Inc,.",
    "001FE9": "Printrex, Inc.",
    "001FEA": "Applied Media Technologies Corporation",
    "001FEB": "Trio Datacom Pty Ltd",
    "001FEC": "Synapse électronique",
    "001FED": "Tecan Systems Inc.",
    "001FEE": "ubisys technologies GmbH",
    "001FEF": "SHINSEI INDUSTRIES CO.,LTD",
    "001FF0": "Audio Partnership",
    "001FF1": "Paradox Hellas S.A.",
    "001FF2": "VIA Technologies, Inc.",
    "001FF3": "Apple, Inc",
    "001FF4": "Power Monitors, Inc.",
    "001FF5": "Kongsberg Defence & Aerospace",
    "001FF6": "PS Audio International",
    "001FF7": "Nakajima All Precision Co., Ltd.",
    "001FF8": "Siemens AG, Sector Industry, Drive Technologies, Motion Control Systems",
    "001FF9": "Advanced Knowledge Associates",
    "001FFA": "Coretree, Co, Ltd",
    "001FFB": "Green Packet Bhd",
    "001FFC": "Riccius+Sohn GmbH",
    "001FFD": "Indigo Mobile Technologies Corp.",
    "001FFE": "ProCurve Networking by HP",
    "001FFF": "Respironics, Inc.",
    "002000": "LEXMARK INTERNATIONAL, INC.",
    "002001": "DSP SOLUTIONS, INC.",
    "002002": "SERITECH ENTERPRISE CO., LTD.",
    "002003": "PIXEL POWER LTD.",
    "002004": "YAMATAKE-HONEYWELL CO., LTD.",
    "002005": "SIMPLE TECHNOLOGY",
    "002006": "GARRETT COMMUNICATIONS, INC.",
    "002007": "SFA, INC.",
    "002008": "CABLE & COMPUTER TECHNOLOGY",
    "002009": "PACKARD BELL ELEC., INC.",
    "00200A": "SOURCE-COMM CORP.",
    "00200B": "OCTAGON SYSTEMS CORP.",
    "00200C": "ADASTRA SYSTEMS CORP.",
    "00200D": "CARL ZEISS",
    "00200E": "SATELLITE TECHNOLOGY MGMT, INC",
    "00200F": "TANBAC CO., LTD.",
    "002010": "JEOL SYSTEM TECHNOLOGY CO. LTD",
    "002011": "CANOPUS CO., LTD.",
    "002012": "CAMTRONICS MEDICAL SYSTEMS",
    "002013": "DIVERSIFIED TECHNOLOGY, INC.",
    "002014": "GLOBAL VIEW CO., LTD.",
    "002015": "ACTIS COMPUTER SA",
    "002016": "SHOWA ELECTRIC WIRE & CABLE CO",
    "002017": "ORBOTECH",
    "002018": "CIS TECHNOLOGY INC.",
    "002019": "OHLER GmbH",
    "00201A": "MRV Communications, Inc.",
    "00201B": "NORTHERN TELECOM/NETWORK",
    "00201C": "EXCEL, INC.",
    "00201D": "KATANA PRODUCTS",
    "00201E": "NETQUEST CORPORATION",
    "00201F": "BEST POWER TECHNOLOGY, INC.",
    "002020": "MEGATRON COMPUTER INDUSTRIES PTY, LTD.",
    "002021": "ALGORITHMS SOFTWARE PVT. LTD.",
    "002022": "NMS Communications",
    "002023": "T.C. TECHNOLOGIES PTY. LTD",
    "002024": "PACIFIC COMMUNICATION SCIENCES",
    "002025": "CONTROL TECHNOLOGY, INC.",
    "002026": "AMKLY SYSTEMS, INC.",
    "002027": "MING FORTUNE INDUSTRY CO., LTD",
    "002028": "WEST EGG SYSTEMS, INC.",
    "002029": "TELEPROCESSING PRODUCTS, INC.",
    "00202A": "N.V. DZINE",
    "00202B": "ADVANCED TELECOMMUNICATIONS MODULES, LTD.",
    "00202C": "WELLTRONIX CO., LTD.",
    "00202D": "TAIYO CORPORATION",
    "00202E": "DAYSTAR DIGITAL",
    "00202F": "ZETA COMMUNICATIONS, LTD.",
    "002030": "ANALOG & DIGITAL SYSTEMS",
    "002031": "ERTEC GmbH",
    "002032": "ALCATEL TAISEL",
    "002033": "SYNAPSE TECHNOLOGIES, INC.",
    "002034": "ROTEC INDUSTRIEAUTOMATION GMBH",
    "002035": "IBM CORPORATION",
    "002036": "BMC SOFTWARE",
    "002037": "SEAGATE TECHNOLOGY",
    "002038": "VME MICROSYSTEMS INTERNATIONAL CORPORATION",
    "002039": "SCINETS",
    "00203A": "DIGITAL BI0METRICS INC.",
    "00203B": "WISDM LTD.",
    "00203C": "EUROTIME AB",
    "00203D": "NOVAR ELECTRONICS CORPORATION",
    "00203E": "LogiCan Technologies, Inc.",
    "00203F": "JUKI CORPORATION",
    "002040": "Motorola Broadband Communications Sector",
    "002041": "DATA NET",
    "002042": "DATAMETRICS CORP.",
    "002043": "NEURON COMPANY LIMITED",
    "002044": "GENITECH PTY LTD",
    "002045": "ION Networks, Inc.",
    "002046": "CIPRICO, INC.",
    "002047": "STEINBRECHER CORP.",
    "002048": "Marconi Communications",
    "002049": "COMTRON, INC.",
    "00204A": "PRONET GMBH",
    "00204B": "AUTOCOMPUTER CO., LTD.",
    "00204C": "MITRON COMPUTER PTE LTD.",
    "00204D": "INOVIS GMBH",
    "00204E": "NETWORK SECURITY SYSTEMS, INC.",
    "00204F": "DEUTSCHE AEROSPACE AG",
    "002050": "KOREA COMPUTER INC.",
    "002051": "Verilink Corporation",
    "002052": "RAGULA SYSTEMS",
    "002053": "HUNTSVILLE MICROSYSTEMS, INC.",
    "002054": "Sycamore Networks",
    "002055": "ALTECH CO., LTD.",
    "002056": "NEOPRODUCTS",
    "002057": "TITZE DATENTECHNIK GmbH",
    "002058": "ALLIED SIGNAL INC.",
    "002059": "MIRO COMPUTER PRODUCTS AG",
    "00205A": "COMPUTER IDENTICS",
    "00205B": "Kentrox, LLC",
    "00205C": "InterNet Systems of Florida, Inc.",
    "00205D": "NANOMATIC OY",
    "00205E": "CASTLE ROCK, INC.",
    "00205F": "GAMMADATA COMPUTER GMBH",
    "002060": "ALCATEL ITALIA S.p.A.",
    "002061": "GarrettCom, Inc.",
    "002062": "SCORPION LOGIC, LTD.",
    "002063": "WIPRO INFOTECH LTD.",
    "002064": "PROTEC MICROSYSTEMS, INC.",
    "002065": "SUPERNET NETWORKING INC.",
    "002066": "GENERAL MAGIC, INC.",
    "002067": "PRIVATE",
    "002068": "ISDYNE",
    "002069": "ISDN SYSTEMS CORPORATION",
    "00206A": "OSAKA COMPUTER CORP.",
    "00206B": "KONICA MINOLTA HOLDINGS, INC.",
    "00206C": "EVERGREEN TECHNOLOGY CORP.",
    "00206D": "DATA RACE, INC.",
    "00206E": "XACT, INC.",
    "00206F": "FLOWPOINT CORPORATION",
    "002070": "HYNET, LTD.",
    "002071": "IBR GMBH",
    "002072": "WORKLINK INNOVATIONS",
    "002073": "FUSION SYSTEMS CORPORATION",
    "002074": "SUNGWOON SYSTEMS",
    "002075": "MOTOROLA COMMUNICATION ISRAEL",
    "002076": "REUDO CORPORATION",
    "002077": "KARDIOS SYSTEMS CORP.",
    "002078": "RUNTOP, INC.",
    "002079": "MIKRON GMBH",
    "00207A": "WiSE Communications, Inc.",
    "00207B": "Intel Corporation",
    "00207C": "AUTEC GmbH",
    "00207D": "ADVANCED COMPUTER APPLICATIONS",
    "00207E": "FINECOM Co., Ltd.",
    "00207F": "KYOEI SANGYO CO., LTD.",
    "002080": "SYNERGY (UK) LTD.",
    "002081": "TITAN ELECTRONICS",
    "002082": "ONEAC CORPORATION",
    "002083": "PRESTICOM INCORPORATED",
    "002084": "OCE PRINTING SYSTEMS, GMBH",
    "002085": "EXIDE ELECTRONICS",
    "002086": "MICROTECH ELECTRONICS LIMITED",
    "002087": "MEMOTEC COMMUNICATIONS CORP.",
    "002088": "GLOBAL VILLAGE COMMUNICATION",
    "002089": "T3PLUS NETWORKING, INC.",
    "00208A": "SONIX COMMUNICATIONS, LTD.",
    "00208B": "LAPIS TECHNOLOGIES, INC.",
    "00208C": "GALAXY NETWORKS, INC.",
    "00208D": "CMD TECHNOLOGY",
    "00208E": "CHEVIN SOFTWARE ENG. LTD.",
    "00208F": "ECI TELECOM LTD.",
    "002090": "ADVANCED COMPRESSION TECHNOLOGY, INC.",
    "002091": "J125, NATIONAL SECURITY AGENCY",
    "002092": "CHESS ENGINEERING B.V.",
    "002093": "LANDINGS TECHNOLOGY CORP.",
    "002094": "CUBIX CORPORATION",
    "002095": "RIVA ELECTRONICS",
    "002096": "Invensys",
    "002097": "APPLIED SIGNAL TECHNOLOGY",
    "002098": "HECTRONIC AB",
    "002099": "BON ELECTRIC CO., LTD.",
    "00209A": "THE 3DO COMPANY",
    "00209B": "ERSAT ELECTRONIC GMBH",
    "00209C": "PRIMARY ACCESS CORP.",
    "00209D": "LIPPERT AUTOMATIONSTECHNIK",
    "00209E": "BROWN'S OPERATING SYSTEM SERVICES, LTD.",
    "00209F": "MERCURY COMPUTER SYSTEMS, INC.",
    "0020A0": "OA LABORATORY CO., LTD.",
    "0020A1": "DOVATRON",
    "0020A2": "GALCOM NETWORKING LTD.",
    "0020A3": "DIVICOM INC.",
    "0020A4": "MULTIPOINT NETWORKS",
    "0020A5": "API ENGINEERING",
    "0020A6": "PROXIM, INC.",
    "0020A7": "PAIRGAIN TECHNOLOGIES, INC.",
    "0020A8": "SAST TECHNOLOGY CORP.",
    "0020A9": "WHITE HORSE INDUSTRIAL",
    "0020AA": "DIGIMEDIA VISION LTD.",
    "0020AB": "MICRO INDUSTRIES CORP.",
    "0020AC": "INTERFLEX DATENSYSTEME GMBH",
    "0020AD": "LINQ SYSTEMS",
    "0020AE": "ORNET DATA COMMUNICATION TECH.",
    "0020AF": "3COM CORPORATION",
    "0020B0": "GATEWAY DEVICES, INC.",
    "0020B1": "COMTECH RESEARCH INC.",
    "0020B2": "GKD Gesellschaft Fur Kommunikation Und Datentechnik",
    "0020B3": "SCLTEC COMMUNICATIONS SYSTEMS",
    "0020B4": "TERMA ELEKTRONIK AS",
    "0020B5": "YASKAWA ELECTRIC CORPORATION",
    "0020B6": "AGILE NETWORKS, INC.",
    "0020B7": "NAMAQUA COMPUTERWARE",
    "0020B8": "PRIME OPTION, INC.",
    "0020B9": "METRICOM, INC.",
    "0020BA": "CENTER FOR HIGH PERFORMANCE",
    "0020BB": "ZAX CORPORATION",
    "0020BC": "Long Reach Networks Pty Ltd",
    "0020BD": "NIOBRARA R & D CORPORATION",
    "0020BE": "LAN ACCESS CORP.",
    "0020BF": "AEHR TEST SYSTEMS",
    "0020C0": "PULSE ELECTRONICS, INC.",
    "0020C1": "SAXA, Inc.",
    "0020C2": "TEXAS MEMORY SYSTEMS, INC.",
    "0020C3": "COUNTER SOLUTIONS LTD.",
    "0020C4": "INET,INC.",
    "0020C5": "EAGLE TECHNOLOGY",
    "0020C6": "NECTEC",
    "0020C7": "AKAI Professional M.I. Corp.",
    "0020C8": "LARSCOM INCORPORATED",
    "0020C9": "VICTRON BV",
    "0020CA": "DIGITAL OCEAN",
    "0020CB": "PRETEC ELECTRONICS CORP.",
    "0020CC": "DIGITAL SERVICES, LTD.",
    "0020CD": "HYBRID NETWORKS, INC.",
    "0020CE": "LOGICAL DESIGN GROUP, INC.",
    "0020CF": "TEST & MEASUREMENT SYSTEMS INC",
    "0020D0": "VERSALYNX CORPORATION",
    "0020D1": "MICROCOMPUTER SYSTEMS (M) SDN.",
    "0020D2": "RAD DATA COMMUNICATIONS, LTD.",
    "0020D3": "OST (OUEST STANDARD TELEMATIQU",
    "0020D4": "CABLETRON - ZEITTNET INC.",
    "0020D5": "VIPA GMBH",
    "0020D6": "BREEZECOM",
    "0020D7": "JAPAN MINICOMPUTER SYSTEMS CO., Ltd.",
    "0020D8": "Nortel Networks",
    "0020D9": "PANASONIC TECHNOLOGIES, INC./MIECO-US",
    "0020DA": "Alcatel North America ESD",
    "0020DB": "XNET TECHNOLOGY, INC.",
    "0020DC": "DENSITRON TAIWAN LTD.",
    "0020DD": "Cybertec Pty Ltd",
    "0020DE": "JAPAN DIGITAL LABORAT'Y CO.LTD",
    "0020DF": "KYOSAN ELECTRIC MFG. CO., LTD.",
    "0020E0": "Actiontec Electronics, Inc.",
    "0020E1": "ALAMAR ELECTRONICS",
    "0020E2": "INFORMATION RESOURCE ENGINEERING",
    "0020E3": "MCD KENCOM CORPORATION",
    "0020E4": "HSING TECH ENTERPRISE CO., LTD",
    "0020E5": "APEX DATA, INC.",
    "0020E6": "LIDKOPING MACHINE TOOLS AB",
    "0020E7": "B&W NUCLEAR SERVICE COMPANY",
    "0020E8": "DATATREK CORPORATION",
    "0020E9": "DANTEL",
    "0020EA": "EFFICIENT NETWORKS, INC.",
    "0020EB": "CINCINNATI MICROWAVE, INC.",
    "0020EC": "TECHWARE SYSTEMS CORP.",
    "0020ED": "GIGA-BYTE TECHNOLOGY CO., LTD.",
    "0020EE": "GTECH CORPORATION",
    "0020EF": "USC CORPORATION",
    "0020F0": "UNIVERSAL MICROELECTRONICS CO.",
    "0020F1": "ALTOS INDIA LIMITED",
    "0020F2": "SUN MICROSYSTEMS, INC.",
    "0020F3": "RAYNET CORPORATION",
    "0020F4": "SPECTRIX CORPORATION",
    "0020F5": "PANDATEL AG",
    "0020F6": "NET TEK AND KARLNET, INC.",
    "0020F7": "CYBERDATA",
    "0020F8": "CARRERA COMPUTERS, INC.",
    "0020F9": "PARALINK NETWORKS, INC.",
    "0020FA": "GDE SYSTEMS, INC.",
    "0020FB": "OCTEL COMMUNICATIONS CORP.",
    "0020FC": "MATROX",
    "0020FD": "ITV TECHNOLOGIES, INC.",
    "0020FE": "TOPWARE INC. / GRAND COMPUTER",
    "0020FF": "SYMMETRICAL TECHNOLOGIES",
    "002100": "GemTek Technology Co., Ltd.",
    "002101": "Aplicaciones Electronicas Quasar (AEQ)",
    "002102": "UpdateLogic Inc.",
    "002103": "GHI Electronics, LLC",
    "002104": "Siemens Home and Office Communication Devices GmbH&Co. KG",
    "002105": "Alcatel-Lucent",
    "002106": "RIM Testing Services",
    "002107": "Seowonintech Co Ltd.",
    "002108": "Nokia Danmark A/S",
    "002109": "Nokia Danmark A/S",
    "00210A": "byd:sign Corporation",
    "00210B": "GEMINI TRAZE RFID PVT. LTD.",
    "00210C": "Cymtec Systems, Inc.",
    "00210D": "SAMSIN INNOTEC",
    "00210E": "Orpak Systems L.T.D.",
    "00210F": "Cernium Corp",
    "002110": "Clearbox Systems",
    "002111": "Uniphone Inc.",
    "002112": "WISCOM SYSTEM CO.,LTD",
    "002113": "Padtec S/A",
    "002114": "Hylab Technology Inc.",
    "002115": "PHYWE Systeme GmbH & Co. KG",
    "002116": "Transcon Electronic Systems, spol. s r. o.",
    "002117": "Tellord",
    "002118": "Athena Tech, Inc.",
    "002119": "Samsung Electro-Mechanics",
    "00211A": "LInTech Corporation",
    "00211B": "Cisco Systems",
    "00211C": "Cisco Systems",
    "00211D": "Dataline AB",
    "00211E": "Motorola CHS",
    "00211F": "SHINSUNG DELTATECH CO.,LTD.",
    "002120": "Sequel Technologies, LLC",
    "002121": "VRmagic GmbH",
    "002122": "Chip-pro Ltd.",
    "002123": "Aerosat Avionics",
    "002124": "Optos Plc",
    "002125": "KUK JE TONG SHIN Co.,LTD",
    "002126": "Shenzhen Torch Equipment Co., Ltd.",
    "002127": "TP-LINK Technology Co., Ltd.",
    "002128": "Sun Microsystems Inc",
    "002129": "Cisco-Linksys, LLC",
    "00212A": "Audiovox Corporation",
    "00212B": "MSA Auer",
    "00212C": "SemIndia System Private Limited",
    "00212D": "SCIMOLEX CORPORATION",
    "00212E": "dresden-elektronik",
    "00212F": "Phoebe Micro Inc.",
    "002130": "Keico Hightech Inc.",
    "002131": "Blynke Inc.",
    "002132": "Masterclock, Inc.",
    "002133": "Building B, Inc",
    "002134": "Brandywine Communications",
    "002135": "ALCATEL-LUCENT",
    "002136": "Motorola Mobile Devices business (MDb)",
    "002137": "Bay Controls, LLC",
    "002138": "Cepheid",
    "002139": "Escherlogic Inc.",
    "00213A": "Winchester Systems Inc.",
    "00213B": "Berkshire Products, Inc",
    "00213C": "AliphCom",
    "00213D": "Cermetek Microelectronics, Inc.",
    "00213E": "TomTom",
    "00213F": "A-Team Technology Ltd.",
    "002140": "EN Technologies Inc.",
    "002141": "RADLIVE",
    "002142": "ABS Control Systems doo",
    "002143": "Motorola CHS",
    "002144": "SS Telecoms",
    "002145": "Semptian Technologies Ltd.",
    "002146": "SCI Technology",
    "002147": "Nintendo Co., Ltd.",
    "002148": "Kaco Solar Korea",
    "002149": "China Daheng Group ,Inc.",
    "00214A": "Pixel Velocity, Inc",
    "00214B": "Shenzhen HAMP Science & Technology Co.,Ltd",
    "00214C": "SAMSUNG ELECTRONICS CO., LTD.",
    "00214D": "Guangzhou Skytone Transmission Technology Com. Ltd.",
    "00214E": "GS Yuasa Power Supply Ltd.",
    "00214F": "ALPS Electric Co., Ltd",
    "002150": "EYEVIEW ELECTRONICS",
    "002151": "Millinet Co., Ltd.",
    "002152": "General Satellite Trading Limited",
    "002153": "SeaMicro Inc.",
    "002154": "D-TACQ Solutions Ltd",
    "002155": "Cisco Systems",
    "002156": "Cisco Systems",
    "002157": "National Datacast, Inc.",
    "002158": "Style Flying Technology Co.",
    "002159": "Juniper Networks",
    "00215A": "Hewlett Packard",
    "00215B": "Inotive",
    "00215C": "Intel Corporate",
    "00215D": "Intel Corporate",
    "00215E": "IBM",
    "00215F": "IHSE GmbH",
    "002160": "Hidea Solutions Co. Ltd.",
    "002161": "Yournet Inc.",
    "002162": "Nortel",
    "002163": "ASKEY COMPUTER CORP",
    "002164": "Special Design Bureau for Seismic Instrumentation",
    "002165": "Presstek Inc.",
    "002166": "NovAtel Inc.",
    "002167": "HWA JIN T&I Corp.",
    "002168": "iVeia, LLC",
    "002169": "Prologix, LLC.",
    "00216A": "Intel Corporate",
    "00216B": "Intel Corporate",
    "00216C": "ODVA",
    "00216D": "Soltech Co., Ltd.",
    "00216E": "Function ATI (Huizhou) Telecommunications Co., Ltd.",
    "00216F": "SymCom, Inc.",
    "002170": "Dell Inc",
    "002171": "Wesung TNC Co., Ltd.",
    "002172": "Seoultek Valley",
    "002173": "Ion Torrent Systems, Inc.",
    "002174": "AvaLAN Wireless",
    "002175": "Pacific Satellite International Ltd.",
    "002176": "YMax Telecom Ltd.",
    "002177": "W. L. Gore & Associates",
    "002178": "Matuschek Messtechnik GmbH",
    "002179": "IOGEAR, Inc.",
    "00217A": "Sejin Electron, Inc.",
    "00217B": "Bastec AB",
    "00217C": "2Wire",
    "00217D": "PYXIS S.R.L.",
    "00217E": "Telit Communication s.p.a",
    "00217F": "Intraco Technology Pte Ltd",
    "002180": "Motorola CHS",
    "002181": "Si2 Microsystems Limited",
    "002182": "SandLinks Systems, Ltd.",
    "002183": "VATECH HYDRO",
    "002184": "POWERSOFT SRL",
    "002185": "MICRO-STAR INT'L CO.,LTD.",
    "002186": "USI",
    "002187": "Imacs GmbH",
    "002188": "Data Domain, Inc.",
    "002189": "AppTech, Inc.",
    "00218A": "Electronic Design and Manufacturing Company",
    "00218B": "Wescon Technology, Inc.",
    "00218C": "TopControl GMBH",
    "00218D": "AP Router Ind. Eletronica LTDA",
    "00218E": "MEKICS CO., LTD.",
    "00218F": "Avantgarde Acoustic Lautsprechersysteme GmbH",
    "002190": "Goliath Solutions",
    "002191": "D-Link Corporation",
    "002192": "Baoding Galaxy Electronic Technology Co.,Ltd",
    "002193": "Videofon MV",
    "002194": "Ping Communication",
    "002195": "GWD Media Limited",
    "002196": "Telsey S.p.A.",
    "002197": "ELITEGROUP COMPUTER SYSTEM",
    "002198": "Thai Radio Co, LTD",
    "002199": "Vacon Plc",
    "00219A": "Cambridge Visual Networks Ltd",
    "00219B": "Dell Inc",
    "00219C": "Honeywld Technology Corp.",
    "00219D": "Adesys BV",
    "00219E": "Sony Ericsson Mobile Communications",
    "00219F": "SATEL OY",
    "0021A0": "Cisco Systems",
    "0021A1": "Cisco Systems",
    "0021A2": "EKE-Electronics Ltd.",
    "0021A3": "Micromint",
    "0021A4": "Dbii Networks",
    "0021A5": "ERLPhase Power Technologies Ltd.",
    "0021A6": "Videotec Spa",
    "0021A7": "Hantle System Co., Ltd.",
    "0021A8": "Telephonics Corporation",
    "0021A9": "Mobilink Telecom Co.,Ltd",
    "0021AA": "Nokia Danmark A/S",
    "0021AB": "Nokia Danmark A/S",
    "0021AC": "Infrared Integrated Systems Ltd",
    "0021AD": "Nordic ID Oy",
    "0021AE": "ALCATEL-LUCENT FRANCE - WTD",
    "0021AF": "Radio Frequency Systems",
    "0021B0": "Tyco Telecommunications",
    "0021B1": "DIGITAL SOLUTIONS LTD",
    "0021B2": "Fiberblaze A/S",
    "0021B3": "Ross Controls",
    "0021B4": "APRO MEDIA CO., LTD",
    "0021B5": "Vyro Games Limited",
    "0021B6": "Triacta Power Technologies Inc.",
    "0021B7": "Lexmark International Inc.",
    "0021B8": "Inphi Corporation",
    "0021B9": "Universal Devices Inc.",
    "0021BA": "Texas Instruments",
    "0021BB": "Riken Keiki Co., Ltd.",
    "0021BC": "ZALA COMPUTER",
    "0021BD": "Nintendo Co., Ltd.",
    "0021BE": "Cisco, Service Provider Video Technology Group",
    "0021BF": "Hitachi High-Tech Control Systems Corporation",
    "0021C0": "Mobile Appliance, Inc.",
    "0021C1": "ABB Oy / Distribution Automation",
    "0021C2": "GL Communications Inc",
    "0021C3": "CORNELL Communications, Inc.",
    "0021C4": "Consilium AB",
    "0021C5": "3DSP Corp",
    "0021C6": "CSJ Global, Inc.",
    "0021C7": "Russound",
    "0021C8": "LOHUIS Networks",
    "0021C9": "Wavecom Asia Pacific Limited",
    "0021CA": "ART System Co., Ltd.",
    "0021CB": "SMS TECNOLOGIA ELETRONICA LTDA",
    "0021CC": "Flextronics International",
    "0021CD": "LiveTV",
    "0021CE": "NTC-Metrotek",
    "0021CF": "The Crypto Group",
    "0021D0": "Global Display Solutions Spa",
    "0021D1": "Samsung Electronics Co.,Ltd",
    "0021D2": "Samsung Electronics Co.,Ltd",
    "0021D3": "BOCOM SECURITY(ASIA PACIFIC) LIMITED",
    "0021D4": "Vollmer Werke GmbH",
    "0021D5": "X2E GmbH",
    "0021D6": "LXI Consortium",
    "0021D7": "Cisco Systems",
    "0021D8": "Cisco Systems",
    "0021D9": "SEKONIC CORPORATION",
    "0021DA": "Automation Products Group Inc.",
    "0021DB": "Santachi Video Technology (Shenzhen) Co., Ltd.",
    "0021DC": "TECNOALARM S.r.l.",
    "0021DD": "Northstar Systems Corp",
    "0021DE": "Firepro Wireless",
    "0021DF": "Martin Christ GmbH",
    "0021E0": "CommAgility Ltd",
    "0021E1": "Nortel Networks",
    "0021E2": "Creative Electronic GmbH",
    "0021E3": "BusTek LLC",
    "0021E4": "I-WIN",
    "0021E5": "Display Solution AG",
    "0021E6": "Starlight Video Limited",
    "0021E7": "Informatics Services Corporation",
    "0021E8": "Murata Manufacturing Co., Ltd.",
    "0021E9": "Apple, Inc",
    "0021EA": "Bystronic Laser AG",
    "0021EB": "ESP SYSTEMS, LLC",
    "0021EC": "Solutronic GmbH",
    "0021ED": "Telegesis",
    "0021EE": "Full Spectrum Inc.",
    "0021EF": "Kapsys",
    "0021F0": "EW3 Technologies LLC",
    "0021F1": "Tutus Data AB",
    "0021F2": "EASY3CALL Technology Limited",
    "0021F3": "M31 SpA",
    "0021F4": "INRange Systems, Inc",
    "0021F5": "Western Engravers Supply, Inc.",
    "0021F6": "Virtual Iron Software",
    "0021F7": "ProCurve Networking by HP",
    "0021F8": "Enseo, Inc.",
    "0021F9": "WIRECOM Technologies",
    "0021FA": "A4SP Technologies Ltd.",
    "0021FB": "LG Electronics",
    "0021FC": "Nokia Danmark A/S",
    "0021FD": "DSTA S.L.",
    "0021FE": "Nokia Danmark A/S",
    "0021FF": "Cyfrowy Polsat SA",
    "002200": "BLADE Network Technology",
    "002201": "Aksys Networks Inc",
    "002202": "Excito Elektronik i Skåne AB",
    "002203": "Glensound Electronics Ltd",
    "002204": "KORATEK",
    "002205": "WeLink Solutions, Inc.",
    "002206": "Cyberdyne Inc.",
    "002207": "Inteno Broadband Technology AB",
    "002208": "Certicom Corp",
    "002209": "Omron Healthcare Co., Ltd",
    "00220A": "Rearden Labs",
    "00220B": "National Source Coding Center",
    "00220C": "Cisco Systems",
    "00220D": "Cisco Systems",
    "00220E": "Indigo Security Co., Ltd.",
    "00220F": "MoCA (Multimedia over Coax Alliance)",
    "002210": "Motorola CHS",
    "002211": "Rohati Systems",
    "002212": "CAI Networks, Inc.",
    "002213": "PCI CORPORATION",
    "002214": "RINNAI KOREA",
    "002215": "ASUSTek COMPUTER INC.",
    "002216": "SHIBAURA VENDING MACHINE CORPORATION",
    "002217": "Neat Electronics",
    "002218": "Verivue Inc.",
    "002219": "Dell Inc",
    "00221A": "Audio Precision",
    "00221B": "Morega Systems",
    "00221C": "PRIVATE",
    "00221D": "Freegene Technology LTD",
    "00221E": "Media Devices Co., Ltd.",
    "00221F": "eSang Technologies Co., Ltd.",
    "002220": "Mitac Technology Corp",
    "002221": "ITOH DENKI CO,LTD.",
    "002222": "Betec Engineering GmbH",
    "002223": "TimeKeeping Systems, Inc.",
    "002224": "Good Will Instrument Co., Ltd.",
    "002225": "Thales Avionics Ltd",
    "002226": "Avaak, Inc.",
    "002227": "uv-electronic GmbH",
    "002228": "Breeze Innovations Ltd.",
    "002229": "Compumedics Ltd",
    "00222A": "SoundEar A/S",
    "00222B": "Nucomm, Inc.",
    "00222C": "Ceton Corp",
    "00222D": "SMC Networks Inc.",
    "00222E": "maintech GmbH",
    "00222F": "Open Grid Computing, Inc.",
    "002230": "FutureLogic Inc.",
    "002231": "SMT&C Co., Ltd.",
    "002232": "Design Design Technology Ltd",
    "002233": "Pirelli Broadband Solutions",
    "002234": "Corventis Inc.",
    "002654": "3Com Corporation",
    "003000": "ALLWELL TECHNOLOGY CORP.",
    "003001": "SMP",
    "003002": "Expand Networks",
    "003003": "Phasys Ltd.",
    "003004": "LEADTEK RESEARCH INC.",
    "003005": "Fujitsu Siemens Computers",
    "003006": "SUPERPOWER COMPUTER",
    "003007": "OPTI, INC.",
    "003008": "AVIO DIGITAL, INC.",
    "003009": "Tachion Networks, Inc.",
    "00300A": "AZTECH SYSTEMS LTD.",
    "00300B": "mPHASE Technologies, Inc.",
    "00300C": "CONGRUENCY, LTD.",
    "00300D": "MMC Technology, Inc.",
    "00300E": "Klotz Digital AG",
    "00300F": "IMT - Information Management T",
    "003010": "VISIONETICS INTERNATIONAL",
    "003011": "HMS FIELDBUS SYSTEMS AB",
    "003012": "DIGITAL ENGINEERING LTD.",
    "003013": "NEC Corporation",
    "003014": "DIVIO, INC.",
    "003015": "CP CLARE CORP.",
    "003016": "ISHIDA CO., LTD.",
    "003017": "BlueArc UK Ltd",
    "003018": "Jetway Information Co., Ltd.",
    "003019": "CISCO SYSTEMS, INC.",
    "00301A": "SMARTBRIDGES PTE. LTD.",
    "00301B": "SHUTTLE, INC.",
    "00301C": "ALTVATER AIRDATA SYSTEMS",
    "00301D": "SKYSTREAM, INC.",
    "00301E": "3COM Europe Ltd.",
    "00301F": "OPTICAL NETWORKS, INC.",
    "003020": "TSI, Inc..",
    "003021": "HSING TECH. ENTERPRISE CO.,LTD",
    "003022": "Fong Kai Industrial Co., Ltd.",
    "003023": "COGENT COMPUTER SYSTEMS, INC.",
    "003024": "CISCO SYSTEMS, INC.",
    "003025": "CHECKOUT COMPUTER SYSTEMS, LTD",
    "003026": "HeiTel Digital Video GmbH",
    "003027": "KERBANGO, INC.",
    "003028": "FASE Saldatura srl",
    "003029": "OPICOM",
    "00302A": "SOUTHERN INFORMATION",
    "00302B": "INALP NETWORKS, INC.",
    "00302C": "SYLANTRO SYSTEMS CORPORATION",
    "00302D": "QUANTUM BRIDGE COMMUNICATIONS",
    "00302E": "Hoft & Wessel AG",
    "00302F": "Smiths Industries",
    "003030": "HARMONIX CORPORATION",
    "003031": "LIGHTWAVE COMMUNICATIONS, INC.",
    "003032": "MagicRam, Inc.",
    "003033": "ORIENT TELECOM CO., LTD.",
    "003034": "SET ENGINEERING",
    "003035": "Corning Incorporated",
    "003036": "RMP ELEKTRONIKSYSTEME GMBH",
    "003037": "Packard Bell Nec Services",
    "003038": "XCP, INC.",
    "003039": "SOFTBOOK PRESS",
    "00303A": "MAATEL",
    "00303B": "PowerCom Technology",
    "00303C": "ONNTO CORP.",
    "00303D": "IVA CORPORATION",
    "00303E": "Radcom Ltd.",
    "00303F": "TurboComm Tech Inc.",
    "003040": "CISCO SYSTEMS, INC.",
    "003041": "SAEJIN T & M CO., LTD.",
    "003042": "DeTeWe-Deutsche Telephonwerke",
    "003043": "IDREAM TECHNOLOGIES, PTE. LTD.",
    "003044": "Portsmith LLC",
    "003045": "Village Networks, Inc. (VNI)",
    "003046": "Controlled Electronic Manageme",
    "003047": "NISSEI ELECTRIC CO., LTD.",
    "003048": "Supermicro Computer, Inc.",
    "003049": "BRYANT TECHNOLOGY, LTD.",
    "00304A": "Fraunhofer IPMS",
    "00304B": "ORBACOM SYSTEMS, INC.",
    "00304C": "APPIAN COMMUNICATIONS, INC.",
    "00304D": "ESI",
    "00304E": "BUSTEC PRODUCTION LTD.",
    "00304F": "PLANET Technology Corporation",
    "003050": "Versa Technology",
    "003051": "ORBIT AVIONIC & COMMUNICATION",
    "003052": "ELASTIC NETWORKS",
    "003053": "Basler AG",
    "003054": "CASTLENET TECHNOLOGY, INC.",
    "003055": "Renesas Technology America, Inc.",
    "003056": "Beck IPC GmbH",
    "003057": "QTelNet, Inc.",
    "003058": "API MOTION",
    "003059": "DIGITAL-LOGIC AG",
    "00305A": "TELGEN CORPORATION",
    "00305B": "Toko Inc.",
    "00305C": "SMAR Laboratories Corp.",
    "00305D": "DIGITRA SYSTEMS, INC.",
    "00305E": "Abelko Innovation",
    "00305F": "Hasselblad",
    "003060": "Powerfile, Inc.",
    "003061": "MobyTEL",
    "003062": "PATH 1 NETWORK TECHNOL'S INC.",
    "003063": "SANTERA SYSTEMS, INC.",
    "003064": "ADLINK TECHNOLOGY, INC.",
    "003065": "APPLE COMPUTER, INC.",
    "003066": "Cirronet, Inc.",
    "003067": "BIOSTAR MICROTECH INT'L CORP.",
    "003068": "CYBERNETICS TECH. CO., LTD.",
    "003069": "IMPACCT TECHNOLOGY CORP.",
    "00306A": "PENTA MEDIA CO., LTD.",
    "00306B": "CMOS SYSTEMS, INC.",
    "00306C": "Hitex Holding GmbH",
    "00306D": "LUCENT TECHNOLOGIES",
    "00306E": "HEWLETT PACKARD",
    "00306F": "SEYEON TECH. CO., LTD.",
    "003070": "1Net Corporation",
    "003071": "Cisco Systems, Inc.",
    "003072": "Intellibyte Inc.",
    "003073": "International Microsystems, In",
    "003074": "EQUIINET LTD.",
    "003075": "ADTECH",
    "003076": "Akamba Corporation",
    "003077": "ONPREM NETWORKS",
    "003078": "Cisco Systems, Inc.",
    "003079": "CQOS, INC.",
    "00307A": "Advanced Technology & Systems",
    "00307B": "Cisco Systems, Inc.",
    "00307C": "ADID SA",
    "00307D": "GRE AMERICA, INC.",
    "00307E": "Redflex Communication Systems",
    "00307F": "IRLAN LTD.",
    "003080": "CISCO SYSTEMS, INC.",
    "003081": "ALTOS C&C",
    "003082": "TAIHAN ELECTRIC WIRE CO., LTD.",
    "003083": "Ivron Systems",
    "003084": "ALLIED TELESYN INTERNAIONAL",
    "003085": "CISCO SYSTEMS, INC.",
    "003086": "Transistor Devices, Inc.",
    "003087": "VEGA GRIESHABER KG",
    "003088": "Siara Systems, Inc.",
    "003089": "Spectrapoint Wireless, LLC",
    "00308A": "NICOTRA SISTEMI S.P.A",
    "00308B": "Brix Networks",
    "00308C": "ADVANCED DIGITAL INFORMATION",
    "00308D": "Pinnacle Systems, Inc.",
    "00308E": "CROSS MATCH TECHNOLOGIES, INC.",
    "00308F": "MICRILOR, Inc.",
    "003090": "CYRA TECHNOLOGIES, INC.",
    "003091": "TAIWAN FIRST LINE ELEC. CORP.",
    "003092": "ModuNORM GmbH",
    "003093": "SONNET TECHNOLOGIES, INC.",
    "003094": "Cisco Systems, Inc.",
    "003095": "Procomp Informatics, Ltd.",
    "003096": "CISCO SYSTEMS, INC.",
    "003097": "AB Regin",
    "003098": "Global Converging Technologies",
    "003099": "BOENIG UND KALLENBACH OHG",
    "00309A": "ASTRO TERRA CORP.",
    "00309B": "Smartware",
    "00309C": "Timing Applications, Inc.",
    "00309D": "Nimble Microsystems, Inc.",
    "00309E": "WORKBIT CORPORATION.",
    "00309F": "AMBER NETWORKS",
    "0030A0": "TYCO SUBMARINE SYSTEMS, LTD.",
    "0030A1": "WEBGATE Inc.",
    "0030A2": "Lightner Engineering",
    "0030A3": "CISCO SYSTEMS, INC.",
    "0030A4": "Woodwind Communications System",
    "0030A5": "ACTIVE POWER",
    "0030A6": "VIANET TECHNOLOGIES, LTD.",
    "0030A7": "SCHWEITZER ENGINEERING",
    "0030A8": "OL'E COMMUNICATIONS, INC.",
    "0030A9": "Netiverse, Inc.",
    "0030AA": "AXUS MICROSYSTEMS, INC.",
    "0030AB": "DELTA NETWORKS, INC.",
    "0030AC": "Systeme Lauer GmbH & Co., Ltd.",
    "0030AD": "SHANGHAI COMMUNICATION",
    "0030AE": "Times N System, Inc.",
    "0030AF": "Honeywell GmbH",
    "0030B0": "Convergenet Technologies",
    "0030B1": "aXess-pro networks GmbH",
    "0030B2": "L-3 Sonoma EO",
    "0030B3": "San Valley Systems, Inc.",
    "0030B4": "INTERSIL CORP.",
    "0030B5": "Tadiran Microwave Networks",
    "0030B6": "CISCO SYSTEMS, INC.",
    "0030B7": "Teletrol Systems, Inc.",
    "0030B8": "RiverDelta Networks",
    "0030B9": "ECTEL",
    "0030BA": "AC&T SYSTEM CO., LTD.",
    "0030BB": "CacheFlow, Inc.",
    "0030BC": "Optronic AG",
    "0030BD": "BELKIN COMPONENTS",
    "0030BE": "City-Net Technology, Inc.",
    "0030BF": "MULTIDATA GMBH",
    "0030C0": "Lara Technology, Inc.",
    "0030C1": "HEWLETT-PACKARD",
    "0030C2": "COMONE",
    "0030C3": "FLUECKIGER ELEKTRONIK AG",
    "0030C4": "Canon Imaging Systems Inc.",
    "0030C5": "CADENCE DESIGN SYSTEMS",
    "0030C6": "CONTROL SOLUTIONS, INC.",
    "0030C7": "Macromate Corp.",
    "0030C8": "GAD LINE, LTD.",
    "0030C9": "LuxN, N",
    "0030CA": "Discovery Com",
    "0030CB": "OMNI FLOW COMPUTERS, INC.",
    "0030CC": "Tenor Networks, Inc.",
    "0030CD": "CONEXANT SYSTEMS, INC.",
    "0030CE": "Zaffire",
    "0030CF": "TWO TECHNOLOGIES, INC.",
    "0030D0": "Tellabs",
    "0030D1": "INOVA CORPORATION",
    "0030D2": "WIN TECHNOLOGIES, CO., LTD.",
    "0030D3": "Agilent Technologies",
    "0030D4": "AAE Systems, Inc",
    "0030D5": "DResearch GmbH",
    "0030D6": "MSC VERTRIEBS GMBH",
    "0030D7": "Innovative Systems, L.L.C.",
    "0030D8": "SITEK",
    "0030D9": "DATACORE SOFTWARE CORP.",
    "0030DA": "COMTREND CO.",
    "0030DB": "Mindready Solutions, Inc.",
    "0030DC": "RIGHTECH CORPORATION",
    "0030DD": "INDIGITA CORPORATION",
    "0030DE": "WAGO Kontakttechnik GmbH",
    "0030DF": "KB/TEL TELECOMUNICACIONES",
    "0030E0": "OXFORD SEMICONDUCTOR LTD.",
    "0030E1": "ACROTRON SYSTEMS, INC.",
    "0030E2": "GARNET SYSTEMS CO., LTD.",
    "0030E3": "SEDONA NETWORKS CORP.",
    "0030E4": "CHIYODA SYSTEM RIKEN",
    "0030E5": "Amper Datos S.A.",
    "0030E6": "Draeger Medical Systems, Inc.",
    "0030E7": "CNF MOBILE SOLUTIONS, INC.",
    "0030E8": "ENSIM CORP.",
    "0030E9": "GMA COMMUNICATION MANUFACT'G",
    "0030EA": "TeraForce Technology Corporation",
    "0030EB": "TURBONET COMMUNICATIONS, INC.",
    "0030EC": "BORGARDT",
    "0030ED": "Expert Magnetics Corp.",
    "0030EE": "DSG Technology, Inc.",
    "0030EF": "NEON TECHNOLOGY, INC.",
    "0030F0": "Uniform Industrial Corp.",
    "0030F1": "Accton Technology Corp.",
    "0030F2": "CISCO SYSTEMS, INC.",
    "0030F3": "At Work Computers",
    "0030F4": "STARDOT TECHNOLOGIES",
    "0030F5": "Wild Lab. Ltd.",
    "0030F6": "SECURELOGIX CORPORATION",
    "0030F7": "RAMIX INC.",
    "0030F8": "Dynapro Systems, Inc.",
    "0030F9": "Sollae Systems Co., Ltd.",
    "0030FA": "TELICA, INC.",
    "0030FB": "AZS Technology AG",
    "0030FC": "Terawave Communications, Inc.",
    "0030FD": "INTEGRATED SYSTEMS DESIGN",
    "0030FE": "DSA GmbH",
    "0030FF": "DATAFAB SYSTEMS, INC.",
    "004000": "PCI COMPONENTES DA AMZONIA LTD",
    "004001": "ZYXEL COMMUNICATIONS, INC.",
    "004002": "PERLE SYSTEMS LIMITED",
    "004003": "Emerson Process Management Power & Water Solutions, Inc.",
    "004004": "ICM CO. LTD.",
    "004005": "ANI COMMUNICATIONS INC.",
    "004006": "SAMPO TECHNOLOGY CORPORATION",
    "004007": "TELMAT INFORMATIQUE",
    "004008": "A PLUS INFO CORPORATION",
    "004009": "TACHIBANA TECTRON CO., LTD.",
    "00400A": "PIVOTAL TECHNOLOGIES, INC.",
    "00400B": "CISCO SYSTEMS, INC.",
    "00400C": "GENERAL MICRO SYSTEMS, INC.",
    "00400D": "LANNET DATA COMMUNICATIONS,LTD",
    "00400E": "MEMOTEC COMMUNICATIONS, INC.",
    "00400F": "DATACOM TECHNOLOGIES",
    "004010": "SONIC SYSTEMS, INC.",
    "004011": "ANDOVER CONTROLS CORPORATION",
    "004012": "WINDATA, INC.",
    "004013": "NTT DATA COMM. SYSTEMS CORP.",
    "004014": "COMSOFT GMBH",
    "004015": "ASCOM INFRASYS AG",
    "004016": "ADC - Global Connectivity Solutions Division",
    "004017": "Silex Technology America",
    "004018": "ADOBE SYSTEMS, INC.",
    "004019": "AEON SYSTEMS, INC.",
    "00401A": "FUJI ELECTRIC CO., LTD.",
    "00401B": "PRINTER SYSTEMS CORP.",
    "00401C": "AST RESEARCH, INC.",
    "00401D": "INVISIBLE SOFTWARE, INC.",
    "00401E": "ICC",
    "00401F": "COLORGRAPH LTD",
    "004020": "Tyco Electronics (UK) Ltd",
    "004021": "RASTER GRAPHICS",
    "004022": "KLEVER COMPUTERS, INC.",
    "004023": "LOGIC CORPORATION",
    "004024": "COMPAC INC.",
    "004025": "MOLECULAR DYNAMICS",
    "004026": "MELCO, INC.",
    "004027": "SMC MASSACHUSETTS, INC.",
    "004028": "NETCOMM LIMITED",
    "004029": "COMPEX",
    "00402A": "CANOGA-PERKINS",
    "00402B": "TRIGEM COMPUTER, INC.",
    "00402C": "ISIS DISTRIBUTED SYSTEMS, INC.",
    "00402D": "HARRIS ADACOM CORPORATION",
    "00402E": "PRECISION SOFTWARE, INC.",
    "00402F": "XLNT DESIGNS INC.",
    "004030": "GK COMPUTER",
    "004031": "KOKUSAI ELECTRIC CO., LTD",
    "004032": "DIGITAL COMMUNICATIONS",
    "004033": "ADDTRON TECHNOLOGY CO., LTD.",
    "004034": "BUSTEK CORPORATION",
    "004035": "OPCOM",
    "004036": "TRIBE COMPUTER WORKS, INC.",
    "004037": "SEA-ILAN, INC.",
    "004038": "TALENT ELECTRIC INCORPORATED",
    "004039": "OPTEC DAIICHI DENKO CO., LTD.",
    "00403A": "IMPACT TECHNOLOGIES",
    "00403B": "SYNERJET INTERNATIONAL CORP.",
    "00403C": "FORKS, INC.",
    "00403D": "TERADATA",
    "00403E": "RASTER OPS CORPORATION",
    "00403F": "SSANGYONG COMPUTER SYSTEMS",
    "004040": "RING ACCESS, INC.",
    "004041": "FUJIKURA LTD.",
    "004042": "N.A.T. GMBH",
    "004043": "Nokia Siemens Networks GmbH & Co. KG.",
    "004044": "QNIX COMPUTER CO., LTD.",
    "004045": "TWINHEAD CORPORATION",
    "004046": "UDC RESEARCH LIMITED",
    "004047": "WIND RIVER SYSTEMS",
    "004048": "SMD INFORMATICA S.A.",
    "004049": "TEGIMENTA AG",
    "00404A": "WEST AUSTRALIAN DEPARTMENT",
    "00404B": "MAPLE COMPUTER SYSTEMS",
    "00404C": "HYPERTEC PTY LTD.",
    "00404D": "TELECOMMUNICATIONS TECHNIQUES",
    "00404E": "FLUENT, INC.",
    "00404F": "SPACE & NAVAL WARFARE SYSTEMS",
    "004050": "IRONICS, INCORPORATED",
    "004051": "GRACILIS, INC.",
    "004052": "STAR TECHNOLOGIES, INC.",
    "004053": "AMPRO COMPUTERS",
    "004054": "CONNECTION MACHINES SERVICES",
    "004055": "METRONIX GMBH",
    "004056": "MCM JAPAN LTD.",
    "004057": "LOCKHEED - SANDERS",
    "004058": "KRONOS, INC.",
    "004059": "YOSHIDA KOGYO K. K.",
    "00405A": "GOLDSTAR INFORMATION & COMM.",
    "00405B": "FUNASSET LIMITED",
    "00405C": "FUTURE SYSTEMS, INC.",
    "00405D": "STAR-TEK, INC.",
    "00405E": "NORTH HILLS ISRAEL",
    "00405F": "AFE COMPUTERS LTD.",
    "004060": "COMENDEC LTD",
    "004061": "DATATECH ENTERPRISES CO., LTD.",
    "004062": "E-SYSTEMS, INC./GARLAND DIV.",
    "004063": "VIA TECHNOLOGIES, INC.",
    "004064": "KLA INSTRUMENTS CORPORATION",
    "004065": "GTE SPACENET",
    "004066": "HITACHI CABLE, LTD.",
    "004067": "OMNIBYTE CORPORATION",
    "004068": "EXTENDED SYSTEMS",
    "004069": "LEMCOM SYSTEMS, INC.",
    "00406A": "KENTEK INFORMATION SYSTEMS,INC",
    "00406B": "SYSGEN",
    "00406C": "COPERNIQUE",
    "00406D": "LANCO, INC.",
    "00406E": "COROLLARY, INC.",
    "00406F": "SYNC RESEARCH INC.",
    "004070": "INTERWARE CO., LTD.",
    "004071": "ATM COMPUTER GMBH",
    "004072": "Applied Innovation Inc.",
    "004073": "BASS ASSOCIATES",
    "004074": "CABLE AND WIRELESS",
    "004075": "M-TRADE (UK) LTD",
    "004076": "Sun Conversion Technologies",
    "004077": "MAXTON TECHNOLOGY CORPORATION",
    "004078": "WEARNES AUTOMATION PTE LTD",
    "004079": "JUKO MANUFACTURE COMPANY, LTD.",
    "00407A": "SOCIETE D'EXPLOITATION DU CNIT",
    "00407B": "SCIENTIFIC ATLANTA",
    "00407C": "QUME CORPORATION",
    "00407D": "EXTENSION TECHNOLOGY CORP.",
    "00407E": "EVERGREEN SYSTEMS, INC.",
    "00407F": "FLIR Systems",
    "004080": "ATHENIX CORPORATION",
    "004081": "MANNESMANN SCANGRAPHIC GMBH",
    "004082": "LABORATORY EQUIPMENT CORP.",
    "004083": "TDA INDUSTRIA DE PRODUTOS",
    "004084": "HONEYWELL INC.",
    "004085": "SAAB INSTRUMENTS AB",
    "004086": "MICHELS & KLEBERHOFF COMPUTER",
    "004087": "UBITREX CORPORATION",
    "004088": "MOBIUS TECHNOLOGIES, INC.",
    "004089": "MEIDENSHA CORPORATION",
    "00408A": "TPS TELEPROCESSING SYS. GMBH",
    "00408B": "RAYLAN CORPORATION",
    "00408C": "AXIS COMMUNICATIONS AB",
    "00408D": "THE GOODYEAR TIRE & RUBBER CO.",
    "00408E": "DIGILOG, INC.",
    "00408F": "WM-DATA MINFO AB",
    "004090": "ANSEL COMMUNICATIONS",
    "004091": "PROCOMP INDUSTRIA ELETRONICA",
    "004092": "ASP COMPUTER PRODUCTS, INC.",
    "004093": "PAXDATA NETWORKS LTD.",
    "004094": "SHOGRAPHICS, INC.",
    "004095": "R.P.T. INTERGROUPS INT'L LTD.",
    "004096": "Cisco Systems, Inc.",
    "004097": "DATEX DIVISION OF",
    "004098": "DRESSLER GMBH & CO.",
    "004099": "NEWGEN SYSTEMS CORP.",
    "00409A": "NETWORK EXPRESS, INC.",
    "00409B": "HAL COMPUTER SYSTEMS INC.",
    "00409C": "TRANSWARE",
    "00409D": "DIGIBOARD, INC.",
    "00409E": "CONCURRENT TECHNOLOGIES LTD.",
    "00409F": "LANCAST/CASAT TECHNOLOGY, INC.",
    "0040A0": "GOLDSTAR CO., LTD.",
    "0040A1": "ERGO COMPUTING",
    "0040A2": "KINGSTAR TECHNOLOGY INC.",
    "0040A3": "MICROUNITY SYSTEMS ENGINEERING",
    "0040A4": "ROSE ELECTRONICS",
    "0040A5": "CLINICOMP INTL.",
    "0040A6": "Cray, Inc.",
    "0040A7": "ITAUTEC PHILCO S.A.",
    "0040A8": "IMF INTERNATIONAL LTD.",
    "0040A9": "DATACOM INC.",
    "0040AA": "VALMET AUTOMATION INC.",
    "0040AB": "ROLAND DG CORPORATION",
    "0040AC": "SUPER WORKSTATION, INC.",
    "0040AD": "SMA REGELSYSTEME GMBH",
    "0040AE": "DELTA CONTROLS, INC.",
    "0040AF": "DIGITAL PRODUCTS, INC.",
    "0040B0": "BYTEX CORPORATION, ENGINEERING",
    "0040B1": "CODONICS INC.",
    "0040B2": "SYSTEMFORSCHUNG",
    "0040B3": "PAR MICROSYSTEMS CORPORATION",
    "0040B4": "NEXTCOM K.K.",
    "0040B5": "VIDEO TECHNOLOGY COMPUTERS LTD",
    "0040B6": "COMPUTERM CORPORATION",
    "0040B7": "STEALTH COMPUTER SYSTEMS",
    "0040B8": "IDEA ASSOCIATES",
    "0040B9": "MACQ ELECTRONIQUE SA",
    "0040BA": "ALLIANT COMPUTER SYSTEMS CORP.",
    "0040BB": "GOLDSTAR CABLE CO., LTD.",
    "0040BC": "ALGORITHMICS LTD.",
    "0040BD": "STARLIGHT NETWORKS, INC.",
    "0040BE": "BOEING DEFENSE & SPACE",
    "0040BF": "CHANNEL SYSTEMS INTERN'L INC.",
    "0040C0": "VISTA CONTROLS CORPORATION",
    "0040C1": "BIZERBA-WERKE WILHEIM KRAUT",
    "0040C2": "APPLIED COMPUTING DEVICES",
    "0040C3": "FISCHER AND PORTER CO.",
    "0040C4": "KINKEI SYSTEM CORPORATION",
    "0040C5": "MICOM COMMUNICATIONS INC.",
    "0040C6": "FIBERNET RESEARCH, INC.",
    "0040C7": "RUBY TECH CORPORATION",
    "0040C8": "MILAN TECHNOLOGY CORPORATION",
    "0040C9": "NCUBE",
    "0040CA": "FIRST INTERNAT'L COMPUTER, INC",
    "0040CB": "LANWAN TECHNOLOGIES",
    "0040CC": "SILCOM MANUF'G TECHNOLOGY INC.",
    "0040CD": "TERA MICROSYSTEMS, INC.",
    "0040CE": "NET-SOURCE, INC.",
    "0040CF": "STRAWBERRY TREE, INC.",
    "0040D0": "MITAC INTERNATIONAL CORP.",
    "0040D1": "FUKUDA DENSHI CO., LTD.",
    "0040D2": "PAGINE CORPORATION",
    "0040D3": "KIMPSION INTERNATIONAL CORP.",
    "0040D4": "GAGE TALKER CORP.",
    "0040D5": "SARTORIUS AG",
    "0040D6": "LOCAMATION B.V.",
    "0040D7": "STUDIO GEN INC.",
    "0040D8": "OCEAN OFFICE AUTOMATION LTD.",
    "0040D9": "AMERICAN MEGATRENDS INC.",
    "0040DA": "TELSPEC LTD",
    "0040DB": "ADVANCED TECHNICAL SOLUTIONS",
    "0040DC": "TRITEC ELECTRONIC GMBH",
    "0040DD": "HONG TECHNOLOGIES",
    "0040DE": "ELETTRONICA SAN GIORGIO",
    "0040DF": "DIGALOG SYSTEMS, INC.",
    "0040E0": "ATOMWIDE LTD.",
    "0040E1": "MARNER INTERNATIONAL, INC.",
    "0040E2": "MESA RIDGE TECHNOLOGIES, INC.",
    "0040E3": "QUIN SYSTEMS LTD",
    "0040E4": "E-M TECHNOLOGY, INC.",
    "0040E5": "SYBUS CORPORATION",
    "0040E6": "C.A.E.N.",
    "0040E7": "ARNOS INSTRUMENTS & COMPUTER",
    "0040E8": "CHARLES RIVER DATA SYSTEMS,INC",
    "0040E9": "ACCORD SYSTEMS, INC.",
    "0040EA": "PLAIN TREE SYSTEMS INC",
    "0040EB": "MARTIN MARIETTA CORPORATION",
    "0040EC": "MIKASA SYSTEM ENGINEERING",
    "0040ED": "NETWORK CONTROLS INT'NATL INC.",
    "0040EE": "OPTIMEM",
    "0040EF": "HYPERCOM, INC.",
    "0040F0": "MICRO SYSTEMS, INC.",
    "0040F1": "CHUO ELECTRONICS CO., LTD.",
    "0040F2": "JANICH & KLASS COMPUTERTECHNIK",
    "0040F3": "NETCOR",
    "0040F4": "CAMEO COMMUNICATIONS, INC.",
    "0040F5": "OEM ENGINES",
    "0040F6": "KATRON COMPUTERS INC.",
    "0040F7": "POLAROID MEDICAL IMAGING SYS.",
    "0040F8": "SYSTEMHAUS DISCOM",
    "0040F9": "COMBINET",
    "0040FA": "MICROBOARDS, INC.",
    "0040FB": "CASCADE COMMUNICATIONS CORP.",
    "0040FC": "IBR COMPUTER TECHNIK GMBH",
    "0040FD": "LXE",
    "0040FE": "SYMPLEX COMMUNICATIONS",
    "0040FF": "TELEBIT CORPORATION",
    "004252": "RLX Technologies",
    "004501": "Versus Technology, Inc.",
    "005000": "NEXO COMMUNICATIONS, INC.",
    "005001": "YAMASHITA SYSTEMS CORP.",
    "005002": "OMNISEC AG",
    "005003": "GRETAG MACBETH AG",
    "005004": "3COM CORPORATION",
    "005006": "TAC AB",
    "005007": "SIEMENS TELECOMMUNICATION SYSTEMS LIMITED",
    "005008": "TIVA MICROCOMPUTER CORP. (TMC)",
    "005009": "PHILIPS BROADBAND NETWORKS",
    "00500A": "IRIS TECHNOLOGIES, INC.",
    "00500B": "CISCO SYSTEMS, INC.",
    "00500C": "e-Tek Labs, Inc.",
    "00500D": "SATORI ELECTORIC CO., LTD.",
    "00500E": "CHROMATIS NETWORKS, INC.",
    "00500F": "CISCO SYSTEMS, INC.",
    "005010": "NovaNET Learning, Inc.",
    "005012": "CBL - GMBH",
    "005013": "Chaparral Network Storage",
    "005014": "CISCO SYSTEMS, INC.",
    "005015": "BRIGHT STAR ENGINEERING",
    "005016": "SST/WOODHEAD INDUSTRIES",
    "005017": "RSR S.R.L.",
    "005018": "AMIT, Inc.",
    "005019": "SPRING TIDE NETWORKS, INC.",
    "00501A": "UISIQN",
    "00501B": "ABL CANADA, INC.",
    "00501C": "JATOM SYSTEMS, INC.",
    "00501E": "Miranda Technologies, Inc.",
    "00501F": "MRG SYSTEMS, LTD.",
    "005020": "MEDIASTAR CO., LTD.",
    "005021": "EIS INTERNATIONAL, INC.",
    "005022": "ZONET TECHNOLOGY, INC.",
    "005023": "PG DESIGN ELECTRONICS, INC.",
    "005024": "NAVIC SYSTEMS, INC.",
    "005026": "COSYSTEMS, INC.",
    "005027": "GENICOM CORPORATION",
    "005028": "AVAL COMMUNICATIONS",
    "005029": "1394 PRINTER WORKING GROUP",
    "00502A": "CISCO SYSTEMS, INC.",
    "00502B": "GENRAD LTD.",
    "00502C": "SOYO COMPUTER, INC.",
    "00502D": "ACCEL, INC.",
    "00502E": "CAMBEX CORPORATION",
    "00502F": "TollBridge Technologies, Inc.",
    "005030": "FUTURE PLUS SYSTEMS",
    "005031": "AEROFLEX LABORATORIES, INC.",
    "005032": "PICAZO COMMUNICATIONS, INC.",
    "005033": "MAYAN NETWORKS",
    "005036": "NETCAM, LTD.",
    "005037": "KOGA ELECTRONICS CO.",
    "005038": "DAIN TELECOM CO., LTD.",
    "005039": "MARINER NETWORKS",
    "00503A": "DATONG ELECTRONICS LTD.",
    "00503B": "MEDIAFIRE CORPORATION",
    "00503C": "TSINGHUA NOVEL ELECTRONICS",
    "00503E": "CISCO SYSTEMS, INC.",
    "00503F": "ANCHOR GAMES",
    "005040": "Matsushita Electric Works, Ltd.",
    "005041": "Coretronic Corporation",
    "005042": "SCI MANUFACTURING SINGAPORE PTE, LTD.",
    "005043": "MARVELL SEMICONDUCTOR, INC.",
    "005044": "ASACA CORPORATION",
    "005045": "RIOWORKS SOLUTIONS, INC.",
    "005046": "MENICX INTERNATIONAL CO., LTD.",
    "005047": "PRIVATE",
    "005048": "INFOLIBRIA",
    "005049": "Arbor Networks Inc",
    "00504A": "ELTECO A.S.",
    "00504B": "BARCONET N.V.",
    "00504C": "Galil Motion Control",
    "00504D": "Tokyo Electron Device Limited",
    "00504E": "SIERRA MONITOR CORP.",
    "00504F": "OLENCOM ELECTRONICS",
    "005050": "CISCO SYSTEMS, INC.",
    "005051": "IWATSU ELECTRIC CO., LTD.",
    "005052": "TIARA NETWORKS, INC.",
    "005053": "CISCO SYSTEMS, INC.",
    "005054": "CISCO SYSTEMS, INC.",
    "005055": "DOMS A/S",
    "005056": "VMWare, Inc.",
    "005057": "BROADBAND ACCESS SYSTEMS",
    "005058": "VegaStream Group Limted",
    "005059": "iBAHN",
    "00505A": "NETWORK ALCHEMY, INC.",
    "00505B": "KAWASAKI LSI U.S.A., INC.",
    "00505C": "TUNDO CORPORATION",
    "00505E": "DIGITEK MICROLOGIC S.A.",
    "00505F": "BRAND INNOVATORS",
    "005060": "TANDBERG TELECOM AS",
    "005062": "KOUWELL ELECTRONICS CORP. **",
    "005063": "OY COMSEL SYSTEM AB",
    "005064": "CAE ELECTRONICS",
    "005065": "DENSEI-LAMBAD Co., Ltd.",
    "005066": "AtecoM GmbH advanced telecomunication modules",
    "005067": "AEROCOMM, INC.",
    "005068": "ELECTRONIC INDUSTRIES ASSOCIATION",
    "005069": "PixStream Incorporated",
    "00506A": "EDEVA, INC.",
    "00506B": "SPX-ATEG",
    "00506C": "G & L BEIJER ELECTRONICS AB",
    "00506D": "VIDEOJET SYSTEMS",
    "00506E": "CORDER ENGINEERING CORPORATION",
    "00506F": "G-CONNECT",
    "005070": "CHAINTECH COMPUTER CO., LTD.",
    "005071": "AIWA CO., LTD.",
    "005072": "CORVIS CORPORATION",
    "005073": "CISCO SYSTEMS, INC.",
    "005074": "ADVANCED HI-TECH CORP.",
    "005075": "KESTREL SOLUTIONS",
    "005076": "IBM",
    "005077": "PROLIFIC TECHNOLOGY, INC.",
    "005078": "MEGATON HOUSE, LTD.",
    "005079": "PRIVATE",
    "00507A": "XPEED, INC.",
    "00507B": "MERLOT COMMUNICATIONS",
    "00507C": "VIDEOCON AG",
    "00507D": "IFP",
    "00507E": "NEWER TECHNOLOGY",
    "00507F": "DrayTek Corp.",
    "005080": "CISCO SYSTEMS, INC.",
    "005081": "MURATA MACHINERY, LTD.",
    "005082": "FORESSON CORPORATION",
    "005083": "GILBARCO, INC.",
    "005084": "ATL PRODUCTS",
    "005086": "TELKOM SA, LTD.",
    "005087": "TERASAKI ELECTRIC CO., LTD.",
    "005088": "AMANO CORPORATION",
    "005089": "SAFETY MANAGEMENT SYSTEMS",
    "00508B": "Hewlett Packard",
    "00508C": "RSI SYSTEMS",
    "00508D": "ABIT COMPUTER CORPORATION",
    "00508E": "OPTIMATION, INC.",
    "00508F": "ASITA TECHNOLOGIES INT'L LTD.",
    "005090": "DCTRI",
    "005091": "NETACCESS, INC.",
    "005092": "RIGAKU INDUSTRIAL CORPORATION",
    "005093": "BOEING",
    "005094": "PACE MICRO TECHNOLOGY PLC",
    "005095": "PERACOM NETWORKS",
    "005096": "SALIX TECHNOLOGIES, INC.",
    "005097": "MMC-EMBEDDED COMPUTERTECHNIK GmbH",
    "005098": "GLOBALOOP, LTD.",
    "005099": "3COM EUROPE, LTD.",
    "00509A": "TAG ELECTRONIC SYSTEMS",
    "00509B": "SWITCHCORE AB",
    "00509C": "BETA RESEARCH",
    "00509D": "THE INDUSTREE B.V.",
    "00509E": "Les Technologies SoftAcoustik Inc.",
    "00509F": "HORIZON COMPUTER",
    "0050A0": "DELTA COMPUTER SYSTEMS, INC.",
    "0050A1": "CARLO GAVAZZI, INC.",
    "0050A2": "CISCO SYSTEMS, INC.",
    "0050A3": "TransMedia Communications, Inc.",
    "0050A4": "IO TECH, INC.",
    "0050A5": "CAPITOL BUSINESS SYSTEMS, LTD.",
    "0050A6": "OPTRONICS",
    "0050A7": "CISCO SYSTEMS, INC.",
    "0050A8": "OpenCon Systems, Inc.",
    "0050A9": "MOLDAT WIRELESS TECHNOLGIES",
    "0050AA": "KONICA MINOLTA HOLDINGS, INC.",
    "0050AB": "NALTEC, Inc.",
    "0050AC": "MAPLE COMPUTER CORPORATION",
    "0050AD": "CommUnique Wireless Corp.",
    "0050AE": "IWAKI ELECTRONICS CO., LTD.",
    "0050AF": "INTERGON, INC.",
    "0050B0": "TECHNOLOGY ATLANTA CORPORATION",
    "0050B1": "GIDDINGS & LEWIS",
    "0050B2": "BRODEL AUTOMATION",
    "0050B3": "VOICEBOARD CORPORATION",
    "0050B4": "SATCHWELL CONTROL SYSTEMS, LTD",
    "0050B5": "FICHET-BAUCHE",
    "0050B6": "GOOD WAY IND. CO., LTD.",
    "0050B7": "BOSER TECHNOLOGY CO., LTD.",
    "0050B8": "INOVA COMPUTERS GMBH & CO. KG",
    "0050B9": "XITRON TECHNOLOGIES, INC.",
    "0050BA": "D-LINK",
    "0050BB": "CMS TECHNOLOGIES",
    "0050BC": "HAMMER STORAGE SOLUTIONS",
    "0050BD": "CISCO SYSTEMS, INC.",
    "0050BE": "FAST MULTIMEDIA AG",
    "0050BF": "MOTOTECH INC.",
    "0050C0": "GATAN, INC.",
    "0050C1": "GEMFLEX NETWORKS, LTD.",
    "0050C2": "IEEE REGISTRATION AUTHORITY",
    "0050C4": "IMD",
    "0050C5": "ADS Technologies, Inc",
    "0050C6": "LOOP TELECOMMUNICATION INTERNATIONAL, INC.",
    "0050C8": "ADDONICS COMMUNICATIONS, INC.",
    "0050C9": "MASPRO DENKOH CORP.",
    "0050CA": "NET TO NET TECHNOLOGIES",
    "0050CB": "JETTER",
    "0050CC": "XYRATEX",
    "0050CD": "DIGIANSWER A/S",
    "0050CE": "LG INTERNATIONAL CORP.",
    "0050CF": "VANLINK COMMUNICATION TECHNOLOGY RESEARCH INSTITUTE",
    "0050D0": "MINERVA SYSTEMS",
    "0050D1": "CISCO SYSTEMS, INC.",
    "0050D2": "CMC Electronics Inc",
    "0050D3": "DIGITAL AUDIO PROCESSING PTY. LTD.",
    "0050D4": "JOOHONG INFORMATION &",
    "0050D5": "AD SYSTEMS CORP.",
    "0050D6": "ATLAS COPCO TOOLS AB",
    "0050D7": "TELSTRAT",
    "0050D8": "UNICORN COMPUTER CORP.",
    "0050D9": "ENGETRON-ENGENHARIA ELETRONICA IND. e COM. LTDA",
    "0050DA": "3COM CORPORATION",
    "0050DB": "CONTEMPORARY CONTROL",
    "0050DC": "TAS TELEFONBAU A. SCHWABE GMBH & CO. KG",
    "0050DD": "SERRA SOLDADURA, S.A.",
    "0050DE": "SIGNUM SYSTEMS CORP.",
    "0050DF": "AirFiber, Inc.",
    "0050E1": "NS TECH ELECTRONICS SDN BHD",
    "0050E2": "CISCO SYSTEMS, INC.",
    "0050E3": "Motorola, Inc.",
    "0050E4": "APPLE COMPUTER, INC.",
    "0050E6": "HAKUSAN CORPORATION",
    "0050E7": "PARADISE INNOVATIONS (ASIA)",
    "0050E8": "NOMADIX INC.",
    "0050EA": "XEL COMMUNICATIONS, INC.",
    "0050EB": "ALPHA-TOP CORPORATION",
    "0050EC": "OLICOM A/S",
    "0050ED": "ANDA NETWORKS",
    "0050EE": "TEK DIGITEL CORPORATION",
    "0050EF": "SPE Systemhaus GmbH",
    "0050F0": "CISCO SYSTEMS, INC.",
    "0050F1": "LIBIT SIGNAL PROCESSING, LTD.",
    "0050F2": "MICROSOFT CORP.",
    "0050F3": "GLOBAL NET INFORMATION CO., Ltd.",
    "0050F4": "SIGMATEK GMBH & CO. KG",
    "0050F6": "PAN-INTERNATIONAL INDUSTRIAL CORP.",
    "0050F7": "VENTURE MANUFACTURING (SINGAPORE) LTD.",
    "0050F8": "ENTREGA TECHNOLOGIES, INC.",
    "0050F9": "SENSORMATIC ACD",
    "0050FA": "OXTEL, LTD.",
    "0050FB": "VSK ELECTRONICS",
    "0050FC": "EDIMAX TECHNOLOGY CO., LTD.",
    "0050FD": "VISIONCOMM CO., LTD.",
    "0050FE": "PCTVnet ASA",
    "0050FF": "HAKKO ELECTRONICS CO., LTD.",
    "006000": "XYCOM INC.",
    "006001": "InnoSys, Inc.",
    "006002": "SCREEN SUBTITLING SYSTEMS, LTD",
    "006003": "TERAOKA WEIGH SYSTEM PTE, LTD.",
    "006004": "COMPUTADORES MODULARES SA",
    "006005": "FEEDBACK DATA LTD.",
    "006006": "SOTEC CO., LTD",
    "006007": "ACRES GAMING, INC.",
    "006008": "3COM CORPORATION",
    "006009": "CISCO SYSTEMS, INC.",
    "00600A": "SORD COMPUTER CORPORATION",
    "00600B": "LOGWARE GmbH",
    "00600C": "APPLIED DATA SYSTEMS, INC.",
    "00600D": "Digital Logic GmbH",
    "00600E": "WAVENET INTERNATIONAL, INC.",
    "00600F": "WESTELL, INC.",
    "006010": "NETWORK MACHINES, INC.",
    "006011": "CRYSTAL SEMICONDUCTOR CORP.",
    "006012": "POWER COMPUTING CORPORATION",
    "006013": "NETSTAL MASCHINEN AG",
    "006014": "EDEC CO., LTD.",
    "006015": "NET2NET CORPORATION",
    "006016": "CLARIION",
    "006017": "TOKIMEC INC.",
    "006018": "STELLAR ONE CORPORATION",
    "006019": "Roche Diagnostics",
    "00601A": "KEITHLEY INSTRUMENTS",
    "00601B": "MESA ELECTRONICS",
    "00601C": "TELXON CORPORATION",
    "00601D": "LUCENT TECHNOLOGIES",
    "00601E": "SOFTLAB, INC.",
    "00601F": "STALLION TECHNOLOGIES",
    "006020": "PIVOTAL NETWORKING, INC.",
    "006021": "DSC CORPORATION",
    "006022": "VICOM SYSTEMS, INC.",
    "006023": "PERICOM SEMICONDUCTOR CORP.",
    "006024": "GRADIENT TECHNOLOGIES, INC.",
    "006025": "ACTIVE IMAGING PLC",
    "006026": "VIKING COMPONENTS, INC.",
    "006027": "Superior Modular Products",
    "006028": "MACROVISION CORPORATION",
    "006029": "CARY PERIPHERALS INC.",
    "00602A": "SYMICRON COMPUTER COMMUNICATIONS, LTD.",
    "00602B": "PEAK AUDIO",
    "00602C": "LINX Data Terminals, Inc.",
    "00602D": "ALERTON TECHNOLOGIES, INC.",
    "00602E": "CYCLADES CORPORATION",
    "00602F": "CISCO SYSTEMS, INC.",
    "006030": "VILLAGE TRONIC ENTWICKLUNG",
    "006031": "HRK SYSTEMS",
    "006032": "I-CUBE, INC.",
    "006033": "ACUITY IMAGING, INC.",
    "006034": "ROBERT BOSCH GmbH",
    "006035": "DALLAS SEMICONDUCTOR, INC.",
    "006036": "AUSTRIAN RESEARCH CENTER SEIBERSDORF",
    "006037": "NXP Semiconductors",
    "006038": "Nortel Networks",
    "006039": "SanCom Technology, Inc.",
    "00603A": "QUICK CONTROLS LTD.",
    "00603B": "AMTEC spa",
    "00603C": "HAGIWARA SYS-COM CO., LTD.",
    "00603D": "3CX",
    "00603E": "CISCO SYSTEMS, INC.",
    "00603F": "PATAPSCO DESIGNS",
    "006040": "NETRO CORP.",
    "006041": "Yokogawa Electric Corporation",
    "006042": "TKS (USA), INC.",
    "006043": "iDirect, INC.",
    "006044": "LITTON/POLY-SCIENTIFIC",
    "006045": "PATHLIGHT TECHNOLOGIES",
    "006046": "VMETRO, INC.",
    "006047": "CISCO SYSTEMS, INC.",
    "006048": "EMC CORPORATION",
    "006049": "VINA TECHNOLOGIES",
    "00604A": "SAIC IDEAS GROUP",
    "00604B": "Safe-com GmbH & Co. KG",
    "00604C": "SAGEM COMMUNICATION",
    "00604D": "MMC NETWORKS, INC.",
    "00604E": "CYCLE COMPUTER CORPORATION, INC.",
    "00604F": "SUZUKI MFG. CO., LTD.",
    "006050": "INTERNIX INC.",
    "006051": "QUALITY SEMICONDUCTOR",
    "006052": "PERIPHERALS ENTERPRISE CO., Ltd.",
    "006053": "TOYODA MACHINE WORKS, LTD.",
    "006054": "CONTROLWARE GMBH",
    "006055": "CORNELL UNIVERSITY",
    "006056": "NETWORK TOOLS, INC.",
    "006057": "MURATA MANUFACTURING CO., LTD.",
    "006058": "COPPER MOUNTAIN COMMUNICATIONS, INC.",
    "006059": "TECHNICAL COMMUNICATIONS CORP.",
    "00605A": "CELCORE, INC.",
    "00605B": "IntraServer Technology, Inc.",
    "00605C": "CISCO SYSTEMS, INC.",
    "00605D": "SCANIVALVE CORP.",
    "00605E": "LIBERTY TECHNOLOGY NETWORKING",
    "00605F": "NIPPON UNISOFT CORPORATION",
    "006060": "DAWNING TECHNOLOGIES, INC.",
    "006061": "WHISTLE COMMUNICATIONS CORP.",
    "006062": "TELESYNC, INC.",
    "006063": "PSION DACOM PLC.",
    "006064": "NETCOMM LIMITED",
    "006065": "BERNECKER & RAINER INDUSTRIE-ELEKTRONIC GmbH",
    "006066": "LACROIX TECHNOLGIE",
    "006067": "ACER NETXUS INC.",
    "006068": "Dialogic Corporation",
    "006069": "Brocade Communications Systems, Inc.",
    "00606A": "MITSUBISHI WIRELESS COMMUNICATIONS. INC.",
    "00606B": "Synclayer Inc.",
    "00606C": "ARESCOM",
    "00606D": "DIGITAL EQUIPMENT CORP.",
    "00606E": "DAVICOM SEMICONDUCTOR, INC.",
    "00606F": "CLARION CORPORATION OF AMERICA",
    "006070": "CISCO SYSTEMS, INC.",
    "006071": "MIDAS LAB, INC.",
    "006072": "VXL INSTRUMENTS, LIMITED",
    "006073": "REDCREEK COMMUNICATIONS, INC.",
    "006074": "QSC AUDIO PRODUCTS",
    "006075": "PENTEK, INC.",
    "006076": "SCHLUMBERGER TECHNOLOGIES RETAIL PETROLEUM SYSTEMS",
    "006077": "PRISA NETWORKS",
    "006078": "POWER MEASUREMENT LTD.",
    "006079": "Mainstream Data, Inc.",
    "00607A": "DVS GmbH",
    "00607B": "FORE SYSTEMS, INC.",
    "00607C": "WaveAccess, Ltd.",
    "00607D": "SENTIENT NETWORKS INC.",
    "00607E": "GIGALABS, INC.",
    "00607F": "AURORA TECHNOLOGIES, INC.",
    "006080": "MICROTRONIX DATACOM LTD.",
    "006081": "TV/COM INTERNATIONAL",
    "006082": "NOVALINK TECHNOLOGIES, INC.",
    "006083": "CISCO SYSTEMS, INC.",
    "006084": "DIGITAL VIDEO",
    "006085": "Storage Concepts",
    "006086": "LOGIC REPLACEMENT TECH. LTD.",
    "006087": "KANSAI ELECTRIC CO., LTD.",
    "006088": "WHITE MOUNTAIN DSP, INC.",
    "006089": "XATA",
    "00608A": "CITADEL COMPUTER",
    "00608B": "ConferTech International",
    "00608C": "3COM CORPORATION",
    "00608D": "UNIPULSE CORP.",
    "00608E": "HE ELECTRONICS, TECHNOLOGIE & SYSTEMTECHNIK GmbH",
    "00608F": "TEKRAM TECHNOLOGY CO., LTD.",
    "006090": "ABLE COMMUNICATIONS, INC.",
    "006091": "FIRST PACIFIC NETWORKS, INC.",
    "006092": "MICRO/SYS, INC.",
    "006093": "VARIAN",
    "006094": "IBM CORP.",
    "006095": "ACCU-TIME SYSTEMS, INC.",
    "006096": "T.S. MICROTECH INC.",
    "006097": "3COM CORPORATION",
    "006098": "HT COMMUNICATIONS",
    "006099": "SBE, Inc.",
    "00609A": "NJK TECHNO CO.",
    "00609B": "ASTRO-MED, INC.",
    "00609C": "Perkin-Elmer Incorporated",
    "00609D": "PMI FOOD EQUIPMENT GROUP",
    "00609E": "ASC X3 - INFORMATION TECHNOLOGY STANDARDS SECRETARIATS",
    "00609F": "PHAST CORPORATION",
    "0060A0": "SWITCHED NETWORK TECHNOLOGIES, INC.",
    "0060A1": "VPNet, Inc.",
    "0060A2": "NIHON UNISYS LIMITED CO.",
    "0060A3": "CONTINUUM TECHNOLOGY CORP.",
    "0060A4": "GRINAKER SYSTEM TECHNOLOGIES",
    "0060A5": "PERFORMANCE TELECOM CORP.",
    "0060A6": "PARTICLE MEASURING SYSTEMS",
    "0060A7": "MICROSENS GmbH & CO. KG",
    "0060A8": "TIDOMAT AB",
    "0060A9": "GESYTEC MbH",
    "0060AA": "INTELLIGENT DEVICES INC. (IDI)",
    "0060AB": "LARSCOM INCORPORATED",
    "0060AC": "RESILIENCE CORPORATION",
    "0060AD": "MegaChips Corporation",
    "0060AE": "TRIO INFORMATION SYSTEMS AB",
    "0060AF": "PACIFIC MICRO DATA, INC.",
    "0060B0": "HEWLETT-PACKARD CO.",
    "0060B1": "INPUT/OUTPUT, INC.",
    "0060B2": "PROCESS CONTROL CORP.",
    "0060B3": "Z-COM, INC.",
    "0060B4": "GLENAYRE R&D INC.",
    "0060B5": "KEBA GmbH",
    "0060B6": "LAND COMPUTER CO., LTD.",
    "0060B7": "CHANNELMATIC, INC.",
    "0060B8": "CORELIS Inc.",
    "0060B9": "NEC Infrontia Corporation",
    "0060BA": "SAHARA NETWORKS, INC.",
    "0060BB": "CABLETRON - NETLINK, INC.",
    "0060BC": "KeunYoung Electronics & Communication Co., Ltd.",
    "0060BD": "HUBBELL-PULSECOM",
    "0060BE": "WEBTRONICS",
    "0060BF": "MACRAIGOR SYSTEMS, INC.",
    "0060C0": "Nera Networks AS",
    "0060C1": "WaveSpan Corporation",
    "0060C2": "MPL AG",
    "0060C3": "NETVISION CORPORATION",
    "0060C4": "SOLITON SYSTEMS K.K.",
    "0060C5": "ANCOT CORP.",
    "0060C6": "DCS AG",
    "0060C7": "AMATI COMMUNICATIONS CORP.",
    "0060C8": "KUKA WELDING SYSTEMS & ROBOTS",
    "0060C9": "ControlNet, Inc.",
    "0060CA": "HARMONIC SYSTEMS INCORPORATED",
    "0060CB": "HITACHI ZOSEN CORPORATION",
    "0060CC": "EMTRAK, INCORPORATED",
    "0060CD": "VideoServer, Inc.",
    "0060CE": "ACCLAIM COMMUNICATIONS",
    "0060CF": "ALTEON NETWORKS, INC.",
    "0060D0": "SNMP RESEARCH INCORPORATED",
    "0060D1": "CASCADE COMMUNICATIONS",
    "0060D2": "LUCENT TECHNOLOGIES TAIWAN TELECOMMUNICATIONS CO., LTD.",
    "0060D3": "AT&T",
    "0060D4": "ELDAT COMMUNICATION LTD.",
    "0060D5": "MIYACHI TECHNOS CORP.",
    "0060D6": "NovAtel Wireless Technologies Ltd.",
    "0060D7": "ECOLE POLYTECHNIQUE FEDERALE DE LAUSANNE (EPFL)",
    "0060D8": "ELMIC SYSTEMS, INC.",
    "0060D9": "TRANSYS NETWORKS INC.",
    "0060DA": "JBM ELECTRONICS CO.",
    "0060DB": "NTP ELEKTRONIK A/S",
    "0060DC": "Toyo Network Systems & System Integration Co. LTD",
    "0060DD": "MYRICOM, INC.",
    "0060DE": "Kayser-Threde GmbH",
    "0060DF": "Brocade Communications Systems, Inc.",
    "0060E0": "AXIOM TECHNOLOGY CO., LTD.",
    "0060E1": "ORCKIT COMMUNICATIONS LTD.",
    "0060E2": "QUEST ENGINEERING & DEVELOPMENT",
    "0060E3": "ARBIN INSTRUMENTS",
    "0060E4": "COMPUSERVE, INC.",
    "0060E5": "FUJI AUTOMATION CO., LTD.",
    "0060E6": "SHOMITI SYSTEMS INCORPORATED",
    "0060E7": "RANDATA",
    "0060E8": "HITACHI COMPUTER PRODUCTS (AMERICA), INC.",
    "0060E9": "ATOP TECHNOLOGIES, INC.",
    "0060EA": "StreamLogic",
    "0060EB": "FOURTHTRACK SYSTEMS",
    "0060EC": "HERMARY OPTO ELECTRONICS INC.",
    "0060ED": "RICARDO TEST AUTOMATION LTD.",
    "0060EE": "APOLLO",
    "0060EF": "FLYTECH TECHNOLOGY CO., LTD.",
    "0060F0": "JOHNSON & JOHNSON MEDICAL, INC",
    "0060F1": "EXP COMPUTER, INC.",
    "0060F2": "LASERGRAPHICS, INC.",
    "0060F3": "Performance Analysis Broadband, Spirent plc",
    "0060F4": "ADVANCED COMPUTER SOLUTIONS, Inc.",
    "0060F5": "ICON WEST, INC.",
    "0060F6": "NEXTEST COMMUNICATIONS PRODUCTS, INC.",
    "0060F7": "DATAFUSION SYSTEMS",
    "0060F8": "Loran International Technologies Inc.",
    "0060F9": "DIAMOND LANE COMMUNICATIONS",
    "0060FA": "EDUCATIONAL TECHNOLOGY RESOURCES, INC.",
    "0060FB": "PACKETEER, INC.",
    "0060FC": "CONSERVATION THROUGH INNOVATION LTD.",
    "0060FD": "NetICs, Inc.",
    "0060FE": "LYNX SYSTEM DEVELOPERS, INC.",
    "0060FF": "QuVis, Inc.",
    "0070B0": "M/A-COM INC. COMPANIES",
    "0070B3": "DATA RECALL LTD.",
    "008000": "MULTITECH SYSTEMS, INC.",
    "008001": "PERIPHONICS CORPORATION",
    "008002": "SATELCOM (UK) LTD",
    "008003": "HYTEC ELECTRONICS LTD.",
    "008004": "ANTLOW COMMUNICATIONS, LTD.",
    "008005": "CACTUS COMPUTER INC.",
    "008006": "COMPUADD CORPORATION",
    "008007": "DLOG NC-SYSTEME",
    "008008": "DYNATECH COMPUTER SYSTEMS",
    "008009": "JUPITER SYSTEMS, INC.",
    "00800A": "JAPAN COMPUTER CORP.",
    "00800B": "CSK CORPORATION",
    "00800C": "VIDECOM LIMITED",
    "00800D": "VOSSWINKEL F.U.",
    "00800E": "ATLANTIX CORPORATION",
    "00800F": "STANDARD MICROSYSTEMS",
    "008010": "COMMODORE INTERNATIONAL",
    "008011": "DIGITAL SYSTEMS INT'L. INC.",
    "008012": "INTEGRATED MEASUREMENT SYSTEMS",
    "008013": "THOMAS-CONRAD CORPORATION",
    "008014": "ESPRIT SYSTEMS",
    "008015": "SEIKO SYSTEMS, INC.",
    "008016": "WANDEL AND GOLTERMANN",
    "008017": "PFU LIMITED",
    "008018": "KOBE STEEL, LTD.",
    "008019": "DAYNA COMMUNICATIONS, INC.",
    "00801A": "BELL ATLANTIC",
    "00801B": "KODIAK TECHNOLOGY",
    "00801C": "NEWPORT SYSTEMS SOLUTIONS",
    "00801D": "INTEGRATED INFERENCE MACHINES",
    "00801E": "XINETRON, INC.",
    "00801F": "KRUPP ATLAS ELECTRONIK GMBH",
    "008020": "NETWORK PRODUCTS",
    "008021": "Alcatel Canada Inc.",
    "008022": "SCAN-OPTICS",
    "008023": "INTEGRATED BUSINESS NETWORKS",
    "008024": "KALPANA, INC.",
    "008025": "STOLLMANN GMBH",
    "008026": "NETWORK PRODUCTS CORPORATION",
    "008027": "ADAPTIVE SYSTEMS, INC.",
    "008028": "TRADPOST (HK) LTD",
    "008029": "EAGLE TECHNOLOGY, INC.",
    "00802A": "TEST SYSTEMS & SIMULATIONS INC",
    "00802B": "INTEGRATED MARKETING CO",
    "00802C": "THE SAGE GROUP PLC",
    "00802D": "XYLOGICS INC",
    "00802E": "CASTLE ROCK COMPUTING",
    "00802F": "NATIONAL INSTRUMENTS CORP.",
    "008030": "NEXUS ELECTRONICS",
    "008031": "BASYS, CORP.",
    "008032": "ACCESS CO., LTD.",
    "008033": "FORMATION, INC.",
    "008034": "SMT GOUPIL",
    "008035": "TECHNOLOGY WORKS, INC.",
    "008036": "REFLEX MANUFACTURING SYSTEMS",
    "008037": "Ericsson Group",
    "008038": "DATA RESEARCH & APPLICATIONS",
    "008039": "ALCATEL STC AUSTRALIA",
    "00803A": "VARITYPER, INC.",
    "00803B": "APT COMMUNICATIONS, INC.",
    "00803C": "TVS ELECTRONICS LTD",
    "00803D": "SURIGIKEN CO., LTD.",
    "00803E": "SYNERNETICS",
    "00803F": "TATUNG COMPANY",
    "008040": "JOHN FLUKE MANUFACTURING CO.",
    "008041": "VEB KOMBINAT ROBOTRON",
    "008042": "Emerson Network Power",
    "008043": "NETWORLD, INC.",
    "008044": "SYSTECH COMPUTER CORP.",
    "008045": "MATSUSHITA ELECTRIC IND. CO",
    "008046": "UNIVERSITY OF TORONTO",
    "008047": "IN-NET CORP.",
    "008048": "COMPEX INCORPORATED",
    "008049": "NISSIN ELECTRIC CO., LTD.",
    "00804A": "PRO-LOG",
    "00804B": "EAGLE TECHNOLOGIES PTY.LTD.",
    "00804C": "CONTEC CO., LTD.",
    "00804D": "CYCLONE MICROSYSTEMS, INC.",
    "00804E": "APEX COMPUTER COMPANY",
    "00804F": "DAIKIN INDUSTRIES, LTD.",
    "008050": "ZIATECH CORPORATION",
    "008051": "FIBERMUX",
    "008052": "TECHNICALLY ELITE CONCEPTS",
    "008053": "INTELLICOM, INC.",
    "008054": "FRONTIER TECHNOLOGIES CORP.",
    "008055": "FERMILAB",
    "008056": "SPHINX ELEKTRONIK GMBH",
    "008057": "ADSOFT, LTD.",
    "008058": "PRINTER SYSTEMS CORPORATION",
    "008059": "STANLEY ELECTRIC CO., LTD",
    "00805A": "TULIP COMPUTERS INTERNAT'L B.V",
    "00805B": "CONDOR SYSTEMS, INC.",
    "00805C": "AGILIS CORPORATION",
    "00805D": "CANSTAR",
    "00805E": "LSI LOGIC CORPORATION",
    "00805F": "Hewlett Packard",
    "008060": "NETWORK INTERFACE CORPORATION",
    "008061": "LITTON SYSTEMS, INC.",
    "008062": "INTERFACE CO.",
    "008063": "RICHARD HIRSCHMANN GMBH & CO.",
    "008064": "WYSE TECHNOLOGY",
    "008065": "CYBERGRAPHIC SYSTEMS PTY LTD.",
    "008066": "ARCOM CONTROL SYSTEMS, LTD.",
    "008067": "SQUARE D COMPANY",
    "008068": "YAMATECH SCIENTIFIC LTD.",
    "008069": "COMPUTONE SYSTEMS",
    "00806A": "ERI (EMPAC RESEARCH INC.)",
    "00806B": "SCHMID TELECOMMUNICATION",
    "00806C": "CEGELEC PROJECTS LTD",
    "00806D": "CENTURY SYSTEMS CORP.",
    "00806E": "NIPPON STEEL CORPORATION",
    "00806F": "ONELAN LTD.",
    "008070": "COMPUTADORAS MICRON",
    "008071": "SAI TECHNOLOGY",
    "008072": "MICROPLEX SYSTEMS LTD.",
    "008073": "DWB ASSOCIATES",
    "008074": "FISHER CONTROLS",
    "008075": "PARSYTEC GMBH",
    "008076": "MCNC",
    "008077": "BROTHER INDUSTRIES, LTD.",
    "008078": "PRACTICAL PERIPHERALS, INC.",
    "008079": "MICROBUS DESIGNS LTD.",
    "00807A": "AITECH SYSTEMS LTD.",
    "00807B": "ARTEL COMMUNICATIONS CORP.",
    "00807C": "FIBERCOM, INC.",
    "00807D": "EQUINOX SYSTEMS INC.",
    "00807E": "SOUTHERN PACIFIC LTD.",
    "00807F": "DY-4 INCORPORATED",
    "008080": "DATAMEDIA CORPORATION",
    "008081": "KENDALL SQUARE RESEARCH CORP.",
    "008082": "PEP MODULAR COMPUTERS GMBH",
    "008083": "AMDAHL",
    "008084": "THE CLOUD INC.",
    "008085": "H-THREE SYSTEMS CORPORATION",
    "008086": "COMPUTER GENERATION INC.",
    "008087": "OKI ELECTRIC INDUSTRY CO., LTD",
    "008088": "VICTOR COMPANY OF JAPAN, LTD.",
    "008089": "TECNETICS (PTY) LTD.",
    "00808A": "SUMMIT MICROSYSTEMS CORP.",
    "00808B": "DACOLL LIMITED",
    "00808C": "NetScout Systems, Inc.",
    "00808D": "WESTCOAST TECHNOLOGY B.V.",
    "00808E": "RADSTONE TECHNOLOGY",
    "00808F": "C. ITOH ELECTRONICS, INC.",
    "008090": "MICROTEK INTERNATIONAL, INC.",
    "008091": "TOKYO ELECTRIC CO.,LTD",
    "008092": "Silex Technology, Inc.",
    "008093": "XYRON CORPORATION",
    "008094": "ALFA LAVAL AUTOMATION AB",
    "008095": "BASIC MERTON HANDELSGES.M.B.H.",
    "008096": "HUMAN DESIGNED SYSTEMS, INC.",
    "008097": "CENTRALP AUTOMATISMES",
    "008098": "TDK CORPORATION",
    "008099": "KLOCKNER MOELLER IPC",
    "00809A": "NOVUS NETWORKS LTD",
    "00809B": "JUSTSYSTEM CORPORATION",
    "00809C": "LUXCOM, INC.",
    "00809D": "Commscraft Ltd.",
    "00809E": "DATUS GMBH",
    "00809F": "ALCATEL BUSINESS SYSTEMS",
    "0080A0": "EDISA HEWLETT PACKARD S/A",
    "0080A1": "MICROTEST, INC.",
    "0080A2": "CREATIVE ELECTRONIC SYSTEMS",
    "0080A3": "Lantronix",
    "0080A4": "LIBERTY ELECTRONICS",
    "0080A5": "SPEED INTERNATIONAL",
    "0080A6": "REPUBLIC TECHNOLOGY, INC.",
    "0080A7": "MEASUREX CORP.",
    "0080A8": "VITACOM CORPORATION",
    "0080A9": "CLEARPOINT RESEARCH",
    "0080AA": "MAXPEED",
    "0080AB": "DUKANE NETWORK INTEGRATION",
    "0080AC": "IMLOGIX, DIVISION OF GENESYS",
    "0080AD": "CNET TECHNOLOGY, INC.",
    "0080AE": "HUGHES NETWORK SYSTEMS",
    "0080AF": "ALLUMER CO., LTD.",
    "0080B0": "ADVANCED INFORMATION",
    "0080B1": "SOFTCOM A/S",
    "0080B2": "NETWORK EQUIPMENT TECHNOLOGIES",
    "0080B3": "AVAL DATA CORPORATION",
    "0080B4": "SOPHIA SYSTEMS",
    "0080B5": "UNITED NETWORKS INC.",
    "0080B6": "THEMIS COMPUTER",
    "0080B7": "STELLAR COMPUTER",
    "0080B8": "BUG, INCORPORATED",
    "0080B9": "ARCHE TECHNOLIGIES INC.",
    "0080BA": "SPECIALIX (ASIA) PTE, LTD",
    "0080BB": "HUGHES LAN SYSTEMS",
    "0080BC": "HITACHI ENGINEERING CO., LTD",
    "0080BD": "THE FURUKAWA ELECTRIC CO., LTD",
    "0080BE": "ARIES RESEARCH",
    "0080BF": "TAKAOKA ELECTRIC MFG. CO. LTD.",
    "0080C0": "PENRIL DATACOMM",
    "0080C1": "LANEX CORPORATION",
    "0080C2": "IEEE 802.1 COMMITTEE",
    "0080C3": "BICC INFORMATION SYSTEMS & SVC",
    "0080C4": "DOCUMENT TECHNOLOGIES, INC.",
    "0080C5": "NOVELLCO DE MEXICO",
    "0080C6": "NATIONAL DATACOMM CORPORATION",
    "0080C7": "XIRCOM",
    "0080C8": "D-LINK SYSTEMS, INC.",
    "0080C9": "ALBERTA MICROELECTRONIC CENTRE",
    "0080CA": "NETCOM RESEARCH INCORPORATED",
    "0080CB": "FALCO DATA PRODUCTS",
    "0080CC": "MICROWAVE BYPASS SYSTEMS",
    "0080CD": "MICRONICS COMPUTER, INC.",
    "0080CE": "BROADCAST TELEVISION SYSTEMS",
    "0080CF": "EMBEDDED PERFORMANCE INC.",
    "0080D0": "COMPUTER PERIPHERALS, INC.",
    "0080D1": "KIMTRON CORPORATION",
    "0080D2": "SHINNIHONDENKO CO., LTD.",
    "0080D3": "SHIVA CORP.",
    "0080D4": "CHASE RESEARCH LTD.",
    "0080D5": "CADRE TECHNOLOGIES",
    "0080D6": "NUVOTECH, INC.",
    "0080D7": "Fantum Engineering",
    "0080D8": "NETWORK PERIPHERALS INC.",
    "0080D9": "EMK ELEKTRONIK",
    "0080DA": "BRUEL & KJAER",
    "0080DB": "GRAPHON CORPORATION",
    "0080DC": "PICKER INTERNATIONAL",
    "0080DD": "GMX INC/GIMIX",
    "0080DE": "GIPSI S.A.",
    "0080DF": "ADC CODENOLL TECHNOLOGY CORP.",
    "0080E0": "XTP SYSTEMS, INC.",
    "0080E1": "STMICROELECTRONICS",
    "0080E2": "T.D.I. CO., LTD.",
    "0080E3": "CORAL NETWORK CORPORATION",
    "0080E4": "NORTHWEST DIGITAL SYSTEMS, INC",
    "0080E5": "LSI Logic Corporation",
    "0080E6": "PEER NETWORKS, INC.",
    "0080E7": "LYNWOOD SCIENTIFIC DEV. LTD.",
    "0080E8": "CUMULUS CORPORATIION",
    "0080E9": "Madge Ltd.",
    "0080EA": "ADVA Optical Networking Ltd.",
    "0080EB": "COMPCONTROL B.V.",
    "0080EC": "SUPERCOMPUTING SOLUTIONS, INC.",
    "0080ED": "IQ TECHNOLOGIES, INC.",
    "0080EE": "THOMSON CSF",
    "0080EF": "RATIONAL",
    "0080F0": "Panasonic Communications Co., Ltd.",
    "0080F1": "OPUS SYSTEMS",
    "0080F2": "RAYCOM SYSTEMS INC",
    "0080F3": "SUN ELECTRONICS CORP.",
    "0080F4": "TELEMECANIQUE ELECTRIQUE",
    "0080F5": "QUANTEL LTD",
    "0080F6": "SYNERGY MICROSYSTEMS",
    "0080F7": "ZENITH ELECTRONICS",
    "0080F8": "MIZAR, INC.",
    "0080F9": "HEURIKON CORPORATION",
    "0080FA": "RWT GMBH",
    "0080FB": "BVM LIMITED",
    "0080FC": "AVATAR CORPORATION",
    "0080FD": "EXSCEED CORPRATION",
    "0080FE": "AZURE TECHNOLOGIES, INC.",
    "0080FF": "SOC. DE TELEINFORMATIQUE RTC",
    "009000": "DIAMOND MULTIMEDIA",
    "009001": "NISHIMU ELECTRONICS INDUSTRIES CO., LTD.",
    "009002": "ALLGON AB",
    "009003": "APLIO",
    "009004": "3COM EUROPE LTD.",
    "009005": "PROTECH SYSTEMS CO., LTD.",
    "009006": "HAMAMATSU PHOTONICS K.K.",
    "009007": "DOMEX TECHNOLOGY CORP.",
    "009008": "HanA Systems Inc.",
    "009009": "i Controls, Inc.",
    "00900A": "PROTON ELECTRONIC INDUSTRIAL CO., LTD.",
    "00900B": "LANNER ELECTRONICS, INC.",
    "00900C": "CISCO SYSTEMS, INC.",
    "00900D": "Overland Storage Inc.",
    "00900E": "HANDLINK TECHNOLOGIES, INC.",
    "00900F": "KAWASAKI HEAVY INDUSTRIES, LTD",
    "009010": "SIMULATION LABORATORIES, INC.",
    "009011": "WAVTrace, Inc.",
    "009012": "GLOBESPAN SEMICONDUCTOR, INC.",
    "009013": "SAMSAN CORP.",
    "009014": "ROTORK INSTRUMENTS, LTD.",
    "009015": "CENTIGRAM COMMUNICATIONS CORP.",
    "009016": "ZAC",
    "009017": "Zypcom, Inc",
    "009018": "ITO ELECTRIC INDUSTRY CO, LTD.",
    "009019": "HERMES ELECTRONICS CO., LTD.",
    "00901A": "UNISPHERE SOLUTIONS",
    "00901B": "DIGITAL CONTROLS",
    "00901C": "mps Software Gmbh",
    "00901D": "PEC (NZ) LTD.",
    "00901E": "Selesta Ingegneria S.p.A.",
    "00901F": "ADTEC PRODUCTIONS, INC.",
    "009020": "PHILIPS ANALYTICAL X-RAY B.V.",
    "009021": "CISCO SYSTEMS, INC.",
    "009022": "IVEX",
    "009023": "ZILOG INC.",
    "009024": "PIPELINKS, INC.",
    "009025": "VISION SYSTEMS LTD. PTY",
    "009026": "ADVANCED SWITCHING COMMUNICATIONS, INC.",
    "009027": "INTEL CORPORATION",
    "009028": "NIPPON SIGNAL CO., LTD.",
    "009029": "CRYPTO AG",
    "00902A": "COMMUNICATION DEVICES, INC.",
    "00902B": "CISCO SYSTEMS, INC.",
    "00902C": "DATA & CONTROL EQUIPMENT LTD.",
    "00902D": "DATA ELECTRONICS (AUST.) PTY, LTD.",
    "00902E": "NAMCO LIMITED",
    "00902F": "NETCORE SYSTEMS, INC.",
    "009030": "HONEYWELL-DATING",
    "009031": "MYSTICOM, LTD.",
    "009032": "PELCOMBE GROUP LTD.",
    "009033": "INNOVAPHONE AG",
    "009034": "IMAGIC, INC.",
    "009035": "ALPHA TELECOM, INC.",
    "009036": "ens, inc.",
    "009037": "ACUCOMM, INC.",
    "009038": "FOUNTAIN TECHNOLOGIES, INC.",
    "009039": "SHASTA NETWORKS",
    "00903A": "NIHON MEDIA TOOL INC.",
    "00903B": "TriEMS Research Lab, Inc.",
    "00903C": "ATLANTIC NETWORK SYSTEMS",
    "00903D": "BIOPAC SYSTEMS, INC.",
    "00903E": "N.V. PHILIPS INDUSTRIAL ACTIVITIES",
    "00903F": "AZTEC RADIOMEDIA",
    "009040": "Siemens Network Convergence LLC",
    "009041": "APPLIED DIGITAL ACCESS",
    "009042": "ECCS, Inc.",
    "009043": "NICHIBEI DENSHI CO., LTD.",
    "009044": "ASSURED DIGITAL, INC.",
    "009045": "Marconi Communications",
    "009046": "DEXDYNE, LTD.",
    "009047": "GIGA FAST E. LTD.",
    "009048": "ZEAL CORPORATION",
    "009049": "ENTRIDIA CORPORATION",
    "00904A": "CONCUR SYSTEM TECHNOLOGIES",
    "00904B": "GemTek Technology Co., Ltd.",
    "00904C": "EPIGRAM, INC.",
    "00904D": "SPEC S.A.",
    "00904E": "DELEM BV",
    "00904F": "ABB POWER T&D COMPANY, INC.",
    "009050": "TELESTE OY",
    "009051": "ULTIMATE TECHNOLOGY CORP.",
    "009052": "SELCOM ELETTRONICA S.R.L.",
    "009053": "DAEWOO ELECTRONICS CO., LTD.",
    "009054": "INNOVATIVE SEMICONDUCTORS, INC",
    "009055": "PARKER HANNIFIN CORPORATION COMPUMOTOR DIVISION",
    "009056": "TELESTREAM, INC.",
    "009057": "AANetcom, Inc.",
    "009058": "Ultra Electronics Ltd., Command and Control Systems",
    "009059": "TELECOM DEVICE K.K.",
    "00905A": "DEARBORN GROUP, INC.",
    "00905B": "RAYMOND AND LAE ENGINEERING",
    "00905C": "EDMI",
    "00905D": "NETCOM SICHERHEITSTECHNIK GmbH",
    "00905E": "RAULAND-BORG CORPORATION",
    "00905F": "CISCO SYSTEMS, INC.",
    "009060": "SYSTEM CREATE CORP.",
    "009061": "PACIFIC RESEARCH & ENGINEERING CORPORATION",
    "009062": "ICP VORTEX COMPUTERSYSTEME GmbH",
    "009063": "COHERENT COMMUNICATIONS SYSTEMS CORPORATION",
    "009064": "Thomson Inc.",
    "009065": "FINISAR CORPORATION",
    "009066": "Troika Networks, Inc.",
    "009067": "WalkAbout Computers, Inc.",
    "009068": "DVT CORP.",
    "009069": "JUNIPER NETWORKS, INC.",
    "00906A": "TURNSTONE SYSTEMS, INC.",
    "00906B": "APPLIED RESOURCES, INC.",
    "00906C": "Sartorius Hamburg GmbH",
    "00906D": "CISCO SYSTEMS, INC.",
    "00906E": "PRAXON, INC.",
    "00906F": "CISCO SYSTEMS, INC.",
    "009070": "NEO NETWORKS, INC.",
    "009071": "Applied Innovation Inc.",
    "009072": "SIMRAD AS",
    "009073": "GAIO TECHNOLOGY",
    "009074": "ARGON NETWORKS, INC.",
    "009075": "NEC DO BRASIL S.A.",
    "009076": "FMT AIRCRAFT GATE SUPPORT SYSTEMS AB",
    "009077": "ADVANCED FIBRE COMMUNICATIONS",
    "009078": "MER TELEMANAGEMENT SOLUTIONS, LTD.",
    "009079": "ClearOne, Inc.",
    "00907A": "Polycom, Inc.",
    "00907B": "E-TECH, INC.",
    "00907C": "DIGITALCAST, INC.",
    "00907D": "Lake Communications",
    "00907E": "VETRONIX CORP.",
    "00907F": "WatchGuard Technologies, Inc.",
    "009080": "NOT LIMITED, INC.",
    "009081": "ALOHA NETWORKS, INC.",
    "009082": "FORCE INSTITUTE",
    "009083": "TURBO COMMUNICATION, INC.",
    "009084": "ATECH SYSTEM",
    "009085": "GOLDEN ENTERPRISES, INC.",
    "009086": "CISCO SYSTEMS, INC.",
    "009087": "ITIS",
    "009088": "BAXALL SECURITY LTD.",
    "009089": "SOFTCOM MICROSYSTEMS, INC.",
    "00908A": "BAYLY COMMUNICATIONS, INC.",
    "00908B": "PFU Systems, Inc.",
    "00908C": "ETREND ELECTRONICS, INC.",
    "00908D": "VICKERS ELECTRONICS SYSTEMS",
    "00908E": "Nortel Networks Broadband Access",
    "00908F": "AUDIO CODES LTD.",
    "009090": "I-BUS",
    "009091": "DigitalScape, Inc.",
    "009092": "CISCO SYSTEMS, INC.",
    "009093": "NANAO CORPORATION",
    "009094": "OSPREY TECHNOLOGIES, INC.",
    "009095": "UNIVERSAL AVIONICS",
    "009096": "ASKEY COMPUTER CORP.",
    "009097": "Sycamore Networks",
    "009098": "SBC DESIGNS, INC.",
    "009099": "ALLIED TELESIS, K.K.",
    "00909A": "ONE WORLD SYSTEMS, INC.",
    "00909B": "IMAJE",
    "00909C": "Motorola, Inc.",
    "00909D": "NovaTech Process Solutions, LLC",
    "00909E": "Critical IO, LLC",
    "00909F": "DIGI-DATA CORPORATION",
    "0090A0": "8X8 INC.",
    "0090A1": "Flying Pig Systems/High End Systems Inc.",
    "0090A2": "CYBERTAN TECHNOLOGY, INC.",
    "0090A3": "Corecess Inc.",
    "0090A4": "ALTIGA NETWORKS",
    "0090A5": "SPECTRA LOGIC",
    "0090A6": "CISCO SYSTEMS, INC.",
    "0090A7": "CLIENTEC CORPORATION",
    "0090A8": "NineTiles Networks, Ltd.",
    "0090A9": "WESTERN DIGITAL",
    "0090AA": "INDIGO ACTIVE VISION SYSTEMS LIMITED",
    "0090AB": "CISCO SYSTEMS, INC.",
    "0090AC": "OPTIVISION, INC.",
    "0090AD": "ASPECT ELECTRONICS, INC.",
    "0090AE": "ITALTEL S.p.A.",
    "0090AF": "J. MORITA MFG. CORP.",
    "0090B0": "VADEM",
    "0090B1": "CISCO SYSTEMS, INC.",
    "0090B2": "AVICI SYSTEMS INC.",
    "0090B3": "AGRANAT SYSTEMS",
    "0090B4": "WILLOWBROOK TECHNOLOGIES",
    "0090B5": "NIKON CORPORATION",
    "0090B6": "FIBEX SYSTEMS",
    "0090B7": "DIGITAL LIGHTWAVE, INC.",
    "0090B8": "ROHDE & SCHWARZ GMBH & CO. KG",
    "0090B9": "BERAN INSTRUMENTS LTD.",
    "0090BA": "VALID NETWORKS, INC.",
    "0090BB": "TAINET COMMUNICATION SYSTEM Corp.",
    "0090BC": "TELEMANN CO., LTD.",
    "0090BD": "OMNIA COMMUNICATIONS, INC.",
    "0090BE": "IBC/INTEGRATED BUSINESS COMPUTERS",
    "0090BF": "CISCO SYSTEMS, INC.",
    "0090C0": "K.J. LAW ENGINEERS, INC.",
    "0090C1": "Peco II, Inc.",
    "0090C2": "JK microsystems, Inc.",
    "0090C3": "TOPIC SEMICONDUCTOR CORP.",
    "0090C4": "JAVELIN SYSTEMS, INC.",
    "0090C5": "INTERNET MAGIC, INC.",
    "0090C6": "OPTIM SYSTEMS, INC.",
    "0090C7": "ICOM INC.",
    "0090C8": "WAVERIDER COMMUNICATIONS (CANADA) INC.",
    "0090C9": "DPAC Technologies",
    "0090CA": "ACCORD VIDEO TELECOMMUNICATIONS, LTD.",
    "0090CB": "Wireless OnLine, Inc.",
    "0090CC": "Planex Communications",
    "0090CD": "ENT-EMPRESA NACIONAL DE TELECOMMUNICACOES, S.A.",
    "0090CE": "TETRA GmbH",
    "0090CF": "NORTEL",
    "0090D0": "Thomson Telecom Belgium",
    "0090D1": "LEICHU ENTERPRISE CO., LTD.",
    "0090D2": "ARTEL VIDEO SYSTEMS",
    "0090D3": "GIESECKE & DEVRIENT GmbH",
    "0090D4": "BindView Development Corp.",
    "0090D5": "EUPHONIX, INC.",
    "0090D6": "CRYSTAL GROUP",
    "0090D7": "NetBoost Corp.",
    "0090D8": "WHITECROSS SYSTEMS",
    "0090D9": "CISCO SYSTEMS, INC.",
    "0090DA": "DYNARC, INC.",
    "0090DB": "NEXT LEVEL COMMUNICATIONS",
    "0090DC": "TECO INFORMATION SYSTEMS",
    "0090DD": "THE MIHARU COMMUNICATIONS CO., LTD.",
    "0090DE": "CARDKEY SYSTEMS, INC.",
    "0090DF": "MITSUBISHI CHEMICAL AMERICA, INC.",
    "0090E0": "SYSTRAN CORP.",
    "0090E1": "TELENA S.P.A.",
    "0090E2": "DISTRIBUTED PROCESSING TECHNOLOGY",
    "0090E3": "AVEX ELECTRONICS INC.",
    "0090E4": "NEC AMERICA, INC.",
    "0090E5": "TEKNEMA, INC.",
    "0090E6": "ALi Corporation",
    "0090E7": "HORSCH ELEKTRONIK AG",
    "0090E8": "MOXA TECHNOLOGIES CORP., LTD.",
    "0090E9": "JANZ COMPUTER AG",
    "0090EA": "ALPHA TECHNOLOGIES, INC.",
    "0090EB": "SENTRY TELECOM SYSTEMS",
    "0090EC": "PYRESCOM",
    "0090ED": "CENTRAL SYSTEM RESEARCH CO., LTD.",
    "0090EE": "PERSONAL COMMUNICATIONS TECHNOLOGIES",
    "0090EF": "INTEGRIX, INC.",
    "0090F0": "Harmonic Video Systems Ltd.",
    "0090F1": "DOT HILL SYSTEMS CORPORATION",
    "0090F2": "CISCO SYSTEMS, INC.",
    "0090F3": "ASPECT COMMUNICATIONS",
    "0090F4": "LIGHTNING INSTRUMENTATION",
    "0090F5": "CLEVO CO.",
    "0090F6": "ESCALATE NETWORKS, INC.",
    "0090F7": "NBASE COMMUNICATIONS LTD.",
    "0090F8": "MEDIATRIX TELECOM",
    "0090F9": "LEITCH",
    "0090FA": "EMULEX Corp",
    "0090FB": "PORTWELL, INC.",
    "0090FC": "NETWORK COMPUTING DEVICES",
    "0090FD": "CopperCom, Inc.",
    "0090FE": "ELECOM CO., LTD. (LANEED DIV.)",
    "0090FF": "TELLUS TECHNOLOGY INC.",
    "0091D6": "Crystal Group, Inc.",
    "009D8E": "CARDIAC RECORDERS, INC.",
    "00A000": "CENTILLION NETWORKS, INC.",
    "00A001": "DRS Signal Solutions",
    "00A002": "LEEDS & NORTHRUP AUSTRALIA PTY LTD",
    "00A003": "STAEFA CONTROL SYSTEM",
    "00A004": "NETPOWER, INC.",
    "00A005": "DANIEL INSTRUMENTS, LTD.",
    "00A006": "IMAGE DATA PROCESSING SYSTEM GROUP",
    "00A007": "APEXX TECHNOLOGY, INC.",
    "00A008": "NETCORP",
    "00A009": "WHITETREE NETWORK",
    "00A00A": "Airspan",
    "00A00B": "COMPUTEX CO., LTD.",
    "00A00C": "KINGMAX TECHNOLOGY, INC.",
    "00A00D": "THE PANDA PROJECT",
    "00A00E": "VISUAL NETWORKS, INC.",
    "00A00F": "Broadband Technologies",
    "00A010": "SYSLOGIC DATENTECHNIK AG",
    "00A011": "MUTOH INDUSTRIES LTD.",
    "00A012": "B.A.T.M. ADVANCED TECHNOLOGIES",
    "00A013": "TELTREND LTD.",
    "00A014": "CSIR",
    "00A015": "WYLE",
    "00A016": "MICROPOLIS CORP.",
    "00A017": "J B M CORPORATION",
    "00A018": "CREATIVE CONTROLLERS, INC.",
    "00A019": "NEBULA CONSULTANTS, INC.",
    "00A01A": "BINAR ELEKTRONIK AB",
    "00A01B": "PREMISYS COMMUNICATIONS, INC.",
    "00A01C": "NASCENT NETWORKS CORPORATION",
    "00A01D": "SIXNET",
    "00A01E": "EST CORPORATION",
    "00A01F": "TRICORD SYSTEMS, INC.",
    "00A020": "CITICORP/TTI",
    "00A021": "General Dynamics",
    "00A022": "CENTRE FOR DEVELOPMENT OF ADVANCED COMPUTING",
    "00A023": "APPLIED CREATIVE TECHNOLOGY, INC.",
    "00A024": "3COM CORPORATION",
    "00A025": "REDCOM LABS INC.",
    "00A026": "TELDAT, S.A.",
    "00A027": "FIREPOWER SYSTEMS, INC.",
    "00A028": "CONNER PERIPHERALS",
    "00A029": "COULTER CORPORATION",
    "00A02A": "TRANCELL SYSTEMS",
    "00A02B": "TRANSITIONS RESEARCH CORP.",
    "00A02C": "interWAVE Communications",
    "00A02D": "1394 Trade Association",
    "00A02E": "BRAND COMMUNICATIONS, LTD.",
    "00A02F": "PIRELLI CAVI",
    "00A030": "CAPTOR NV/SA",
    "00A031": "HAZELTINE CORPORATION, MS 1-17",
    "00A032": "GES SINGAPORE PTE. LTD.",
    "00A033": "imc MeBsysteme GmbH",
    "00A034": "AXEL",
    "00A035": "CYLINK CORPORATION",
    "00A036": "APPLIED NETWORK TECHNOLOGY",
    "00A037": "Datascope Corporation",
    "00A038": "EMAIL ELECTRONICS",
    "00A039": "ROSS TECHNOLOGY, INC.",
    "00A03A": "KUBOTEK CORPORATION",
    "00A03B": "TOSHIN ELECTRIC CO., LTD.",
    "00A03C": "EG&G NUCLEAR INSTRUMENTS",
    "00A03D": "OPTO-22",
    "00A03E": "ATM FORUM",
    "00A03F": "COMPUTER SOCIETY MICROPROCESSOR & MICROPROCESSOR STANDARDS C",
    "00A040": "APPLE COMPUTER",
    "00A041": "INFICON",
    "00A042": "SPUR PRODUCTS CORP.",
    "00A043": "AMERICAN TECHNOLOGY LABS, INC.",
    "00A044": "NTT IT CO., LTD.",
    "00A045": "PHOENIX CONTACT GMBH & CO.",
    "00A046": "SCITEX CORP. LTD.",
    "00A047": "INTEGRATED FITNESS CORP.",
    "00A048": "QUESTECH, LTD.",
    "00A049": "DIGITECH INDUSTRIES, INC.",
    "00A04A": "NISSHIN ELECTRIC CO., LTD.",
    "00A04B": "TFL LAN INC.",
    "00A04C": "INNOVATIVE SYSTEMS & TECHNOLOGIES, INC.",
    "00A04D": "EDA INSTRUMENTS, INC.",
    "00A04E": "VOELKER TECHNOLOGIES, INC.",
    "00A04F": "AMERITEC CORP.",
    "00A050": "CYPRESS SEMICONDUCTOR",
    "00A051": "ANGIA COMMUNICATIONS. INC.",
    "00A052": "STANILITE ELECTRONICS PTY. LTD",
    "00A053": "COMPACT DEVICES, INC.",
    "00A054": "PRIVATE",
    "00A055": "Data Device Corporation",
    "00A056": "MICROPROSS",
    "00A057": "LANCOM Systems GmbH",
    "00A058": "GLORY, LTD.",
    "00A059": "HAMILTON HALLMARK",
    "00A05A": "KOFAX IMAGE PRODUCTS",
    "00A05B": "MARQUIP, INC.",
    "00A05C": "INVENTORY CONVERSION, INC./",
    "00A05D": "CS COMPUTER SYSTEME GmbH",
    "00A05E": "MYRIAD LOGIC INC.",
    "00A05F": "BTG Electronics Design BV",
    "00A060": "ACER PERIPHERALS, INC.",
    "00A061": "PURITAN BENNETT",
    "00A062": "AES PRODATA",
    "00A063": "JRL SYSTEMS, INC.",
    "00A064": "KVB/ANALECT",
    "00A065": "Symantec Corporation",
    "00A066": "ISA CO., LTD.",
    "00A067": "NETWORK SERVICES GROUP",
    "00A068": "BHP LIMITED",
    "00A069": "Symmetricom, Inc.",
    "00A06A": "Verilink Corporation",
    "00A06B": "DMS DORSCH MIKROSYSTEM GMBH",
    "00A06C": "SHINDENGEN ELECTRIC MFG. CO., LTD.",
    "00A06D": "MANNESMANN TALLY CORPORATION",
    "00A06E": "AUSTRON, INC.",
    "00A06F": "THE APPCON GROUP, INC.",
    "00A070": "COASTCOM",
    "00A071": "VIDEO LOTTERY TECHNOLOGIES,INC",
    "00A072": "OVATION SYSTEMS LTD.",
    "00A073": "COM21, INC.",
    "00A074": "PERCEPTION TECHNOLOGY",
    "00A075": "MICRON TECHNOLOGY, INC.",
    "00A076": "CARDWARE LAB, INC.",
    "00A077": "FUJITSU NEXION, INC.",
    "00A078": "Marconi Communications",
    "00A079": "ALPS ELECTRIC (USA), INC.",
    "00A07A": "ADVANCED PERIPHERALS TECHNOLOGIES, INC.",
    "00A07B": "DAWN COMPUTER INCORPORATION",
    "00A07C": "TONYANG NYLON CO., LTD.",
    "00A07D": "SEEQ TECHNOLOGY, INC.",
    "00A07E": "AVID TECHNOLOGY, INC.",
    "00A07F": "GSM-SYNTEL, LTD.",
    "00A080": "SBE, Inc.",
    "00A081": "ALCATEL DATA NETWORKS",
    "00A082": "NKT ELEKTRONIK A/S",
    "00A083": "ASIMMPHONY TURKEY",
    "00A084": "DATAPLEX PTY. LTD.",
    "00A085": "PRIVATE",
    "00A086": "AMBER WAVE SYSTEMS, INC.",
    "00A087": "Zarlink Semiconductor Ltd.",
    "00A088": "ESSENTIAL COMMUNICATIONS",
    "00A089": "XPOINT TECHNOLOGIES, INC.",
    "00A08A": "BROOKTROUT TECHNOLOGY, INC.",
    "00A08B": "ASTON ELECTRONIC DESIGNS LTD.",
    "00A08C": "MultiMedia LANs, Inc.",
    "00A08D": "JACOMO CORPORATION",
    "00A08E": "Nokia Internet Communications",
    "00A08F": "DESKNET SYSTEMS, INC.",
    "00A090": "TimeStep Corporation",
    "00A091": "APPLICOM INTERNATIONAL",
    "00A092": "H. BOLLMANN MANUFACTURERS, LTD",
    "00A093": "B/E AEROSPACE, Inc.",
    "00A094": "COMSAT CORPORATION",
    "00A095": "ACACIA NETWORKS, INC.",
    "00A096": "MITUMI ELECTRIC CO., LTD.",
    "00A097": "JC INFORMATION SYSTEMS",
    "00A098": "NETWORK APPLIANCE CORP.",
    "00A099": "K-NET LTD.",
    "00A09A": "NIHON KOHDEN AMERICA",
    "00A09B": "QPSX COMMUNICATIONS, LTD.",
    "00A09C": "Xyplex, Inc.",
    "00A09D": "JOHNATHON FREEMAN TECHNOLOGIES",
    "00A09E": "ICTV",
    "00A09F": "COMMVISION CORP.",
    "00A0A0": "COMPACT DATA, LTD.",
    "00A0A1": "EPIC DATA INC.",
    "00A0A2": "DIGICOM S.P.A.",
    "00A0A3": "RELIABLE POWER METERS",
    "00A0A4": "MICROS SYSTEMS, INC.",
    "00A0A5": "TEKNOR MICROSYSTEME, INC.",
    "00A0A6": "M.I. SYSTEMS, K.K.",
    "00A0A7": "VORAX CORPORATION",
    "00A0A8": "RENEX CORPORATION",
    "00A0A9": "NAVTEL COMMUNICATIONS INC.",
    "00A0AA": "SPACELABS MEDICAL",
    "00A0AB": "NETCS INFORMATIONSTECHNIK GMBH",
    "00A0AC": "GILAT SATELLITE NETWORKS, LTD.",
    "00A0AD": "MARCONI SPA",
    "00A0AE": "NUCOM SYSTEMS, INC.",
    "00A0AF": "WMS INDUSTRIES",
    "00A0B0": "I-O DATA DEVICE, INC.",
    "00A0B1": "FIRST VIRTUAL CORPORATION",
    "00A0B2": "SHIMA SEIKI",
    "00A0B3": "ZYKRONIX",
    "00A0B4": "TEXAS MICROSYSTEMS, INC.",
    "00A0B5": "3H TECHNOLOGY",
    "00A0B6": "SANRITZ AUTOMATION CO., LTD.",
    "00A0B7": "CORDANT, INC.",
    "00A0B8": "SYMBIOS LOGIC INC.",
    "00A0B9": "EAGLE TECHNOLOGY, INC.",
    "00A0BA": "PATTON ELECTRONICS CO.",
    "00A0BB": "HILAN GMBH",
    "00A0BC": "VIASAT, INCORPORATED",
    "00A0BD": "I-TECH CORP.",
    "00A0BE": "INTEGRATED CIRCUIT SYSTEMS, INC. COMMUNICATIONS GROUP",
    "00A0BF": "WIRELESS DATA GROUP MOTOROLA",
    "00A0C0": "DIGITAL LINK CORP.",
    "00A0C1": "ORTIVUS MEDICAL AB",
    "00A0C2": "R.A. SYSTEMS CO., LTD.",
    "00A0C3": "UNICOMPUTER GMBH",
    "00A0C4": "CRISTIE ELECTRONICS LTD.",
    "00A0C5": "ZYXEL COMMUNICATION",
    "00A0C6": "QUALCOMM INCORPORATED",
    "00A0C7": "TADIRAN TELECOMMUNICATIONS",
    "00A0C8": "ADTRAN INC.",
    "00A0C9": "INTEL CORPORATION - HF1-06",
    "00A0CA": "FUJITSU DENSO LTD.",
    "00A0CB": "ARK TELECOMMUNICATIONS, INC.",
    "00A0CC": "LITE-ON COMMUNICATIONS, INC.",
    "00A0CD": "DR. JOHANNES HEIDENHAIN GmbH",
    "00A0CE": "ASTROCOM CORPORATION",
    "00A0CF": "SOTAS, INC.",
    "00A0D0": "TEN X TECHNOLOGY, INC.",
    "00A0D1": "INVENTEC CORPORATION",
    "00A0D2": "ALLIED TELESIS INTERNATIONAL CORPORATION",
    "00A0D3": "INSTEM COMPUTER SYSTEMS, LTD.",
    "00A0D4": "RADIOLAN, INC.",
    "00A0D5": "SIERRA WIRELESS INC.",
    "00A0D6": "SBE, INC.",
    "00A0D7": "KASTEN CHASE APPLIED RESEARCH",
    "00A0D8": "SPECTRA - TEK",
    "00A0D9": "CONVEX COMPUTER CORPORATION",
    "00A0DA": "INTEGRATED SYSTEMS Technology, Inc.",
    "00A0DB": "FISHER & PAYKEL PRODUCTION",
    "00A0DC": "O.N. ELECTRONIC CO., LTD.",
    "00A0DD": "AZONIX CORPORATION",
    "00A0DE": "YAMAHA CORPORATION",
    "00A0DF": "STS TECHNOLOGIES, INC.",
    "00A0E0": "TENNYSON TECHNOLOGIES PTY LTD",
    "00A0E1": "WESTPORT RESEARCH ASSOCIATES, INC.",
    "00A0E2": "Keisokugiken Corporation",
    "00A0E3": "XKL SYSTEMS CORP.",
    "00A0E4": "OPTIQUEST",
    "00A0E5": "NHC COMMUNICATIONS",
    "00A0E6": "DIALOGIC CORPORATION",
    "00A0E7": "CENTRAL DATA CORPORATION",
    "00A0E8": "REUTERS HOLDINGS PLC",
    "00A0E9": "ELECTRONIC RETAILING SYSTEMS INTERNATIONAL",
    "00A0EA": "ETHERCOM CORP.",
    "00A0EB": "Encore Networks",
    "00A0EC": "TRANSMITTON LTD.",
    "00A0ED": "Brooks Automation, Inc.",
    "00A0EE": "NASHOBA NETWORKS",
    "00A0EF": "LUCIDATA LTD.",
    "00A0F0": "TORONTO MICROELECTRONICS INC.",
    "00A0F1": "MTI",
    "00A0F2": "INFOTEK COMMUNICATIONS, INC.",
    "00A0F3": "STAUBLI",
    "00A0F4": "GE",
    "00A0F5": "RADGUARD LTD.",
    "00A0F6": "AutoGas Systems Inc.",
    "00A0F7": "V.I COMPUTER CORP.",
    "00A0F8": "SYMBOL TECHNOLOGIES, INC.",
    "00A0F9": "BINTEC COMMUNICATIONS GMBH",
    "00A0FA": "Marconi Communication GmbH",
    "00A0FB": "TORAY ENGINEERING CO., LTD.",
    "00A0FC": "IMAGE SCIENCES, INC.",
    "00A0FD": "SCITEX DIGITAL PRINTING, INC.",
    "00A0FE": "BOSTON TECHNOLOGY, INC.",
    "00A0FF": "TELLABS OPERATIONS, INC.",
    "00AA00": "INTEL CORPORATION",
    "00AA01": "INTEL CORPORATION",
    "00AA02": "INTEL CORPORATION",
    "00AA3C": "OLIVETTI TELECOM SPA (OLTECO)",
    "00B009": "Grass Valley Group",
    "00B017": "InfoGear Technology Corp.",
    "00B019": "Casi-Rusco",
    "00B01C": "Westport Technologies",
    "00B01E": "Rantic Labs, Inc.",
    "00B02A": "ORSYS GmbH",
    "00B02D": "ViaGate Technologies, Inc.",
    "00B03B": "HiQ Networks",
    "00B048": "Marconi Communications Inc.",
    "00B04A": "Cisco Systems, Inc.",
    "00B052": "Intellon Corporation",
    "00B064": "Cisco Systems, Inc.",
    "00B069": "Honewell Oy",
    "00B06D": "Jones Futurex Inc.",
    "00B080": "Mannesmann Ipulsys B.V.",
    "00B086": "LocSoft Limited",
    "00B08E": "Cisco Systems, Inc.",
    "00B091": "Transmeta Corp.",
    "00B094": "Alaris, Inc.",
    "00B09A": "Morrow Technologies Corp.",
    "00B09D": "Point Grey Research Inc.",
    "00B0AC": "SIAE-Microelettronica S.p.A.",
    "00B0AE": "Symmetricom",
    "00B0B3": "Xstreamis PLC",
    "00B0C2": "Cisco Systems, Inc.",
    "00B0C7": "Tellabs Operations, Inc.",
    "00B0CE": "TECHNOLOGY RESCUE",
    "00B0D0": "Dell Computer Corp.",
    "00B0DB": "Nextcell, Inc.",
    "00B0DF": "RELDATA Inc",
    "00B0E7": "British Federal Ltd.",
    "00B0EC": "EACEM",
    "00B0EE": "Ajile Systems, Inc.",
    "00B0F0": "CALY NETWORKS",
    "00B0F5": "NetWorth Technologies, Inc.",
    "00BAC0": "Biometric Access Company",
    "00BB01": "OCTOTHORPE CORP.",
    "00BBF0": "UNGERMANN-BASS INC.",
    "00C000": "LANOPTICS, LTD.",
    "00C001": "DIATEK PATIENT MANAGMENT",
    "00C002": "SERCOMM CORPORATION",
    "00C003": "GLOBALNET COMMUNICATIONS",
    "00C004": "JAPAN BUSINESS COMPUTER CO.LTD",
    "00C005": "LIVINGSTON ENTERPRISES, INC.",
    "00C006": "NIPPON AVIONICS CO., LTD.",
    "00C007": "PINNACLE DATA SYSTEMS, INC.",
    "00C008": "SECO SRL",
    "00C009": "KT TECHNOLOGY (S) PTE LTD",
    "00C00A": "MICRO CRAFT",
    "00C00B": "NORCONTROL A.S.",
    "00C00C": "RELIA TECHNOLGIES",
    "00C00D": "ADVANCED LOGIC RESEARCH, INC.",
    "00C00E": "PSITECH, INC.",
    "00C00F": "QUANTUM SOFTWARE SYSTEMS LTD.",
    "00C010": "HIRAKAWA HEWTECH CORP.",
    "00C011": "INTERACTIVE COMPUTING DEVICES",
    "00C012": "NETSPAN CORPORATION",
    "00C013": "NETRIX",
    "00C014": "TELEMATICS CALABASAS INT'L,INC",
    "00C015": "NEW MEDIA CORPORATION",
    "00C016": "ELECTRONIC THEATRE CONTROLS",
    "00C017": "Fluke Corporation",
    "00C018": "LANART CORPORATION",
    "00C019": "LEAP TECHNOLOGY, INC.",
    "00C01A": "COROMETRICS MEDICAL SYSTEMS",
    "00C01B": "SOCKET COMMUNICATIONS, INC.",
    "00C01C": "INTERLINK COMMUNICATIONS LTD.",
    "00C01D": "GRAND JUNCTION NETWORKS, INC.",
    "00C01E": "LA FRANCAISE DES JEUX",
    "00C01F": "S.E.R.C.E.L.",
    "00C020": "ARCO ELECTRONIC, CONTROL LTD.",
    "00C021": "NETEXPRESS",
    "00C022": "LASERMASTER TECHNOLOGIES, INC.",
    "00C023": "TUTANKHAMON ELECTRONICS",
    "00C024": "EDEN SISTEMAS DE COMPUTACAO SA",
    "00C025": "DATAPRODUCTS CORPORATION",
    "00C026": "LANS TECHNOLOGY CO., LTD.",
    "00C027": "CIPHER SYSTEMS, INC.",
    "00C028": "JASCO CORPORATION",
    "00C029": "Nexans Deutschland AG - ANS",
    "00C02A": "OHKURA ELECTRIC CO., LTD.",
    "00C02B": "GERLOFF GESELLSCHAFT FUR",
    "00C02C": "CENTRUM COMMUNICATIONS, INC.",
    "00C02D": "FUJI PHOTO FILM CO., LTD.",
    "00C02E": "NETWIZ",
    "00C02F": "OKUMA CORPORATION",
    "00C030": "INTEGRATED ENGINEERING B. V.",
    "00C031": "DESIGN RESEARCH SYSTEMS, INC.",
    "00C032": "I-CUBED LIMITED",
    "00C033": "TELEBIT COMMUNICATIONS APS",
    "00C034": "TRANSACTION NETWORK",
    "00C035": "QUINTAR COMPANY",
    "00C036": "RAYTECH ELECTRONIC CORP.",
    "00C037": "DYNATEM",
    "00C038": "RASTER IMAGE PROCESSING SYSTEM",
    "00C039": "Teridian Semiconductor Corporation",
    "00C03A": "MEN-MIKRO ELEKTRONIK GMBH",
    "00C03B": "MULTIACCESS COMPUTING CORP.",
    "00C03C": "TOWER TECH S.R.L.",
    "00C03D": "WIESEMANN & THEIS GMBH",
    "00C03E": "FA. GEBR. HELLER GMBH",
    "00C03F": "STORES AUTOMATED SYSTEMS, INC.",
    "00C040": "ECCI",
    "00C041": "DIGITAL TRANSMISSION SYSTEMS",
    "00C042": "DATALUX CORP.",
    "00C043": "STRATACOM",
    "00C044": "EMCOM CORPORATION",
    "00C045": "ISOLATION SYSTEMS, LTD.",
    "00C046": "Blue Chip Technology Ltd",
    "00C047": "UNIMICRO SYSTEMS, INC.",
    "00C048": "BAY TECHNICAL ASSOCIATES",
    "00C049": "U.S. ROBOTICS, INC.",
    "00C04A": "GROUP 2000 AG",
    "00C04B": "CREATIVE MICROSYSTEMS",
    "00C04C": "DEPARTMENT OF FOREIGN AFFAIRS",
    "00C04D": "MITEC, INC.",
    "00C04E": "COMTROL CORPORATION",
    "00C04F": "DELL COMPUTER CORPORATION",
    "00C050": "TOYO DENKI SEIZO K.K.",
    "00C051": "ADVANCED INTEGRATION RESEARCH",
    "00C052": "BURR-BROWN",
    "00C053": "Aspect Software Inc.",
    "00C054": "NETWORK PERIPHERALS, LTD.",
    "00C055": "MODULAR COMPUTING TECHNOLOGIES",
    "00C056": "SOMELEC",
    "00C057": "MYCO ELECTRONICS",
    "00C058": "DATAEXPERT CORP.",
    "00C059": "NIPPON DENSO CO., LTD.",
    "00C05A": "SEMAPHORE COMMUNICATIONS CORP.",
    "00C05B": "NETWORKS NORTHWEST, INC.",
    "00C05C": "ELONEX PLC",
    "00C05D": "L&N TECHNOLOGIES",
    "00C05E": "VARI-LITE, INC.",
    "00C05F": "FINE-PAL COMPANY LIMITED",
    "00C060": "ID SCANDINAVIA AS",
    "00C061": "SOLECTEK CORPORATION",
    "00C062": "IMPULSE TECHNOLOGY",
    "00C063": "MORNING STAR TECHNOLOGIES, INC",
    "00C064": "GENERAL DATACOMM IND. INC.",
    "00C065": "SCOPE COMMUNICATIONS, INC.",
    "00C066": "DOCUPOINT, INC.",
    "00C067": "UNITED BARCODE INDUSTRIES",
    "00C068": "PHILIP DRAKE ELECTRONICS LTD.",
    "00C069": "Axxcelera Broadband Wireless",
    "00C06A": "ZAHNER-ELEKTRIK GMBH & CO. KG",
    "00C06B": "OSI PLUS CORPORATION",
    "00C06C": "SVEC COMPUTER CORP.",
    "00C06D": "BOCA RESEARCH, INC.",
    "00C06E": "HAFT TECHNOLOGY, INC.",
    "00C06F": "KOMATSU LTD.",
    "00C070": "SECTRA SECURE-TRANSMISSION AB",
    "00C071": "AREANEX COMMUNICATIONS, INC.",
    "00C072": "KNX LTD.",
    "00C073": "XEDIA CORPORATION",
    "00C074": "TOYODA AUTOMATIC LOOM",
    "00C075": "XANTE CORPORATION",
    "00C076": "I-DATA INTERNATIONAL A-S",
    "00C077": "DAEWOO TELECOM LTD.",
    "00C078": "COMPUTER SYSTEMS ENGINEERING",
    "00C079": "FONSYS CO.,LTD.",
    "00C07A": "PRIVA B.V.",
    "00C07B": "ASCEND COMMUNICATIONS, INC.",
    "00C07C": "HIGHTECH INFORMATION",
    "00C07D": "RISC DEVELOPMENTS LTD.",
    "00C07E": "KUBOTA CORPORATION ELECTRONIC",
    "00C07F": "NUPON COMPUTING CORP.",
    "00C080": "NETSTAR, INC.",
    "00C081": "METRODATA LTD.",
    "00C082": "MOORE PRODUCTS CO.",
    "00C083": "TRACE MOUNTAIN PRODUCTS, INC.",
    "00C084": "DATA LINK CORP. LTD.",
    "00C085": "ELECTRONICS FOR IMAGING, INC.",
    "00C086": "THE LYNK CORPORATION",
    "00C087": "UUNET TECHNOLOGIES, INC.",
    "00C088": "EKF ELEKTRONIK GMBH",
    "00C089": "TELINDUS DISTRIBUTION",
    "00C08A": "LAUTERBACH DATENTECHNIK GMBH",
    "00C08B": "RISQ MODULAR SYSTEMS, INC.",
    "00C08C": "PERFORMANCE TECHNOLOGIES, INC.",
    "00C08D": "TRONIX PRODUCT DEVELOPMENT",
    "00C08E": "NETWORK INFORMATION TECHNOLOGY",
    "00C08F": "Matsushita Electric Works, Ltd.",
    "00C090": "PRAIM S.R.L.",
    "00C091": "JABIL CIRCUIT, INC.",
    "00C092": "MENNEN MEDICAL INC.",
    "00C093": "ALTA RESEARCH CORP.",
    "00C094": "VMX INC.",
    "00C095": "ZNYX",
    "00C096": "TAMURA CORPORATION",
    "00C097": "ARCHIPEL SA",
    "00C098": "CHUNTEX ELECTRONIC CO., LTD.",
    "00C099": "YOSHIKI INDUSTRIAL CO.,LTD.",
    "00C09A": "PHOTONICS CORPORATION",
    "00C09B": "RELIANCE COMM/TEC, R-TEC",
    "00C09C": "HIOKI E.E. CORPORATION",
    "00C09D": "DISTRIBUTED SYSTEMS INT'L, INC",
    "00C09E": "CACHE COMPUTERS, INC.",
    "00C09F": "QUANTA COMPUTER, INC.",
    "00C0A0": "ADVANCE MICRO RESEARCH, INC.",
    "00C0A1": "TOKYO DENSHI SEKEI CO.",
    "00C0A2": "INTERMEDIUM A/S",
    "00C0A3": "DUAL ENTERPRISES CORPORATION",
    "00C0A4": "UNIGRAF OY",
    "00C0A5": "DICKENS DATA SYSTEMS",
    "00C0A6": "EXICOM AUSTRALIA PTY. LTD",
    "00C0A7": "SEEL LTD.",
    "00C0A8": "GVC CORPORATION",
    "00C0A9": "BARRON MCCANN LTD.",
    "00C0AA": "SILICON VALLEY COMPUTER",
    "00C0AB": "Telco Systems, Inc.",
    "00C0AC": "GAMBIT COMPUTER COMMUNICATIONS",
    "00C0AD": "MARBEN COMMUNICATION SYSTEMS",
    "00C0AE": "TOWERCOM CO. INC. DBA PC HOUSE",
    "00C0AF": "TEKLOGIX INC.",
    "00C0B0": "GCC TECHNOLOGIES,INC.",
    "00C0B1": "GENIUS NET CO.",
    "00C0B2": "NORAND CORPORATION",
    "00C0B3": "COMSTAT DATACOMM CORPORATION",
    "00C0B4": "MYSON TECHNOLOGY, INC.",
    "00C0B5": "CORPORATE NETWORK SYSTEMS,INC.",
    "00C0B6": "Adaptec, Inc.",
    "00C0B7": "AMERICAN POWER CONVERSION CORP",
    "00C0B8": "FRASER'S HILL LTD.",
    "00C0B9": "FUNK SOFTWARE, INC.",
    "00C0BA": "NETVANTAGE",
    "00C0BB": "FORVAL CREATIVE, INC.",
    "00C0BC": "TELECOM AUSTRALIA/CSSC",
    "00C0BD": "INEX TECHNOLOGIES, INC.",
    "00C0BE": "ALCATEL - SEL",
    "00C0BF": "TECHNOLOGY CONCEPTS, LTD.",
    "00C0C0": "SHORE MICROSYSTEMS, INC.",
    "00C0C1": "QUAD/GRAPHICS, INC.",
    "00C0C2": "INFINITE NETWORKS LTD.",
    "00C0C3": "ACUSON COMPUTED SONOGRAPHY",
    "00C0C4": "COMPUTER OPERATIONAL",
    "00C0C5": "SID INFORMATICA",
    "00C0C6": "PERSONAL MEDIA CORP.",
    "00C0C7": "SPARKTRUM MICROSYSTEMS, INC.",
    "00C0C8": "MICRO BYTE PTY. LTD.",
    "00C0C9": "ELSAG BAILEY PROCESS",
    "00C0CA": "ALFA, INC.",
    "00C0CB": "CONTROL TECHNOLOGY CORPORATION",
    "00C0CC": "TELESCIENCES CO SYSTEMS, INC.",
    "00C0CD": "COMELTA, S.A.",
    "00C0CE": "CEI SYSTEMS & ENGINEERING PTE",
    "00C0CF": "IMATRAN VOIMA OY",
    "00C0D0": "RATOC SYSTEM INC.",
    "00C0D1": "COMTREE TECHNOLOGY CORPORATION",
    "00C0D2": "SYNTELLECT, INC.",
    "00C0D3": "OLYMPUS IMAGE SYSTEMS, INC.",
    "00C0D4": "AXON NETWORKS, INC.",
    "00C0D5": "QUANCOM ELECTRONIC GMBH",
    "00C0D6": "J1 SYSTEMS, INC.",
    "00C0D7": "TAIWAN TRADING CENTER DBA",
    "00C0D8": "UNIVERSAL DATA SYSTEMS",
    "00C0D9": "QUINTE NETWORK CONFIDENTIALITY",
    "00C0DA": "NICE SYSTEMS LTD.",
    "00C0DB": "IPC CORPORATION (PTE) LTD.",
    "00C0DC": "EOS TECHNOLOGIES, INC.",
    "00C0DD": "QLogic Corporation",
    "00C0DE": "ZCOMM, INC.",
    "00C0DF": "KYE Systems Corp.",
    "00C0E0": "DSC COMMUNICATION CORP.",
    "00C0E1": "SONIC SOLUTIONS",
    "00C0E2": "CALCOMP, INC.",
    "00C0E3": "OSITECH COMMUNICATIONS, INC.",
    "00C0E4": "SIEMENS BUILDING",
    "00C0E5": "GESPAC, S.A.",
    "00C0E6": "Verilink Corporation",
    "00C0E7": "FIBERDATA AB",
    "00C0E8": "PLEXCOM, INC.",
    "00C0E9": "OAK SOLUTIONS, LTD.",
    "00C0EA": "ARRAY TECHNOLOGY LTD.",
    "00C0EB": "SEH COMPUTERTECHNIK GMBH",
    "00C0EC": "DAUPHIN TECHNOLOGY",
    "00C0ED": "US ARMY ELECTRONIC",
    "00C0EE": "KYOCERA CORPORATION",
    "00C0EF": "ABIT CORPORATION",
    "00C0F0": "KINGSTON TECHNOLOGY CORP.",
    "00C0F1": "SHINKO ELECTRIC CO., LTD.",
    "00C0F2": "TRANSITION NETWORKS",
    "00C0F3": "NETWORK COMMUNICATIONS CORP.",
    "00C0F4": "INTERLINK SYSTEM CO., LTD.",
    "00C0F5": "METACOMP, INC.",
    "00C0F6": "CELAN TECHNOLOGY INC.",
    "00C0F7": "ENGAGE COMMUNICATION, INC.",
    "00C0F8": "ABOUT COMPUTING INC.",
    "00C0F9": "Motorola Embedded Computing Group",
    "00C0FA": "CANARY COMMUNICATIONS, INC.",
    "00C0FB": "ADVANCED TECHNOLOGY LABS",
    "00C0FC": "ELASTIC REALITY, INC.",
    "00C0FD": "PROSUM",
    "00C0FE": "APTEC COMPUTER SYSTEMS, INC.",
    "00C0FF": "DOT HILL SYSTEMS CORPORATION",
    "00CBBD": "Cambridge Broadband Networks Ltd.",
    "00CF1C": "COMMUNICATION MACHINERY CORP.",
    "00D000": "FERRAN SCIENTIFIC, INC.",
    "00D001": "VST TECHNOLOGIES, INC.",
    "00D002": "DITECH CORPORATION",
    "00D003": "COMDA ENTERPRISES CORP.",
    "00D004": "PENTACOM LTD.",
    "00D005": "ZHS ZEITMANAGEMENTSYSTEME",
    "00D006": "CISCO SYSTEMS, INC.",
    "00D007": "MIC ASSOCIATES, INC.",
    "00D008": "MACTELL CORPORATION",
    "00D009": "HSING TECH. ENTERPRISE CO. LTD",
    "00D00A": "LANACCESS TELECOM S.A.",
    "00D00B": "RHK TECHNOLOGY, INC.",
    "00D00C": "SNIJDER MICRO SYSTEMS",
    "00D00D": "MICROMERITICS INSTRUMENT",
    "00D00E": "PLURIS, INC.",
    "00D00F": "SPEECH DESIGN GMBH",
    "00D010": "CONVERGENT NETWORKS, INC.",
    "00D011": "PRISM VIDEO, INC.",
    "00D012": "GATEWORKS CORP.",
    "00D013": "PRIMEX AEROSPACE COMPANY",
    "00D014": "ROOT, INC.",
    "00D015": "UNIVEX MICROTECHNOLOGY CORP.",
    "00D016": "SCM MICROSYSTEMS, INC.",
    "00D017": "SYNTECH INFORMATION CO., LTD.",
    "00D018": "QWES. COM, INC.",
    "00D019": "DAINIPPON SCREEN CORPORATE",
    "00D01A": "URMET TLC S.P.A.",
    "00D01B": "MIMAKI ENGINEERING CO., LTD.",
    "00D01C": "SBS TECHNOLOGIES,",
    "00D01D": "FURUNO ELECTRIC CO., LTD.",
    "00D01E": "PINGTEL CORP.",
    "00D01F": "CTAM PTY. LTD.",
    "00D020": "AIM SYSTEM, INC.",
    "00D021": "REGENT ELECTRONICS CORP.",
    "00D022": "INCREDIBLE TECHNOLOGIES, INC.",
    "00D023": "INFORTREND TECHNOLOGY, INC.",
    "00D024": "Cognex Corporation",
    "00D025": "XROSSTECH, INC.",
    "00D026": "HIRSCHMANN AUSTRIA GMBH",
    "00D027": "APPLIED AUTOMATION, INC.",
    "00D028": "OMNEON VIDEO NETWORKS",
    "00D029": "WAKEFERN FOOD CORPORATION",
    "00D02A": "Voxent Systems Ltd.",
    "00D02B": "JETCELL, INC.",
    "00D02C": "CAMPBELL SCIENTIFIC, INC.",
    "00D02D": "ADEMCO",
    "00D02E": "COMMUNICATION AUTOMATION CORP.",
    "00D02F": "VLSI TECHNOLOGY INC.",
    "00D030": "SAFETRAN SYSTEMS CORP.",
    "00D031": "INDUSTRIAL LOGIC CORPORATION",
    "00D032": "YANO ELECTRIC CO., LTD.",
    "00D033": "DALIAN DAXIAN NETWORK",
    "00D034": "ORMEC SYSTEMS CORP.",
    "00D035": "BEHAVIOR TECH. COMPUTER CORP.",
    "00D036": "TECHNOLOGY ATLANTA CORP.",
    "00D037": "Pace France",
    "00D038": "FIVEMERE, LTD.",
    "00D039": "UTILICOM, INC.",
    "00D03A": "ZONEWORX, INC.",
    "00D03B": "VISION PRODUCTS PTY. LTD.",
    "00D03C": "Vieo, Inc.",
    "00D03D": "GALILEO TECHNOLOGY, LTD.",
    "00D03E": "ROCKETCHIPS, INC.",
    "00D03F": "AMERICAN COMMUNICATION",
    "00D040": "SYSMATE CO., LTD.",
    "00D041": "AMIGO TECHNOLOGY CO., LTD.",
    "00D042": "MAHLO GMBH & CO. UG",
    "00D043": "ZONAL RETAIL DATA SYSTEMS",
    "00D044": "ALIDIAN NETWORKS, INC.",
    "00D045": "KVASER AB",
    "00D046": "DOLBY LABORATORIES, INC.",
    "00D047": "XN TECHNOLOGIES",
    "00D048": "ECTON, INC.",
    "00D049": "IMPRESSTEK CO., LTD.",
    "00D04A": "PRESENCE TECHNOLOGY GMBH",
    "00D04B": "LA CIE GROUP S.A.",
    "00D04C": "EUROTEL TELECOM LTD.",
    "00D04D": "DIV OF RESEARCH & STATISTICS",
    "00D04E": "LOGIBAG",
    "00D04F": "BITRONICS, INC.",
    "00D050": "ISKRATEL",
    "00D051": "O2 MICRO, INC.",
    "00D052": "ASCEND COMMUNICATIONS, INC.",
    "00D053": "CONNECTED SYSTEMS",
    "00D054": "SAS INSTITUTE INC.",
    "00D055": "KATHREIN-WERKE KG",
    "00D056": "SOMAT CORPORATION",
    "00D057": "ULTRAK, INC.",
    "00D058": "CISCO SYSTEMS, INC.",
    "00D059": "AMBIT MICROSYSTEMS CORP.",
    "00D05A": "SYMBIONICS, LTD.",
    "00D05B": "ACROLOOP MOTION CONTROL",
    "00D05C": "TECHNOTREND SYSTEMTECHNIK GMBH",
    "00D05D": "INTELLIWORXX, INC.",
    "00D05E": "STRATABEAM TECHNOLOGY, INC.",
    "00D05F": "VALCOM, INC.",
    "00D060": "Panasonic Europe Ltd.",
    "00D061": "TREMON ENTERPRISES CO., LTD.",
    "00D062": "DIGIGRAM",
    "00D063": "CISCO SYSTEMS, INC.",
    "00D064": "MULTITEL",
    "00D065": "TOKO ELECTRIC",
    "00D066": "WINTRISS ENGINEERING CORP.",
    "00D067": "CAMPIO COMMUNICATIONS",
    "00D068": "IWILL CORPORATION",
    "00D069": "TECHNOLOGIC SYSTEMS",
    "00D06A": "LINKUP SYSTEMS CORPORATION",
    "00D06B": "SR TELECOM INC.",
    "00D06C": "SHAREWAVE, INC.",
    "00D06D": "ACRISON, INC.",
    "00D06E": "TRENDVIEW RECORDERS LTD.",
    "00D06F": "KMC CONTROLS",
    "00D070": "LONG WELL ELECTRONICS CORP.",
    "00D071": "ECHELON CORP.",
    "00D072": "BROADLOGIC",
    "00D073": "ACN ADVANCED COMMUNICATIONS",
    "00D074": "TAQUA SYSTEMS, INC.",
    "00D075": "ALARIS MEDICAL SYSTEMS, INC.",
    "00D076": "Merrill Lynch & Co., Inc.",
    "00D077": "LUCENT TECHNOLOGIES",
    "00D078": "ELTEX OF SWEDEN AB",
    "00D079": "CISCO SYSTEMS, INC.",
    "00D07A": "AMAQUEST COMPUTER CORP.",
    "00D07B": "COMCAM INTERNATIONAL LTD.",
    "00D07C": "KOYO ELECTRONICS INC. CO.,LTD.",
    "00D07D": "COSINE COMMUNICATIONS",
    "00D07E": "KEYCORP LTD.",
    "00D07F": "STRATEGY & TECHNOLOGY, LIMITED",
    "00D080": "EXABYTE CORPORATION",
    "00D081": "RTD Embedded Technologies, Inc.",
    "00D082": "IOWAVE INC.",
    "00D083": "INVERTEX, INC.",
    "00D084": "NEXCOMM SYSTEMS, INC.",
    "00D085": "OTIS ELEVATOR COMPANY",
    "00D086": "FOVEON, INC.",
    "00D087": "MICROFIRST INC.",
    "00D088": "Motorola, Inc.",
    "00D089": "DYNACOLOR, INC.",
    "00D08A": "PHOTRON USA",
    "00D08B": "ADVA Optical Networking Ltd",
    "00D08C": "GENOA TECHNOLOGY, INC.",
    "00D08D": "PHOENIX GROUP, INC.",
    "00D08E": "NVISION INC.",
    "00D08F": "ARDENT TECHNOLOGIES, INC.",
    "00D090": "CISCO SYSTEMS, INC.",
    "00D091": "SMARTSAN SYSTEMS, INC.",
    "00D092": "GLENAYRE WESTERN MULTIPLEX",
    "00D093": "TQ - COMPONENTS GMBH",
    "00D094": "TIMELINE VISTA, INC.",
    "00D095": "Alcatel-Lucent, Enterprise Business Group",
    "00D096": "3COM EUROPE LTD.",
    "00D097": "CISCO SYSTEMS, INC.",
    "00D098": "Photon Dynamics Canada Inc.",
    "00D099": "ELCARD OY",
    "00D09A": "FILANET CORPORATION",
    "00D09B": "SPECTEL LTD.",
    "00D09C": "KAPADIA COMMUNICATIONS",
    "00D09D": "VERIS INDUSTRIES",
    "00D09E": "2WIRE, INC.",
    "00D09F": "NOVTEK TEST SYSTEMS",
    "00D0A0": "MIPS DENMARK",
    "00D0A1": "OSKAR VIERLING GMBH + CO. KG",
    "00D0A2": "INTEGRATED DEVICE",
    "00D0A3": "VOCAL DATA, INC.",
    "00D0A4": "ALANTRO COMMUNICATIONS",
    "00D0A5": "AMERICAN ARIUM",
    "00D0A6": "LANBIRD TECHNOLOGY CO., LTD.",
    "00D0A7": "TOKYO SOKKI KENKYUJO CO., LTD.",
    "00D0A8": "NETWORK ENGINES, INC.",
    "00D0A9": "SHINANO KENSHI CO., LTD.",
    "00D0AA": "CHASE COMMUNICATIONS",
    "00D0AB": "DELTAKABEL TELECOM CV",
    "00D0AC": "GRAYSON WIRELESS",
    "00D0AD": "TL INDUSTRIES",
    "00D0AE": "ORESIS COMMUNICATIONS, INC.",
    "00D0AF": "CUTLER-HAMMER, INC.",
    "00D0B0": "BITSWITCH LTD.",
    "00D0B1": "OMEGA ELECTRONICS SA",
    "00D0B2": "XIOTECH CORPORATION",
    "00D0B3": "DRS FLIGHT SAFETY AND",
    "00D0B4": "KATSUJIMA CO., LTD.",
    "00D0B5": "IPricot formerly DotCom",
    "00D0B6": "CRESCENT NETWORKS, INC.",
    "00D0B7": "INTEL CORPORATION",
    "00D0B8": "Iomega Corporation",
    "00D0B9": "MICROTEK INTERNATIONAL, INC.",
    "00D0BA": "CISCO SYSTEMS, INC.",
    "00D0BB": "CISCO SYSTEMS, INC.",
    "00D0BC": "CISCO SYSTEMS, INC.",
    "00D0BD": "Silicon Image GmbH",
    "00D0BE": "EMUTEC INC.",
    "00D0BF": "PIVOTAL TECHNOLOGIES",
    "00D0C0": "CISCO SYSTEMS, INC.",
    "00D0C1": "HARMONIC DATA SYSTEMS, LTD.",
    "00D0C2": "BALTHAZAR TECHNOLOGY AB",
    "00D0C3": "VIVID TECHNOLOGY PTE, LTD.",
    "00D0C4": "TERATECH CORPORATION",
    "00D0C5": "COMPUTATIONAL SYSTEMS, INC.",
    "00D0C6": "THOMAS & BETTS CORP.",
    "00D0C7": "PATHWAY, INC.",
    "00D0C8": "I/O CONSULTING A/S",
    "00D0C9": "ADVANTECH CO., LTD.",
    "00D0CA": "Intrinsyc Software int'l, Inc.",
    "00D0CB": "DASAN CO., LTD.",
    "00D0CC": "TECHNOLOGIES LYRE INC.",
    "00D0CD": "ATAN TECHNOLOGY INC.",
    "00D0CE": "ASYST ELECTRONIC",
    "00D0CF": "MORETON BAY",
    "00D0D0": "ZHONGXING TELECOM LTD.",
    "00D0D1": "Sycamore Networks",
    "00D0D2": "EPILOG CORPORATION",
    "00D0D3": "CISCO SYSTEMS, INC.",
    "00D0D4": "V-BITS, INC.",
    "00D0D5": "GRUNDIG AG",
    "00D0D6": "AETHRA TELECOMUNICAZIONI",
    "00D0D7": "B2C2, INC.",
    "00D0D8": "3Com Corporation",
    "00D0D9": "DEDICATED MICROCOMPUTERS",
    "00D0DA": "TAICOM DATA SYSTEMS CO., LTD.",
    "00D0DB": "MCQUAY INTERNATIONAL",
    "00D0DC": "MODULAR MINING SYSTEMS, INC.",
    "00D0DD": "SUNRISE TELECOM, INC.",
    "00D0DE": "PHILIPS MULTIMEDIA NETWORK",
    "00D0DF": "KUZUMI ELECTRONICS, INC.",
    "00D0E0": "DOOIN ELECTRONICS CO.",
    "00D0E1": "AVIONITEK ISRAEL INC.",
    "00D0E2": "MRT MICRO, INC.",
    "00D0E3": "ELE-CHEM ENGINEERING CO., LTD.",
    "00D0E4": "CISCO SYSTEMS, INC.",
    "00D0E5": "SOLIDUM SYSTEMS CORP.",
    "00D0E6": "IBOND INC.",
    "00D0E7": "VCON TELECOMMUNICATION LTD.",
    "00D0E8": "MAC SYSTEM CO., LTD.",
    "00D0E9": "Advantage Century Telecommunication Corp.",
    "00D0EA": "NEXTONE COMMUNICATIONS, INC.",
    "00D0EB": "LIGHTERA NETWORKS, INC.",
    "00D0EC": "NAKAYO TELECOMMUNICATIONS, INC",
    "00D0ED": "XIOX",
    "00D0EE": "DICTAPHONE CORPORATION",
    "00D0EF": "IGT",
    "00D0F0": "CONVISION TECHNOLOGY GMBH",
    "00D0F1": "SEGA ENTERPRISES, LTD.",
    "00D0F2": "MONTEREY NETWORKS",
    "00D0F3": "SOLARI DI UDINE SPA",
    "00D0F4": "CARINTHIAN TECH INSTITUTE",
    "00D0F5": "ORANGE MICRO, INC.",
    "00D0F6": "Alcatel Canada",
    "00D0F7": "NEXT NETS CORPORATION",
    "00D0F8": "FUJIAN STAR TERMINAL",
    "00D0F9": "ACUTE COMMUNICATIONS CORP.",
    "00D0FA": "Thales e-Security Ltd.",
    "00D0FB": "TEK MICROSYSTEMS, INCORPORATED",
    "00D0FC": "GRANITE MICROSYSTEMS",
    "00D0FD": "OPTIMA TELE.COM, INC.",
    "00D0FE": "ASTRAL POINT",
    "00D0FF": "CISCO SYSTEMS, INC.",
    "00DD00": "UNGERMANN-BASS INC.",
    "00DD01": "UNGERMANN-BASS INC.",
    "00DD02": "UNGERMANN-BASS INC.",
    "00DD03": "UNGERMANN-BASS INC.",
    "00DD04": "UNGERMANN-BASS INC.",
    "00DD05": "UNGERMANN-BASS INC.",
    "00DD06": "UNGERMANN-BASS INC.",
    "00DD07": "UNGERMANN-BASS INC.",
    "00DD08": "UNGERMANN-BASS INC.",
    "00DD09": "UNGERMANN-BASS INC.",
    "00DD0A": "UNGERMANN-BASS INC.",
    "00DD0B": "UNGERMANN-BASS INC.",
    "00DD0C": "UNGERMANN-BASS INC.",
    "00DD0D": "UNGERMANN-BASS INC.",
    "00DD0E": "UNGERMANN-BASS INC.",
    "00DD0F": "UNGERMANN-BASS INC.",
    "00E000": "FUJITSU, LTD",
    "00E001": "STRAND LIGHTING LIMITED",
    "00E002": "CROSSROADS SYSTEMS, INC.",
    "00E003": "NOKIA WIRELESS BUSINESS COMMUN",
    "00E004": "PMC-SIERRA, INC.",
    "00E005": "TECHNICAL CORP.",
    "00E006": "SILICON INTEGRATED SYS. CORP.",
    "00E007": "NETWORK ALCHEMY LTD.",
    "00E008": "AMAZING CONTROLS! INC.",
    "00E009": "MARATHON TECHNOLOGIES CORP.",
    "00E00A": "DIBA, INC.",
    "00E00B": "ROOFTOP COMMUNICATIONS CORP.",
    "00E00C": "MOTOROLA",
    "00E00D": "RADIANT SYSTEMS",
    "00E00E": "AVALON IMAGING SYSTEMS, INC.",
    "00E00F": "SHANGHAI BAUD DATA",
    "00E010": "HESS SB-AUTOMATENBAU GmbH",
    "00E011": "UNIDEN SAN DIEGO R&D CENTER, INC.",
    "00E012": "PLUTO TECHNOLOGIES INTERNATIONAL INC.",
    "00E013": "EASTERN ELECTRONIC CO., LTD.",
    "00E014": "CISCO SYSTEMS, INC.",
    "00E015": "HEIWA CORPORATION",
    "00E016": "RAPID CITY COMMUNICATIONS",
    "00E017": "EXXACT GmbH",
    "00E018": "ASUSTEK COMPUTER INC.",
    "00E019": "ING. GIORDANO ELETTRONICA",
    "00E01A": "COMTEC SYSTEMS. CO., LTD.",
    "00E01B": "SPHERE COMMUNICATIONS, INC.",
    "00E01C": "Cradlepoint, Inc",
    "00E01D": "WebTV NETWORKS, INC.",
    "00E01E": "CISCO SYSTEMS, INC.",
    "00E01F": "AVIDIA Systems, Inc.",
    "00E020": "TECNOMEN OY",
    "00E021": "FREEGATE CORP.",
    "00E022": "Analog Devices Inc.",
    "00E023": "TELRAD",
    "00E024": "GADZOOX NETWORKS",
    "00E025": "dit CO., LTD.",
    "00E026": "Redlake MASD LLC",
    "00E027": "DUX, INC.",
    "00E028": "APTIX CORPORATION",
    "00E029": "STANDARD MICROSYSTEMS CORP.",
    "00E02A": "TANDBERG TELEVISION AS",
    "00E02B": "EXTREME NETWORKS",
    "00E02C": "AST COMPUTER",
    "00E02D": "InnoMediaLogic, Inc.",
    "00E02E": "SPC ELECTRONICS CORPORATION",
    "00E02F": "MCNS HOLDINGS, L.P.",
    "00E030": "MELITA INTERNATIONAL CORP.",
    "00E031": "HAGIWARA ELECTRIC CO., LTD.",
    "00E032": "MISYS FINANCIAL SYSTEMS, LTD.",
    "00E033": "E.E.P.D. GmbH",
    "00E034": "CISCO SYSTEMS, INC.",
    "00E035": "Emerson Network Power",
    "00E036": "PIONEER CORPORATION",
    "00E037": "CENTURY CORPORATION",
    "00E038": "PROXIMA CORPORATION",
    "00E039": "PARADYNE CORP.",
    "00E03A": "CABLETRON SYSTEMS, INC.",
    "00E03B": "PROMINET CORPORATION",
    "00E03C": "AdvanSys",
    "00E03D": "FOCON ELECTRONIC SYSTEMS A/S",
    "00E03E": "ALFATECH, INC.",
    "00E03F": "JATON CORPORATION",
    "00E040": "DeskStation Technology, Inc.",
    "00E041": "CSPI",
    "00E042": "Pacom Systems Ltd.",
    "00E043": "VitalCom",
    "00E044": "LSICS CORPORATION",
    "00E045": "TOUCHWAVE, INC.",
    "00E046": "BENTLY NEVADA CORP.",
    "00E047": "INFOCUS SYSTEMS",
    "00E048": "SDL COMMUNICATIONS, INC.",
    "00E049": "MICROWI ELECTRONIC GmbH",
    "00E04A": "ENHANCED MESSAGING SYSTEMS, INC",
    "00E04B": "JUMP INDUSTRIELLE COMPUTERTECHNIK GmbH",
    "00E04C": "REALTEK SEMICONDUCTOR CORP.",
    "00E04D": "INTERNET INITIATIVE JAPAN, INC",
    "00E04E": "SANYO DENKI CO., LTD.",
    "00E04F": "CISCO SYSTEMS, INC.",
    "00E050": "EXECUTONE INFORMATION SYSTEMS, INC.",
    "00E051": "TALX CORPORATION",
    "00E052": "FOUNDRY NETWORKS, INC.",
    "00E053": "CELLPORT LABS, INC.",
    "00E054": "KODAI HITEC CO., LTD.",
    "00E055": "INGENIERIA ELECTRONICA COMERCIAL INELCOM S.A.",
    "00E056": "HOLONTECH CORPORATION",
    "00E057": "HAN MICROTELECOM. CO., LTD.",
    "00E058": "PHASE ONE DENMARK A/S",
    "00E059": "CONTROLLED ENVIRONMENTS, LTD.",
    "00E05A": "GALEA NETWORK SECURITY",
    "00E05B": "WEST END SYSTEMS CORP.",
    "00E05C": "MATSUSHITA KOTOBUKI ELECTRONICS INDUSTRIES, LTD.",
    "00E05D": "UNITEC CO., LTD.",
    "00E05E": "JAPAN AVIATION ELECTRONICS INDUSTRY, LTD.",
    "00E05F": "e-Net, Inc.",
    "00E060": "SHERWOOD",
    "00E061": "EdgePoint Networks, Inc.",
    "00E062": "HOST ENGINEERING",
    "00E063": "CABLETRON - YAGO SYSTEMS, INC.",
    "00E064": "SAMSUNG ELECTRONICS",
    "00E065": "OPTICAL ACCESS INTERNATIONAL",
    "00E066": "ProMax Systems, Inc.",
    "00E067": "eac AUTOMATION-CONSULTING GmbH",
    "00E068": "MERRIMAC SYSTEMS INC.",
    "00E069": "JAYCOR",
    "00E06A": "KAPSCH AG",
    "00E06B": "W&G SPECIAL PRODUCTS",
    "00E06C": "AEP Systems International Ltd",
    "00E06D": "COMPUWARE CORPORATION",
    "00E06E": "FAR SYSTEMS S.p.A.",
    "00E06F": "Motorola, Inc.",
    "00E070": "DH TECHNOLOGY",
    "00E071": "EPIS MICROCOMPUTER",
    "00E072": "LYNK",
    "00E073": "NATIONAL AMUSEMENT NETWORK, INC.",
    "00E074": "TIERNAN COMMUNICATIONS, INC.",
    "00E075": "Verilink Corporation",
    "00E076": "DEVELOPMENT CONCEPTS, INC.",
    "00E077": "WEBGEAR, INC.",
    "00E078": "BERKELEY NETWORKS",
    "00E079": "A.T.N.R.",
    "00E07A": "MIKRODIDAKT AB",
    "00E07B": "BAY NETWORKS",
    "00E07C": "METTLER-TOLEDO, INC.",
    "00E07D": "NETRONIX, INC.",
    "00E07E": "WALT DISNEY IMAGINEERING",
    "00E07F": "LOGISTISTEM s.r.l.",
    "00E080": "CONTROL RESOURCES CORPORATION",
    "00E081": "TYAN COMPUTER CORP.",
    "00E082": "ANERMA",
    "00E083": "JATO TECHNOLOGIES, INC.",
    "00E084": "COMPULITE R&D",
    "00E085": "GLOBAL MAINTECH, INC.",
    "00E086": "CYBEX COMPUTER PRODUCTS",
    "00E087": "LeCroy - Networking Productions Division",
    "00E088": "LTX CORPORATION",
    "00E089": "ION Networks, Inc.",
    "00E08A": "GEC AVERY, LTD.",
    "00E08B": "QLogic Corp.",
    "00E08C": "NEOPARADIGM LABS, INC.",
    "00E08D": "PRESSURE SYSTEMS, INC.",
    "00E08E": "UTSTARCOM",
    "00E08F": "CISCO SYSTEMS, INC.",
    "00E090": "BECKMAN LAB. AUTOMATION DIV.",
    "00E091": "LG ELECTRONICS, INC.",
    "00E092": "ADMTEK INCORPORATED",
    "00E093": "ACKFIN NETWORKS",
    "00E094": "OSAI SRL",
    "00E095": "ADVANCED-VISION TECHNOLGIES CORP.",
    "00E096": "SHIMADZU CORPORATION",
    "00E097": "CARRIER ACCESS CORPORATION",
    "00E098": "AboCom Systems, Inc.",
    "00E099": "SAMSON AG",
    "00E09A": "POSITRON INDUSTRIES, INC.",
    "00E09B": "ENGAGE NETWORKS, INC.",
    "00E09C": "MII",
    "00E09D": "SARNOFF CORPORATION",
    "00E09E": "QUANTUM CORPORATION",
    "00E09F": "PIXEL VISION",
    "00E0A0": "WILTRON CO.",
    "00E0A1": "HIMA PAUL HILDEBRANDT GmbH Co. KG",
    "00E0A2": "MICROSLATE INC.",
    "00E0A3": "CISCO SYSTEMS, INC.",
    "00E0A4": "ESAOTE S.p.A.",
    "00E0A5": "ComCore Semiconductor, Inc.",
    "00E0A6": "TELOGY NETWORKS, INC.",
    "00E0A7": "IPC INFORMATION SYSTEMS, INC.",
    "00E0A8": "SAT GmbH & Co.",
    "00E0A9": "FUNAI ELECTRIC CO., LTD.",
    "00E0AA": "ELECTROSONIC LTD.",
    "00E0AB": "DIMAT S.A.",
    "00E0AC": "MIDSCO, INC.",
    "00E0AD": "EES TECHNOLOGY, LTD.",
    "00E0AE": "XAQTI CORPORATION",
    "00E0AF": "GENERAL DYNAMICS INFORMATION SYSTEMS",
    "00E0B0": "CISCO SYSTEMS, INC.",
    "00E0B1": "Alcatel-Lucent, Enterprise Business Group",
    "00E0B2": "TELMAX COMMUNICATIONS CORP.",
    "00E0B3": "EtherWAN Systems, Inc.",
    "00E0B4": "TECHNO SCOPE CO., LTD.",
    "00E0B5": "ARDENT COMMUNICATIONS CORP.",
    "00E0B6": "Entrada Networks",
    "00E0B7": "PI GROUP, LTD.",
    "00E0B8": "GATEWAY 2000",
    "00E0B9": "BYAS SYSTEMS",
    "00E0BA": "BERGHOF AUTOMATIONSTECHNIK GmbH",
    "00E0BB": "NBX CORPORATION",
    "00E0BC": "SYMON COMMUNICATIONS, INC.",
    "00E0BD": "INTERFACE SYSTEMS, INC.",
    "00E0BE": "GENROCO INTERNATIONAL, INC.",
    "00E0BF": "TORRENT NETWORKING TECHNOLOGIES CORP.",
    "00E0C0": "SEIWA ELECTRIC MFG. CO., LTD.",
    "00E0C1": "MEMOREX TELEX JAPAN, LTD.",
    "00E0C2": "NECSY S.p.A.",
    "00E0C3": "SAKAI SYSTEM DEVELOPMENT CORP.",
    "00E0C4": "HORNER ELECTRIC, INC.",
    "00E0C5": "BCOM ELECTRONICS INC.",
    "00E0C6": "LINK2IT, L.L.C.",
    "00E0C7": "EUROTECH SRL",
    "00E0C8": "VIRTUAL ACCESS, LTD.",
    "00E0C9": "AutomatedLogic Corporation",
    "00E0CA": "BEST DATA PRODUCTS",
    "00E0CB": "RESON, INC.",
    "00E0CC": "HERO SYSTEMS, LTD.",
    "00E0CD": "SENSIS CORPORATION",
    "00E0CE": "ARN",
    "00E0CF": "INTEGRATED DEVICE TECHNOLOGY, INC.",
    "00E0D0": "NETSPEED, INC.",
    "00E0D1": "TELSIS LIMITED",
    "00E0D2": "VERSANET COMMUNICATIONS, INC.",
    "00E0D3": "DATENTECHNIK GmbH",
    "00E0D4": "EXCELLENT COMPUTER",
    "00E0D5": "Emulex Corporation",
    "00E0D6": "COMPUTER & COMMUNICATION RESEARCH LAB.",
    "00E0D7": "SUNSHINE ELECTRONICS, INC.",
    "00E0D8": "LANBit Computer, Inc.",
    "00E0D9": "TAZMO CO., LTD.",
    "00E0DA": "Alcatel North America ESD",
    "00E0DB": "ViaVideo Communications, Inc.",
    "00E0DC": "NEXWARE CORP.",
    "00E0DD": "ZENITH ELECTRONICS CORPORATION",
    "00E0DE": "DATAX NV",
    "00E0DF": "KE KOMMUNIKATIONS-ELECTRONIK",
    "00E0E0": "SI ELECTRONICS, LTD.",
    "00E0E1": "G2 NETWORKS, INC.",
    "00E0E2": "INNOVA CORP.",
    "00E0E3": "SK-ELEKTRONIK GmbH",
    "00E0E4": "FANUC ROBOTICS NORTH AMERICA, Inc.",
    "00E0E5": "CINCO NETWORKS, INC.",
    "00E0E6": "INCAA DATACOM B.V.",
    "00E0E7": "RAYTHEON E-SYSTEMS, INC.",
    "00E0E8": "GRETACODER Data Systems AG",
    "00E0E9": "DATA LABS, INC.",
    "00E0EA": "INNOVAT COMMUNICATIONS, INC.",
    "00E0EB": "DIGICOM SYSTEMS, INCORPORATED",
    "00E0EC": "CELESTICA INC.",
    "00E0ED": "SILICOM, LTD.",
    "00E0EE": "MAREL HF",
    "00E0EF": "DIONEX",
    "00E0F0": "ABLER TECHNOLOGY, INC.",
    "00E0F1": "THAT CORPORATION",
    "00E0F2": "ARLOTTO COMNET, INC.",
    "00E0F3": "WebSprint Communications, Inc.",
    "00E0F4": "INSIDE Technology A/S",
    "00E0F5": "TELES AG",
    "00E0F6": "DECISION EUROPE",
    "00E0F7": "CISCO SYSTEMS, INC.",
    "00E0F8": "DICNA CONTROL AB",
    "00E0F9": "CISCO SYSTEMS, INC.",
    "00E0FA": "TRL TECHNOLOGY, LTD.",
    "00E0FB": "LEIGHTRONIX, INC.",
    "00E0FC": "HUAWEI TECHNOLOGIES CO., LTD.",
    "00E0FD": "A-TREND TECHNOLOGY CO., LTD.",
    "00E0FE": "CISCO SYSTEMS, INC.",
    "00E0FF": "SECURITY DYNAMICS TECHNOLOGIES, Inc.",
    "00E6D3": "NIXDORF COMPUTER CORP.",
    "020701": "RACAL-DATACOM",
    "021C7C": "PERQ SYSTEMS CORPORATION",
    "026086": "LOGIC REPLACEMENT TECH. LTD.",
    "02608C": "3COM CORPORATION",
    "027001": "RACAL-DATACOM",
    "0270B0": "M/A-COM INC. COMPANIES",
    "0270B3": "DATA RECALL LTD",
    "029D8E": "CARDIAC RECORDERS INC.",
    "02AA3C": "OLIVETTI TELECOMM SPA (OLTECO)",
    "02BB01": "OCTOTHORPE CORP.",
    "02C08C": "3COM CORPORATION",
    "02CF1C": "COMMUNICATION MACHINERY CORP.",
    "02E6D3": "NIXDORF COMPUTER CORPORATION",
    "040AE0": "XMIT AG COMPUTER NETWORKS",
    "04E0C4": "TRIUMPH-ADLER AG",
    "080001": "COMPUTERVISION CORPORATION",
    "080002": "BRIDGE COMMUNICATIONS INC.",
    "080003": "ADVANCED COMPUTER COMM.",
    "080004": "CROMEMCO INCORPORATED",
    "080005": "SYMBOLICS INC.",
    "080006": "SIEMENS AG",
    "080007": "APPLE COMPUTER INC.",
    "080008": "BOLT BERANEK AND NEWMAN INC.",
    "080009": "HEWLETT PACKARD",
    "08000A": "NESTAR SYSTEMS INCORPORATED",
    "08000B": "UNISYS CORPORATION",
    "08000C": "MIKLYN DEVELOPMENT CO.",
    "08000D": "INTERNATIONAL COMPUTERS LTD.",
    "08000E": "NCR CORPORATION",
    "08000F": "MITEL CORPORATION",
    "080011": "TEKTRONIX INC.",
    "080012": "BELL ATLANTIC INTEGRATED SYST.",
    "080013": "EXXON",
    "080014": "EXCELAN",
    "080015": "STC BUSINESS SYSTEMS",
    "080016": "BARRISTER INFO SYS CORP",
    "080017": "NATIONAL SEMICONDUCTOR",
    "080018": "PIRELLI FOCOM NETWORKS",
    "080019": "GENERAL ELECTRIC CORPORATION",
    "08001A": "TIARA/ 10NET",
    "08001B": "EMC Corporation",
    "08001C": "KDD-KOKUSAI DEBNSIN DENWA CO.",
    "08001D": "ABLE COMMUNICATIONS INC.",
    "08001E": "APOLLO COMPUTER INC.",
    "08001F": "SHARP CORPORATION",
    "080020": "SUN MICROSYSTEMS INC.",
    "080021": "3M COMPANY",
    "080022": "NBI INC.",
    "080023": "Panasonic Communications Co., Ltd.",
    "080024": "10NET COMMUNICATIONS/DCA",
    "080025": "CONTROL DATA",
    "080026": "NORSK DATA A.S.",
    "080027": "CADMUS COMPUTER SYSTEMS",
    "080028": "Texas Instruments",
    "080029": "MEGATEK CORPORATION",
    "08002A": "MOSAIC TECHNOLOGIES INC.",
    "08002B": "DIGITAL EQUIPMENT CORPORATION",
    "08002C": "BRITTON LEE INC.",
    "08002D": "LAN-TEC INC.",
    "08002E": "METAPHOR COMPUTER SYSTEMS",
    "08002F": "PRIME COMPUTER INC.",
    "080030": "NETWORK RESEARCH CORPORATION",
    "080030": "CERN",
    "080030": "ROYAL MELBOURNE INST OF TECH",
    "080031": "LITTLE MACHINES INC.",
    "080032": "TIGAN INCORPORATED",
    "080033": "BAUSCH & LOMB",
    "080034": "FILENET CORPORATION",
    "080035": "MICROFIVE CORPORATION",
    "080036": "INTERGRAPH CORPORATION",
    "080037": "FUJI-XEROX CO. LTD.",
    "080038": "CII HONEYWELL BULL",
    "080039": "SPIDER SYSTEMS LIMITED",
    "08003A": "ORCATECH INC.",
    "08003B": "TORUS SYSTEMS LIMITED",
    "08003C": "SCHLUMBERGER WELL SERVICES",
    "08003D": "CADNETIX CORPORATIONS",
    "08003E": "CODEX CORPORATION",
    "08003F": "FRED KOSCHARA ENTERPRISES",
    "080040": "FERRANTI COMPUTER SYS. LIMITED",
    "080041": "RACAL-MILGO INFORMATION SYS..",
    "080042": "JAPAN MACNICS CORP.",
    "080043": "PIXEL COMPUTER INC.",
    "080044": "DAVID SYSTEMS INC.",
    "080045": "CONCURRENT COMPUTER CORP.",
    "080046": "SONY CORPORATION LTD.",
    "080047": "SEQUENT COMPUTER SYSTEMS INC.",
    "080048": "EUROTHERM GAUGING SYSTEMS",
    "080049": "UNIVATION",
    "08004A": "BANYAN SYSTEMS INC.",
    "08004B": "PLANNING RESEARCH CORP.",
    "08004C": "HYDRA COMPUTER SYSTEMS INC.",
    "08004D": "CORVUS SYSTEMS INC.",
    "08004E": "3COM EUROPE LTD.",
    "08004F": "CYGNET SYSTEMS",
    "080050": "DAISY SYSTEMS CORP.",
    "080051": "EXPERDATA",
    "080052": "INSYSTEC",
    "080053": "MIDDLE EAST TECH. UNIVERSITY",
    "080055": "STANFORD TELECOMM. INC.",
    "080056": "STANFORD LINEAR ACCEL. CENTER",
    "080057": "EVANS & SUTHERLAND",
    "080058": "SYSTEMS CONCEPTS",
    "080059": "A/S MYCRON",
    "08005A": "IBM CORPORATION",
    "08005B": "VTA TECHNOLOGIES INC.",
    "08005C": "FOUR PHASE SYSTEMS",
    "08005D": "GOULD INC.",
    "08005E": "COUNTERPOINT COMPUTER INC.",
    "08005F": "SABER TECHNOLOGY CORP.",
    "080060": "INDUSTRIAL NETWORKING INC.",
    "080061": "JAROGATE LTD.",
    "080062": "GENERAL DYNAMICS",
    "080063": "PLESSEY",
    "080064": "AUTOPHON AG",
    "080065": "GENRAD INC.",
    "080066": "AGFA CORPORATION",
    "080067": "COMDESIGN",
    "080068": "RIDGE COMPUTERS",
    "080069": "SILICON GRAPHICS INC.",
    "08006A": "ATT BELL LABORATORIES",
    "08006B": "ACCEL TECHNOLOGIES INC.",
    "08006C": "SUNTEK TECHNOLOGY INT'L",
    "08006D": "WHITECHAPEL COMPUTER WORKS",
    "08006E": "MASSCOMP",
    "08006F": "PHILIPS APELDOORN B.V.",
    "080070": "MITSUBISHI ELECTRIC CORP.",
    "080071": "MATRA (DSIE)",
    "080072": "XEROX CORP UNIV GRANT PROGRAM",
    "080073": "TECMAR INC.",
    "080074": "CASIO COMPUTER CO. LTD.",
    "080075": "DANSK DATA ELECTRONIK",
    "080076": "PC LAN TECHNOLOGIES",
    "080077": "TSL COMMUNICATIONS LTD.",
    "080078": "ACCELL CORPORATION",
    "080079": "THE DROID WORKS",
    "08007A": "INDATA",
    "08007B": "SANYO ELECTRIC CO. LTD.",
    "08007C": "VITALINK COMMUNICATIONS CORP.",
    "08007E": "AMALGAMATED WIRELESS(AUS) LTD",
    "08007F": "CARNEGIE-MELLON UNIVERSITY",
    "080080": "AES DATA INC.",
    "080081": ",ASTECH INC.",
    "080082": "VERITAS SOFTWARE",
    "080083": "Seiko Instruments Inc.",
    "080084": "TOMEN ELECTRONICS CORP.",
    "080085": "ELXSI",
    "080086": "KONICA MINOLTA HOLDINGS, INC.",
    "080087": "XYPLEX",
    "080088": "Brocade Communications Systems, Inc.",
    "080089": "KINETICS",
    "08008A": "PERFORMANCE TECHNOLOGY",
    "08008B": "PYRAMID TECHNOLOGY CORP.",
    "08008C": "NETWORK RESEARCH CORPORATION",
    "08008D": "XYVISION INC.",
    "08008E": "TANDEM COMPUTERS",
    "08008F": "CHIPCOM CORPORATION",
    "080090": "SONOMA SYSTEMS",
    "081443": "UNIBRAIN S.A.",
    "08BBCC": "AK-NORD EDV VERTRIEBSGES. mbH",
    "100000": "PRIVATE",
    "10005A": "IBM CORPORATION",
    "1000E8": "NATIONAL SEMICONDUCTOR",
    "1100AA": "PRIVATE",
    "800010": "ATT BELL LABORATORIES",
    "A06A00": "Verilink Corporation",
    "AA0000": "DIGITAL EQUIPMENT CORPORATION",
    "AA0001": "DIGITAL EQUIPMENT CORPORATION",
    "AA0002": "DIGITAL EQUIPMENT CORPORATION",
    "AA0003": "DIGITAL EQUIPMENT CORPORATION",
    "AA0004": "DIGITAL EQUIPMENT CORPORATION",
    "ACDE48": "PRIVATE",
}

from gozerbot.plughelp import plughelp
from gozerbot.commands import cmnds
from gozerbot.examples import examples

plughelp.add('mac', 'find manifacturer of network card')

def handle_mac(bot, ievent):
    mac = ' '.join(ievent.args).replace(' ', '').replace(':', '').replace('-', '').upper()
    if len(mac) < 6:
        ievent.reply('invalid mac address')
    else:
        part = mac[:6]
        if part in vendors:
            ievent.reply(vendors[part])
        else:
            ievent.reply('not found')

cmnds.add('mac', handle_mac, 'USER')
examples.add('mac', 'find manufacturer of network card', 'mac AA0003')
